@@ -44,6 +44,14 @@ DIE=0
      }
 }
 
+(pkg-config --version) < /dev/null > /dev/null 2>&1 || {
+        echo
+        echo "You must have pkg-config installed to compile $package."
+        echo "Download the appropriate package for your distribution."
+               result="no"
+        DIE=1
+}
+
 (automake --version) < /dev/null > /dev/null 2>&1 || {
 	echo
 	echo "**Error**: You must have \`automake' (1.7 or later) installed."
@@ -83,6 +91,10 @@ autoheader
 automake --add-missing --copy --gnu
 autoconf
 
+if [ "$NOCONFIGURE" = 1 ]; then
+    echo "Done. configure skipped."
+    exit 0;
+fi
 echo "Running $srcdir/configure $@ ..."
 $srcdir/configure "$@" && echo "Now type 'make' to compile." || exit 1
 
