@@ -2,13 +2,20 @@
 [styling]
 # styling for PHP/HTML is done in filetypes.xml
 
+[lexer_properties]
+phpscript.mode=1
 
 [settings]
+# default extension used when saving files
+extension=php
+
 # the following characters are these which a "word" can contains, see documentation
 #wordchars=_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789
 
-# if only single comment char is supported like # in this file, leave comment_close blank
 # these comments are used for PHP, the comments used in HTML are in filetypes.xml
+# single comments, like # in this file
+comment_single=//
+# multiline comments
 comment_open=/*
 comment_close=*/
 
@@ -23,6 +30,15 @@ comment_use_indent=true
 # context action command (please see Geany's main documentation for details)
 context_action_cmd=
 
+# if this setting is set to true, a new line after a line ending with an
+# unclosed tag will be automatically indented
+xml_indent_tags=true
+
+[indentation]
+#width=4
+# 0 is spaces, 1 is tabs, 2 is tab & spaces
+#type=1
+
 [build_settings]
 # %f will be replaced by the complete filename
 # %e will be replaced by the filename without extension
