@@ -12,6 +12,9 @@ identifier=0x000000;0xffffff;false;false
 backticks=0x000000;0xd0d0d0;false;false
 param=0x009f00;0xffffff;false;false
 scalar=0x105090;0xffffff;false;false
+error=0xff0000;0xffffff;false;false
+here_delim=0x000000;0xddd0dd;false;false
+here_q=0x7f007f;0xddd0dd;false;false
 
 [keywords]
 primary=break case continue do done elif else esac eval exit export fi for function goto if in integer return set shift then until while
@@ -19,14 +22,16 @@ primary=break case continue do done elif else esac eval exit export fi for funct
 
 [settings]
 # default extension used when saving files
-#extension=sh
+extension=sh
 
 # the following characters are these which a "word" can contains, see documentation
 #wordchars=_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789
 
-# if only single comment char is supported like # in this file, leave comment_close blank
-comment_open=#
-comment_close=
+# single comments, like # in this file
+comment_single=#
+# multiline comments
+#comment_open=
+#comment_close=
 
 # set to false if a comment character/string should start a column 0 of a line, true uses any
 # indentation of the line, e.g. setting to true causes the following on pressing CTRL+d
@@ -39,6 +44,11 @@ comment_use_indent=true
 # context action command (please see Geany's main documentation for details)
 context_action_cmd=
 
+[indentation]
+#width=4
+# 0 is spaces, 1 is tabs, 2 is tab & spaces
+#type=1
+
 [build_settings]
 # %f will be replaced by the complete filename
 # %e will be replaced by the filename without extension
