@@ -1,8 +1,8 @@
 /*
  *      pluginsignals.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2008-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2008-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2008-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2008-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: pluginsignals.c 5235 2010-09-16 15:14:09Z ntrel $
+ * $Id: pluginsignals.c 5710 2011-04-11 14:40:27Z ntrel $
  */
 
 /* Note: this file is for Doxygen only. */
@@ -63,8 +63,6 @@ PluginCallback plugin_callbacks[] =
  */
 
 /** Sent when a new document is created.
- *
- *  You need to include "document.h" for the declaration of GeanyDocument.
  *
  *  @param obj a GeanyObject instance, should be ignored.
  *  @param doc the new document.
@@ -73,8 +71,6 @@ PluginCallback plugin_callbacks[] =
 signal void (*document_new)(GObject *obj, GeanyDocument *doc, gpointer user_data);
 
 /** Sent when a new document is opened.
- *
- *  You need to include "document.h" for the declaration of GeanyDocument.
  *
  *  @param obj a GeanyObject instance, should be ignored.
  *  @param doc the opened document.
@@ -82,9 +78,17 @@ signal void (*document_new)(GObject *obj, GeanyDocument *doc, gpointer user_data
  */
 signal void (*document_open)(GObject *obj, GeanyDocument *doc, gpointer user_data);
 
-/** Sent before a document is saved.
+/** Sent when an existing document is reloaded.
  *
- *  You need to include "document.h" for the declaration of GeanyDocument.
+ *  @param obj a GeanyObject instance, should be ignored.
+ *  @param doc the re-opened document.
+ *  @param user_data user data.
+ *
+ *  @since 0.21
+ */
+signal void (*document_reload)(GObject *obj, GeanyDocument *doc, gpointer user_data);
+
+/** Sent before a document is saved.
  *
  *  @param obj a GeanyObject instance, should be ignored.
  *  @param doc the document to be saved.
@@ -93,8 +97,6 @@ signal void (*document_open)(GObject *obj, GeanyDocument *doc, gpointer user_dat
 signal void (*document_before_save)(GObject *obj, GeanyDocument *doc, gpointer user_data);
 
 /** Sent when a new document is saved.
- *
- *  You need to include "document.h" for the declaration of GeanyDocument.
  *
  *  @param obj a GeanyObject instance, should be ignored.
  *  @param doc the saved document.
@@ -117,8 +119,6 @@ signal void (*document_save)(GObject *obj, GeanyDocument *doc, gpointer user_dat
 signal void (*document_filetype_set)(GObject *obj, GeanyDocument *doc, GeanyFiletype *filetype_old, gpointer user_data);
 
 /** Sent when switching notebook pages.
- *
- *  You need to include "document.h" for the declaration of GeanyDocument.
  *
  *  @param obj a GeanyObject instance, should be ignored.
  *  @param doc the current document.
@@ -127,8 +127,6 @@ signal void (*document_filetype_set)(GObject *obj, GeanyDocument *doc, GeanyFile
 signal void (*document_activate)(GObject *obj, GeanyDocument *doc, gpointer user_data);
 
 /** Sent before closing a document.
- *
- *  You need to include "document.h" for the declaration of GeanyDocument.
  *
  *  @param obj a GeanyObject instance, should be ignored.
  *  @param doc the document about to be closed.
@@ -203,8 +201,6 @@ signal void (*build_start)(GObject *obj, gpointer user_data);
  *  @note You can add menu items from @c plugin_init() using @c geany->main_widgets->editor_menu,
  *  remembering to destroy them in @c plugin_cleanup().
  *
- *  You need to include "document.h" for the declaration of GeanyDocument.
- *
  *  @param obj a GeanyObject instance, should be ignored.
  *  @param word the current word (in UTF-8 encoding) below the cursor position
 		   where the popup menu will be opened.
@@ -238,9 +234,6 @@ signal void (*update_editor_menu)(GObject *obj, const gchar *word, gint pos, Gea
  *           care about the return value; make sure to return TRUE only if it is necessary
  *           and in the correct situations.
  *
- *  You need to include "editor.h" for the declaration of GeanyEditor and "Scintilla.h" for
- *  SCNotification.
- *
  *  @param obj a GeanyObject instance, should be ignored.
  *  @param editor The current GeanyEditor.
  *  @param nt A pointer to the SCNotification struct which holds additional information for
