@@ -1,8 +1,8 @@
 /*
  *      filebrowser.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2007-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2007-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2007-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2007-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -19,11 +19,15 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: filebrowser.c 5052 2010-06-18 17:03:35Z ntrel $
+ * $Id: filebrowser.c 5882 2011-08-03 15:20:26Z colombanw $
  */
 
 /* Sidebar file browser plugin. */
 
+#ifdef HAVE_CONFIG_H
+#	include "config.h"
+#endif
+
 #include "geanyplugin.h"
 #include <string.h>
 
@@ -74,13 +78,15 @@ static GtkListStore *file_store;
 static GtkTreeIter *last_dir_iter = NULL;
 static GtkEntryCompletion *entry_completion = NULL;
 
+static GtkWidget *filter_combo;
 static GtkWidget *filter_entry;
 static GtkWidget *path_combo;
 static GtkWidget *path_entry;
 static gchar *current_dir = NULL; /* in locale-encoding */
 static gchar *open_cmd; /* in locale-encoding */
 static gchar *config_file;
-static gchar *filter = NULL;
+static gchar **filter = NULL;
+static gchar *hidden_file_extensions = NULL;
 
 static gint page_number = 0;
 
@@ -95,6 +101,7 @@ static struct
 
 static void project_change_cb(GObject *obj, GKeyFile *config, gpointer data);
 
+/* note: other callbacks connected in plugin_init */
 PluginCallback plugin_callbacks[] =
 {
 	{ "project-open", (GCallback) &project_change_cb, TRUE, NULL },
@@ -122,9 +129,6 @@ static gboolean check_hidden(const gchar *filename, const gchar *base_name)
 {
 	gsize len;
 
-	if (! NZV(base_name))
-		return FALSE;
-
 #ifdef G_OS_WIN32
 	if (win32_check_hidden(filename))
 		return TRUE;
@@ -134,38 +138,45 @@ static gboolean check_hidden(const gchar *filename, const gchar *base_name)
 #endif
 
 	len = strlen(base_name);
-	if (base_name[len - 1] == '~')
-		return TRUE;
+	return base_name[len - 1] == '~';
+}
 
-	if (hide_object_files)
+
+static gboolean check_object(const gchar *base_name)
+{
+	gboolean ret = FALSE;
+	gchar **ptr;
+	gchar **exts = g_strsplit(hidden_file_extensions, " ", -1);
+
+	foreach_strv(ptr, exts)
 	{
-		const gchar *exts[] = {".o", ".obj", ".so", ".dll", ".a", ".lib"};
-		guint i, exts_len;
-
-		exts_len = G_N_ELEMENTS(exts);
-		for (i = 0; i < exts_len; i++)
+		if (g_str_has_suffix(base_name, *ptr))
 		{
-			const gchar *ext = exts[i];
-
-			if (g_str_has_suffix(base_name, ext))
-				return TRUE;
+			ret = TRUE;
+			break;
 		}
 	}
-	return FALSE;
+	g_strfreev(exts);
+	return ret;
 }
 
 
 /* Returns: whether filename should be removed. */
 static gboolean check_filtered(const gchar *base_name)
 {
+	gchar **filter_item;
+
 	if (filter == NULL)
 		return FALSE;
 
-	if (! utils_str_equal(base_name, "*") && ! g_pattern_match_simple(filter, base_name))
+	foreach_strv(filter_item, filter)
 	{
-		return TRUE;
+		if (utils_str_equal(*filter_item, "*") || g_pattern_match_simple(*filter_item, base_name))
+		{
+			return FALSE;
+		}
 	}
-	return FALSE;
+	return TRUE;
 }
 
 
@@ -177,6 +188,10 @@ static void add_item(const gchar *name)
 	const gchar *sep;
 	gboolean dir;
 
+	if (G_UNLIKELY(! NZV(name)))
+		return;
+
+	/* root directory doesn't need separator */
 	sep = (utils_str_equal(current_dir, "/")) ? "" : G_DIR_SEPARATOR_S;
 	fname = g_strconcat(current_dir, sep, name, NULL);
 	dir = g_file_test(fname, G_FILE_TEST_IS_DIR);
@@ -184,12 +199,8 @@ static void add_item(const gchar *name)
 	utf8_name = utils_get_utf8_from_locale(name);
 	g_free(fname);
 
-	if (! show_hidden_files && check_hidden(utf8_fullname, name))
-	{
-		g_free(utf8_name);
-		g_free(utf8_fullname);
-		return;
-	}
+	if (! show_hidden_files && check_hidden(utf8_fullname, utf8_name))
+		goto done;
 
 	if (dir)
 	{
@@ -204,12 +215,11 @@ static void add_item(const gchar *name)
 	}
 	else
 	{
+		if (! show_hidden_files && hide_object_files && check_object(utf8_name))
+			goto done;
 		if (check_filtered(utf8_name))
-		{
-			g_free(utf8_name);
-			g_free(utf8_fullname);
-			return;
-		}
+			goto done;
+
 		gtk_list_store_append(file_store, &iter);
 	}
 	gtk_list_store_set(file_store, &iter,
@@ -217,6 +227,7 @@ static void add_item(const gchar *name)
 		FILEVIEW_COLUMN_NAME, utf8_name,
 		FILEVIEW_COLUMN_FILENAME, utf8_fullname,
 		-1);
+done:
 	g_free(utf8_name);
 	g_free(utf8_fullname);
 }
@@ -485,8 +496,9 @@ static void open_selected_files(GList *list, gboolean do_not_focus)
 		GtkTreePath *treepath = item->data;
 		gchar *fname = get_tree_path_filename(treepath);
 
-		files = g_slist_append(files, fname);
+		files = g_slist_prepend(files, fname);
 	}
+	files = g_slist_reverse(files);
 	document_open_files(files, FALSE, NULL, NULL);
 	doc = document_get_current();
 	if (doc != NULL && ! do_not_focus)
@@ -728,9 +740,19 @@ static gboolean on_key_press(GtkWidget *widget, GdkEventKey *event, gpointer dat
 }
 
 
+static void clear_filter(void)
+{
+	if (filter != NULL)
+	{
+		g_strfreev(filter);
+		filter = NULL;
+	}
+}
+
+
 static void on_clear_filter(GtkEntry *entry, gpointer user_data)
 {
-	setptr(filter, NULL);
+	clear_filter();
 
 	gtk_entry_set_text(GTK_ENTRY(filter_entry), "");
 
@@ -767,23 +789,24 @@ static void on_path_entry_activate(GtkEntry *entry, gpointer user_data)
 }
 
 
-static void on_path_combo_changed(GtkComboBox *combo, gpointer user_data)
+static void ui_combo_box_changed(GtkComboBox *combo, gpointer user_data)
 {
 	/* we get this callback on typing as well as choosing an item */
 	if (gtk_combo_box_get_active(combo) >= 0)
-		on_path_entry_activate(GTK_ENTRY(path_entry), NULL);
+		gtk_widget_activate(GTK_BIN(combo)->child);
 }
 
 
 static void on_filter_activate(GtkEntry *entry, gpointer user_data)
 {
-	setptr(filter, g_strdup(gtk_entry_get_text(entry)));
-
-	if (! NZV(filter))
+	/* We use spaces for consistency with Find in Files file patterns
+	 * ';' also supported like original patch. */
+	filter = g_strsplit_set(gtk_entry_get_text(entry), "; ", -1);
+	if (filter == NULL || g_strv_length(filter) == 0)
 	{
-		setptr(filter, g_strdup("*"));
+		clear_filter();
 	}
-
+	ui_combo_box_add_to_history(GTK_COMBO_BOX_ENTRY(filter_combo), NULL, 0);
 	refresh();
 }
 
@@ -791,8 +814,7 @@ static void on_filter_activate(GtkEntry *entry, gpointer user_data)
 static void on_filter_clear(GtkEntry *entry, gint icon_pos,
 							GdkEvent *event, gpointer data)
 {
-	setptr(filter, g_strdup("*"));
-
+	clear_filter();
 	refresh();
 }
 
@@ -823,14 +845,14 @@ static void prepare_file_view(void)
 
 	ui_widget_modify_font_from_string(file_view, geany->interface_prefs->tagbar_font);
 
-	/* GTK 2.12 tooltips */
-	if (gtk_check_version(2, 12, 0) == NULL)
-		g_object_set(file_view, "has-tooltip", TRUE, "tooltip-column", FILEVIEW_COLUMN_FILENAME, NULL);
+	/* tooltips */
+	gtk_tree_view_set_tooltip_column(GTK_TREE_VIEW(file_view), FILEVIEW_COLUMN_FILENAME);
 
 	/* selection handling */
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(file_view));
 	gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
 
+	/* Show the current path when the FB is first needed */
 	g_signal_connect(file_view, "realize", G_CALLBACK(on_current_path), NULL);
 	g_signal_connect(selection, "changed", G_CALLBACK(on_tree_selection_changed), NULL);
 	g_signal_connect(file_view, "button-press-event", G_CALLBACK(on_button_press), NULL);
@@ -847,22 +869,22 @@ static GtkWidget *make_toolbar(void)
 	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
 
 	wid = GTK_WIDGET(gtk_tool_button_new_from_stock(GTK_STOCK_GO_UP));
-	ui_widget_set_tooltip_text(wid, _("Up"));
+	gtk_widget_set_tooltip_text(wid, _("Up"));
 	g_signal_connect(wid, "clicked", G_CALLBACK(on_go_up), NULL);
 	gtk_container_add(GTK_CONTAINER(toolbar), wid);
 
 	wid = GTK_WIDGET(gtk_tool_button_new_from_stock(GTK_STOCK_REFRESH));
-	ui_widget_set_tooltip_text(wid, _("Refresh"));
+	gtk_widget_set_tooltip_text(wid, _("Refresh"));
 	g_signal_connect(wid, "clicked", G_CALLBACK(refresh), NULL);
 	gtk_container_add(GTK_CONTAINER(toolbar), wid);
 
 	wid = GTK_WIDGET(gtk_tool_button_new_from_stock(GTK_STOCK_HOME));
-	ui_widget_set_tooltip_text(wid, _("Home"));
+	gtk_widget_set_tooltip_text(wid, _("Home"));
 	g_signal_connect(wid, "clicked", G_CALLBACK(on_go_home), NULL);
 	gtk_container_add(GTK_CONTAINER(toolbar), wid);
 
 	wid = GTK_WIDGET(gtk_tool_button_new_from_stock(GTK_STOCK_JUMP_TO));
-	ui_widget_set_tooltip_text(wid, _("Set path from document"));
+	gtk_widget_set_tooltip_text(wid, _("Set path from document"));
 	g_signal_connect(wid, "clicked", G_CALLBACK(on_current_path), NULL);
 	gtk_container_add(GTK_CONTAINER(toolbar), wid);
 
@@ -872,7 +894,7 @@ static GtkWidget *make_toolbar(void)
 		gtk_container_add(GTK_CONTAINER(toolbar), wid);
 
 		wid = GTK_WIDGET(gtk_tool_button_new_from_stock(GTK_STOCK_CLEAR));
-		ui_widget_set_tooltip_text(wid, _("Clear the filter"));
+		gtk_widget_set_tooltip_text(wid, _("Clear the filter"));
 		g_signal_connect(wid, "clicked", G_CALLBACK(on_clear_filter), NULL);
 		gtk_container_add(GTK_CONTAINER(toolbar), wid);
 	}
@@ -888,19 +910,21 @@ static GtkWidget *make_filterbar(void)
 
 	label = gtk_label_new(_("Filter:"));
 
-	filter_entry = gtk_entry_new();
+	filter_combo = gtk_combo_box_entry_new_text();
+	filter_entry = GTK_BIN(filter_combo)->child;
 
 	if (gtk_check_version(2, 15, 2) == NULL)
 	{
 		ui_entry_add_clear_icon(GTK_ENTRY(filter_entry));
 		g_signal_connect(filter_entry, "icon-release", G_CALLBACK(on_filter_clear), NULL);
 	}
-	ui_widget_set_tooltip_text(filter_entry,
-		_("Filter your files with usual wildcards"));
+	gtk_widget_set_tooltip_text(filter_entry,
+		_("Filter your files with the usual wildcards. Separate multiple patterns with a space."));
 	g_signal_connect(filter_entry, "activate", G_CALLBACK(on_filter_activate), NULL);
+	g_signal_connect(filter_combo, "changed", G_CALLBACK(ui_combo_box_changed), NULL);
 
 	gtk_box_pack_start(GTK_BOX(filterbar), label, FALSE, FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(filterbar), filter_entry, TRUE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(filterbar), filter_combo, TRUE, TRUE, 0);
 
 	return filterbar;
 }
@@ -969,39 +993,22 @@ static void completion_create(void)
 }
 
 
-#define CHECK_READ_SETTING(var, error, tmp) \
-	if ((error) != NULL) \
-	{ \
-		g_error_free((error)); \
-		(error) = NULL; \
-	} \
-	else \
-		(var) = (tmp);
-
 static void load_settings(void)
 {
 	GKeyFile *config = g_key_file_new();
-	GError *error = NULL;
-	gboolean tmp;
 
 	config_file = g_strconcat(geany->app->configdir, G_DIR_SEPARATOR_S, "plugins", G_DIR_SEPARATOR_S,
 		"filebrowser", G_DIR_SEPARATOR_S, "filebrowser.conf", NULL);
 	g_key_file_load_from_file(config, config_file, G_KEY_FILE_NONE, NULL);
-	open_cmd = g_key_file_get_string(config, "filebrowser", "open_command", &error);
-	if (error != NULL)
-	{
-		open_cmd = g_strdup("nautilus \"%d\"");
-		g_error_free(error);
-		error = NULL;
-	}
-	tmp = g_key_file_get_boolean(config, "filebrowser", "show_hidden_files", &error);
-	CHECK_READ_SETTING(show_hidden_files, error, tmp);
-	tmp = g_key_file_get_boolean(config, "filebrowser", "hide_object_files", &error);
-	CHECK_READ_SETTING(hide_object_files, error, tmp);
-	tmp = g_key_file_get_boolean(config, "filebrowser", "fb_follow_path", &error);
-	CHECK_READ_SETTING(fb_follow_path, error, tmp);
-	tmp = g_key_file_get_boolean(config, "filebrowser", "fb_set_project_base_path", &error);
-	CHECK_READ_SETTING(fb_set_project_base_path, error, tmp);
+
+	open_cmd = utils_get_setting_string(config, "filebrowser", "open_command", "nautilus \"%d\"");
+	/* g_key_file_get_boolean defaults to FALSE */
+	show_hidden_files = g_key_file_get_boolean(config, "filebrowser", "show_hidden_files", NULL);
+	hide_object_files = utils_get_setting_boolean(config, "filebrowser", "hide_object_files", TRUE);
+	hidden_file_extensions = utils_get_setting_string(config, "filebrowser", "hidden_file_extensions",
+		".o .obj .so .dll .a .lib .pyc");
+	fb_follow_path = g_key_file_get_boolean(config, "filebrowser", "fb_follow_path", NULL);
+	fb_set_project_base_path = g_key_file_get_boolean(config, "filebrowser", "fb_set_project_base_path", NULL);
 
 	g_key_file_free(config);
 }
@@ -1039,11 +1046,15 @@ static void project_change_cb(G_GNUC_UNUSED GObject *obj, G_GNUC_UNUSED GKeyFile
 }
 
 
+static gpointer last_activate_path = NULL;
+
 static void document_activate_cb(G_GNUC_UNUSED GObject *obj, GeanyDocument *doc,
 								 G_GNUC_UNUSED gpointer data)
 {
 	gchar *new_dir;
 
+	last_activate_path = doc->real_path;
+
 	if (! fb_follow_path || doc->file_name == NULL || ! g_path_is_absolute(doc->file_name))
 		return;
 
@@ -1060,6 +1071,13 @@ static void document_activate_cb(G_GNUC_UNUSED GObject *obj, GeanyDocument *doc,
 }
 
 
+static void document_save_cb(GObject *obj, GeanyDocument *doc, gpointer user_data)
+{
+	if (!last_activate_path)
+		document_activate_cb(obj, doc, user_data);
+}
+
+
 static void kb_activate(guint key_id)
 {
 	gtk_notebook_set_current_page(GTK_NOTEBOOK(geany->main_widgets->sidebar_notebook), page_number);
@@ -1090,7 +1108,7 @@ void plugin_init(GeanyData *data)
 
 	path_combo = gtk_combo_box_entry_new_text();
 	gtk_box_pack_start(GTK_BOX(file_view_vbox), path_combo, FALSE, FALSE, 2);
-	g_signal_connect(path_combo, "changed", G_CALLBACK(on_path_combo_changed), NULL);
+	g_signal_connect(path_combo, "changed", G_CALLBACK(ui_combo_box_changed), NULL);
 	path_entry = GTK_BIN(path_combo)->child;
 	g_signal_connect(path_entry, "activate", G_CALLBACK(on_path_entry_activate), NULL);
 
@@ -1107,12 +1125,13 @@ void plugin_init(GeanyData *data)
 	gtk_container_add(GTK_CONTAINER(scrollwin), file_view);
 	gtk_container_add(GTK_CONTAINER(file_view_vbox), scrollwin);
 
+	/* load settings before file_view "realize" callback */
+	load_settings();
+
 	gtk_widget_show_all(file_view_vbox);
 	page_number = gtk_notebook_append_page(GTK_NOTEBOOK(geany->main_widgets->sidebar_notebook),
 		file_view_vbox, gtk_label_new(_("Files")));
 
-	load_settings();
-
 	/* setup keybindings */
 	keybindings_set_item(plugin_key_group, KB_FOCUS_FILE_LIST, kb_activate,
 		0, 0, "focus_file_list", _("Focus File List"), NULL);
@@ -1121,6 +1140,8 @@ void plugin_init(GeanyData *data)
 
 	plugin_signal_connect(geany_plugin, NULL, "document-activate", TRUE,
 		(GCallback) &document_activate_cb, NULL);
+	plugin_signal_connect(geany_plugin, NULL, "document-save", TRUE,
+		(GCallback) &document_save_cb, NULL);
 }
 
 
@@ -1135,6 +1156,7 @@ static void save_settings(void)
 	g_key_file_set_string(config, "filebrowser", "open_command", open_cmd);
 	g_key_file_set_boolean(config, "filebrowser", "show_hidden_files", show_hidden_files);
 	g_key_file_set_boolean(config, "filebrowser", "hide_object_files", hide_object_files);
+	g_key_file_set_string(config, "filebrowser", "hidden_file_extensions", hidden_file_extensions);
 	g_key_file_set_boolean(config, "filebrowser", "fb_follow_path", fb_follow_path);
 	g_key_file_set_boolean(config, "filebrowser", "fb_set_project_base_path",
 		fb_set_project_base_path);
@@ -1161,6 +1183,7 @@ static struct
 	GtkWidget *open_cmd_entry;
 	GtkWidget *show_hidden_checkbox;
 	GtkWidget *hide_objects_checkbox;
+	GtkWidget *hidden_files_entry;
 	GtkWidget *follow_path_checkbox;
 	GtkWidget *set_project_base_path_checkbox;
 }
@@ -1175,6 +1198,8 @@ on_configure_response(GtkDialog *dialog, gint response, gpointer user_data)
 		open_cmd = g_strdup(gtk_entry_get_text(GTK_ENTRY(pref_widgets.open_cmd_entry)));
 		show_hidden_files = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pref_widgets.show_hidden_checkbox));
 		hide_object_files = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pref_widgets.hide_objects_checkbox));
+		g_free(hidden_file_extensions);
+		hidden_file_extensions = g_strdup(gtk_entry_get_text(GTK_ENTRY(pref_widgets.hidden_files_entry)));
 		fb_follow_path = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pref_widgets.follow_path_checkbox));
 		fb_set_project_base_path = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
 			pref_widgets.set_project_base_path_checkbox));
@@ -1185,10 +1210,21 @@ on_configure_response(GtkDialog *dialog, gint response, gpointer user_data)
 }
 
 
+static void on_toggle_hidden(void)
+{
+	gboolean enabled = !gtk_toggle_button_get_active(
+		GTK_TOGGLE_BUTTON(pref_widgets.show_hidden_checkbox));
+
+	gtk_widget_set_sensitive(pref_widgets.hide_objects_checkbox, enabled);
+	enabled &= gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pref_widgets.hide_objects_checkbox));
+	gtk_widget_set_sensitive(pref_widgets.hidden_files_entry, enabled);
+}
+
+
 GtkWidget *plugin_configure(GtkDialog *dialog)
 {
 	GtkWidget *label, *entry, *checkbox_of, *checkbox_hf, *checkbox_fp, *checkbox_pb, *vbox;
-	GtkWidget *box;
+	GtkWidget *box, *align;
 
 	vbox = gtk_vbox_new(FALSE, 6);
 	box = gtk_vbox_new(FALSE, 3);
@@ -1198,32 +1234,43 @@ GtkWidget *plugin_configure(GtkDialog *dialog)
 	gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 0);
 
 	entry = gtk_entry_new();
-	gtk_widget_show(entry);
 	if (open_cmd != NULL)
 		gtk_entry_set_text(GTK_ENTRY(entry), open_cmd);
-	ui_widget_set_tooltip_text(entry,
+	gtk_widget_set_tooltip_text(entry,
 		_("The command to execute when using \"Open with\". You can use %f and %d wildcards.\n"
 		  "%f will be replaced with the filename including full path\n"
 		  "%d will be replaced with the path name of the selected file without the filename"));
 	gtk_box_pack_start(GTK_BOX(box), entry, FALSE, FALSE, 0);
 	pref_widgets.open_cmd_entry = entry;
 
-	gtk_box_pack_start(GTK_BOX(vbox), box, FALSE, FALSE, 6);
+	gtk_box_pack_start(GTK_BOX(vbox), box, FALSE, FALSE, 3);
 
 	checkbox_hf = gtk_check_button_new_with_label(_("Show hidden files"));
 	gtk_button_set_focus_on_click(GTK_BUTTON(checkbox_hf), FALSE);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbox_hf), show_hidden_files);
 	gtk_box_pack_start(GTK_BOX(vbox), checkbox_hf, FALSE, FALSE, 0);
 	pref_widgets.show_hidden_checkbox = checkbox_hf;
+	g_signal_connect(checkbox_hf, "toggled", on_toggle_hidden, NULL);
 
-	checkbox_of = gtk_check_button_new_with_label(_("Hide object files"));
+	box = gtk_vbox_new(FALSE, 3);
+	checkbox_of = gtk_check_button_new_with_label(_("Hide file extensions:"));
 	gtk_button_set_focus_on_click(GTK_BUTTON(checkbox_of), FALSE);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbox_of), hide_object_files);
-	ui_widget_set_tooltip_text(checkbox_of,
-		_("Don't show generated object files in the file browser, this includes "
-		  "*.o, *.obj. *.so, *.dll, *.a, *.lib"));
-	gtk_box_pack_start(GTK_BOX(vbox), checkbox_of, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(box), checkbox_of, FALSE, FALSE, 0);
 	pref_widgets.hide_objects_checkbox = checkbox_of;
+	g_signal_connect(checkbox_of, "toggled", on_toggle_hidden, NULL);
+
+	entry = gtk_entry_new();
+	if (hidden_file_extensions != NULL)
+		gtk_entry_set_text(GTK_ENTRY(entry), hidden_file_extensions);
+	gtk_box_pack_start(GTK_BOX(box), entry, FALSE, FALSE, 0);
+	pref_widgets.hidden_files_entry = entry;
+
+	align = gtk_alignment_new(1, 0.5, 1, 1);
+	gtk_alignment_set_padding(GTK_ALIGNMENT(align), 0, 0, 12, 0);
+	gtk_container_add(GTK_CONTAINER(align), box);
+	gtk_box_pack_start(GTK_BOX(vbox), align, FALSE, FALSE, 0);
+	on_toggle_hidden();
 
 	checkbox_fp = gtk_check_button_new_with_label(_("Follow the path of the current file"));
 	gtk_button_set_focus_on_click(GTK_BUTTON(checkbox_fp), FALSE);
@@ -1234,7 +1281,7 @@ GtkWidget *plugin_configure(GtkDialog *dialog)
 	checkbox_pb = gtk_check_button_new_with_label(_("Use the project's base directory"));
 	gtk_button_set_focus_on_click(GTK_BUTTON(checkbox_pb), FALSE);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbox_pb), fb_set_project_base_path);
-	ui_widget_set_tooltip_text(checkbox_pb,
+	gtk_widget_set_tooltip_text(checkbox_pb,
 		_("Change the directory to the base directory of the currently opened project"));
 	gtk_box_pack_start(GTK_BOX(vbox), checkbox_pb, FALSE, FALSE, 0);
 	pref_widgets.set_project_base_path_checkbox = checkbox_pb;
@@ -1252,7 +1299,8 @@ void plugin_cleanup(void)
 
 	g_free(config_file);
 	g_free(open_cmd);
-	g_free(filter);
+	g_free(hidden_file_extensions);
+	clear_filter();
 	gtk_widget_destroy(file_view_vbox);
 	g_object_unref(G_OBJECT(entry_completion));
 }
