@@ -1,9 +1,9 @@
 /*
  *      htmlchars.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2009-2010 Frank Lanitz <frank(at)frank(dot)uvena(dot)de>
- *      Copyright 2006-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2007-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2009-2011 Frank Lanitz <frank(at)frank(dot)uvena(dot)de>
+ *      Copyright 2006-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2007-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -20,11 +20,15 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: htmlchars.c 5354 2010-11-02 13:05:12Z ntrel $
+ * $Id: htmlchars.c 5882 2011-08-03 15:20:26Z colombanw $
  */
 
 /* HTML Characters plugin (Inserts HTML character entities like '&amp;') */
 
+#ifdef HAVE_CONFIG_H
+#	include "config.h"
+#endif
+
 #include "geanyplugin.h"
 #include <string.h>
 #include "SciLexer.h"
@@ -649,10 +653,10 @@ static void replace_special_character(void)
 
 	if (doc != NULL && sci_has_selection(doc->editor->sci))
 	{
-		guint selection_len;
+		gsize selection_len;
 		gchar *selection;
 		GString *replacement = g_string_new(NULL);
-		guint i;
+		gsize i;
 		gchar *new;
 		const gchar *entity = NULL;
 		gchar buf[7];
@@ -664,7 +668,7 @@ static void replace_special_character(void)
 		for (i = 0; i < selection_len; i++)
 		{
 			len = g_unichar_to_utf8(g_utf8_get_char(selection + i), buf);
-			i = len - 1 + i;
+			i = (guint)len - 1 + i;
 
 			buf[len] = '\0';
 			entity = get_entity(buf);
