@@ -1,8 +1,8 @@
 /*
  *      saveactions.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2007-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2007-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2007-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2007-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -19,10 +19,14 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: saveactions.c 5323 2010-10-25 17:24:47Z ntrel $
+ * $Id: saveactions.c 5882 2011-08-03 15:20:26Z colombanw $
  */
 
 
+#ifdef HAVE_CONFIG_H
+#	include "config.h"
+#endif
+
 #include "geanyplugin.h"
 
 #include <unistd.h>
@@ -74,7 +78,7 @@ static gboolean enable_backupcopy;
 static gint autosave_interval;
 static gboolean autosave_print_msg;
 static gboolean autosave_save_all;
-static guint autosave_src_id = G_MAXUINT;
+static guint autosave_src_id = 0;
 
 static gchar *instantsave_default_ft;
 
@@ -91,7 +95,7 @@ static gboolean backupcopy_set_backup_dir(const gchar *utf8_dir)
 {
 	gchar *tmp;
 
-	if (! NZV(utf8_dir))
+	if (G_UNLIKELY(! NZV(utf8_dir)))
 		return FALSE;
 
 	tmp = utils_get_locale_from_utf8(utf8_dir);
@@ -321,7 +325,7 @@ static void autosave_set_timeout(void)
 	if (! enable_autosave)
 		return;
 
-	if (autosave_src_id != G_MAXUINT)
+	if (autosave_src_id != 0)
 		g_source_remove(autosave_src_id);
 	autosave_src_id = g_timeout_add(autosave_interval * 1000, (GSourceFunc) auto_save, NULL);
 }
@@ -347,7 +351,7 @@ void plugin_init(GeanyData *data)
 	instantsave_default_ft = utils_get_setting_string(config, "instantsave", "default_ft",
 		filetypes[GEANY_FILETYPES_NONE]->name);
 
-	autosave_src_id = G_MAXUINT; /* mark as invalid */
+	autosave_src_id = 0; /* mark as invalid */
 	autosave_interval = utils_get_setting_integer(config, "autosave", "interval", 300);
 	autosave_print_msg = utils_get_setting_boolean(config, "autosave", "print_messages", FALSE);
 	autosave_save_all = utils_get_setting_boolean(config, "autosave", "save_all", FALSE);
@@ -453,14 +457,17 @@ static void configure_response_cb(GtkDialog *dialog, gint response, G_GNUC_UNUSE
 		g_key_file_set_integer(config, "backupcopy", "dir_levels", backupcopy_dir_levels);
 		g_key_file_set_string(config, "backupcopy", "time_fmt", text_time);
 		setptr(backupcopy_time_fmt, g_strdup(text_time));
-		if (*text_dir != '\0' && backupcopy_set_backup_dir(text_dir))
+		if (enable_backupcopy)
 		{
-			g_key_file_set_string(config, "backupcopy", "backup_dir", text_dir);
-		}
-		else
-		{
-			dialogs_show_msgbox(GTK_MESSAGE_ERROR,
-					_("Backup directory does not exist or is not writable."));
+			if (NZV(text_dir) && backupcopy_set_backup_dir(text_dir))
+			{
+				g_key_file_set_string(config, "backupcopy", "backup_dir", text_dir);
+			}
+			else
+			{
+				dialogs_show_msgbox(GTK_MESSAGE_ERROR,
+						_("Backup directory does not exist or is not writable."));
+			}
 		}
 
 
@@ -595,7 +602,7 @@ GtkWidget *plugin_configure(GtkDialog *dialog)
 	{
 		GtkWidget *combo;
 		guint i;
-		GSList *node;
+		const GSList *node;
 
 		notebook_vbox = gtk_vbox_new(FALSE, 2);
 		inner_vbox = gtk_vbox_new(FALSE, 5);
@@ -618,7 +625,7 @@ GtkWidget *plugin_configure(GtkDialog *dialog)
 
 		pref_widgets.instantsave_ft_combo = combo = gtk_combo_box_new_text();
 		i = 0;
-		foreach_slist(node, geany->filetypes_by_title)
+		foreach_slist(node, filetypes_get_sorted_by_name())
 		{
 			GeanyFiletype *ft = node->data;
 
@@ -716,7 +723,7 @@ GtkWidget *plugin_configure(GtkDialog *dialog)
 
 void plugin_cleanup(void)
 {
-	if (autosave_src_id != G_MAXUINT)
+	if (autosave_src_id != 0)
 		g_source_remove(autosave_src_id);
 
 	g_free(instantsave_default_ft);
