@@ -1,18 +1,18 @@
-/* Scintilla source code edit control */
+// Scintilla source code edit control
 /** @file Platform.h
  ** Interface to platform facilities. Also includes some basic utilities.
  ** Implemented in PlatGTK.cxx for GTK+/Linux, PlatWin.cxx for Windows, and PlatWX.cxx for wxWindows.
  **/
-/* Copyright 1998-2003 by Neil Hodgson <neilh@scintilla.org>
- * The License.txt file describes the conditions under which this software may be distributed. */
+// Copyright 1998-2009 by Neil Hodgson <neilh@scintilla.org>
+// The License.txt file describes the conditions under which this software may be distributed.
 
 #ifndef PLATFORM_H
 #define PLATFORM_H
 
-/* PLAT_GTK = GTK+ on Linux or Win32
- * PLAT_GTK_WIN32 is defined additionally when running PLAT_GTK under Win32
- * PLAT_WIN = Win32 API on Win32 OS
- * PLAT_WX is wxWindows on any supported platform */
+// PLAT_GTK = GTK+ on Linux or Win32
+// PLAT_GTK_WIN32 is defined additionally when running PLAT_GTK under Win32
+// PLAT_WIN = Win32 API on Win32 OS
+// PLAT_WX is wxWindows on any supported platform
 
 #define PLAT_GTK 0
 #define PLAT_GTK_WIN32 0
@@ -38,7 +38,8 @@
 #define PLAT_GTK_WIN32 1
 #endif
 
-#elif defined(MACOSX)
+#elif defined(__APPLE__)
+
 #undef PLAT_MACOSX
 #define PLAT_MACOSX 1
 
@@ -52,8 +53,8 @@
 namespace Scintilla {
 #endif
 
-/* Underlying the implementation of the platform classes are platform specific types.
- * Sometimes these need to be passed around by client code so they are defined here */
+// Underlying the implementation of the platform classes are platform specific types.
+// Sometimes these need to be passed around by client code so they are defined here
 
 typedef void *FontID;
 typedef void *SurfaceID;
@@ -75,7 +76,7 @@ public:
 	explicit Point(int x_=0, int y_=0) : x(x_), y(y_) {
 	}
 
-	/* Other automatically defined methods (assignment, copy constructor, destructor) are fine */
+	// Other automatically defined methods (assignment, copy constructor, destructor) are fine
 
 	static Point FromLong(long lpoint);
 };
@@ -96,7 +97,7 @@ public:
 		left(left_), top(top_), right(right_), bottom(bottom_) {
 	}
 
-	/* Other automatically defined methods (assignment, copy constructor, destructor) are fine */
+	// Other automatically defined methods (assignment, copy constructor, destructor) are fine
 
 	bool operator==(PRectangle &rc) {
 		return (rc.left == left) && (rc.right == right) &&
@@ -240,7 +241,7 @@ struct ColourPair {
 	}
 };
 
-class Window;	/* Forward declaration for Palette */
+class Window;	// Forward declaration for Palette
 
 /**
  * Colour palette management.
@@ -250,12 +251,12 @@ class Palette {
 	int size;
 	ColourPair *entries;
 #if PLAT_GTK
-	void *allocatedPalette; /* GdkColor * */
+	void *allocatedPalette; // GdkColor *
 	int allocatedLen;
 #endif
-	/* Private so Palette objects can not be copied */
-	Palette(const Palette &) {}
-	Palette &operator=(const Palette &) { return *this; }
+	// Private so Palette objects can not be copied
+	Palette(const Palette &);
+	Palette &operator=(const Palette &);
 public:
 #if PLAT_WIN
 	void *hpal;
@@ -282,24 +283,24 @@ public:
  */
 class Font {
 protected:
-	FontID id;
+	FontID fid;
 #if PLAT_WX
 	int ascent;
 #endif
-	/* Private so Font objects can not be copied */
-	Font(const Font &) {}
-	Font &operator=(const Font &) { id=0; return *this; }
+	// Private so Font objects can not be copied
+	Font(const Font &);
+	Font &operator=(const Font &);
 public:
 	Font();
 	virtual ~Font();
 
 	virtual void Create(const char *faceName, int characterSet, int size,
-		bool bold, bool italic, bool extraFontFlag=false);
+		bool bold, bool italic, int extraFontFlag=0);
 	virtual void Release();
 
-	FontID GetID() { return id; }
-	/* Alias another font - caller guarantees not to Release */
-	void SetID(FontID id_) { id = id_; }
+	FontID GetID() { return fid; }
+	// Alias another font - caller guarantees not to Release
+	void SetID(FontID fid_) { fid = fid_; }
 	friend class Surface;
         friend class SurfaceImpl;
 };
@@ -309,12 +310,12 @@ public:
  */
 class Surface {
 private:
-	/* Private so Surface objects can not be copied */
+	// Private so Surface objects can not be copied
 	Surface(const Surface &) {}
 	Surface &operator=(const Surface &) { return *this; }
 public:
-	Surface() {};
-	virtual ~Surface() {};
+	Surface() {}
+	virtual ~Surface() {}
 	static Surface *Allocate();
 
 	virtual void Init(WindowID wid)=0;
@@ -370,31 +371,31 @@ typedef void (*CallBackAction)(void*);
  */
 class Window {
 protected:
-	WindowID id;
+	WindowID wid;
 #if PLAT_MACOSX
 	void *windowRef;
 	void *control;
 #endif
 public:
-	Window() : id(0), cursorLast(cursorInvalid) {
+	Window() : wid(0), cursorLast(cursorInvalid) {
 #if PLAT_MACOSX
 	  windowRef = 0;
 	  control = 0;
 #endif
 	}
-	Window(const Window &source) : id(source.id), cursorLast(cursorInvalid) {
+	Window(const Window &source) : wid(source.wid), cursorLast(cursorInvalid) {
 #if PLAT_MACOSX
 	  windowRef = 0;
 	  control = 0;
 #endif
 	}
 	virtual ~Window();
-	Window &operator=(WindowID id_) {
-		id = id_;
+	Window &operator=(WindowID wid_) {
+		wid = wid_;
 		return *this;
 	}
-	WindowID GetID() const { return id; }
-	bool Created() const { return id != 0; }
+	WindowID GetID() const { return wid; }
+	bool Created() const { return wid != 0; }
 	void Destroy();
 	bool HasFocus();
 	PRectangle GetPosition();
@@ -451,10 +452,10 @@ public:
  * Menu management.
  */
 class Menu {
-	MenuID id;
+	MenuID mid;
 public:
 	Menu();
-	MenuID GetID() { return id; }
+	MenuID GetID() { return mid; }
 	void CreatePopUp();
 	void Destroy();
 	void Show(Point pt, Window &w);
@@ -473,15 +474,15 @@ public:
  */
 class DynamicLibrary {
 public:
-	virtual ~DynamicLibrary() {};
+	virtual ~DynamicLibrary() {}
 
-	/** @return Pointer to function "name", or NULL on failure. */
+	/// @return Pointer to function "name", or NULL on failure.
 	virtual Function FindFunction(const char *name) = 0;
 
-	/** @return true if the library was loaded successfully. */
+	/// @return true if the library was loaded successfully.
 	virtual bool IsValid() = 0;
 
-	/** @return An instance of a DynamicLibrary subclass with "modulePath" loaded. */
+	/// @return An instance of a DynamicLibrary subclass with "modulePath" loaded.
 	static DynamicLibrary *Load(const char *modulePath);
 };
 
@@ -490,12 +491,12 @@ public:
  * and chrome colour. Not a creatable object, more of a module with several functions.
  */
 class Platform {
-	/* Private so Platform objects can not be copied */
+	// Private so Platform objects can not be copied
 	Platform(const Platform &) {}
 	Platform &operator=(const Platform &) { return *this; }
 public:
-	/* Should be private because no new Platforms are ever created
-	 * but gcc warns about this */
+	// Should be private because no new Platforms are ever created
+	// but gcc warns about this
 	Platform() {}
 	~Platform() {}
 	static ColourDesired Chrome();
@@ -514,10 +515,10 @@ public:
 	static int DBCSCharLength(int codePage, const char *s);
 	static int DBCSCharMaxLength();
 
-	/* These are utility functions not really tied to a platform */
+	// These are utility functions not really tied to a platform
 	static int Minimum(int a, int b);
 	static int Maximum(int a, int b);
-	/* Next three assume 16 bit shorts and 32 bit longs */
+	// Next three assume 16 bit shorts and 32 bit longs
 	static long LongFromTwoShorts(short a,short b) {
 		return (a) | ((b) << 16);
 	}
@@ -547,7 +548,7 @@ public:
 }
 #endif
 
-/* Shut up annoying Visual C++ warnings: */
+// Shut up annoying Visual C++ warnings:
 #ifdef _MSC_VER
 #pragma warning(disable: 4244 4309 4514 4710)
 #endif
