@@ -1,8 +1,8 @@
 /*
  *      about.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: about.c 5321 2010-10-25 16:24:55Z ntrel $
+ * $Id: about.c 5949 2011-09-22 18:04:03Z colombanw $
  */
 
 /*
@@ -40,7 +40,7 @@
 #define INFO "<span size=\"larger\" weight=\"bold\">%s</span>"
 #define CODENAME "<span weight=\"bold\">\"" GEANY_CODENAME "\"</span>"
 #define BUILDDATE "<span size=\"smaller\">%s</span>"
-#define COPYRIGHT "Copyright (c)  2005-2010\nNick Treleaven\nEnrico Tröger\nFrank Lanitz\nAll rights reserved."
+#define COPYRIGHT "Copyright (c)  2005-2011\nNick Treleaven\nColomban Wendling\nEnrico Tröger\nFrank Lanitz\nAll rights reserved."
 
 const gchar *translators[][2] = {
 	{ "ast", "Marcos Costales &lt;marcoscostales@gmail.com&gt;"},
@@ -52,6 +52,7 @@ const gchar *translators[][2] = {
 	{ "el", "Stavros Temertzidis &lt;bullgr@gmail.com&gt;" },
 	{ "en_GB", "Jeff Bailes &lt;thepizzaking@gmail.com&gt;" },
 	{ "es", "Antonio Jiménez González &lt;tonificante@hotmail.com&gt;\nLucas Vieites &lt;lucasvieites@gmail.com&gt;" },
+	{ "fa", "Moein Owhadi Kareshk &lt;moein.owhadi@gmail.com&gt;"},
 	{ "fi", "Harri Koskinen &lt;harri@fastmonkey.org&gt;\nJari Rahkonen &lt;jari.rahkonen@pp1.inet.fi&gt;" },
 	{ "fr", "Jean-Philippe Moal &lt;skateinmars@skateinmars.net&gt;" },
 	{ "gl", "José Manuel Castroagudín Silva &lt;chavescesures@gmail.com&gt;"},
@@ -63,7 +64,9 @@ const gchar *translators[][2] = {
 	{ "lb", "Laurent Hoeltgen &lt;hoeltgman@gmail.com&gt;" },
 	{ "nl", "Peter Scholtens &lt;peter.scholtens@xs4all.nl&gt;\nAyke van Laethem &lt;aykevanlaethem@gmail.com&gt;" },
 	{ "pl_PL", "Jarosław Foksa &lt;jfoksa@gmail.com&gt;" },
-	{ "pt_BR", "Alexandra Moreire &lt;alexandream@gmail.com&gt;\nAdrovane Marques Kade &lt;adrovane@gmail.com&gt;" },
+	{ "pt_BR", "Alexandra Moreire &lt;alexandream@gmail.com&gt;\n"
+			   "Adrovane Marques Kade &lt;adrovane@gmail.com&gt;\n"
+			   "Rafael Peregrino da Silva &lt;rperegrino@linuxnewmedia.com.br&gt;"},
 	{ "ro", "Alex Eftimie &lt;alex@rosedu.org&gt;" },
 	{ "ru_RU", "brahmann_ &lt;brahmann@pisem.net&gt;,\nNikita E. Shalaev &lt;nshalaev@eu.spb.ru&gt;" },
 	{ "sl", "Jože Klepec &lt;joze.klepec@siol.net&gt;"},
@@ -72,7 +75,7 @@ const gchar *translators[][2] = {
 	{ "uk", "Boris Dibrov &lt;dibrov.bor@gmail.com&gt;" },
 	{ "vi_VN", "Clytie Siddall &lt;clytie@riverland.net.au&gt;" },
 	{ "zh_CN", "Dormouse Young &lt;mouselinux@163.com&gt;,\nXhacker Liu &lt;liu.dongyuan@gmail.com&gt;" },
-	{ "zh_TW", "KoViCH &lt;kovich.ian@gmail.com&gt;" }
+	{ "zh_TW", "KoViCH &lt;kovich.ian@gmail.com&gt;\nWei-Lun Chao &lt;chaoweilun@gmail.com&gt;" }
 };
 static const gint translators_len = G_N_ELEMENTS(translators);
 
@@ -95,8 +98,9 @@ static const gchar *contributors =
 "Jean-François Wauthy, Jeff Pohlmeyer, Jesse Mayes, Jiří Techet, "
 "John Gabriele, Jon Senior, Jon Strait, Josef Whiter, "
 "Jörn Reder, Kelvin Gardiner, Kevin Ellwood, Kristoffer A. Tjernås, Lex Trotman, "
-"Manuel Bua, Mário Silva, Marko Peric, Matti Mårds, "
-"Moritz Barsnick, Ondrej Donek, Peter Strand, Philipp Gildein, Pierre Joye, Rob van der Linde, "
+"Manuel Bua, Mário Silva, Marko Peric, Matthew Brush, Matti Mårds, "
+"Moritz Barsnick, Nicolas Sierro, Ondrej Donek, Peter Strand, Philipp Gildein, "
+"Pierre Joye, Rob van der Linde, "
 "Robert McGinley, Roland Baudin, Ross McKay, S Jagannathan, Saleem Abdulrasool, "
 "Sebastian Kraft, Shiv, Slava Semushin, Stefan Oltmanns, Tamim, Taylor Venable, "
 "Thomas Huth, Thomas Martitz, Tomás Vírseda, "
@@ -134,7 +138,6 @@ static GtkWidget *create_dialog(void)
 	GtkWidget *table;
 	GtkWidget *license_scrollwin;
 	GtkWidget *info_box;
-	GtkWidget *button;
 	GtkWidget *header_hbox;
 	GtkWidget *header_eventbox;
 	GdkPixbuf *icon;
@@ -151,7 +154,7 @@ static GtkWidget *create_dialog(void)
 	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER_ON_PARENT);
 	gtk_window_set_title(GTK_WINDOW(dialog), _("About Geany"));
 	gtk_widget_set_name(dialog, "GeanyDialog");
-	button = gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
+	gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
 	gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_CLOSE);
 	gtk_dialog_set_has_separator(GTK_DIALOG(dialog), FALSE);
 	g_signal_connect(dialog, "key-press-event", G_CALLBACK(gb_on_key_pressed), NULL);
@@ -230,7 +233,7 @@ static GtkWidget *create_dialog(void)
 	box = gtk_hbutton_box_new();
 	url_button = gtk_button_new();
 	gtk_button_set_relief(GTK_BUTTON(url_button), GTK_RELIEF_NONE);
-	g_signal_connect(url_button, "clicked", G_CALLBACK(homepage_clicked), GEANY_HOMEPAGE);
+	g_signal_connect(url_button, "clicked", G_CALLBACK(homepage_clicked), (gpointer)GEANY_HOMEPAGE);
 	label = gtk_label_new(NULL);
 	gtk_label_set_text(GTK_LABEL(label), GEANY_HOMEPAGE);
 	gtk_widget_show(label);
@@ -262,7 +265,7 @@ static GtkWidget *create_dialog(void)
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(credits_scrollwin),
 		GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 
-	table = gtk_table_new(18 + translators_len + prev_translators_len, 3, FALSE);
+	table = gtk_table_new(20 + translators_len + prev_translators_len, 3, FALSE);
 	gtk_table_set_col_spacings(GTK_TABLE(table), 10);
 
 	row = 0;
@@ -284,6 +287,14 @@ static GtkWidget *create_dialog(void)
 	ROW("", row, 0, 0, 0, 0);
 	row++;
 
+	g_snprintf(buffer, sizeof(buffer), "Colomban Wendling - %s", _("developer"));
+	ROW(buffer, row, 0, 0, 0, 1);
+	row++;
+	ROW("&lt;lists.ban@herbesfolles.org&gt;", row, 0, 0, 0, 1);
+	row++;
+	ROW("", row, 0, 0, 0, 0);
+	row++;
+
 	g_snprintf(buffer, sizeof(buffer), "Enrico Tröger - %s", _("developer"));
 	ROW(buffer, row, 0, 0, 0, 1);
 	row++;
@@ -293,7 +304,7 @@ static GtkWidget *create_dialog(void)
 	row++;
 
 	g_snprintf(buffer, sizeof(buffer), "Frank Lanitz - %s", _("translation maintainer"));
-	ROW(buffer, 7, 0, 0, 0, 1);
+	ROW(buffer, row, 0, 0, 0, 1);
 	row++;
 	ROW("&lt;frank@frank.uvena.de&gt;", row, 0, 0, 0, 1);
 	row++;
@@ -460,4 +471,3 @@ static void homepage_clicked(GtkButton *button, gpointer data)
 {
 	utils_open_browser(data);
 }
-
