@@ -1,8 +1,8 @@
 /*
  *      build.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *      Copyright 2009 Lex Trotman <elextr(at)gmail(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
@@ -19,7 +19,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: build.c 5481 2010-12-09 15:34:28Z ntrel $
+ * $Id: build.c 5947 2011-09-22 15:55:10Z ntrel $
  */
 
 /*
@@ -189,10 +189,10 @@ static void set_command(GeanyBuildCommand *bc, gint id, gchar *str)
 }
 
 
-static const gchar *config_keys[] = {
-	[GEANY_BC_LABEL] = "LB",
-	[GEANY_BC_COMMAND] = "CM",
-	[GEANY_BC_WORKING_DIR] = "WD"
+static const gchar *config_keys[GEANY_BC_CMDENTRIES_COUNT] = {
+	"LB", /* label */
+	"CM", /* command */
+	"WD"  /* working directory */
 };
 
 /*-----------------------------------------------------
@@ -1654,11 +1654,8 @@ static void on_toolbutton_build_activate(GtkWidget *menuitem, gpointer user_data
 static void on_toolbutton_make_activate(GtkWidget *menuitem, gpointer user_data)
 {
 	gchar *msg;
-	gint grp, cmd;
 
 	last_toolbutton_action = user_data;
-	grp = GPOINTER_TO_GRP(user_data);
-	cmd = GPOINTER_TO_CMD(user_data);
 	if (last_toolbutton_action == GBO_TO_POINTER(GEANY_GBO_MAKE_ALL))
 			msg = _("Build the current file with Make and the default target");
 	else if (last_toolbutton_action == GBO_TO_POINTER(GEANY_GBO_CUSTOM))
@@ -1762,7 +1759,7 @@ static void set_row_color(RowWidgets *r, GdkColor *color )
 
 	for (i = 0; i < GEANY_BC_CMDENTRIES_COUNT; i++)
 		gtk_widget_modify_text(r->entries[i], GTK_STATE_NORMAL, color);
-};
+}
 
 
 static void set_build_command_entry_text(GtkWidget *wid, const gchar *text)
@@ -1845,8 +1842,10 @@ static const gchar *colheads[] =
 	N_("Label"),
 	N_("Command"),
 	N_("Working directory"),
-	N_("Clear"),
-	NULL };
+	N_("Reset"),
+	NULL
+};
+
 /* column names */
 #define DC_ITEM 0
 #define DC_ENTRIES 1
@@ -1888,7 +1887,7 @@ static RowWidgets *build_add_dialog_row(GeanyDocument *doc, GtkTable *table, gui
 		{
 			GtkWidget *wid = roww->entries[i] = gtk_button_new();
 			gtk_button_set_use_underline(GTK_BUTTON(wid), TRUE);
-			ui_widget_set_tooltip_text(wid, _("Click to set menu item label"));
+			gtk_widget_set_tooltip_text(wid, _("Click to set menu item label"));
 			g_signal_connect(wid, "clicked", G_CALLBACK(on_label_button_clicked), roww);
 		}
 		else
@@ -1972,9 +1971,10 @@ GtkWidget *build_commands_table(GeanyDocument *doc, GeanyBuildSource dst, BuildT
 	gtk_table_attach(table, sep, 0, DC_N_COL, 1, 2, GTK_FILL, GTK_FILL | GTK_EXPAND,
 		entry_x_padding, sep_padding);
 	if (ft != NULL && ft->id != GEANY_FILETYPES_NONE)
-		txt = g_strdup_printf(_("%s Commands"), ft->title);
+		txt = g_strdup_printf(_("%s commands"), ft->name);
 	else
-		txt = g_strdup_printf(_("%s Commands"), _("No Filetype"));
+		txt = g_strdup_printf(_("%s commands"), _("No filetype"));
+
 	label = ui_label_new_bold(txt);
 	g_free(txt);
 	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
@@ -1982,7 +1982,7 @@ GtkWidget *build_commands_table(GeanyDocument *doc, GeanyBuildSource dst, BuildT
 		entry_x_padding, entry_y_padding);
 	for (row = 3, cmdindex = 0, cmd = 0; cmd < build_groups_count[GEANY_GBG_FT]; ++row, ++cmdindex, ++cmd)
 		fields->rows[cmdindex] = build_add_dialog_row(doc, table, row, dst, GEANY_GBG_FT, cmd, FALSE);
-	label = gtk_label_new(_("Error Regular Expression:"));
+	label = gtk_label_new(_("Error regular expression:"));
 	gtk_table_attach(table, label, 0, DC_ENTRIES + 1, row, row + 1, GTK_FILL, GTK_FILL | GTK_EXPAND,
 		entry_x_padding, entry_y_padding);
 	fields->fileregex = gtk_entry_new();
@@ -2010,14 +2010,14 @@ GtkWidget *build_commands_table(GeanyDocument *doc, GeanyBuildSource dst, BuildT
 	gtk_table_attach(table, sep, 0, DC_N_COL, row, row + 1, GTK_FILL, GTK_FILL | GTK_EXPAND,
 		entry_x_padding, sep_padding);
 	++row;
-	label = ui_label_new_bold(_("Independent Commands"));
+	label = ui_label_new_bold(_("Independent commands"));
 	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
 	gtk_table_attach(table, label, 0, DC_N_COL, row, row + 1, GTK_FILL, GTK_FILL | GTK_EXPAND,
 		entry_x_padding, entry_y_padding);
 	for (++row, cmd = 0; cmd < build_groups_count[GEANY_GBG_NON_FT]; ++row, ++cmdindex, ++cmd)
 		fields->rows[cmdindex] = build_add_dialog_row(
 			doc, table, row, dst, GEANY_GBG_NON_FT, cmd, TRUE);
-	label = gtk_label_new(_("Error Regular Expression:"));
+	label = gtk_label_new(_("Error regular expression:"));
 	gtk_table_attach(table, label, 0, DC_ENTRIES + 1, row, row + 1, GTK_FILL,
 		GTK_FILL | GTK_EXPAND, entry_x_padding, entry_y_padding);
 	fields->nonfileregex = gtk_entry_new();
@@ -2051,7 +2051,7 @@ GtkWidget *build_commands_table(GeanyDocument *doc, GeanyBuildSource dst, BuildT
 	gtk_table_attach(table, sep, 0, DC_N_COL, row, row + 1, GTK_FILL, GTK_FILL | GTK_EXPAND,
 		entry_x_padding, sep_padding);
 	++row;
-	label = ui_label_new_bold(_("Execute Commands"));
+	label = ui_label_new_bold(_("Execute commands"));
 	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
 	gtk_table_attach(table, label, 0, DC_N_COL, row, row + 1, GTK_FILL, GTK_FILL | GTK_EXPAND,
 		entry_x_padding, entry_y_padding);
@@ -2113,11 +2113,8 @@ static gboolean read_row(BuildDestination *dst, BuildTableData table_data, gint
 {
 	gchar *entries[GEANY_BC_CMDENTRIES_COUNT];
 	gboolean changed = FALSE;
-	GeanyBuildSource src;
 	enum GeanyBuildCmdEntries i;
 
-	src = table_data->rows[drow]->src;
-
 	for (i = 0; i < GEANY_BC_CMDENTRIES_COUNT; i++)
 	{
 		entries[i] = g_strdup(get_build_command_entry_text(table_data->rows[drow]->entries[i]));
@@ -2256,7 +2253,7 @@ static void show_build_commands_dialog(void)
 	}
 	prefdsts.nonfileregexstr = &regex_pref;
 	if (build_read_commands(&prefdsts, table_data, response) && ft != NULL)
-		ft->home_save_needed = TRUE;
+		filetypes_save_commands(ft);
 	build_free_fields(table_data);
 
 	build_menu_update(doc);
@@ -2743,7 +2740,7 @@ void build_init(void)
 	gtk_container_add(GTK_CONTAINER(toolmenu), item);
 
 	/* arguments */
-	item = ui_image_menu_item_new(GTK_STOCK_PREFERENCES, _("_Set Build Menu Commands"));
+	item = ui_image_menu_item_new(GTK_STOCK_PREFERENCES, _("_Set Build Commands"));
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(toolmenu), item);
 	g_signal_connect(item, "activate", G_CALLBACK(on_set_build_commands_activate), NULL);
