@@ -1,8 +1,8 @@
 /*
  *      callbacks.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: callbacks.h 5427 2010-11-23 12:50:24Z ntrel $
+ * $Id: callbacks.h 5904 2011-08-25 20:15:02Z colombanw $
  */
 
 
@@ -142,6 +142,11 @@ on_toolbar_search_entry_changed        (GtkAction *action,
                                         const gchar *text,
                                         gpointer user_data);
 
+void
+on_toolbar_search_entry_activate       (GtkAction *action,
+                                        const gchar *text,
+                                        gpointer user_data);
+
 void
 on_toggle_case1_activate               (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
@@ -657,3 +662,11 @@ on_reset_indentation1_activate         (GtkMenuItem     *menuitem,
 void
 on_mark_all1_activate                  (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
+
+void
+on_detect_type_from_file_activate      (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_detect_width_from_file_activate     (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
