@@ -1,8 +1,8 @@
 /*
  *      document.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: document.c 5466 2010-12-06 18:04:42Z ntrel $
+ * $Id: document.c 5904 2011-08-25 20:15:02Z colombanw $
  */
 
 /*
@@ -48,13 +48,7 @@
 
 /* uncomment to use GIO based file monitoring, though it is not completely stable yet */
 /*#define USE_GIO_FILEMON 1*/
-#ifdef HAVE_GIO
-# include <gio/gio.h>
-#else
-# if USE_GIO_FILEMON
-#  undef USE_GIO_FILEMON
-# endif
-#endif
+#include <gio/gio.h>
 
 #include "document.h"
 #include "documentprivate.h"
@@ -143,7 +137,7 @@ GeanyDocument* document_find_by_real_path(const gchar *realname)
 	{
 		GeanyDocument *doc = documents[i];
 
-		if (! doc->is_valid || G_UNLIKELY(! doc->real_path))
+		if (! doc->is_valid || ! doc->real_path)
 			continue;
 
 		if (filenamecmp(realname, doc->real_path) == 0)
@@ -191,7 +185,7 @@ GeanyDocument *document_find_by_filename(const gchar *utf8_filename)
 	{
 		doc = documents[i];
 
-		if (! doc->is_valid || G_UNLIKELY(doc->file_name == NULL))
+		if (! doc->is_valid || doc->file_name == NULL)
 			continue;
 
 		if (filenamecmp(utf8_filename, doc->file_name) == 0)
@@ -313,7 +307,7 @@ gchar *document_get_basename_for_display(GeanyDocument *doc, gint length)
 		length = 30;
 
 	base_name = g_path_get_basename(DOC_FILENAME(doc));
-	short_name = utils_str_middle_truncate(base_name, length);
+	short_name = utils_str_middle_truncate(base_name, (guint)length);
 
 	g_free(base_name);
 
@@ -336,7 +330,7 @@ void document_update_tab_label(GeanyDocument *doc)
 
 	gtk_label_set_text(GTK_LABEL(doc->priv->tab_label), short_name);
 
-	ui_widget_set_tooltip_text(parent, DOC_FILENAME(doc));
+	gtk_widget_set_tooltip_text(parent, DOC_FILENAME(doc));
 
 	g_free(short_name);
 }
@@ -395,7 +389,8 @@ static void init_doc_struct(GeanyDocument *new_doc)
 	priv->undo_actions = NULL;
 	priv->redo_actions = NULL;
 	priv->line_count = 0;
-#if ! defined(USE_GIO_FILEMON)
+	priv->tag_list_update_source = 0;
+#ifndef USE_GIO_FILEMON
 	priv->last_check = time(NULL);
 #endif
 }
@@ -431,7 +426,7 @@ static void queue_colourise(GeanyDocument *doc)
 }
 
 
-#if USE_GIO_FILEMON
+#ifdef USE_GIO_FILEMON
 static void monitor_file_changed_cb(G_GNUC_UNUSED GFileMonitor *monitor, G_GNUC_UNUSED GFile *file,
 									G_GNUC_UNUSED GFile *other_file, GFileMonitorEvent event,
 									GeanyDocument *doc)
@@ -490,7 +485,7 @@ static void monitor_file_setup(GeanyDocument *doc)
 	 * doesn't work at all for remote files and legacy polling is too slow. */
 	if (! doc->priv->is_remote)
 	{
-#if USE_GIO_FILEMON
+#ifdef USE_GIO_FILEMON
 		gchar *locale_filename;
 
 		/* stop any previous monitoring */
@@ -780,7 +775,7 @@ GeanyDocument *document_new_file(const gchar *utf8_filename, GeanyFiletype *ft,
 	editor_goto_pos(doc->editor, 0, FALSE);
 	document_try_focus(doc, NULL);
 
-#if USE_GIO_FILEMON
+#ifdef USE_GIO_FILEMON
 	monitor_file_setup(doc);
 #else
 	doc->priv->mtime = time(NULL);
@@ -819,7 +814,6 @@ GeanyDocument *document_open_file(const gchar *locale_filename, gboolean readonl
 typedef struct
 {
 	gchar		*data;	/* null-terminated file data */
-	gsize		 size;	/* actual file size on disk */
 	gsize		 len;	/* string length of data */
 	gchar		*enc;
 	gboolean	 bom;
@@ -828,133 +822,12 @@ typedef struct
 } FileData;
 
 
-/* reload file with specified encoding */
-static gboolean
-handle_forced_encoding(FileData *filedata, const gchar *forced_enc)
-{
-	GeanyEncodingIndex enc_idx;
-
-	if (utils_str_equal(forced_enc, "UTF-8"))
-	{
-		if (! g_utf8_validate(filedata->data, filedata->len, NULL))
-		{
-			return FALSE;
-		}
-	}
-	else
-	{
-		gchar *converted_text = encodings_convert_to_utf8_from_charset(
-										filedata->data, filedata->size, forced_enc, FALSE);
-		if (converted_text == NULL)
-		{
-			return FALSE;
-		}
-		else
-		{
-			g_free(filedata->data);
-			filedata->data = converted_text;
-			filedata->len = strlen(converted_text);
-		}
-	}
-	enc_idx = encodings_scan_unicode_bom(filedata->data, filedata->size, NULL);
-	filedata->bom = (enc_idx == GEANY_ENCODING_UTF_8);
-	filedata->enc = g_strdup(forced_enc);
-	return TRUE;
-}
-
-
-/* detect encoding and convert to UTF-8 if necessary */
-static gboolean
-handle_encoding(FileData *filedata, GeanyEncodingIndex enc_idx)
-{
-	g_return_val_if_fail(filedata->enc == NULL, FALSE);
-	g_return_val_if_fail(filedata->bom == FALSE, FALSE);
-
-	if (filedata->size == 0)
-	{
-		/* we have no data so assume UTF-8, filedata->len can be 0 even we have an empty
-		 * e.g. UTF32 file with a BOM(so size is 4, len is 0) */
-		filedata->enc = g_strdup("UTF-8");
-	}
-	else
-	{
-		/* first check for a BOM */
-		if (enc_idx != GEANY_ENCODING_NONE)
-		{
-			filedata->enc = g_strdup(encodings[enc_idx].charset);
-			filedata->bom = TRUE;
-
-			if (enc_idx != GEANY_ENCODING_UTF_8) /* the BOM indicated something else than UTF-8 */
-			{
-				gchar *converted_text = encodings_convert_to_utf8_from_charset(
-										filedata->data, filedata->size, filedata->enc, FALSE);
-				if (converted_text != NULL)
-				{
-					g_free(filedata->data);
-					filedata->data = converted_text;
-					filedata->len = strlen(converted_text);
-				}
-				else
-				{
-					/* there was a problem converting data from BOM encoding type */
-					g_free(filedata->enc);
-					filedata->enc = NULL;
-					filedata->bom = FALSE;
-				}
-			}
-		}
-
-		if (filedata->enc == NULL)	/* either there was no BOM or the BOM encoding failed */
-		{
-			/* try UTF-8 first */
-			if ((filedata->size == filedata->len) &&
-				g_utf8_validate(filedata->data, filedata->len, NULL))
-			{
-				filedata->enc = g_strdup("UTF-8");
-			}
-			else
-			{
-				/* detect the encoding */
-				gchar *converted_text = encodings_convert_to_utf8(filedata->data,
-					filedata->size, &filedata->enc);
-
-				if (converted_text == NULL)
-				{
-					return FALSE;
-				}
-				g_free(filedata->data);
-				filedata->data = converted_text;
-				filedata->len = strlen(converted_text);
-			}
-		}
-	}
-	return TRUE;
-}
-
-
-static void
-handle_bom(FileData *filedata)
-{
-	guint bom_len;
-
-	encodings_scan_unicode_bom(filedata->data, filedata->size, &bom_len);
-	g_return_if_fail(bom_len != 0);
-
-	/* use filedata->len here because the contents are already converted into UTF-8 */
-	filedata->len -= bom_len;
-	/* overwrite the BOM with the remainder of the file contents, plus the NULL terminator. */
-	g_memmove(filedata->data, filedata->data + bom_len, filedata->len + 1);
-	filedata->data = g_realloc(filedata->data, filedata->len + 1);
-}
-
-
 /* loads textfile data, verifies and converts to forced_enc or UTF-8. Also handles BOM. */
 static gboolean load_text_file(const gchar *locale_filename, const gchar *display_filename,
 	FileData *filedata, const gchar *forced_enc)
 {
 	GError *err = NULL;
 	struct stat st;
-	GeanyEncodingIndex tmp_enc_idx;
 
 	filedata->data = NULL;
 	filedata->len = 0;
@@ -978,20 +851,26 @@ static gboolean load_text_file(const gchar *locale_filename, const gchar *displa
 		return FALSE;
 	}
 
-	/* use strlen to check for null chars */
-	filedata->size = (gsize) st.st_size;
-	filedata->len = strlen(filedata->data);
+	filedata->len = (gsize) st.st_size;
+	if (! encodings_convert_to_utf8_auto(&filedata->data, &filedata->len, forced_enc,
+				&filedata->enc, &filedata->bom, &filedata->readonly))
+	{
+		if (forced_enc)
+		{
+			ui_set_statusbar(TRUE, _("The file \"%s\" is not valid %s."),
+				display_filename, forced_enc);
+		}
+		else
+		{
+			ui_set_statusbar(TRUE,
+	_("The file \"%s\" does not look like a text file or the file encoding is not supported."),
+			display_filename);
+		}
+		g_free(filedata->data);
+		return FALSE;
+	}
 
-	/* temporarily retrieve the encoding idx based on the BOM to suppress the following warning
-	 * if we have a BOM */
-	tmp_enc_idx = encodings_scan_unicode_bom(filedata->data, filedata->size, NULL);
-
-	/* check whether the size of the loaded data is equal to the size of the file in the
-	 * filesystem file size may be 0 to allow opening files in /proc/ which have typically a
-	 * file size of 0 bytes */
-	if (filedata->len != filedata->size && filedata->size != 0 && (
-		tmp_enc_idx == GEANY_ENCODING_UTF_8 || /* tmp_enc_idx can be UTF-7/8/16/32, UCS and None */
-		tmp_enc_idx == GEANY_ENCODING_UTF_7))  /* filter UTF-7/8 where no NULL bytes are allowed */
+	if (filedata->readonly)
 	{
 		const gchar *warn_msg = _(
 			"The file \"%s\" could not be opened properly and has been truncated. " \
@@ -1002,43 +881,8 @@ static gboolean load_text_file(const gchar *locale_filename, const gchar *displa
 			dialogs_show_msgbox(GTK_MESSAGE_WARNING, warn_msg, display_filename);
 
 		ui_set_statusbar(TRUE, warn_msg, display_filename);
-
-		/* set the file to read-only mode because saving it is probably dangerous */
-		filedata->readonly = TRUE;
 	}
 
-	/* Determine character encoding and convert to UTF-8 */
-	if (forced_enc != NULL)
-	{
-		/* the encoding should be ignored(requested by user), so open the file "as it is" */
-		if (utils_str_equal(forced_enc, encodings[GEANY_ENCODING_NONE].charset))
-		{
-			filedata->bom = FALSE;
-			filedata->enc = g_strdup(encodings[GEANY_ENCODING_NONE].charset);
-		}
-		else if (! handle_forced_encoding(filedata, forced_enc))
-		{
-			/* For translators: the second wildcard is an encoding name, e.g.
-			 * The file \"test.txt\" is not valid UTF-8. */
-			ui_set_statusbar(TRUE, _("The file \"%s\" is not valid %s."),
-				display_filename, forced_enc);
-			utils_beep();
-			g_free(filedata->data);
-			return FALSE;
-		}
-	}
-	else if (! handle_encoding(filedata, tmp_enc_idx))
-	{
-		ui_set_statusbar(TRUE,
-	_("The file \"%s\" does not look like a text file or the file encoding is not supported."),
-			display_filename);
-		utils_beep();
-		g_free(filedata->data);
-		return FALSE;
-	}
-
-	if (filedata->bom)
-		handle_bom(filedata);
 	return TRUE;
 }
 
@@ -1082,7 +926,7 @@ static gboolean detect_tabs_and_spaces(GeanyEditor *editor)
 	ScintillaObject *sci = editor->sci;
 	gsize count = 0;
 	struct Sci_TextToFind ttf;
-	gchar *soft_tab = g_strnfill(iprefs->width, ' ');
+	gchar *soft_tab = g_strnfill((gsize)iprefs->width, ' ');
 	gchar *regex = g_strconcat("^\t+", soft_tab, "[^ ]", NULL);
 
 	g_free(soft_tab);
@@ -1106,16 +950,20 @@ static gboolean detect_tabs_and_spaces(GeanyEditor *editor)
 }
 
 
-/* Detect the indent type based on counting the leading indent characters for each line. */
-static GeanyIndentType detect_indent_type(GeanyEditor *editor)
+/* Detect the indent type based on counting the leading indent characters for each line.
+ * Returns whether detection succeeded, and the detected type in *type_ upon success */
+gboolean document_detect_indent_type(GeanyDocument *doc, GeanyIndentType *type_)
 {
-	const GeanyIndentPrefs *iprefs = editor_get_indent_prefs(editor);
+	GeanyEditor *editor = doc->editor;
 	ScintillaObject *sci = editor->sci;
-	guint line, line_count;
+	gint line, line_count;
 	gsize tabs = 0, spaces = 0;
 
 	if (detect_tabs_and_spaces(editor))
-		return GEANY_INDENT_TYPE_BOTH;
+	{
+		*type_ = GEANY_INDENT_TYPE_BOTH;
+		return TRUE;
+	}
 
 	line_count = sci_get_line_count(sci);
 	for (line = 0; line < line_count; line++)
@@ -1131,24 +979,84 @@ static GeanyIndentType detect_indent_type(GeanyEditor *editor)
 		c = sci_get_char_at(sci, pos);
 		if (c == '\t')
 			tabs++;
-		else
-		if (c == ' ')
-		{
-			/* check for at least 2 spaces */
-			if (sci_get_char_at(sci, pos + 1) == ' ')
-				spaces++;
-		}
+		/* check for at least 2 spaces */
+		else if (c == ' ' && sci_get_char_at(sci, pos + 1) == ' ')
+			spaces++;
 	}
 	if (spaces == 0 && tabs == 0)
-		return iprefs->type;
+		return FALSE;
 
 	/* the factors may need to be tweaked */
 	if (spaces > tabs * 4)
-		return GEANY_INDENT_TYPE_SPACES;
+		*type_ = GEANY_INDENT_TYPE_SPACES;
 	else if (tabs > spaces * 4)
-		return GEANY_INDENT_TYPE_TABS;
+		*type_ = GEANY_INDENT_TYPE_TABS;
 	else
-		return GEANY_INDENT_TYPE_BOTH;
+		*type_ = GEANY_INDENT_TYPE_BOTH;
+
+	return TRUE;
+}
+
+
+/* Detect the indent width based on counting the leading indent characters for each line.
+ * Returns whether detection succeeded, and the detected width in *width_ upon success */
+static gboolean detect_indent_width(GeanyEditor *editor, GeanyIndentType type, gint *width_)
+{
+	const GeanyIndentPrefs *iprefs = editor_get_indent_prefs(editor);
+	ScintillaObject *sci = editor->sci;
+	gint line, line_count;
+	gint widths[7] = { 0 }; /* width can be from 2 to 8 */
+	gint count, width, i;
+
+	/* can't easily detect the supposed width of a tab, guess the default is OK */
+	if (type == GEANY_INDENT_TYPE_TABS)
+		return FALSE;
+
+	/* force 8 at detection time for tab & spaces -- anyway we don't use tabs at this point */
+	sci_set_tab_width(sci, 8);
+
+	line_count = sci_get_line_count(sci);
+	for (line = 0; line < line_count; line++)
+	{
+		width = sci_get_line_indentation(sci, line);
+		/* most code will have indent total <= 24, otherwise it's more likely to be
+		 * alignment than indentation */
+		if (width > 24)
+			continue;
+		/* < 2 is no indentation */
+		if (width < 2)
+			continue;
+
+		for (i = G_N_ELEMENTS(widths) - 1; i >= 0; i--)
+		{
+			if ((width % (i + 2)) == 0)
+				widths[i]++;
+		}
+	}
+	count = 0;
+	width = iprefs->width;
+	for (i = G_N_ELEMENTS(widths) - 1; i >= 0; i--)
+	{
+		/* give large indents higher weight not to be fooled by spurious indents */
+		if (widths[i] >= count * 1.5)
+		{
+			width = i + 2;
+			count = widths[i];
+		}
+	}
+
+	if (count == 0)
+		return FALSE;
+
+	*width_ = width;
+	return TRUE;
+}
+
+
+/* same as detect_indent_width() but uses editor's indent type */
+gboolean document_detect_indent_width(GeanyDocument *doc, gint *width_)
+{
+	return detect_indent_width(doc->editor, doc->editor->indent_type, width_);
 }
 
 
@@ -1156,24 +1064,10 @@ void document_apply_indent_settings(GeanyDocument *doc)
 {
 	const GeanyIndentPrefs *iprefs = editor_get_indent_prefs(NULL);
 	GeanyIndentType type = iprefs->type;
+	gint width = iprefs->width;
 
-	switch (doc->file_type->id)
+	if (iprefs->detect_type && document_detect_indent_type(doc, &type))
 	{
-		case GEANY_FILETYPES_MAKE:
-			/* force using tabs for indentation for Makefiles */
-			editor_set_indent(doc->editor, GEANY_INDENT_TYPE_TABS, iprefs->width);
-			return;
-		case GEANY_FILETYPES_F77:
-			/* force using spaces for indentation for Fortran 77 */
-			editor_set_indent(doc->editor, GEANY_INDENT_TYPE_SPACES, iprefs->width);
-			return;
-		default:
-			break;
-	}
-	if (iprefs->detect_type)
-	{
-		type = detect_indent_type(doc->editor);
-
 		if (type != iprefs->type)
 		{
 			const gchar *name = NULL;
@@ -1196,29 +1090,24 @@ void document_apply_indent_settings(GeanyDocument *doc)
 				DOC_FILENAME(doc));
 		}
 	}
-	editor_set_indent(doc->editor, type, iprefs->width);
+	else if (doc->file_type->indent_type > -1)
+		type = doc->file_type->indent_type;
+
+	if (iprefs->detect_width && detect_indent_width(doc->editor, type, &width))
+	{
+		if (width != iprefs->width)
+		{
+			ui_set_statusbar(TRUE, _("Setting indentation width to %d for %s."), width,
+				DOC_FILENAME(doc));
+		}
+	}
+	else if (doc->file_type->indent_width > -1)
+		width = doc->file_type->indent_width;
+
+	editor_set_indent(doc->editor, type, width);
 }
 
 
-#if 0
-static gboolean auto_update_tag_list(gpointer data)
-{
-	GeanyDocument *doc = data;
-
-	if (! doc || ! doc->is_valid || doc->tm_file == NULL)
-		return FALSE;
-
-	if (gtk_window_get_focus(GTK_WINDOW(main_widgets.window)) != GTK_WIDGET(doc->editor->sci))
-		return TRUE;
-
-	if (update_tags_from_buffer(doc))
-		sidebar_update_tag_list(doc, TRUE);
-
-	return TRUE;
-}
-#endif
-
-
 /* To open a new file, set doc to NULL; filename should be locale encoded.
  * To reload a file, set the doc for the document to be reloaded; filename should be NULL.
  * pos is the cursor position, which can be overridden by --line and --column.
@@ -1356,28 +1245,27 @@ GeanyDocument *document_open_file_full(GeanyDocument *doc, const gchar *filename
 		if (! main_status.opening_session_files)
 			ui_add_recent_file(utf8_filename);
 
-		if (! reload)
-			g_signal_emit_by_name(geany_object, "document-open", doc);
-
 		if (reload)
+		{
+			g_signal_emit_by_name(geany_object, "document-reload", doc);
 			ui_set_statusbar(TRUE, _("File %s reloaded."), display_filename);
+		}
 		else
+		{
+			g_signal_emit_by_name(geany_object, "document-open", doc);
 			/* For translators: this is the status window message for opening a file. %d is the number
 			 * of the newly opened file, %s indicates whether the file is opened read-only
 			 * (it is replaced with the string ", read-only"). */
 			msgwin_status_add(_("File %s opened(%d%s)."),
 				display_filename, gtk_notebook_get_n_pages(GTK_NOTEBOOK(main_widgets.notebook)),
 				(readonly) ? _(", read-only") : "");
+		}
 	}
 
 	g_free(display_filename);
 	g_free(utf8_filename);
 	g_free(locale_filename);
 
-	/* TODO This could be used to automatically update the symbol list,
-	 * based on a configurable interval */
-	/*g_timeout_add(10000, auto_update_tag_list, doc);*/
-
 	/* set the cursor position according to pos, cl_options.goto_line and cl_options.goto_column */
 	pos = set_cursor_position(doc->editor, pos);
 	/* now bring the file in front */
@@ -1391,30 +1279,19 @@ GeanyDocument *document_open_file_full(GeanyDocument *doc, const gchar *filename
 
 
 /* Takes a new line separated list of filename URIs and opens each file.
- * length is the length of the string or -1 if it should be detected */
-void document_open_file_list(const gchar *data, gssize length)
+ * length is the length of the string */
+void document_open_file_list(const gchar *data, gsize length)
 {
-	gint i;
+	guint i;
 	gchar *filename;
 	gchar **list;
 
 	g_return_if_fail(data != NULL);
 
-	if (length < 0)
-		length = strlen(data);
+	list = g_strsplit(data, utils_get_eol_char(utils_get_line_endings(data, length)), 0);
 
-	switch (utils_get_line_endings(data, length))
+	for (i = 0; list[i] != NULL; i++)
 	{
-		case SC_EOL_CR: list = g_strsplit(data, "\r", 0); break;
-		case SC_EOL_CRLF: list = g_strsplit(data, "\r\n", 0); break;
-		case SC_EOL_LF: list = g_strsplit(data, "\n", 0); break;
-		default: list = g_strsplit(data, "\n", 0);
-	}
-
-	for (i = 0; ; i++)
-	{
-		if (list[i] == NULL)
-			break;
 		filename = g_filename_from_uri(list[i], NULL, NULL);
 		if (G_UNLIKELY(filename == NULL))
 			continue;
@@ -1473,7 +1350,7 @@ gboolean document_reload_file(GeanyDocument *doc, const gchar *forced_enc)
 
 static gboolean document_update_timestamp(GeanyDocument *doc, const gchar *locale_filename)
 {
-#if ! USE_GIO_FILEMON
+#ifndef USE_GIO_FILEMON
 	struct stat st;
 
 	g_return_val_if_fail(doc != NULL, FALSE);
@@ -1714,17 +1591,27 @@ static gchar *write_data_to_disk(const gchar *locale_filename,
 {
 	GError *error = NULL;
 
-	if (! file_prefs.use_safe_file_saving)
+	if (file_prefs.use_safe_file_saving)
+	{
+		/* Use old GLib API for safe saving (GVFS-safe, but alters ownership and permissons).
+		 * This is the only option that handles disk space exhaustion. */
+		if (g_file_set_contents(locale_filename, data, len, &error))
+			geany_debug("Wrote %s with g_file_set_contents().", locale_filename);
+	}
+	else if (file_prefs.use_gio_unsafe_file_saving)
 	{
-#ifdef HAVE_GIO
 		GFile *fp;
 
-		/* Use GIO API to save file (GVFS-safe) */
+		/* Use GIO API to save file (GVFS-safe)
+		 * It is best in most GVFS setups but don't seem to work correctly on some more complex
+		 * setups (saving from some VM to their host, over some SMB shares, etc.) */
 		fp = g_file_new_for_path(locale_filename);
 		g_file_replace_contents(fp, data, len, NULL, file_prefs.gio_unsafe_save_backup,
 			G_FILE_CREATE_NONE, NULL, NULL, &error);
 		g_object_unref(fp);
-#else
+	}
+	else
+	{
 		FILE *fp;
 		int save_errno;
 		gchar *display_name = g_filename_display_name(locale_filename);
@@ -1779,14 +1666,6 @@ static gchar *write_data_to_disk(const gchar *locale_filename,
 		}
 
 		g_free(display_name);
-#endif
-	}
-	else
-	{
-		/* Use old GLib API for safe saving (GVFS-safe, but alters ownership and permissons).
-		 * This is the only option that handles disk space exhaustion. */
-		if (g_file_set_contents(locale_filename, data, len, &error))
-			geany_debug("Wrote %s with g_file_set_contents().", locale_filename);
 	}
 	if (error != NULL)
 	{
@@ -1968,7 +1847,8 @@ gboolean document_save_file(GeanyDocument *doc, gboolean force)
 /* special search function, used from the find entry in the toolbar
  * return TRUE if text was found otherwise FALSE
  * return also TRUE if text is empty  */
-gboolean document_search_bar_find(GeanyDocument *doc, const gchar *text, gint flags, gboolean inc)
+gboolean document_search_bar_find(GeanyDocument *doc, const gchar *text, gint flags, gboolean inc,
+		gboolean backwards)
 {
 	gint start_pos, search_pos;
 	struct Sci_TextToFind ttf;
@@ -1978,20 +1858,28 @@ gboolean document_search_bar_find(GeanyDocument *doc, const gchar *text, gint fl
 	if (! *text)
 		return TRUE;
 
-	start_pos = (inc) ? sci_get_selection_start(doc->editor->sci) :
+	start_pos = (inc || backwards) ? sci_get_selection_start(doc->editor->sci) :
 		sci_get_selection_end(doc->editor->sci);	/* equal if no selection */
 
-	/* search cursor to end */
+	/* search cursor to end or start */
 	ttf.chrg.cpMin = start_pos;
-	ttf.chrg.cpMax = sci_get_length(doc->editor->sci);
+	ttf.chrg.cpMax = backwards ? 0 : sci_get_length(doc->editor->sci);
 	ttf.lpstrText = (gchar *)text;
 	search_pos = sci_find_text(doc->editor->sci, flags, &ttf);
 
-	/* if no match, search start to cursor */
+	/* if no match, search start (or end) to cursor */
 	if (search_pos == -1)
 	{
-		ttf.chrg.cpMin = 0;
-		ttf.chrg.cpMax = start_pos + strlen(text);
+		if (backwards)
+		{
+			ttf.chrg.cpMin = sci_get_length(doc->editor->sci);
+			ttf.chrg.cpMax = start_pos;
+		}
+		else
+		{
+			ttf.chrg.cpMin = 0;
+			ttf.chrg.cpMax = start_pos + strlen(text);
+		}
 		search_pos = sci_find_text(doc->editor->sci, flags, &ttf);
 	}
 
@@ -2030,9 +1918,13 @@ gboolean document_search_bar_find(GeanyDocument *doc, const gchar *text, gint fl
 
 /* General search function, used from the find dialog.
  * Returns -1 on failure or the start position of the matching text.
- * Will skip past any selection, ignoring it. */
-gint document_find_text(GeanyDocument *doc, const gchar *text, gint flags, gboolean search_backwards,
-		gboolean scroll, GtkWidget *parent)
+ * Will skip past any selection, ignoring it.
+ *
+ * @param text Text to find.
+ * @param original_text Text as it was entered by user, or @c NULL to use @c text
+ */
+gint document_find_text(GeanyDocument *doc, const gchar *text, const gchar *original_text,
+		gint flags, gboolean search_backwards, gboolean scroll, GtkWidget *parent)
 {
 	gint selection_end, selection_start, search_pos;
 
@@ -2044,6 +1936,9 @@ gint document_find_text(GeanyDocument *doc, const gchar *text, gint flags, gbool
 	if (flags & SCFIND_REGEXP)
 		search_backwards = FALSE;
 
+	if (!original_text)
+		original_text = text;
+
 	selection_start = sci_get_selection_start(doc->editor->sci);
 	selection_end = sci_get_selection_end(doc->editor->sci);
 	if ((selection_end - selection_start) > 0)
@@ -2076,7 +1971,7 @@ gint document_find_text(GeanyDocument *doc, const gchar *text, gint flags, gbool
 		if ((selection_end == 0 && ! search_backwards) ||
 			(selection_end == sci_len && search_backwards))
 		{
-			ui_set_statusbar(FALSE, _("\"%s\" was not found."), text);
+			ui_set_statusbar(FALSE, _("\"%s\" was not found."), original_text);
 			utils_beep();
 			return -1;
 		}
@@ -2084,12 +1979,12 @@ gint document_find_text(GeanyDocument *doc, const gchar *text, gint flags, gbool
 		/* we searched only part of the document, so ask whether to wraparound. */
 		if (search_prefs.suppress_dialogs ||
 			dialogs_show_question_full(parent, GTK_STOCK_FIND, GTK_STOCK_CANCEL,
-				_("Wrap search and find again?"), _("\"%s\" was not found."), text))
+				_("Wrap search and find again?"), _("\"%s\" was not found."), original_text))
 		{
 			gint ret;
 
 			sci_set_current_position(doc->editor->sci, (search_backwards) ? sci_len : 0, FALSE);
-			ret = document_find_text(doc, text, flags, search_backwards, scroll, parent);
+			ret = document_find_text(doc, text, original_text, flags, search_backwards, scroll, parent);
 			if (ret == -1)
 			{	/* return to original cursor position if not found */
 				sci_set_current_position(doc->editor->sci, selection_start, FALSE);
@@ -2102,9 +1997,13 @@ gint document_find_text(GeanyDocument *doc, const gchar *text, gint flags, gbool
 
 
 /* Replaces the selection if it matches, otherwise just finds the next match.
- * Returns: start of replaced text, or -1 if no replacement was made */
-gint document_replace_text(GeanyDocument *doc, const gchar *find_text, const gchar *replace_text,
-		gint flags, gboolean search_backwards)
+ * Returns: start of replaced text, or -1 if no replacement was made
+ *
+ * @param find_text Text to find.
+ * @param original_find_text Text to find as it was entered by user, or @c NULL to use @c find_text
+ */
+gint document_replace_text(GeanyDocument *doc, const gchar *find_text, const gchar *original_find_text,
+		const gchar *replace_text, gint flags, gboolean search_backwards)
 {
 	gint selection_end, selection_start, search_pos;
 
@@ -2117,12 +2016,15 @@ gint document_replace_text(GeanyDocument *doc, const gchar *find_text, const gch
 	if (flags & SCFIND_REGEXP)
 		search_backwards = FALSE;
 
+	if (!original_find_text)
+		original_find_text = find_text;
+
 	selection_start = sci_get_selection_start(doc->editor->sci);
 	selection_end = sci_get_selection_end(doc->editor->sci);
 	if (selection_end == selection_start)
 	{
 		/* no selection so just find the next match */
-		document_find_text(doc, find_text, flags, search_backwards, TRUE, NULL);
+		document_find_text(doc, find_text, original_find_text, flags, search_backwards, TRUE, NULL);
 		return -1;
 	}
 	/* there's a selection so go to the start before finding to search through it
@@ -2132,7 +2034,7 @@ gint document_replace_text(GeanyDocument *doc, const gchar *find_text, const gch
 	else
 		sci_goto_pos(doc->editor->sci, selection_start, TRUE);
 
-	search_pos = document_find_text(doc, find_text, flags, search_backwards, TRUE, NULL);
+	search_pos = document_find_text(doc, find_text, original_find_text, flags, search_backwards, TRUE, NULL);
 	/* return if the original selected text did not match (at the start of the selection) */
 	if (search_pos != selection_start)
 		return -1;
@@ -2156,37 +2058,22 @@ gint document_replace_text(GeanyDocument *doc, const gchar *find_text, const gch
 }
 
 
-static void show_replace_summary(GeanyDocument *doc, gint count, const gchar *find_text,
-		const gchar *replace_text, gboolean escaped_chars)
+static void show_replace_summary(GeanyDocument *doc, gint count, const gchar *original_find_text,
+	const gchar *original_replace_text)
 {
-	gchar *escaped_find_text, *escaped_replace_text, *filename;
+	gchar *filename;
 
 	if (count == 0)
 	{
-		ui_set_statusbar(FALSE, _("No matches found for \"%s\"."), find_text);
+		ui_set_statusbar(FALSE, _("No matches found for \"%s\"."), original_find_text);
 		return;
 	}
 
 	filename = g_path_get_basename(DOC_FILENAME(doc));
-
-	if (escaped_chars)
-	{	/* escape special characters for showing */
-		escaped_find_text = g_strescape(find_text, NULL);
-		escaped_replace_text = g_strescape(replace_text, NULL);
-		ui_set_statusbar(TRUE, ngettext(
-			"%s: replaced %d occurrence of \"%s\" with \"%s\".",
-			"%s: replaced %d occurrences of \"%s\" with \"%s\".",
-			count),	filename, count, escaped_find_text, escaped_replace_text);
-		g_free(escaped_find_text);
-		g_free(escaped_replace_text);
-	}
-	else
-	{
-		ui_set_statusbar(TRUE, ngettext(
-			"%s: replaced %d occurrence of \"%s\" with \"%s\".",
-			"%s: replaced %d occurrences of \"%s\" with \"%s\".",
-			count), filename, count, find_text, replace_text);
-	}
+	ui_set_statusbar(TRUE, ngettext(
+		"%s: replaced %d occurrence of \"%s\" with \"%s\".",
+		"%s: replaced %d occurrences of \"%s\" with \"%s\".",
+		count), filename, count, original_find_text, original_replace_text);
 	g_free(filename);
 }
 
@@ -2236,7 +2123,7 @@ document_replace_range(GeanyDocument *doc, const gchar *find_text, const gchar *
 
 
 void document_replace_sel(GeanyDocument *doc, const gchar *find_text, const gchar *replace_text,
-						  gint flags, gboolean escaped_chars)
+						  const gchar *original_find_text, const gchar *original_replace_text, gint flags)
 {
 	gint selection_end, selection_start, selection_mode, selected_lines, last_line = 0;
 	gint max_column = 0, count = 0;
@@ -2333,13 +2220,13 @@ void document_replace_sel(GeanyDocument *doc, const gchar *find_text, const gcha
 	else /* no replacements */
 		utils_beep();
 
-	show_replace_summary(doc, count, find_text, replace_text, escaped_chars);
+	show_replace_summary(doc, count, original_find_text, original_replace_text);
 }
 
 
 /* returns number of replacements made. */
 gint document_replace_all(GeanyDocument *doc, const gchar *find_text, const gchar *replace_text,
-		gint flags, gboolean escaped_chars)
+		const gchar *original_find_text, const gchar *original_replace_text, gint flags)
 {
 	gint len, count;
 	g_return_val_if_fail(doc != NULL && find_text != NULL && replace_text != NULL, FALSE);
@@ -2351,7 +2238,7 @@ gint document_replace_all(GeanyDocument *doc, const gchar *find_text, const gcha
 	count = document_replace_range(
 			doc, find_text, replace_text, flags, 0, len, TRUE, NULL);
 
-	show_replace_summary(doc, count, find_text, replace_text, escaped_chars);
+	show_replace_summary(doc, count, original_find_text, original_replace_text);
 	return count;
 }
 
@@ -2359,7 +2246,7 @@ gint document_replace_all(GeanyDocument *doc, const gchar *find_text, const gcha
 static gboolean update_tags_from_buffer(GeanyDocument *doc)
 {
 	gboolean result;
-#if 1
+#if 0
 		/* old code */
 		result = tm_source_file_update(doc->tm_file, TRUE, FALSE, TRUE);
 #else
@@ -2427,6 +2314,33 @@ void document_update_tag_list(GeanyDocument *doc, gboolean update)
 }
 
 
+static gboolean on_document_update_tag_list_idle(gpointer data)
+{
+	GeanyDocument *doc = data;
+
+	if (! DOC_VALID(doc))
+		return FALSE;
+
+	if (! main_status.quitting)
+		document_update_tag_list(doc, TRUE);
+
+	doc->priv->tag_list_update_source = 0;
+	return FALSE;
+}
+
+
+void document_update_tag_list_in_idle(GeanyDocument *doc)
+{
+	if (editor_prefs.autocompletion_update_freq <= 0 || ! filetype_has_tags(doc->file_type))
+		return;
+
+	if (doc->priv->tag_list_update_source != 0)
+		g_source_remove(doc->priv->tag_list_update_source);
+	doc->priv->tag_list_update_source = g_timeout_add_full(G_PRIORITY_LOW,
+		editor_prefs.autocompletion_update_freq, on_document_update_tag_list_idle, doc, NULL);
+}
+
+
 /* Caches the list of project typenames, as a space separated GString.
  * Returns: TRUE if typenames have changed.
  * (*types) is set to the list of typenames, or NULL if there are none. */
@@ -2585,6 +2499,18 @@ void document_set_filetype(GeanyDocument *doc, GeanyFiletype *type)
 
 	if (ft_changed)
 	{
+		const GeanyIndentPrefs *iprefs = editor_get_indent_prefs(NULL);
+
+		/* assume that if previous filetype was none and the settings are the default ones, this
+		 * is the first time the filetype is carefully set, so we should apply indent settings */
+		if (old_ft && old_ft->id == GEANY_FILETYPES_NONE &&
+			doc->editor->indent_type == iprefs->type &&
+			doc->editor->indent_width == iprefs->width)
+		{
+			document_apply_indent_settings(doc);
+			ui_document_show_hide(doc);
+		}
+
 		sidebar_openfiles_update(doc); /* to update the icon */
 		g_signal_emit_by_name(geany_object, "document-filetype-set", doc, old_ft);
 	}
@@ -2861,7 +2787,7 @@ const GdkColor *document_get_status_color(GeanyDocument *doc)
 {
 	static GdkColor red = {0, 0xFFFF, 0, 0};
 	static GdkColor green = {0, 0, 0x7FFF, 0};
-#if USE_GIO_FILEMON
+#ifdef USE_GIO_FILEMON
 	static GdkColor orange = {0, 0xFFFF, 0x7FFF, 0};
 #endif
 	GdkColor *color = NULL;
@@ -2870,7 +2796,7 @@ const GdkColor *document_get_status_color(GeanyDocument *doc)
 
 	if (doc->changed)
 		color = &red;
-#if USE_GIO_FILEMON
+#ifdef USE_GIO_FILEMON
 	else if (doc->priv->file_disk_status == FILE_CHANGED)
 		color = &orange;
 #endif
@@ -3084,8 +3010,8 @@ gboolean document_check_disk_status(GeanyDocument *doc, gboolean force)
 		/* doc may be closed now */
 		ret = TRUE;
 	}
-	else if (! use_gio_filemon && /* ignore these checks when using GIO */
-			 (G_UNLIKELY(doc->priv->mtime > cur_time) || G_UNLIKELY(st.st_mtime > cur_time)))
+	else if (G_UNLIKELY(! use_gio_filemon && /* ignore these checks when using GIO */
+			 (doc->priv->mtime > cur_time || st.st_mtime > cur_time)))
 	{
 		g_warning("%s: Something is wrong with the time stamps.", G_STRFUNC);
 	}
@@ -3109,3 +3035,97 @@ gboolean document_check_disk_status(GeanyDocument *doc, gboolean force)
 }
 
 
+/** Compares documents by their display names.
+ * This matches @c GCompareFunc for use with e.g. @c g_ptr_array_sort().
+ * @note 'Display name' means the base name of the document's filename.
+ *
+ * @param a @c GeanyDocument**.
+ * @param b @c GeanyDocument**.
+ * @warning The arguments take the address of each document pointer.
+ * @return Negative value if a < b; zero if a = b; positive value if a > b.
+ *
+ * @since 0.21
+ */
+gint document_compare_by_display_name(gconstpointer a, gconstpointer b)
+{
+	GeanyDocument *doc_a = *((GeanyDocument**) a);
+	GeanyDocument *doc_b = *((GeanyDocument**) b);
+	gchar *base_name_a, *base_name_b;
+	gint result;
+
+	base_name_a = g_path_get_basename(DOC_FILENAME(doc_a));
+	base_name_b = g_path_get_basename(DOC_FILENAME(doc_b));
+
+	result = strcmp(base_name_a, base_name_b);
+
+	g_free(base_name_a);
+	g_free(base_name_b);
+
+	return result;
+}
+
+
+/** Compares documents by their tab order.
+ * This matches @c GCompareFunc for use with e.g. @c g_ptr_array_sort().
+ *
+ * @param a @c GeanyDocument**.
+ * @param b @c GeanyDocument**.
+ * @warning The arguments take the address of each document pointer.
+ * @return Negative value if a < b; zero if a = b; positive value if a > b.
+ *
+ * @since 0.21 (GEANY_API_VERSION 209)
+ */
+gint document_compare_by_tab_order(gconstpointer a, gconstpointer b)
+{
+	GeanyDocument *doc_a = *((GeanyDocument**) a);
+	GeanyDocument *doc_b = *((GeanyDocument**) b);
+	gint notebook_position_doc_a;
+	gint notebook_position_doc_b;
+
+	notebook_position_doc_a = document_get_notebook_page(doc_a);
+	notebook_position_doc_b = document_get_notebook_page(doc_b);
+
+	if (notebook_position_doc_a < notebook_position_doc_b)
+		return -1;
+	if (notebook_position_doc_a > notebook_position_doc_b)
+		return 1;
+	/* equality */
+	return 0;
+}
+
+
+/** Compares documents by their tab order, in reverse order.
+ * This matches @c GCompareFunc for use with e.g. @c g_ptr_array_sort().
+ *
+ * @param a @c GeanyDocument**.
+ * @param b @c GeanyDocument**.
+ * @warning The arguments take the address of each document pointer.
+ * @return Negative value if a < b; zero if a = b; positive value if a > b.
+ *
+ * @since 0.21 (GEANY_API_VERSION 209)
+ */
+gint document_compare_by_tab_order_reverse(gconstpointer a, gconstpointer b)
+{
+	GeanyDocument *doc_a = *((GeanyDocument**) a);
+	GeanyDocument *doc_b = *((GeanyDocument**) b);
+	gint notebook_position_doc_a;
+	gint notebook_position_doc_b;
+
+	notebook_position_doc_a = document_get_notebook_page(doc_a);
+	notebook_position_doc_b = document_get_notebook_page(doc_b);
+
+	if (notebook_position_doc_a < notebook_position_doc_b)
+		return 1;
+	if (notebook_position_doc_a > notebook_position_doc_b)
+		return -1;
+	/* equality */
+	return 0;
+}
+
+
+void document_grab_focus(GeanyDocument *doc)
+{
+	g_return_if_fail(doc != NULL);
+
+	gtk_widget_grab_focus(GTK_WIDGET(doc->editor->sci));
+}
