@@ -1,8 +1,8 @@
 /*
  *      document.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: document.h 3010 2008-09-26 17:28:50Z ntrel $
+ *  $Id: document.h 5904 2011-08-25 20:15:02Z colombanw $
  */
 
 /**
@@ -33,6 +33,7 @@
 
 #include "Scintilla.h"
 #include "ScintillaWidget.h"
+#include "editor.h"
 
 #if defined(G_OS_WIN32)
 # define GEANY_DEFAULT_EOL_CHARACTER SC_EOL_CRLF
@@ -43,6 +44,7 @@
 #endif
 
 
+/** File Prefs. */
 typedef struct GeanyFilePrefs
 {
 	gint			default_new_encoding;
@@ -51,11 +53,16 @@ typedef struct GeanyFilePrefs
 	gboolean		strip_trailing_spaces;
 	gboolean		replace_tabs;
 	gboolean		tab_order_ltr;
+	gboolean		tab_order_beside;
 	gboolean		show_tab_cross;
 	guint			mru_length;
 	gint			default_eol_character;
 	gint			disk_check_timeout;
 	gboolean		cmdline_new_files;	/* New file if command-line filename doesn't exist */
+	gboolean		use_safe_file_saving;
+	gboolean		ensure_convert_new_lines;
+	gboolean		gio_unsafe_save_backup;
+	gboolean		use_gio_unsafe_file_saving; /* whether to use GIO as the unsafe backend */
 }
 GeanyFilePrefs;
 
@@ -70,7 +77,7 @@ struct GeanyDocument
 	/** General flag to represent this document is active and all properties are set correctly. */
 	gboolean		 is_valid;
 	gint			 index;		/**< Index in the documents array. */
-	/** Whether this %document support source code symbols(tags) to show in the sidebar. */
+	/** Whether this document supports source code symbols(tags) to show in the sidebar. */
 	gboolean		 has_tags;
 	/** The UTF-8 encoded file name.
 	 * Be careful; glibc and GLib file functions expect the locale representation of the
@@ -78,25 +85,21 @@ struct GeanyDocument
 	 * For conversion into locale encoding, you can use @ref utils_get_locale_from_utf8().
 	 * @see real_path. */
 	gchar 			*file_name;
-	/** The encoding of the %document, must be a valid string representation of an encoding, can
+	/** The encoding of the document, must be a valid string representation of an encoding, can
 	 *  be retrieved with @ref encodings_get_charset_from_index. */
 	gchar 			*encoding;
-	/** Internally used flag to indicate whether the file of this %document has a byte-order-mark. */
+	/** Internally used flag to indicate whether the file of this document has a byte-order-mark. */
 	gboolean		 has_bom;
 	struct GeanyEditor *editor;	/**< The editor associated with the document. */
-	/** The filetype for this %document, it's only a reference to one of the elements of the global
+	/** The filetype for this document, it's only a reference to one of the elements of the global
 	 *  filetypes array. */
 	GeanyFiletype	*file_type;
-	/** TMWorkObject object for this %document. */
+	/** TMWorkObject object for this document, or @c NULL. */
 	TMWorkObject	*tm_file;
-	/** Whether this %document is read-only. */
+	/** Whether this document is read-only. */
 	gboolean		 readonly;
-	/** Whether this %document has been changed since it was last saved. */
+	/** Whether this document has been changed since it was last saved. */
 	gboolean		 changed;
-	/** Time of the last disk check. */
-	time_t			 last_check;
-	/** Modification time of this %document on disk. */
-	time_t			 mtime;
 	/** The link-dereferenced, locale-encoded file name.
 	 * If non-NULL, this indicates the file once existed on disk (not just as an
 	 * unsaved document with a filename set).
@@ -112,27 +115,43 @@ struct GeanyDocument
 extern GPtrArray *documents_array;
 
 
-/**
- *  This wraps documents_array so it can be used with C array syntax.
- *  Example: documents[0]->sci = NULL;
- **/
-#define documents ((GeanyDocument **)documents_array->pdata)
+/** Wraps documents_array so it can be used with C array syntax.
+ * Example: documents[0]->sci = NULL;
+ * @see document_index(). */
+#define documents ((GeanyDocument **)GEANY(documents_array)->pdata)
 
-/** Check that the @a doc_ptr document still exists (has not been closed).
+/** @deprecated Use @ref foreach_document() instead.
+ * Iterates all valid documents.
+ * Use like a @c for statement.
+ * @param i @c guint index for document_index(). */
+#ifndef GEANY_DISABLE_DEPRECATED
+#define documents_foreach(i) foreach_document(i)
+#endif
+
+/** Iterates all valid documents.
+ * Use like a @c for statement.
+ * @param i @c guint index for document_index(). */
+#define foreach_document(i) \
+	for (i = 0; i < GEANY(documents_array)->len; i++)\
+		if (!documents[i]->is_valid)\
+			{}\
+		else /* prevent outside 'else' matching our macro 'if' */
+
+/** @c NULL-safe way to check @c doc_ptr->is_valid.
  * This is useful when @a doc_ptr was stored some time earlier and documents may have been
  * closed since then.
  * @note This should not be used to check the result of the main API functions,
- * these only need a NULL-pointer check - @c p_document->get_current() != @c NULL. */
+ * these only need a NULL-pointer check - @c document_get_current() != @c NULL. */
 #define DOC_VALID(doc_ptr) \
-	((doc_ptr) != NULL && (doc_ptr)->is_valid)
+	(G_LIKELY((doc_ptr) != NULL && (doc_ptr)->is_valid))
 
 /**
- *  DOC_FILENAME returns the filename of the document passed or
- *  GEANY_STRING_UNTITLED (e.g. _("untitled")) if the %document's filename was not yet set.
- *  This macro never returns NULL.
+ *  Returns the filename of the document passed or @c GEANY_STRING_UNTITLED
+ *  (e.g. _("untitled")) if the document's filename was not yet set.
+ *  This macro never returns @c NULL.
  **/
 #define DOC_FILENAME(doc) \
-	((doc->file_name != NULL) ? (doc->file_name) : GEANY_STRING_UNTITLED)
+	(G_LIKELY((doc)->file_name != NULL) ? ((doc)->file_name) : GEANY_STRING_UNTITLED)
 
 
 
@@ -140,7 +159,7 @@ extern GPtrArray *documents_array;
 
 GeanyDocument* document_new_file(const gchar *filename, GeanyFiletype *ft, const gchar *text);
 
-GeanyDocument* document_new_file_if_non_open();
+GeanyDocument* document_new_file_if_non_open(void);
 
 GeanyDocument* document_find_by_filename(const gchar *utf8_filename);
 
@@ -159,6 +178,11 @@ void document_set_text_changed(GeanyDocument *doc, gboolean changed);
 
 void document_set_filetype(GeanyDocument *doc, GeanyFiletype *type);
 
+void document_reload_config(GeanyDocument *doc);
+
+void document_rename_file(GeanyDocument *doc, const gchar *new_filename);
+
+GeanyDocument *document_index(gint idx);
 
 GeanyDocument *document_find_by_sci(ScintillaObject *sci);
 
@@ -174,6 +198,8 @@ void document_finalize(void);
 
 gboolean document_remove_page(guint page_num);
 
+void document_try_focus(GeanyDocument *doc, GtkWidget *source_widget);
+
 gboolean document_close(GeanyDocument *doc);
 
 gboolean document_account_for_unsaved(void);
@@ -185,27 +211,30 @@ GeanyDocument *document_clone(GeanyDocument *old_doc, const gchar *utf8_filename
 GeanyDocument *document_open_file_full(GeanyDocument *doc, const gchar *filename, gint pos,
 		gboolean readonly, GeanyFiletype *ft, const gchar *forced_enc);
 
-void document_open_file_list(const gchar *data, gssize length);
+void document_open_file_list(const gchar *data, gsize length);
 
 void document_open_files(const GSList *filenames, gboolean readonly, GeanyFiletype *ft,
 		const gchar *forced_enc);
 
-gboolean document_search_bar_find(GeanyDocument *doc, const gchar *text, gint flags, gboolean inc);
+gboolean document_search_bar_find(GeanyDocument *doc, const gchar *text, gint flags, gboolean inc,
+		gboolean backwards);
 
-gint document_find_text(GeanyDocument *doc, const gchar *text, gint flags, gboolean search_backwards,
-		gboolean scroll, GtkWidget *parent);
+gint document_find_text(GeanyDocument *doc, const gchar *text, const gchar *original_text,
+		gint flags, gboolean search_backwards, gboolean scroll, GtkWidget *parent);
 
-gint document_replace_text(GeanyDocument *doc, const gchar *find_text, const gchar *replace_text,
-		gint flags, gboolean search_backwards);
+gint document_replace_text(GeanyDocument *doc, const gchar *find_text, const gchar *original_find_text,
+		const gchar *replace_text, gint flags, gboolean search_backwards);
 
-gboolean document_replace_all(GeanyDocument *doc, const gchar *find_text, const gchar *replace_text,
-		gint flags, gboolean escaped_chars);
+gint document_replace_all(GeanyDocument *doc, const gchar *find_text, const gchar *replace_text,
+		const gchar *original_find_text, const gchar *original_replace_text, gint flags);
 
-void document_replace_sel(GeanyDocument *doc, const gchar *find_text, const gchar *replace_text, gint flags,
-						  gboolean escaped_chars);
+void document_replace_sel(GeanyDocument *doc, const gchar *find_text, const gchar *replace_text,
+						  const gchar *original_find_text, const gchar *original_replace_text, gint flags);
 
 void document_update_tag_list(GeanyDocument *doc, gboolean update);
 
+void document_update_tag_list_in_idle(GeanyDocument *doc);
+
 void document_set_encoding(GeanyDocument *doc, const gchar *new_encoding);
 
 gboolean document_check_disk_status(GeanyDocument *doc, gboolean force);
@@ -224,7 +253,26 @@ void document_redo(GeanyDocument *doc);
 
 void document_undo_add(GeanyDocument *doc, guint type, gpointer data);
 
+void document_update_tab_label(GeanyDocument *doc);
 
-GdkColor *document_get_status_color(GeanyDocument *doc);
+const GdkColor *document_get_status_color(GeanyDocument *doc);
+
+gchar *document_get_basename_for_display(GeanyDocument *doc, gint length);
+
+gboolean document_need_save_as(GeanyDocument *doc);
+
+gboolean document_detect_indent_type(GeanyDocument *doc, GeanyIndentType *type_);
+
+gboolean document_detect_indent_width(GeanyDocument *doc, gint *width_);
+
+void document_apply_indent_settings(GeanyDocument *doc);
+
+gint document_compare_by_display_name(gconstpointer a, gconstpointer b);
+
+gint document_compare_by_tab_order(gconstpointer a, gconstpointer b);
+
+gint document_compare_by_tab_order_reverse(gconstpointer a, gconstpointer b);
+
+void document_grab_focus(GeanyDocument *doc);
 
 #endif
