@@ -1,8 +1,8 @@
 /*
  *      document-private.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2008-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2008-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2008-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2008-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- *  $Id: documentprivate.h 4063 2009-08-09 20:31:03Z eht16 $
+ *  $Id: documentprivate.h 5742 2011-04-26 16:21:47Z colombanw $
  */
 
 
@@ -84,6 +84,8 @@ typedef struct GeanyDocumentPrivate
 	time_t			 last_check;
 	/* Modification time of the document on disk, only used when legacy file monitoring is used. */
 	time_t			 mtime;
+	/* ID of the idle callback updating the tag list */
+	guint			 tag_list_update_source;
 }
 GeanyDocumentPrivate;
 
