@@ -1,8 +1,8 @@
 /*
  *      editor.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: editor.h 4025 2009-07-24 15:44:14Z ntrel $
+ * $Id: editor.h 5903 2011-08-25 20:14:40Z colombanw $
  */
 
 
@@ -28,7 +28,7 @@
 #include "Scintilla.h"
 #include "ScintillaWidget.h"
 
-/** Default character set to define which characters shoud be treated as part of a word. */
+/** Default character set to define which characters should be treated as part of a word. */
 #define GEANY_WORDCHARS					"_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"
 #define GEANY_MAX_WORD_LENGTH			192
 
@@ -51,6 +51,14 @@ typedef enum
 }
 GeanyAutoIndent;
 
+typedef enum
+{
+	GEANY_VIRTUAL_SPACE_DISABLED = 0,
+	GEANY_VIRTUAL_SPACE_SELECTION = 1,
+	GEANY_VIRTUAL_SPACE_ALWAYS = 3
+}
+GeanyVirtualSpace;
+
 
 /* Auto-close brackets/quotes */
 enum {
@@ -88,25 +96,30 @@ typedef struct GeanyIndentPrefs
 	gint			hard_tab_width;
 	GeanyAutoIndent	auto_indent_mode;
 	gboolean		detect_type;
+	gboolean		detect_width;
 }
 GeanyIndentPrefs;
 
 
-/* Default prefs when creating a new editor window.
- * Some of these can be overridden per document. */
+/** Default prefs when creating a new editor window.
+ * Some of these can be overridden per document or per project. */
+/* See editor_get_prefs(). */
 typedef struct GeanyEditorPrefs
 {
-	GeanyIndentPrefs *indentation;	/*< Default indentation prefs. @see editor_get_indent_prefs(). */
+	GeanyIndentPrefs *indentation;	/* Default indentation prefs. Use editor_get_indent_prefs(). */
 	gboolean	show_white_space;
 	gboolean	show_indent_guide;
 	gboolean	show_line_endings;
-	gint		long_line_type; /* 0 - line, 1 - background, 2 - none */
+	/* 0 - line, 1 - background, 2 - disabled.
+	 * This setting may be overridden when a project is opened. Use @c editor_get_prefs(). */
+	gint		long_line_type;
+	/* This setting may be overridden when a project is opened. Use @c editor_get_prefs(). */
 	gint		long_line_column;
 	gchar		*long_line_color;
 	gboolean	show_markers_margin;		/* view menu */
 	gboolean	show_linenumber_margin;		/* view menu */
 	gboolean	show_scrollbars;			/* hidden pref */
-	gboolean	scroll_stop_at_last_line;	/* hidden pref */
+	gboolean	scroll_stop_at_last_line;
 	gboolean	line_wrapping;
 	gboolean	use_indicators;
 	gboolean	folding;
@@ -120,7 +133,6 @@ typedef struct GeanyEditorPrefs
 	gboolean	complete_snippets;
 	gint		symbolcompletion_min_chars;
 	gint		symbolcompletion_max_height;
-	GHashTable	*snippets;
 	gboolean	brace_match_ltgt;	/* whether to highlight < and > chars (hidden pref) */
 	gboolean	use_gtk_word_boundaries;	/* hidden pref */
 	gboolean	complete_snippets_whilst_editing;	/* hidden pref */
@@ -131,6 +143,11 @@ typedef struct GeanyEditorPrefs
 	guint		autoclose_chars;
 	gboolean	autocomplete_doc_words;
 	gboolean	completion_drops_rest_of_word;
+	gchar		*color_scheme;
+	gint 		show_virtual_space;
+	/* This setting may be overridden when a project is opened. Use @c editor_get_prefs(). */
+	gboolean	long_line_enabled;
+	gint		autocompletion_update_freq;
 }
 GeanyEditorPrefs;
 
@@ -148,6 +165,7 @@ struct GeanyEditor
 	gfloat			 scroll_percent;
 	GeanyIndentType	 indent_type;	/* Use editor_get_indent_prefs() instead. */
 	gboolean		 line_breaking;	/**< Whether to split long lines as you type. */
+	gint			 indent_width;
 };
 
 
@@ -174,7 +192,9 @@ void editor_sci_notify_cb(GtkWidget *widget, gint scn, gpointer scnt, gpointer d
 
 gboolean editor_start_auto_complete(GeanyEditor *editor, gint pos, gboolean force);
 
-void snippet_goto_next_cursor(ScintillaObject *sci, gint current_pos);
+gboolean editor_complete_word_part(GeanyEditor *editor);
+
+void editor_goto_next_snippet_cursor(GeanyEditor *editor);
 
 gboolean editor_complete_snippet(GeanyEditor *editor, gint pos);
 
@@ -212,21 +232,30 @@ void editor_snippets_init(void);
 
 void editor_snippets_free(void);
 
+const GeanyEditorPrefs *editor_get_prefs(GeanyEditor *editor);
+
+
 /* General editing functions */
 
-void editor_find_current_word(GeanyEditor *editor, gint pos, gchar *word, size_t wordlen,
+void editor_find_current_word(GeanyEditor *editor, gint pos, gchar *word, gsize wordlen,
 	const gchar *wc);
 
+void editor_find_current_word_sciwc(GeanyEditor *editor, gint pos, gchar *word, gsize wordlen);
+
 gchar *editor_get_word_at_pos(GeanyEditor *editor, gint pos, const gchar *wordchars);
 
 gchar *editor_get_default_selection(GeanyEditor *editor, gboolean use_current_word, const gchar *wordchars);
 
+
 void editor_select_word(GeanyEditor *editor);
 
 void editor_select_lines(GeanyEditor *editor, gboolean extra_line);
 
 void editor_select_paragraph(GeanyEditor *editor);
 
+void editor_select_indent_block(GeanyEditor *editor);
+
+
 void editor_set_font(GeanyEditor *editor, const gchar *font);
 
 void editor_indicator_set_on_line(GeanyEditor *editor, gint indic, gint line);
@@ -237,6 +266,8 @@ void editor_indicator_set_on_range(GeanyEditor *editor, gint indic, gint start,
 
 void editor_indicator_clear(GeanyEditor *editor, gint indic);
 
+gint editor_get_eol_char_mode(GeanyEditor *editor);
+
 const gchar *editor_get_eol_char_name(GeanyEditor *editor);
 
 gint editor_get_eol_char_len(GeanyEditor *editor);
@@ -263,11 +294,15 @@ const GeanyIndentPrefs *editor_get_indent_prefs(GeanyEditor *editor);
 
 void editor_set_indent_type(GeanyEditor *editor, GeanyIndentType type);
 
+void editor_set_indent_width(GeanyEditor *editor, gint width);
+
+void editor_set_indent(GeanyEditor *editor, GeanyIndentType type, gint width);
+
 void editor_set_line_wrapping(GeanyEditor *editor, gboolean wrap);
 
 gboolean editor_goto_pos(GeanyEditor *editor, gint pos, gboolean mark);
 
-gboolean editor_goto_line(GeanyEditor *editor, gint line_no);
+gboolean editor_goto_line(GeanyEditor *editor, gint line_no, gint offset);
 
 void editor_set_indentation_guides(GeanyEditor *editor);
 
@@ -275,4 +310,14 @@ void editor_apply_update_prefs(GeanyEditor *editor);
 
 gchar *editor_get_calltip_text(GeanyEditor *editor, const TMTag *tag);
 
+void editor_insert_text_block(GeanyEditor *editor, const gchar *text,
+	 						  gint insert_pos, gint cursor_index,
+	 						  gint newline_indent_size, gboolean replace_newlines);
+
+void editor_toggle_fold(GeanyEditor *editor, gint line, gint modifiers);
+
+const gchar *editor_find_snippet(GeanyEditor *editor, const gchar *snippet_name);
+
+void editor_insert_snippet(GeanyEditor *editor, gint pos, const gchar *snippet);
+
 #endif
