@@ -1,8 +1,8 @@
 /*
  *      encodings.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: encodings.c 5221 2010-09-09 17:21:05Z ntrel $
+ *  $Id: encodings.c 5908 2011-09-02 21:45:20Z colombanw $
  */
 
 /*
@@ -51,9 +51,9 @@
 #endif
 
 /* <meta http-equiv="content-type" content="text/html; charset=UTF-8" /> */
-#define PATTERN_HTMLMETA "<meta[ \t\n\r\f]http-equiv[ \t\n\r\f]*=[ \t\n\r\f]*\"content-type\"[ \t\n\r\f]+content[ \t\n\r\f]*=[ \t\n\r\f]*\"text/x?html;[ \t\n\r\f]*charset=([a-z0-9_-]+)\"[ \t\n\r\f]*/?>"
+#define PATTERN_HTMLMETA "<meta[ \t\n\r\f]+http-equiv[ \t\n\r\f]*=[ \t\n\r\f]*\"?content-type\"?[ \t\n\r\f]+content[ \t\n\r\f]*=[ \t\n\r\f]*\"text/x?html;[ \t\n\r\f]*charset=([a-z0-9_-]+)\"[ \t\n\r\f]*/?>"
 /* " geany_encoding=utf-8 " or " coding: utf-8 " */
-#define PATTERN_CODING "coding[\t ]*[:=][\t ]*([a-z0-9-]+)[\t ]*"
+#define PATTERN_CODING "coding[\t ]*[:=][\t ]*\"?([a-z0-9-]+)\"?[\t ]*"
 
 /* precompiled regexps */
 static regex_t pregs[2];
@@ -72,78 +72,128 @@ GeanyEncoding encodings[GEANY_ENCODINGS_MAX];
 
 static void init_encodings(void)
 {
-	fill(0, WESTEUROPEAN, GEANY_ENCODING_ISO_8859_14, "ISO-8859-14", _("Celtic"));
-	fill(1, WESTEUROPEAN, GEANY_ENCODING_ISO_8859_7, "ISO-8859-7", _("Greek"));
-	fill(2, WESTEUROPEAN, GEANY_ENCODING_WINDOWS_1253, "WINDOWS-1253", _("Greek"));
-	fill(3, WESTEUROPEAN, GEANY_ENCODING_ISO_8859_10, "ISO-8859-10", _("Nordic"));
-	fill(4, WESTEUROPEAN, GEANY_ENCODING_ISO_8859_3, "ISO-8859-3", _("South European"));
-	fill(5, WESTEUROPEAN, GEANY_ENCODING_IBM_850, "IBM850", _("Western"));
-	fill(6, WESTEUROPEAN, GEANY_ENCODING_ISO_8859_1, "ISO-8859-1", _("Western"));
-	fill(7, WESTEUROPEAN, GEANY_ENCODING_ISO_8859_15, "ISO-8859-15", _("Western"));
-	fill(8, WESTEUROPEAN, GEANY_ENCODING_WINDOWS_1252, "WINDOWS-1252", _("Western"));
+	fill(0,		WESTEUROPEAN,	GEANY_ENCODING_ISO_8859_14,		"ISO-8859-14",		_("Celtic"));
+	fill(1,		WESTEUROPEAN,	GEANY_ENCODING_ISO_8859_7,		"ISO-8859-7",		_("Greek"));
+	fill(2,		WESTEUROPEAN,	GEANY_ENCODING_WINDOWS_1253,	"WINDOWS-1253",		_("Greek"));
+	fill(3,		WESTEUROPEAN,	GEANY_ENCODING_ISO_8859_10,		"ISO-8859-10",		_("Nordic"));
+	fill(4,		WESTEUROPEAN,	GEANY_ENCODING_ISO_8859_3,		"ISO-8859-3",		_("South European"));
+	fill(5,		WESTEUROPEAN,	GEANY_ENCODING_IBM_850,			"IBM850",			_("Western"));
+	fill(6,		WESTEUROPEAN,	GEANY_ENCODING_ISO_8859_1,		"ISO-8859-1",		_("Western"));
+	fill(7,		WESTEUROPEAN,	GEANY_ENCODING_ISO_8859_15,		"ISO-8859-15",		_("Western"));
+	fill(8,		WESTEUROPEAN,	GEANY_ENCODING_WINDOWS_1252,	"WINDOWS-1252",		_("Western"));
 
-	fill(0, EASTEUROPEAN, GEANY_ENCODING_ISO_8859_4, "ISO-8859-4", _("Baltic"));
-	fill(1, EASTEUROPEAN, GEANY_ENCODING_ISO_8859_13, "ISO-8859-13", _("Baltic"));
-	fill(2, EASTEUROPEAN, GEANY_ENCODING_WINDOWS_1257, "WINDOWS-1257", _("Baltic"));
-	fill(3, EASTEUROPEAN, GEANY_ENCODING_IBM_852, "IBM852", _("Central European"));
-	fill(4, EASTEUROPEAN, GEANY_ENCODING_ISO_8859_2, "ISO-8859-2", _("Central European"));
-	fill(5, EASTEUROPEAN, GEANY_ENCODING_WINDOWS_1250, "WINDOWS-1250", _("Central European"));
-	fill(6, EASTEUROPEAN, GEANY_ENCODING_IBM_855, "IBM855", _("Cyrillic"));
-	fill(7, EASTEUROPEAN, GEANY_ENCODING_ISO_8859_5, "ISO-8859-5", _("Cyrillic"));
+	fill(0,		EASTEUROPEAN,	GEANY_ENCODING_ISO_8859_4,		"ISO-8859-4",		_("Baltic"));
+	fill(1,		EASTEUROPEAN,	GEANY_ENCODING_ISO_8859_13,		"ISO-8859-13",		_("Baltic"));
+	fill(2,		EASTEUROPEAN,	GEANY_ENCODING_WINDOWS_1257,	"WINDOWS-1257",		_("Baltic"));
+	fill(3,		EASTEUROPEAN,	GEANY_ENCODING_IBM_852,			"IBM852",			_("Central European"));
+	fill(4,		EASTEUROPEAN,	GEANY_ENCODING_ISO_8859_2,		"ISO-8859-2",		_("Central European"));
+	fill(5,		EASTEUROPEAN,	GEANY_ENCODING_WINDOWS_1250,	"WINDOWS-1250",		_("Central European"));
+	fill(6,		EASTEUROPEAN,	GEANY_ENCODING_IBM_855,			"IBM855",			_("Cyrillic"));
+	fill(7,		EASTEUROPEAN,	GEANY_ENCODING_ISO_8859_5,		"ISO-8859-5",		_("Cyrillic"));
 	/* ISO-IR-111 not available on Windows */
-	fill(8, EASTEUROPEAN, GEANY_ENCODING_ISO_IR_111, "ISO-IR-111", _("Cyrillic"));
-	fill(9, EASTEUROPEAN, GEANY_ENCODING_KOI8_R, "KOI8-R", _("Cyrillic"));
-	fill(10, EASTEUROPEAN, GEANY_ENCODING_WINDOWS_1251, "WINDOWS-1251", _("Cyrillic"));
-	fill(11, EASTEUROPEAN, GEANY_ENCODING_CP_866, "CP866", _("Cyrillic/Russian"));
-	fill(12, EASTEUROPEAN, GEANY_ENCODING_KOI8_U, "KOI8-U", _("Cyrillic/Ukrainian"));
-	fill(13, EASTEUROPEAN, GEANY_ENCODING_ISO_8859_16, "ISO-8859-16", _("Romanian"));
+	fill(8,		EASTEUROPEAN,	GEANY_ENCODING_ISO_IR_111,		"ISO-IR-111",		_("Cyrillic"));
+	fill(9,		EASTEUROPEAN,	GEANY_ENCODING_KOI8_R,			"KOI8-R",			_("Cyrillic"));
+	fill(10,	EASTEUROPEAN,	GEANY_ENCODING_WINDOWS_1251,	"WINDOWS-1251",		_("Cyrillic"));
+	fill(11,	EASTEUROPEAN,	GEANY_ENCODING_CP_866,			"CP866",			_("Cyrillic/Russian"));
+	fill(12,	EASTEUROPEAN,	GEANY_ENCODING_KOI8_U,			"KOI8-U",			_("Cyrillic/Ukrainian"));
+	fill(13,	EASTEUROPEAN,	GEANY_ENCODING_ISO_8859_16,		"ISO-8859-16",		_("Romanian"));
 
-	fill(0, MIDDLEEASTERN, GEANY_ENCODING_IBM_864, "IBM864", _("Arabic"));
-	fill(1, MIDDLEEASTERN, GEANY_ENCODING_ISO_8859_6, "ISO-8859-6", _("Arabic"));
-	fill(2, MIDDLEEASTERN, GEANY_ENCODING_WINDOWS_1256, "WINDOWS-1256", _("Arabic"));
-	fill(3, MIDDLEEASTERN, GEANY_ENCODING_IBM_862, "IBM862", _("Hebrew"));
+	fill(0,		MIDDLEEASTERN,	GEANY_ENCODING_IBM_864,			"IBM864",			_("Arabic"));
+	fill(1,		MIDDLEEASTERN,	GEANY_ENCODING_ISO_8859_6,		"ISO-8859-6",		_("Arabic"));
+	fill(2,		MIDDLEEASTERN,	GEANY_ENCODING_WINDOWS_1256,	"WINDOWS-1256",		_("Arabic"));
+	fill(3,		MIDDLEEASTERN,	GEANY_ENCODING_IBM_862,			"IBM862",			_("Hebrew"));
 	/* not available at all, ? */
-	fill(4, MIDDLEEASTERN, GEANY_ENCODING_ISO_8859_8_I, "ISO-8859-8-I", _("Hebrew"));
-	fill(5, MIDDLEEASTERN, GEANY_ENCODING_WINDOWS_1255, "WINDOWS-1255", _("Hebrew"));
-	fill(6, MIDDLEEASTERN, GEANY_ENCODING_ISO_8859_8, "ISO-8859-8", _("Hebrew Visual"));
+	fill(4,		MIDDLEEASTERN,	GEANY_ENCODING_ISO_8859_8_I,	"ISO-8859-8-I",		_("Hebrew"));
+	fill(5,		MIDDLEEASTERN,	GEANY_ENCODING_WINDOWS_1255,	"WINDOWS-1255",		_("Hebrew"));
+	fill(6,		MIDDLEEASTERN,	GEANY_ENCODING_ISO_8859_8,		"ISO-8859-8",		_("Hebrew Visual"));
 
-	fill(0, ASIAN, GEANY_ENCODING_ARMSCII_8, "ARMSCII-8", _("Armenian"));
-	fill(1, ASIAN, GEANY_ENCODING_GEOSTD8, "GEORGIAN-ACADEMY", _("Georgian"));
-	fill(2, ASIAN, GEANY_ENCODING_TIS_620, "TIS-620", _("Thai"));
-	fill(3, ASIAN, GEANY_ENCODING_IBM_857, "IBM857", _("Turkish"));
-	fill(4, ASIAN, GEANY_ENCODING_WINDOWS_1254, "WINDOWS-1254", _("Turkish"));
-	fill(5, ASIAN, GEANY_ENCODING_ISO_8859_9, "ISO-8859-9", _("Turkish"));
-	fill(6, ASIAN, GEANY_ENCODING_TCVN, "TCVN", _("Vietnamese"));
-	fill(7, ASIAN, GEANY_ENCODING_VISCII, "VISCII", _("Vietnamese"));
-	fill(8, ASIAN, GEANY_ENCODING_WINDOWS_1258, "WINDOWS-1258", _("Vietnamese"));
+	fill(0,		ASIAN,			GEANY_ENCODING_ARMSCII_8,		"ARMSCII-8",		_("Armenian"));
+	fill(1,		ASIAN,			GEANY_ENCODING_GEOSTD8,			"GEORGIAN-ACADEMY",	_("Georgian"));
+	fill(2,		ASIAN,			GEANY_ENCODING_TIS_620,			"TIS-620",			_("Thai"));
+	fill(3,		ASIAN,			GEANY_ENCODING_IBM_857,			"IBM857",			_("Turkish"));
+	fill(4,		ASIAN,			GEANY_ENCODING_WINDOWS_1254,	"WINDOWS-1254",		_("Turkish"));
+	fill(5,		ASIAN,			GEANY_ENCODING_ISO_8859_9,		"ISO-8859-9",		_("Turkish"));
+	fill(6,		ASIAN,			GEANY_ENCODING_TCVN,			"TCVN",				_("Vietnamese"));
+	fill(7,		ASIAN,			GEANY_ENCODING_VISCII,			"VISCII",			_("Vietnamese"));
+	fill(8,		ASIAN,			GEANY_ENCODING_WINDOWS_1258,	"WINDOWS-1258",		_("Vietnamese"));
 
-	fill(0, UNICODE, GEANY_ENCODING_UTF_7, "UTF-7", _("Unicode"));
-	fill(1, UNICODE, GEANY_ENCODING_UTF_8, "UTF-8", _("Unicode"));
-	fill(2, UNICODE, GEANY_ENCODING_UTF_16LE, "UTF-16LE", _("Unicode"));
-	fill(3, UNICODE, GEANY_ENCODING_UTF_16BE, "UTF-16BE", _("Unicode"));
-	fill(4, UNICODE, GEANY_ENCODING_UCS_2LE, "UCS-2LE", _("Unicode"));
-	fill(5, UNICODE, GEANY_ENCODING_UCS_2BE, "UCS-2BE", _("Unicode"));
-	fill(6, UNICODE, GEANY_ENCODING_UTF_32LE, "UTF-32LE", _("Unicode"));
-	fill(7, UNICODE, GEANY_ENCODING_UTF_32BE, "UTF-32BE", _("Unicode"));
+	fill(0,		UNICODE,		GEANY_ENCODING_UTF_7,			"UTF-7",			_("Unicode"));
+	fill(1,		UNICODE,		GEANY_ENCODING_UTF_8,			"UTF-8",			_("Unicode"));
+	fill(2,		UNICODE,		GEANY_ENCODING_UTF_16LE,		"UTF-16LE",			_("Unicode"));
+	fill(3,		UNICODE,		GEANY_ENCODING_UTF_16BE,		"UTF-16BE",			_("Unicode"));
+	fill(4,		UNICODE,		GEANY_ENCODING_UCS_2LE,			"UCS-2LE",			_("Unicode"));
+	fill(5,		UNICODE,		GEANY_ENCODING_UCS_2BE,			"UCS-2BE",			_("Unicode"));
+	fill(6,		UNICODE,		GEANY_ENCODING_UTF_32LE,		"UTF-32LE",			_("Unicode"));
+	fill(7,		UNICODE,		GEANY_ENCODING_UTF_32BE,		"UTF-32BE",			_("Unicode"));
 
-	fill(0, EASTASIAN, GEANY_ENCODING_GB18030, "GB18030", _("Chinese Simplified"));
-	fill(1, EASTASIAN, GEANY_ENCODING_GB2312, "GB2312", _("Chinese Simplified"));
-	fill(2, EASTASIAN, GEANY_ENCODING_GBK, "GBK", _("Chinese Simplified"));
+	fill(0,		EASTASIAN,		GEANY_ENCODING_GB18030,			"GB18030",			_("Chinese Simplified"));
+	fill(1,		EASTASIAN,		GEANY_ENCODING_GB2312,			"GB2312",			_("Chinese Simplified"));
+	fill(2,		EASTASIAN,		GEANY_ENCODING_GBK,				"GBK",				_("Chinese Simplified"));
 	/* maybe not available on Linux */
-	fill(3, EASTASIAN, GEANY_ENCODING_HZ, "HZ", _("Chinese Simplified"));
-	fill(4, EASTASIAN, GEANY_ENCODING_BIG5, "BIG5", _("Chinese Traditional"));
-	fill(5, EASTASIAN, GEANY_ENCODING_BIG5_HKSCS, "BIG5-HKSCS", _("Chinese Traditional"));
-	fill(6, EASTASIAN, GEANY_ENCODING_EUC_TW, "EUC-TW", _("Chinese Traditional"));
-	fill(7, EASTASIAN, GEANY_ENCODING_EUC_JP, "EUC-JP", _("Japanese"));
-	fill(8, EASTASIAN, GEANY_ENCODING_ISO_2022_JP, "ISO-2022-JP", _("Japanese"));
-	fill(9, EASTASIAN, GEANY_ENCODING_SHIFT_JIS, "SHIFT_JIS", _("Japanese"));
-	fill(10, EASTASIAN, GEANY_ENCODING_CP_932, "CP932", _("Japanese"));
-	fill(11, EASTASIAN, GEANY_ENCODING_EUC_KR, "EUC-KR", _("Korean"));
-	fill(12, EASTASIAN, GEANY_ENCODING_ISO_2022_KR, "ISO-2022-KR", _("Korean"));
-	fill(13, EASTASIAN, GEANY_ENCODING_JOHAB, "JOHAB", _("Korean"));
-	fill(14, EASTASIAN, GEANY_ENCODING_UHC, "UHC", _("Korean"));
+	fill(3,		EASTASIAN,		GEANY_ENCODING_HZ,				"HZ",				_("Chinese Simplified"));
+	fill(4,		EASTASIAN,		GEANY_ENCODING_BIG5,			"BIG5",				_("Chinese Traditional"));
+	fill(5,		EASTASIAN,		GEANY_ENCODING_BIG5_HKSCS,		"BIG5-HKSCS",		_("Chinese Traditional"));
+	fill(6,		EASTASIAN,		GEANY_ENCODING_EUC_TW,			"EUC-TW",			_("Chinese Traditional"));
+	fill(7,		EASTASIAN,		GEANY_ENCODING_EUC_JP,			"EUC-JP",			_("Japanese"));
+	fill(8,		EASTASIAN,		GEANY_ENCODING_ISO_2022_JP,		"ISO-2022-JP",		_("Japanese"));
+	fill(9,		EASTASIAN,		GEANY_ENCODING_SHIFT_JIS,		"SHIFT_JIS",		_("Japanese"));
+	fill(10,	EASTASIAN,		GEANY_ENCODING_CP_932,			"CP932",			_("Japanese"));
+	fill(11,	EASTASIAN,		GEANY_ENCODING_EUC_KR,			"EUC-KR",			_("Korean"));
+	fill(12,	EASTASIAN,		GEANY_ENCODING_ISO_2022_KR,		"ISO-2022-KR",		_("Korean"));
+	fill(13,	EASTASIAN,		GEANY_ENCODING_JOHAB,			"JOHAB",			_("Korean"));
+	fill(14,	EASTASIAN,		GEANY_ENCODING_UHC,				"UHC",				_("Korean"));
 
-	fill(0, NONE, GEANY_ENCODING_NONE, "None", _("Without encoding"));
+	fill(0,		NONE,			GEANY_ENCODING_NONE,			"None",				_("Without encoding"));
+}
+
+
+/* compares two encoding names in a permissive fashion.
+ * e.g. "utf8" matches "UTF-8", "iso8859_1" matches "ISO-8859-1", etc. */
+static gboolean encodings_charset_equals(const gchar *a, const gchar *b)
+{
+	gboolean was_alpha = FALSE; /* whether last character of previous word was a letter */
+	gboolean need_sep = FALSE; /* whether we're expecting an implicit separator */
+
+	while (*a && *b)
+	{
+		gboolean is_alpha;
+
+		if (g_ascii_toupper(*a) == g_ascii_toupper(*b) &&
+			((is_alpha = g_ascii_isalpha(*a)) || g_ascii_isdigit(*a)))
+		{
+			/* either there was a real separator, or we need a implicit one (a chage from alpha to
+			 * numeric or so) */
+			if (! need_sep || (was_alpha != is_alpha))
+			{
+				a++;
+				b++;
+				was_alpha = is_alpha;
+				need_sep = FALSE;
+			}
+			else
+				return FALSE;
+		}
+		else
+		{
+			guint n_sep = 0;
+
+			if (! g_ascii_isalnum(*a))
+			{
+				a++;
+				n_sep++;
+			}
+			if (! g_ascii_isalnum(*b))
+			{
+				b++;
+				n_sep++;
+			}
+			if (n_sep < 1)
+				return FALSE;
+			else if (n_sep < 2)
+				need_sep = TRUE;
+		}
+	}
+	return *a == *b;
 }
 
 
@@ -157,7 +207,7 @@ GeanyEncodingIndex encodings_get_idx_from_charset(const gchar *charset)
 	i = 0;
 	while (i < GEANY_ENCODINGS_MAX)
 	{
-		if (strcmp(charset, encodings[i].charset) == 0)
+		if (encodings_charset_equals(charset, encodings[i].charset))
 			return i;
 
 		++i;
@@ -176,7 +226,7 @@ const GeanyEncoding *encodings_get_from_charset(const gchar *charset)
 	i = 0;
 	while (i < GEANY_ENCODINGS_MAX)
 	{
-		if (strcmp(charset, encodings[i].charset) == 0)
+		if (encodings_charset_equals(charset, encodings[i].charset))
 			return &encodings[i];
 
 		++i;
@@ -186,6 +236,18 @@ const GeanyEncoding *encodings_get_from_charset(const gchar *charset)
 }
 
 
+static const gchar *encodings_normalize_charset(const gchar *charset)
+{
+	const GeanyEncoding *encoding;
+
+	encoding = encodings_get_from_charset(charset);
+	if (encoding != NULL)
+		return encoding->charset;
+
+	return NULL;
+}
+
+
 const GeanyEncoding *encodings_get_from_index(gint idx)
 {
 	g_return_val_if_fail(idx >= 0 && idx < GEANY_ENCODINGS_MAX, NULL);
@@ -219,7 +281,7 @@ gchar *encodings_to_string(const GeanyEncoding* enc)
 	g_return_val_if_fail(enc->name != NULL, NULL);
 	g_return_val_if_fail(enc->charset != NULL, NULL);
 
-    return g_strdup_printf("%s (%s)", enc->name, enc->charset);
+	return g_strdup_printf("%s (%s)", enc->name, enc->charset);
 }
 
 
@@ -282,7 +344,7 @@ static gchar *regex_match(regex_t *preg, const gchar *buffer, gsize size)
 	gchar *encoding = NULL;
 	regmatch_t pmatch[10];
 
-	if (G_UNLIKELY(! pregs_loaded) || G_UNLIKELY(buffer == NULL))
+	if (G_UNLIKELY(! pregs_loaded || buffer == NULL))
 		return NULL;
 
 	if (size > 512)
@@ -500,42 +562,38 @@ gchar *encodings_convert_to_utf8_from_charset(const gchar *buffer, gsize size,
 }
 
 
-/**
- *  Tries to convert @a buffer into UTF-8 encoding and store the detected original encoding in
- *  @a used_encoding.
- *
- *  @param buffer the input string to convert.
- *  @param size the length of the string, or -1 if the string is nul-terminated.
- *  @param used_encoding return location of the detected encoding of the input string, or @c NULL.
- *
- *  @return If the conversion was successful, a newly allocated nul-terminated string,
- *    which must be freed with @c g_free(). Otherwise @c NULL.
- **/
-gchar *encodings_convert_to_utf8(const gchar *buffer, gsize size, gchar **used_encoding)
+static gchar *encodings_check_regexes(const gchar *buffer, gsize size)
 {
-	gchar *locale_charset = NULL;
-	gchar *regex_charset = NULL;
+	guint i;
+
+	for (i = 0; i < G_N_ELEMENTS(pregs); i++)
+	{
+		gchar *charset;
+
+		if ((charset = regex_match(&pregs[i], buffer, size)) != NULL)
+			return charset;
+	}
+	return NULL;
+}
+
+
+static gchar *encodings_convert_to_utf8_with_suggestion(const gchar *buffer, gsize size,
+		const gchar *suggested_charset, gchar **used_encoding)
+{
+	const gchar *locale_charset = NULL;
 	const gchar *charset;
 	gchar *utf8_content;
-	gboolean check_regex = FALSE;
+	gboolean check_suggestion = suggested_charset != NULL;
 	gboolean check_locale = FALSE;
-	gint i, len, preferred_charset;
+	gint i, preferred_charset;
 
 	if ((gint)size == -1)
 	{
 		size = strlen(buffer);
 	}
 
-	/* first try to read the encoding from the file content */
-	len = (gint) G_N_ELEMENTS(pregs);
-	for (i = 0; i < len && ! check_regex; i++)
-	{
-		if ((regex_charset = regex_match(&pregs[i], buffer, size)) != NULL)
-			check_regex = TRUE;
-	}
-
 	/* current locale is not UTF-8, we have to check this charset */
-	check_locale = ! g_get_charset((const gchar**) &charset);
+	check_locale = ! g_get_charset(&locale_charset);
 
 	/* First check for preferred charset, if specified */
 	preferred_charset = file_prefs.default_open_encoding;
@@ -553,10 +611,12 @@ gchar *encodings_convert_to_utf8(const gchar *buffer, gsize size, gchar **used_e
 		if (G_UNLIKELY(i == encodings[GEANY_ENCODING_NONE].idx))
 			continue;
 
-		if (check_regex)
+		if (check_suggestion)
 		{
-			check_regex = FALSE;
-			charset = regex_charset;
+			check_suggestion = FALSE;
+			charset = encodings_normalize_charset(suggested_charset);
+			if (charset == NULL) /* we failed at normalizing suggested encoding, try it as is */
+				charset = suggested_charset;
 			i = -2; /* keep i below the start value to have it again at -1 on the next loop run */
 		}
 		else if (check_locale)
@@ -598,16 +658,39 @@ gchar *encodings_convert_to_utf8(const gchar *buffer, gsize size, gchar **used_e
 				}
 				*used_encoding = g_strdup(charset);
 			}
-			g_free(regex_charset);
 			return utf8_content;
 		}
 	}
-	g_free(regex_charset);
 
 	return NULL;
 }
 
 
+/**
+ *  Tries to convert @a buffer into UTF-8 encoding and store the detected original encoding in
+ *  @a used_encoding.
+ *
+ *  @param buffer the input string to convert.
+ *  @param size the length of the string, or -1 if the string is nul-terminated.
+ *  @param used_encoding return location of the detected encoding of the input string, or @c NULL.
+ *
+ *  @return If the conversion was successful, a newly allocated nul-terminated string,
+ *    which must be freed with @c g_free(). Otherwise @c NULL.
+ **/
+gchar *encodings_convert_to_utf8(const gchar *buffer, gsize size, gchar **used_encoding)
+{
+	gchar *regex_charset;
+	gchar *utf8;
+
+	/* first try to read the encoding from the file content */
+	regex_charset = encodings_check_regexes(buffer, size);
+	utf8 = encodings_convert_to_utf8_with_suggestion(buffer, size, regex_charset, used_encoding);
+	g_free(regex_charset);
+
+	return utf8;
+}
+
+
 /* If there's a BOM, return a corresponding GEANY_ENCODING_UTF_* index,
  * otherwise GEANY_ENCODING_NONE.
  * */
@@ -676,3 +759,223 @@ gboolean encodings_is_unicode_charset(const gchar *string)
 }
 
 
+typedef struct
+{
+	gchar		*data;	/* null-terminated data */
+	gsize		 size;	/* actual data size */
+	gsize		 len;	/* string length of data */
+	gchar		*enc;
+	gboolean	 bom;
+	gboolean	 partial;
+} BufferData;
+
+
+/* convert data with the specified encoding */
+static gboolean
+handle_forced_encoding(BufferData *buffer, const gchar *forced_enc)
+{
+	GeanyEncodingIndex enc_idx;
+
+	if (utils_str_equal(forced_enc, "UTF-8"))
+	{
+		if (! g_utf8_validate(buffer->data, buffer->len, NULL))
+		{
+			return FALSE;
+		}
+	}
+	else
+	{
+		gchar *converted_text = encodings_convert_to_utf8_from_charset(
+										buffer->data, buffer->size, forced_enc, FALSE);
+		if (converted_text == NULL)
+		{
+			return FALSE;
+		}
+		else
+		{
+			setptr(buffer->data, converted_text);
+			buffer->len = strlen(converted_text);
+		}
+	}
+	enc_idx = encodings_scan_unicode_bom(buffer->data, buffer->size, NULL);
+	buffer->bom = (enc_idx == GEANY_ENCODING_UTF_8);
+	buffer->enc = g_strdup(forced_enc);
+	return TRUE;
+}
+
+
+/* detect encoding and convert to UTF-8 if necessary */
+static gboolean
+handle_encoding(BufferData *buffer, GeanyEncodingIndex enc_idx)
+{
+	g_return_val_if_fail(buffer->enc == NULL, FALSE);
+	g_return_val_if_fail(buffer->bom == FALSE, FALSE);
+
+	if (buffer->size == 0)
+	{
+		/* we have no data so assume UTF-8, buffer->len can be 0 even we have an empty
+		 * e.g. UTF32 file with a BOM(so size is 4, len is 0) */
+		buffer->enc = g_strdup("UTF-8");
+	}
+	else
+	{
+		/* first check for a BOM */
+		if (enc_idx != GEANY_ENCODING_NONE)
+		{
+			buffer->enc = g_strdup(encodings[enc_idx].charset);
+			buffer->bom = TRUE;
+
+			if (enc_idx != GEANY_ENCODING_UTF_8) /* the BOM indicated something else than UTF-8 */
+			{
+				gchar *converted_text = encodings_convert_to_utf8_from_charset(
+										buffer->data, buffer->size, buffer->enc, FALSE);
+				if (converted_text != NULL)
+				{
+					setptr(buffer->data, converted_text);
+					buffer->len = strlen(converted_text);
+				}
+				else
+				{
+					/* there was a problem converting data from BOM encoding type */
+					setptr(buffer->enc, NULL);
+					buffer->bom = FALSE;
+				}
+			}
+		}
+
+		if (buffer->enc == NULL)	/* either there was no BOM or the BOM encoding failed */
+		{
+			/* first try to read the encoding from the file content */
+			gchar *regex_charset = encodings_check_regexes(buffer->data, buffer->size);
+
+			/* try UTF-8 first */
+			if (encodings_get_idx_from_charset(regex_charset) == GEANY_ENCODING_UTF_8 &&
+				(buffer->size == buffer->len) && g_utf8_validate(buffer->data, buffer->len, NULL))
+			{
+				buffer->enc = g_strdup("UTF-8");
+			}
+			else
+			{
+				/* detect the encoding */
+				gchar *converted_text = encodings_convert_to_utf8_with_suggestion(buffer->data,
+					buffer->size, regex_charset, &buffer->enc);
+
+				if (converted_text == NULL)
+				{
+					g_free(regex_charset);
+					return FALSE;
+				}
+				setptr(buffer->data, converted_text);
+				buffer->len = strlen(converted_text);
+			}
+			g_free(regex_charset);
+		}
+	}
+	return TRUE;
+}
+
+
+static void
+handle_bom(BufferData *buffer)
+{
+	guint bom_len;
+
+	encodings_scan_unicode_bom(buffer->data, buffer->size, &bom_len);
+	g_return_if_fail(bom_len != 0);
+
+	/* use filedata->len here because the contents are already converted into UTF-8 */
+	buffer->len -= bom_len;
+	/* overwrite the BOM with the remainder of the file contents, plus the NULL terminator. */
+	g_memmove(buffer->data, buffer->data + bom_len, buffer->len + 1);
+	buffer->data = g_realloc(buffer->data, buffer->len + 1);
+}
+
+
+/* loads textfile data, verifies and converts to forced_enc or UTF-8. Also handles BOM. */
+static gboolean handle_buffer(BufferData *buffer, const gchar *forced_enc)
+{
+	GeanyEncodingIndex tmp_enc_idx;
+
+	/* temporarily retrieve the encoding idx based on the BOM to suppress the following warning
+	 * if we have a BOM */
+	tmp_enc_idx = encodings_scan_unicode_bom(buffer->data, buffer->size, NULL);
+
+	/* check whether the size of the loaded data is equal to the size of the file in the
+	 * filesystem file size may be 0 to allow opening files in /proc/ which have typically a
+	 * file size of 0 bytes */
+	if (buffer->len != buffer->size && buffer->size != 0 && (
+		tmp_enc_idx == GEANY_ENCODING_UTF_8 || /* tmp_enc_idx can be UTF-7/8/16/32, UCS and None */
+		tmp_enc_idx == GEANY_ENCODING_UTF_7))  /* filter UTF-7/8 where no NULL bytes are allowed */
+	{
+		buffer->partial = TRUE;
+	}
+
+	/* Determine character encoding and convert to UTF-8 */
+	if (forced_enc != NULL)
+	{
+		/* the encoding should be ignored(requested by user), so open the file "as it is" */
+		if (utils_str_equal(forced_enc, encodings[GEANY_ENCODING_NONE].charset))
+		{
+			buffer->bom = FALSE;
+			buffer->enc = g_strdup(encodings[GEANY_ENCODING_NONE].charset);
+		}
+		else if (! handle_forced_encoding(buffer, forced_enc))
+		{
+			return FALSE;
+		}
+	}
+	else if (! handle_encoding(buffer, tmp_enc_idx))
+	{
+		return FALSE;
+	}
+
+	if (buffer->bom)
+		handle_bom(buffer);
+	return TRUE;
+}
+
+
+/*
+ * Tries to convert @a buffer into UTF-8 encoding. Unlike encodings_convert_to_utf8()
+ * and encodings_convert_to_utf8_from_charset() it handles the possible BOM in the data.
+ *
+ * @param buf a pointer to modifiable null-terminated buffer to convert.
+ *   It may or may not be modified, and should be freed whatever happens.
+ * @param size a pointer to the size of the buffer (expected to be e.g. the on-disk
+ *   file size). It will be updated to the new size.
+ * @param forced_enc forced encoding to use, or @c NULL
+ * @param used_encoding return location for the actually used encoding, or @c NULL
+ * @param has_bom return location to store whether the data had a BOM, or @c NULL
+ * @param partial return location to store whether the conversion may be partial, or @c NULL
+ *
+ * @return @C TRUE if the conversion succeeded, @c FALSE otherwise.
+ */
+gboolean encodings_convert_to_utf8_auto(gchar **buf, gsize *size, const gchar *forced_enc,
+		gchar **used_encoding, gboolean *has_bom, gboolean *partial)
+{
+	BufferData buffer;
+
+	buffer.data = *buf;
+	buffer.size = *size;
+	/* use strlen to check for null chars */
+	buffer.len = strlen(buffer.data);
+	buffer.enc = NULL;
+	buffer.bom = FALSE;
+	buffer.partial = FALSE;
+
+	if (! handle_buffer(&buffer, forced_enc))
+		return FALSE;
+
+	*size = buffer.len;
+	if (used_encoding)
+		*used_encoding = buffer.enc;
+	else
+		g_free(buffer.enc);
+	if (has_bom)
+		*has_bom = buffer.bom;
+	if (partial)
+		*partial = buffer.partial;
+
+	*buf = buffer.data;
+	return TRUE;
+}
