@@ -1,8 +1,8 @@
 /*
  *      filetypes.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: filetypes.c 5393 2010-11-10 18:04:15Z ntrel $
+ * $Id: filetypes.c 5964 2011-09-28 10:47:43Z ntrel $
  */
 
 /**
@@ -56,12 +56,16 @@ static GHashTable *filetypes_hash = NULL;	/* Hash of filetype pointers based on
 /** List of filetype pointers sorted by name, but with @c filetypes_index(GEANY_FILETYPES_NONE)
  * first, as this is usually treated specially.
  * The list does not change (after filetypes have been initialized), so you can use
- * @code g_slist_nth_data(filetypes_by_title, n) @endcode and expect the same result at different times. */
+ * @code g_slist_nth_data(filetypes_by_title, n) @endcode and expect the same result at different times.
+ * @see filetypes_get_sorted_by_name(). */
 GSList *filetypes_by_title = NULL;
 
 
 static void create_radio_menu_item(GtkWidget *menu, GeanyFiletype *ftype);
 
+static gchar *filetypes_get_conf_extension(const GeanyFiletype *ft);
+static void read_filetype_config(void);
+
 
 enum TitleType
 {
@@ -275,7 +279,8 @@ static void init_builtin_filetypes(void)
 	ft->lang = 36;
 	ft->name = g_strdup("Markdown");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
-	ft->group = GEANY_FILETYPE_GROUP_MISC;
+	ft->mime_type = g_strdup("text/x-markdown");
+	ft->group = GEANY_FILETYPE_GROUP_MARKUP;
 
 #define TXT2TAGS
 	ft = filetypes[GEANY_FILETYPES_TXT2TAGS];
@@ -283,20 +288,20 @@ static void init_builtin_filetypes(void)
 	ft->name = g_strdup("Txt2tags");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
 	ft->mime_type = g_strdup("text/x-txt2tags");
-	ft->group = GEANY_FILETYPE_GROUP_MISC;
+	ft->group = GEANY_FILETYPE_GROUP_MARKUP;
 
 #define ABC
 	ft = filetypes[GEANY_FILETYPES_ABC];
 	ft->lang = 38;
 	ft->name = g_strdup("Abc");
-	filetype_make_title(ft, TITLE_SOURCE_FILE);
+	filetype_make_title(ft, TITLE_FILE);
 	ft->group = GEANY_FILETYPE_GROUP_MISC;
 
 #define SH
 	ft = filetypes[GEANY_FILETYPES_SH];
 	ft->lang = 16;
 	ft->name = g_strdup("Sh");
-	ft->title = g_strdup(_("Shell script file"));
+	ft->title = g_strdup(_("Shell script"));
 	ft->mime_type = g_strdup("application/x-shellscript");
 	ft->group = GEANY_FILETYPE_GROUP_SCRIPT;
 
@@ -343,17 +348,25 @@ static void init_builtin_filetypes(void)
 	ft = filetypes[GEANY_FILETYPES_SQL];
 	ft->lang = 11;
 	ft->name = g_strdup("SQL");
-	ft->title = g_strdup(_("SQL Dump file"));
+	filetype_make_title(ft, TITLE_FILE);
 	ft->mime_type = g_strdup("text/x-sql");
 	ft->group = GEANY_FILETYPE_GROUP_MISC;
 
+#define COBOL
+	ft = filetypes[GEANY_FILETYPES_COBOL];
+	ft->lang = 41;
+	ft->name = g_strdup("COBOL");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
+	ft->mime_type = g_strdup("text/x-cobol");
+	ft->group = GEANY_FILETYPE_GROUP_COMPILED;
+
 #define LATEX
 	ft = filetypes[GEANY_FILETYPES_LATEX];
 	ft->lang = 8;
 	ft->name = g_strdup("LaTeX");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
 	ft->mime_type = g_strdup("text/x-tex");
-	ft->group = GEANY_FILETYPE_GROUP_MISC;
+	ft->group = GEANY_FILETYPE_GROUP_MARKUP;
 
 #define VHDL
 	ft = filetypes[GEANY_FILETYPES_VHDL];
@@ -368,6 +381,7 @@ static void init_builtin_filetypes(void)
 	ft->lang = 39;
 	ft->name = g_strdup("Verilog");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
+	ft->mime_type = g_strdup("text/x-verilog");
 	ft->group = GEANY_FILETYPE_GROUP_COMPILED;
 
 #define DIFF
@@ -424,15 +438,15 @@ static void init_builtin_filetypes(void)
 	ft = filetypes[GEANY_FILETYPES_R];
 	ft->lang = 40;
 	ft->name = g_strdup("R");
-	ft->title = g_strdup_printf(_("%s script file"), "R");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
 	ft->group = GEANY_FILETYPE_GROUP_SCRIPT;
 
 #define REST
 	ft = filetypes[GEANY_FILETYPES_REST];
 	ft->lang = 28;
 	ft->name = g_strdup("reStructuredText");
-	filetype_make_title(ft, TITLE_FILE);
-	ft->group = GEANY_FILETYPE_GROUP_MISC;
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
+	ft->group = GEANY_FILETYPE_GROUP_MARKUP;
 
 #define MATLAB
 	ft = filetypes[GEANY_FILETYPES_MATLAB];
@@ -445,7 +459,8 @@ static void init_builtin_filetypes(void)
 #define YAML
 	ft = filetypes[GEANY_FILETYPES_YAML];
 	ft->name = g_strdup("YAML");
-	filetype_make_title(ft, TITLE_SOURCE_FILE);
+	filetype_make_title(ft, TITLE_FILE);
+	ft->mime_type = g_strdup("application/x-yaml");
 	ft->group = GEANY_FILETYPE_GROUP_MISC;
 
 #define CMAKE
@@ -453,14 +468,14 @@ static void init_builtin_filetypes(void)
 	ft->name = g_strdup("CMake");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
 	ft->mime_type = g_strdup("text/x-cmake");
-	ft->group = GEANY_FILETYPE_GROUP_MISC;
+	ft->group = GEANY_FILETYPE_GROUP_SCRIPT;
 
 #define NSIS
 	ft = filetypes[GEANY_FILETYPES_NSIS];
 	ft->lang = 35;
 	ft->name = g_strdup("NSIS");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
-	ft->group = GEANY_FILETYPE_GROUP_MISC;
+	ft->group = GEANY_FILETYPE_GROUP_SCRIPT;
 
 #define ADA
 	ft = filetypes[GEANY_FILETYPES_ADA];
@@ -482,18 +497,22 @@ static GeanyFiletype *filetype_new(void)
 {
 	GeanyFiletype *ft = g_new0(GeanyFiletype, 1);
 
+	ft->group = GEANY_FILETYPE_GROUP_NONE;
 	ft->lang = -2;	/* assume no tagmanager parser */
 	/* pattern must not be null */
 	ft->pattern = g_new0(gchar*, 1);
 	ft->project_list_entry = -1; /* no entry */
+	ft->indent_width = -1;
+	ft->indent_type = -1;
 
 	ft->priv = g_new0(GeanyFiletypePrivate, 1);
 	return ft;
 }
 
 
-static gint cmp_filetype(gconstpointer pft1, gconstpointer pft2)
+static gint cmp_filetype(gconstpointer pft1, gconstpointer pft2, gpointer data)
 {
+	gboolean by_name = GPOINTER_TO_INT(data);
 	const GeanyFiletype *ft1 = pft1, *ft2 = pft2;
 
 	if (G_UNLIKELY(ft1->id == GEANY_FILETYPES_NONE))
@@ -501,7 +520,28 @@ static gint cmp_filetype(gconstpointer pft1, gconstpointer pft2)
 	if (G_UNLIKELY(ft2->id == GEANY_FILETYPES_NONE))
 		return 1;
 
-	return utils_str_casecmp(ft1->title, ft2->title);
+	return by_name ?
+		utils_str_casecmp(ft1->name, ft2->name) :
+		utils_str_casecmp(ft1->title, ft2->title);
+}
+
+
+/** Gets a list of filetype pointers sorted by name.
+ * The list does not change on subsequent calls.
+ * @return The list - do not free.
+ * @see filetypes_by_title. */
+const GSList *filetypes_get_sorted_by_name(void)
+{
+	static GSList *list = NULL;
+
+	g_return_val_if_fail(filetypes_by_title, NULL);
+
+	if (!list)
+	{
+		list = g_slist_copy(filetypes_by_title);
+		list = g_slist_sort_with_data(list, cmp_filetype, GINT_TO_POINTER(TRUE));
+	}
+	return list;
 }
 
 
@@ -521,8 +561,6 @@ static void filetype_add(GeanyFiletype *ft)
 
 	if (!ft->mime_type)
 		ft->mime_type = g_strdup("text/plain");
-
-	ft->icon = ui_get_mime_icon(ft->mime_type, GTK_ICON_SIZE_MENU);
 }
 
 
@@ -542,7 +580,6 @@ static void add_custom_filetype(const gchar *filename)
 	ft = filetype_new();
 	ft->name = g_strdup(fn);
 	filetype_make_title(ft, TITLE_FILE);
-	ft->group = GEANY_FILETYPE_GROUP_CUSTOM;
 	ft->priv->custom = TRUE;
 	filetype_add(ft);
 	geany_debug("Added filetype %s (%d).", ft->name, ft->id);
@@ -574,10 +611,12 @@ static void init_custom_filetypes(const gchar *path)
 }
 
 
-/* Create the filetypes array and fill it with the known filetypes. */
+/* Create the filetypes array and fill it with the known filetypes.
+ * Warning: GTK isn't necessarily initialized yet. */
 void filetypes_init_types()
 {
 	filetype_id ft_id;
+	gchar *f;
 
 	g_return_if_fail(filetypes_array == NULL);
 	g_return_if_fail(filetypes_hash == NULL);
@@ -598,22 +637,31 @@ void filetypes_init_types()
 		filetype_add(filetypes[ft_id]);
 	}
 	init_custom_filetypes(app->datadir);
-	init_custom_filetypes(utils_build_path(app->configdir, GEANY_FILEDEFS_SUBDIR, NULL));
+	f = utils_build_path(app->configdir, GEANY_FILEDEFS_SUBDIR, NULL);
+	init_custom_filetypes(f);
+	g_free(f);
 
 	/* sort last instead of on insertion to prevent exponential time */
-	filetypes_by_title = g_slist_sort(filetypes_by_title, cmp_filetype);
+	filetypes_by_title = g_slist_sort_with_data(filetypes_by_title,
+		cmp_filetype, GINT_TO_POINTER(FALSE));
+
+	read_filetype_config();
 }
 
 
 static void on_document_save(G_GNUC_UNUSED GObject *object, GeanyDocument *doc)
 {
+	gchar *f;
+
 	g_return_if_fail(NZV(doc->real_path));
 
-	if (utils_str_equal(doc->real_path,
-		utils_build_path(app->configdir, "filetype_extensions.conf", NULL)))
-		filetypes_read_extensions();
-	else if (utils_str_equal(doc->real_path,
-		utils_build_path(app->configdir, GEANY_FILEDEFS_SUBDIR, "filetypes.common", NULL)))
+	f = utils_build_path(app->configdir, "filetype_extensions.conf", NULL);
+	if (utils_str_equal(doc->real_path, f))
+		filetypes_reload_extensions();
+
+	g_free(f);
+	f = utils_build_path(app->configdir, GEANY_FILEDEFS_SUBDIR, "filetypes.common", NULL);
+	if (utils_str_equal(doc->real_path, f))
 	{
 		guint i;
 
@@ -624,15 +672,19 @@ static void on_document_save(G_GNUC_UNUSED GObject *object, GeanyDocument *doc)
 		foreach_document(i)
 			document_reload_config(documents[i]);
 	}
+	g_free(f);
 }
 
 
 static void setup_config_file_menus(void)
 {
-	ui_add_config_file_menu_item(
-		utils_build_path(app->configdir, "filetype_extensions.conf", NULL), NULL, NULL);
-	ui_add_config_file_menu_item(
-		utils_build_path(app->configdir, GEANY_FILEDEFS_SUBDIR, "filetypes.common", NULL), NULL, NULL);
+	gchar *f;
+
+	f = utils_build_path(app->configdir, "filetype_extensions.conf", NULL);
+	ui_add_config_file_menu_item(f, NULL, NULL);
+	setptr(f, utils_build_path(app->configdir, GEANY_FILEDEFS_SUBDIR, "filetypes.common", NULL));
+	ui_add_config_file_menu_item(f, NULL, NULL);
+	g_free(f);
 
 	g_signal_connect(geany_object, "document-save", G_CALLBACK(on_document_save), NULL);
 }
@@ -661,8 +713,7 @@ static void create_set_filetype_menu(void)
 	create_sub_menu(filetype_menu, GEANY_FILETYPE_GROUP_COMPILED, _("_Programming Languages"));
 	create_sub_menu(filetype_menu, GEANY_FILETYPE_GROUP_SCRIPT, _("_Scripting Languages"));
 	create_sub_menu(filetype_menu, GEANY_FILETYPE_GROUP_MARKUP, _("_Markup Languages"));
-	create_sub_menu(filetype_menu, GEANY_FILETYPE_GROUP_MISC, _("M_iscellaneous Languages"));
-	create_sub_menu(filetype_menu, GEANY_FILETYPE_GROUP_CUSTOM, _("_Custom Filetypes"));
+	create_sub_menu(filetype_menu, GEANY_FILETYPE_GROUP_MISC, _("M_iscellaneous"));
 
 	/* Append all filetypes to the filetype menu */
 	foreach_slist(node, filetypes_by_title)
@@ -679,8 +730,16 @@ static void create_set_filetype_menu(void)
 
 void filetypes_init()
 {
+	GSList *node;
+
 	filetypes_init_types();
 
+	/* this has to be here as GTK isn't initialized in filetypes_init_types(). */
+	foreach_slist(node, filetypes_by_title)
+	{
+		GeanyFiletype *ft = node->data;
+		ft->icon = ui_get_mime_icon(ft->mime_type, GTK_ICON_SIZE_MENU);
+	}
 	create_set_filetype_menu();
 	setup_config_file_menus();
 }
@@ -731,7 +790,7 @@ static gboolean match_basename(gconstpointer pft, gconstpointer user_data)
 static GeanyFiletype *check_builtin_filenames(const gchar *utf8_filename)
 {
 	gchar *lfn = NULL;
-	const gchar *path;
+	gchar *path;
 	gboolean found = FALSE;
 
 #ifdef G_OS_WIN32
@@ -746,10 +805,11 @@ static GeanyFiletype *check_builtin_filenames(const gchar *utf8_filename)
 	if (g_str_has_prefix(lfn, path))
 		found = TRUE;
 
-	path = utils_build_path(app->datadir, "filetypes.", NULL);
+	setptr(path, utils_build_path(app->datadir, "filetypes.", NULL));
 	if (g_str_has_prefix(lfn, path))
 		found = TRUE;
 
+	g_free(path);
 	g_free(lfn);
 	return found ? filetypes[GEANY_FILETYPES_CONF] : NULL;
 }
@@ -975,8 +1035,7 @@ void filetypes_select_radio_item(const GeanyFiletype *ft)
 
 
 static void
-on_filetype_change                     (GtkCheckMenuItem     *menuitem,
-                                        gpointer         user_data)
+on_filetype_change(GtkCheckMenuItem *menuitem, gpointer user_data)
 {
 	GeanyDocument *doc = document_get_current();
 	if (ignore_callback || doc == NULL || ! gtk_check_menu_item_get_active(menuitem))
@@ -1000,20 +1059,6 @@ static void create_radio_menu_item(GtkWidget *menu, GeanyFiletype *ftype)
 }
 
 
-#if 0
-/* Remove a filetype pointer from the list of available filetypes. */
-static void filetype_remove(GeanyFiletype *ft)
-{
-	g_return_if_fail(ft);
-
-	g_ptr_array_remove(filetypes_array, ft);
-
-	if (!g_hash_table_remove(filetypes_hash, ft))
-		g_warning("Could not remove filetype %p!", ft);
-}
-#endif
-
-
 static void set_error_regex(GeanyFiletype *ft, gchar *string)
 {
 	setptr(ft->error_regex_string, string);
@@ -1038,6 +1083,7 @@ static void filetype_free(gpointer data, G_GNUC_UNUSED gpointer user_data)
 	g_free(ft->mime_type);
 	g_free(ft->comment_open);
 	g_free(ft->comment_close);
+	g_free(ft->comment_single);
 	g_free(ft->context_action_cmd);
 	g_free(ft->filecmds);
 	g_free(ft->ftdefcmds);
@@ -1064,61 +1110,76 @@ void filetypes_free_types(void)
 }
 
 
+static void load_indent_settings(GeanyFiletype *ft, GKeyFile *config, GKeyFile *configh)
+{
+	ft->indent_width = utils_get_setting(integer, configh, config, "indentation", "width", -1);
+	ft->indent_type = utils_get_setting(integer, configh, config, "indentation", "type", -1);
+	/* check whether the indent type is OK */
+	switch (ft->indent_type)
+	{
+		case GEANY_INDENT_TYPE_TABS:
+		case GEANY_INDENT_TYPE_SPACES:
+		case GEANY_INDENT_TYPE_BOTH:
+		case -1:
+			break;
+
+		default:
+			g_warning("Invalid indent type %d in file type %s", ft->indent_type, ft->name);
+			ft->indent_type = -1;
+			break;
+	}
+}
+
+
 static void load_settings(gint ft_id, GKeyFile *config, GKeyFile *configh)
 {
 	GeanyFiletype *ft = filetypes[ft_id];
 	gchar *result;
-	GError *error = NULL;
-	gboolean tmp;
 
 	/* default extension */
-	result = g_key_file_get_string(configh, "settings", "extension", NULL);
-	if (result == NULL) result = g_key_file_get_string(config, "settings", "extension", NULL);
-	if (G_LIKELY(result != NULL))
+	result = utils_get_setting(string, configh, config, "settings", "extension", NULL);
+	if (result != NULL)
 	{
 		setptr(filetypes[ft_id]->extension, result);
 	}
 
 	/* read comment notes */
-	result = g_key_file_get_string(configh, "settings", "comment_open", NULL);
-	if (result == NULL) result = g_key_file_get_string(config, "settings", "comment_open", NULL);
-	if (G_LIKELY(result != NULL))
+	result = utils_get_setting(string, configh, config, "settings", "comment_open", NULL);
+	if (result != NULL)
 	{
-		g_free(filetypes[ft_id]->comment_open);
-		filetypes[ft_id]->comment_open = result;
+		setptr(filetypes[ft_id]->comment_open, result);
 	}
 
-	result = g_key_file_get_string(configh, "settings", "comment_close", NULL);
-	if (result == NULL) result = g_key_file_get_string(config, "settings", "comment_close", NULL);
-	if (G_LIKELY(result != NULL))
+	result = utils_get_setting(string, configh, config, "settings", "comment_close", NULL);
+	if (result != NULL)
 	{
-		g_free(filetypes[ft_id]->comment_close);
-		filetypes[ft_id]->comment_close = result;
+		setptr(filetypes[ft_id]->comment_close, result);
 	}
 
-	tmp = g_key_file_get_boolean(configh, "settings", "comment_use_indent", &error);
-	if (error)
+	result = utils_get_setting(string, configh, config, "settings", "comment_single", NULL);
+	if (result != NULL)
 	{
-		g_error_free(error);
-		error = NULL;
-		tmp = g_key_file_get_boolean(config, "settings", "comment_use_indent", &error);
-		if (error) g_error_free(error);
-		else filetypes[ft_id]->comment_use_indent = tmp;
+		setptr(filetypes[ft_id]->comment_single, result);
 	}
-	else filetypes[ft_id]->comment_use_indent = tmp;
+	/* import correctly filetypes that use old-style single comments */
+	else if (! NZV(filetypes[ft_id]->comment_close))
+	{
+		setptr(filetypes[ft_id]->comment_single, filetypes[ft_id]->comment_open);
+		filetypes[ft_id]->comment_open = NULL;
+	}
+
+	filetypes[ft_id]->comment_use_indent = utils_get_setting(boolean, configh, config,
+			"settings", "comment_use_indent", FALSE);
 
 	/* read context action */
-	result = g_key_file_get_string(configh, "settings", "context_action_cmd", NULL);
-	if (result == NULL) result = g_key_file_get_string(config, "settings", "context_action_cmd", NULL);
-	if (G_LIKELY(result != NULL))
+	result = utils_get_setting(string, configh, config, "settings", "context_action_cmd", NULL);
+	if (result != NULL)
 	{
 		setptr(filetypes[ft_id]->context_action_cmd, result);
 	}
 
-	result = utils_get_setting_string(configh, "settings", "tag_parser", NULL);
-	if (!result)
-		result = utils_get_setting_string(config, "settings", "tag_parser", NULL);
-	if (result)
+	result = utils_get_setting(string, configh, config, "settings", "tag_parser", NULL);
+	if (result != NULL)
 	{
 		ft->lang = tm_source_file_get_named_lang(result);
 		if (ft->lang < 0)
@@ -1126,10 +1187,8 @@ static void load_settings(gint ft_id, GKeyFile *config, GKeyFile *configh)
 		g_free(result);
 	}
 
-	result = utils_get_setting_string(configh, "settings", "lexer_filetype", NULL);
-	if (!result)
-		result = utils_get_setting_string(config, "settings", "lexer_filetype", NULL);
-	if (result)
+	result = utils_get_setting(string, configh, config, "settings", "lexer_filetype", NULL);
+	if (result != NULL)
 	{
 		ft->lexer_filetype = filetypes_lookup_by_name(result);
 		if (!ft->lexer_filetype)
@@ -1142,12 +1201,105 @@ static void load_settings(gint ft_id, GKeyFile *config, GKeyFile *configh)
 	ft->priv->xml_indent_tags = utils_get_setting(boolean, configh, config, "settings",
 		"xml_indent_tags", FALSE);
 
+	/* read indent settings */
+	load_indent_settings(ft, config, configh);
+
 	/* read build settings */
 	build_load_menu(config, GEANY_BCS_FT, (gpointer)ft);
 	build_load_menu(configh, GEANY_BCS_HOME_FT, (gpointer)ft);
 }
 
 
+static void add_keys(GKeyFile *dest, const gchar *group, GKeyFile *src)
+{
+	gchar **keys = g_key_file_get_keys(src, group, NULL, NULL);
+	gchar **ptr;
+
+	foreach_strv(ptr, keys)
+	{
+		gchar *key = *ptr;
+		gchar *value = g_key_file_get_value(src, group, key, NULL);
+
+		g_key_file_set_value(dest, group, key, value);
+		g_free(value);
+	}
+	g_strfreev(keys);
+}
+
+
+static gchar *filetypes_get_filename(GeanyFiletype *ft, gboolean user)
+{
+	gchar *ext = filetypes_get_conf_extension(ft);
+	gchar *f;
+
+	if (user)
+		f = utils_make_filename(app->configdir,
+			GEANY_FILEDEFS_SUBDIR G_DIR_SEPARATOR_S, "filetypes.", ext, NULL);
+	else
+		f = utils_make_filename(app->datadir, "filetypes.", ext, NULL);
+
+	g_free(ext);
+	return f;
+}
+
+
+static void add_group_keys(GKeyFile *kf, const gchar *group, GeanyFiletype *ft)
+{
+	gchar *files[2];
+	gboolean loaded = FALSE;
+	guint i;
+
+	files[0] = filetypes_get_filename(ft, FALSE);
+	files[1] = filetypes_get_filename(ft, TRUE);
+
+	for (i = 0; i < G_N_ELEMENTS(files); i++)
+	{
+		GKeyFile *src = g_key_file_new();
+
+		if (g_key_file_load_from_file(src, files[i], G_KEY_FILE_NONE, NULL))
+		{
+			add_keys(kf, group, src);
+			loaded = TRUE;
+		}
+		g_key_file_free(src);
+	}
+
+	if (!loaded)
+		geany_debug("Could not read config file %s for [%s=%s]!", files[0], group, ft->name);
+
+	g_free(files[0]);
+	g_free(files[1]);
+}
+
+
+static void copy_ft_groups(GKeyFile *kf)
+{
+	gchar **groups = g_key_file_get_groups(kf, NULL);
+	gchar **ptr;
+
+	foreach_strv(ptr, groups)
+	{
+		gchar *group = *ptr;
+		gchar *name = strstr(*ptr, "=");
+		GeanyFiletype *ft;
+
+		if (!name)
+			continue;
+
+		/* terminate group at '=' */
+		*name = 0;
+		name++;
+		if (!name[0])
+			continue;
+
+		ft = filetypes_lookup_by_name(name);
+		if (ft)
+			add_group_keys(kf, group, ft);
+	}
+	g_strfreev(groups);
+}
+
+
 /* simple wrapper function to print file errors in DEBUG mode */
 static void load_system_keyfile(GKeyFile *key_file, const gchar *file, GKeyFileFlags flags,
 		GeanyFiletype *ft)
@@ -1181,11 +1333,11 @@ void filetypes_load_config(gint ft_id, gboolean reload)
 	pft = ft->priv;
 
 	/* when reloading, proceed only if the settings were already loaded */
-	if (reload && G_UNLIKELY(! pft->keyfile_loaded))
+	if (G_UNLIKELY(reload && ! pft->keyfile_loaded))
 		return;
 
 	/* when not reloading, load the settings only once */
-	if (! reload && G_LIKELY(pft->keyfile_loaded))
+	if (G_LIKELY(! reload && pft->keyfile_loaded))
 		return;
 	pft->keyfile_loaded = TRUE;
 
@@ -1193,18 +1345,18 @@ void filetypes_load_config(gint ft_id, gboolean reload)
 	config_home = g_key_file_new();
 	{
 		/* highlighting uses GEANY_FILETYPES_NONE for common settings */
-		gchar *ext = filetypes_get_conf_extension(ft_id);
-		gchar *f0 = g_strconcat(app->datadir, G_DIR_SEPARATOR_S "filetypes.", ext, NULL);
-		gchar *f = g_strconcat(app->configdir,
-			G_DIR_SEPARATOR_S GEANY_FILEDEFS_SUBDIR G_DIR_SEPARATOR_S "filetypes.", ext, NULL);
+		gchar *f;
 
-		load_system_keyfile(config, f0, G_KEY_FILE_KEEP_COMMENTS, ft);
+		f = filetypes_get_filename(ft, FALSE);
+		load_system_keyfile(config, f, G_KEY_FILE_KEEP_COMMENTS, ft);
+
+		setptr(f, filetypes_get_filename(ft, TRUE));
 		g_key_file_load_from_file(config_home, f, G_KEY_FILE_KEEP_COMMENTS, NULL);
-
-		g_free(ext);
 		g_free(f);
-		g_free(f0);
 	}
+	/* Copy keys for any groups with [group=C] from system keyfile */
+	copy_ft_groups(config);
+	copy_ft_groups(config_home);
 
 	load_settings(ft_id, config, config_home);
 	highlighting_init_styles(ft_id, config, config_home);
@@ -1214,60 +1366,44 @@ void filetypes_load_config(gint ft_id, gboolean reload)
 }
 
 
-gchar *filetypes_get_conf_extension(gint filetype_idx)
+static gchar *filetypes_get_conf_extension(const GeanyFiletype *ft)
 {
-	gchar *result, *ptr;
-	GeanyFiletype *ft = filetypes[filetype_idx];
+	gchar *result;
 
 	if (ft->priv->custom)
 		return g_strconcat(ft->name, ".conf", NULL);
 
 	/* Handle any special extensions different from lowercase filetype->name */
-	switch (filetype_idx)
+	switch (ft->id)
 	{
 		case GEANY_FILETYPES_CPP: result = g_strdup("cpp"); break;
 		case GEANY_FILETYPES_CS: result = g_strdup("cs"); break;
 		case GEANY_FILETYPES_MAKE: result = g_strdup("makefile"); break;
 		case GEANY_FILETYPES_NONE: result = g_strdup("common"); break;
+		/* name is Matlab/Octave */
+		case GEANY_FILETYPES_MATLAB: result = g_strdup("matlab"); break;
 		default:
 			result = g_ascii_strdown(ft->name, -1);
-			/* truncate at slash (e.g. for Matlab/Octave) */
-			ptr = strstr(result, "/");
-			if (ptr)
-				*ptr = 0x0;
 			break;
 	}
 	return result;
 }
 
 
-void filetypes_save_commands(void)
+void filetypes_save_commands(GeanyFiletype *ft)
 {
-	gchar *conf_prefix = g_strconcat(app->configdir,
-		G_DIR_SEPARATOR_S GEANY_FILEDEFS_SUBDIR G_DIR_SEPARATOR_S "filetypes.", NULL);
-	guint i;
+	GKeyFile *config_home;
+	gchar *fname, *data;
 
-	for (i = 0; i < filetypes_array->len; i++)
-	{
-		GKeyFile *config_home;
-		gchar *fname, *ext, *data;
-
-		if (filetypes[i]->home_save_needed)
-		{
-			ext = filetypes_get_conf_extension(i);
-			fname = g_strconcat(conf_prefix, ext, NULL);
-			g_free(ext);
-			config_home = g_key_file_new();
-			g_key_file_load_from_file(config_home, fname, G_KEY_FILE_KEEP_COMMENTS, NULL);
-			build_save_menu(config_home, (gpointer)(filetypes[i]), GEANY_BCS_HOME_FT);
-			data = g_key_file_to_data(config_home, NULL, NULL);
-			utils_write_file(fname, data);
-			g_free(data);
-			g_key_file_free(config_home);
-			g_free(fname);
-		}
-	}
-	g_free(conf_prefix);
+	fname = filetypes_get_filename(ft, TRUE);
+	config_home = g_key_file_new();
+	g_key_file_load_from_file(config_home, fname, G_KEY_FILE_KEEP_COMMENTS, NULL);
+	build_save_menu(config_home, ft, GEANY_BCS_HOME_FT);
+	data = g_key_file_to_data(config_home, NULL, NULL);
+	utils_write_file(fname, data);
+	g_free(data);
+	g_key_file_free(config_home);
+	g_free(fname);
 }
 
 
@@ -1392,7 +1528,7 @@ gboolean filetypes_parse_error_message(GeanyFiletype *ft, const gchar *message,
 	*filename = NULL;
 	*line = -1;
 
-	if (!NZV(regstr))
+	if (G_UNLIKELY(! NZV(regstr)))
 		return FALSE;
 
 	if (!ft->priv->error_regex_compiled || regstr != ft->priv->last_string)
@@ -1452,19 +1588,10 @@ static void convert_filetype_extensions_to_lower_case(gchar **patterns, gsize le
 #endif
 
 
-void filetypes_read_extensions(void)
+static void read_extensions(GKeyFile *sysconfig, GKeyFile *userconfig)
 {
 	guint i;
 	gsize len = 0;
-	gchar *sysconfigfile = g_strconcat(app->datadir, G_DIR_SEPARATOR_S,
-		"filetype_extensions.conf", NULL);
-	gchar *userconfigfile = g_strconcat(app->configdir, G_DIR_SEPARATOR_S,
-		"filetype_extensions.conf", NULL);
-	GKeyFile *sysconfig = g_key_file_new();
-	GKeyFile *userconfig = g_key_file_new();
-
-	g_key_file_load_from_file(sysconfig, sysconfigfile, G_KEY_FILE_NONE, NULL);
-	g_key_file_load_from_file(userconfig, userconfigfile, G_KEY_FILE_NONE, NULL);
 
 	/* read the keys */
 	for (i = 0; i < filetypes_array->len; i++)
@@ -1484,11 +1611,73 @@ void filetypes_read_extensions(void)
 		convert_filetype_extensions_to_lower_case(filetypes[i]->pattern, len);
 #endif
 	}
+}
+
+
+static void read_group(GKeyFile *config, const gchar *group_name, gint group_id)
+{
+	gchar **names = g_key_file_get_string_list(config, "Groups", group_name, NULL, NULL);
+	gchar **name;
+
+	foreach_strv(name, names)
+	{
+		GeanyFiletype *ft = filetypes_lookup_by_name(*name);
+
+		if (ft)
+		{
+			ft->group = group_id;
+			if (ft->priv->custom &&
+				(group_id == GEANY_FILETYPE_GROUP_COMPILED || group_id == GEANY_FILETYPE_GROUP_SCRIPT))
+			{
+				setptr(ft->title, NULL);
+				filetype_make_title(ft, TITLE_SOURCE_FILE);
+			}
+		}
+		else
+			geany_debug("Filetype '%s' not found for group '%s'!", *name, group_name);
+	}
+	g_strfreev(names);
+}
+
+
+static void read_groups(GKeyFile *config)
+{
+	read_group(config, "Programming", GEANY_FILETYPE_GROUP_COMPILED);
+	read_group(config, "Script", GEANY_FILETYPE_GROUP_SCRIPT);
+	read_group(config, "Markup", GEANY_FILETYPE_GROUP_MARKUP);
+	read_group(config, "Misc", GEANY_FILETYPE_GROUP_MISC);
+	read_group(config, "None", GEANY_FILETYPE_GROUP_NONE);
+}
+
+
+static void read_filetype_config(void)
+{
+	gchar *sysconfigfile = g_strconcat(app->datadir, G_DIR_SEPARATOR_S,
+		"filetype_extensions.conf", NULL);
+	gchar *userconfigfile = g_strconcat(app->configdir, G_DIR_SEPARATOR_S,
+		"filetype_extensions.conf", NULL);
+	GKeyFile *sysconfig = g_key_file_new();
+	GKeyFile *userconfig = g_key_file_new();
+
+	g_key_file_load_from_file(sysconfig, sysconfigfile, G_KEY_FILE_NONE, NULL);
+	g_key_file_load_from_file(userconfig, userconfigfile, G_KEY_FILE_NONE, NULL);
+
+	read_extensions(sysconfig, userconfig);
+	read_groups(sysconfig);
+	read_groups(userconfig);
 
 	g_free(sysconfigfile);
 	g_free(userconfigfile);
 	g_key_file_free(sysconfig);
 	g_key_file_free(userconfig);
+}
+
+
+void filetypes_reload_extensions(void)
+{
+	guint i;
+
+	read_filetype_config();
 
 	/* Redetect filetype of any documents with none set */
 	foreach_document(i)
@@ -1519,9 +1708,6 @@ void filetypes_reload(void)
 	guint i;
 	GeanyDocument *current_doc;
 
-	/* save possibly changed commands before re-reading them */
-	filetypes_save_commands();
-
 	/* reload filetype configs */
 	for (i = 0; i < filetypes_array->len; i++)
 	{
