@@ -1,8 +1,8 @@
 /*
  *      filetypes.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: filetypes.h 4066 2009-08-11 13:22:42Z ntrel $
+ * $Id: filetypes.h 5902 2011-08-25 20:14:17Z colombanw $
  */
 
 
@@ -28,10 +28,15 @@
 #include "Scintilla.h"
 #include "ScintillaWidget.h"
 
+#ifdef GEANY_PRIVATE
+#include "build.h"
+#endif
 
+
+/* Do not change the order, only append. */
 typedef enum
 {
-	GEANY_FILETYPES_NONE = 0,	/* first filetype is always None */
+	GEANY_FILETYPES_NONE = 0,	/* first filetype is always None & must be 0 */
 
 	GEANY_FILETYPES_PHP,
 	GEANY_FILETYPES_BASIC,	/* FreeBasic */
@@ -77,6 +82,13 @@ typedef enum
 	GEANY_FILETYPES_ADA,
 	GEANY_FILETYPES_CMAKE,
 	GEANY_FILETYPES_MARKDOWN,
+	GEANY_FILETYPES_TXT2TAGS,
+	GEANY_FILETYPES_ABC,
+	GEANY_FILETYPES_VERILOG,
+	GEANY_FILETYPES_FORTH,
+	GEANY_FILETYPES_LISP,
+	GEANY_FILETYPES_ERLANG,
+	GEANY_FILETYPES_COBOL,
 	/* ^ append items here */
 	GEANY_MAX_BUILT_IN_FILETYPES	/* Don't use this, use filetypes_array->len instead */
 }
@@ -99,23 +111,6 @@ GeanyFiletypeGroupID;
 #define FILETYPE_ID(filetype_ptr) \
 	(((filetype_ptr) != NULL) ? (filetype_ptr)->id : GEANY_FILETYPES_NONE)
 
-
-struct build_actions
-{
-	gboolean	can_compile;
-	gboolean	can_link;
-	gboolean	can_exec;
-};
-
-struct build_programs
-{
-	gchar *compiler;
-	gchar *linker;
-	gchar *run_cmd;
-	gchar *run_cmd2;
-	gboolean modified;
-};
-
 /** Represents a filetype. */
 struct GeanyFiletype
 {
@@ -123,25 +118,48 @@ struct GeanyFiletype
 	/** Represents the langType of tagmanager (see the table
 	 * in tagmanager/parsers.h), -1 represents all, -2 none. */
 	langType 		  lang;
-	gchar			 *name;				/**< Used as name for tagmanager. E.g. "C". */
-	gchar			 *title;			/**< Shown in the file open dialog. E.g. "C source file". */
-	gchar			 *extension;		/**< Default file extension for new files. */
+	/** Untranslated short name, such as "C", "None".
+	 * Must not be translated as it's used for hash table lookups - use
+	 * filetypes_get_display_name() instead. */
+	gchar			 *name;
+	/** Shown in the file open dialog, such as "C source file". */
+	gchar			 *title;
+	gchar			 *extension;		/**< Default file extension for new files, or @c NULL. */
 	gchar			**pattern;			/**< Array of filename-matching wildcard strings. */
 	gchar			 *context_action_cmd;
 	gchar			 *comment_open;
 	gchar			 *comment_close;
 	gboolean		  comment_use_indent;
-	struct build_programs	*programs;
-	struct build_actions	*actions;	/* TODO: make private */
-	GeanyFiletypeGroupID	group;
+	GeanyFiletypeGroupID group;
 	gchar			 *error_regex_string;
+	GeanyFiletype	 *lexer_filetype;
+	gchar			 *mime_type;
+	GdkPixbuf		 *icon;
+	gchar			 *comment_single; /* single-line comment */
+	/* filetype indent settings, -1 if not set */
+	gint			  indent_type;
+	gint			  indent_width;
 
 	struct GeanyFiletypePrivate	*priv;	/* must be last, append fields before this item */
+#ifdef GEANY_PRIVATE
+	/* Do not use following fields in plugins */
+	/* TODO: move these fields into filetypesprivate.h */
+	GeanyBuildCommand *filecmds;
+	GeanyBuildCommand *ftdefcmds;
+	GeanyBuildCommand *execcmds;
+	GeanyBuildCommand *homefilecmds;
+	GeanyBuildCommand *homeexeccmds;
+	GeanyBuildCommand *projfilecmds;
+	GeanyBuildCommand *projexeccmds;
+	gint			 project_list_entry;
+	gchar			 *projerror_regex_string;
+	gchar			 *homeerror_regex_string;
+#endif
 };
 
 extern GPtrArray *filetypes_array;
 
-/** Wrap filetypes_array so it can be used with C array syntax.
+/** Wraps filetypes_array so it can be used with C array syntax.
  * Example: filetypes[GEANY_FILETYPES_C]->name = ...;
  * @see filetypes_index(). */
 #define filetypes	((GeanyFiletype **)GEANY(filetypes_array)->pdata)
@@ -158,10 +176,17 @@ void filetypes_init(void);
 
 void filetypes_init_types(void);
 
-void filetypes_read_extensions(void);
+void filetypes_reload_extensions(void);
+
+void filetypes_reload(void);
+
 
 GeanyFiletype *filetypes_index(gint idx);
 
+const GSList *filetypes_get_sorted_by_name(void);
+
+const gchar *filetypes_get_display_name(GeanyFiletype *ft);
+
 GeanyFiletype *filetypes_detect_from_document(GeanyDocument *doc);
 
 GeanyFiletype *filetypes_detect_from_extension(const gchar *utf8_filename);
@@ -172,7 +197,7 @@ void filetypes_free_types(void);
 
 void filetypes_load_config(gint ft_id, gboolean reload);
 
-void filetypes_save_commands(void);
+void filetypes_save_commands(GeanyFiletype *ft);
 
 void filetypes_select_radio_item(const GeanyFiletype *ft);
 
@@ -180,8 +205,6 @@ GtkFileFilter *filetypes_create_file_filter(const GeanyFiletype *ft);
 
 GtkFileFilter *filetypes_create_file_filter_all_source(void);
 
-gchar *filetypes_get_conf_extension(gint filetype_idx);
-
 gboolean filetype_has_tags(GeanyFiletype *ft);
 
 gboolean filetypes_parse_error_message(GeanyFiletype *ft, const gchar *message,
