@@ -1,8 +1,8 @@
 /*
  *      geanyentryaction.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2008-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2008-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2008-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2008-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,6 +17,8 @@
  *      You should have received a copy of the GNU General Public License
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *
+ * $Id: geanyentryaction.c 5621 2011-03-24 16:51:02Z colombanw $
  */
 
 /* GtkAction subclass to provide a GtkEntry in a toolbar.
@@ -45,6 +47,7 @@ struct _GeanyEntryActionPrivate
 enum
 {
 	ENTRY_ACTIVATE,
+	ENTRY_ACTIVATE_BACKWARD,
 	ENTRY_CHANGED,
 
 	LAST_SIGNAL
@@ -65,6 +68,7 @@ static GtkWidget *geany_entry_action_create_tool_item(GtkAction *action)
 		gtk_entry_set_width_chars(GTK_ENTRY(priv->entry), 9);
 
 	ui_entry_add_clear_icon(GTK_ENTRY(priv->entry));
+	ui_entry_add_activate_backward_signal(GTK_ENTRY(priv->entry));
 
 	gtk_widget_show(priv->entry);
 
@@ -84,6 +88,15 @@ static void delegate_entry_activate_cb(GtkEntry *entry, GeanyEntryAction *action
 }
 
 
+static void delegate_entry_activate_backward_cb(GtkEntry *entry, GeanyEntryAction *action)
+{
+	GeanyEntryActionPrivate *priv = GEANY_ENTRY_ACTION_GET_PRIVATE(action);
+	const gchar *text = gtk_entry_get_text(GTK_ENTRY(priv->entry));
+
+	g_signal_emit(action, signals[ENTRY_ACTIVATE_BACKWARD], 0, text);
+}
+
+
 static void delegate_entry_changed_cb(GtkEditable *editable, GeanyEntryAction *action)
 {
 	GeanyEntryActionPrivate *priv = GEANY_ENTRY_ACTION_GET_PRIVATE(action);
@@ -102,8 +115,10 @@ static void geany_entry_action_connect_proxy(GtkAction *action, GtkWidget *widge
 			G_CALLBACK(ui_editable_insert_text_callback), NULL);
 	g_signal_connect(priv->entry, "changed", G_CALLBACK(delegate_entry_changed_cb), action);
 	g_signal_connect(priv->entry, "activate", G_CALLBACK(delegate_entry_activate_cb), action);
+	g_signal_connect(priv->entry, "activate-backward",
+		G_CALLBACK(delegate_entry_activate_backward_cb), action);
 
-    GTK_ACTION_CLASS(geany_entry_action_parent_class)->connect_proxy(action, widget);
+	GTK_ACTION_CLASS(geany_entry_action_parent_class)->connect_proxy(action, widget);
 }
 
 
@@ -119,18 +134,26 @@ static void geany_entry_action_class_init(GeanyEntryActionClass *klass)
 
 	signals[ENTRY_CHANGED] = g_signal_new("entry-changed",
 									G_TYPE_FROM_CLASS(klass),
-									(GSignalFlags) 0,
-									0,
+									G_SIGNAL_RUN_LAST,
 									0,
 									NULL,
+									NULL,
 									g_cclosure_marshal_VOID__STRING,
 									G_TYPE_NONE, 1, G_TYPE_STRING);
 	signals[ENTRY_ACTIVATE] = g_signal_new("entry-activate",
 									G_TYPE_FROM_CLASS(klass),
-									(GSignalFlags) 0,
+									G_SIGNAL_RUN_LAST,
 									0,
+									NULL,
+									NULL,
+									g_cclosure_marshal_VOID__STRING,
+									G_TYPE_NONE, 1, G_TYPE_STRING);
+	signals[ENTRY_ACTIVATE_BACKWARD] = g_signal_new("entry-activate-backward",
+									G_TYPE_FROM_CLASS(klass),
+									G_SIGNAL_RUN_LAST,
 									0,
 									NULL,
+									NULL,
 									g_cclosure_marshal_VOID__STRING,
 									G_TYPE_NONE, 1, G_TYPE_STRING);
 }
