@@ -1,8 +1,8 @@
 /*
  *      geanymenubuttonaction.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2009-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2009-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2009-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2009-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,6 +17,8 @@
  *      You should have received a copy of the GNU General Public License
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *
+ * $Id: geanymenubuttonaction.c 5845 2011-06-13 21:01:42Z colombanw $
  */
 
 /* GtkAction subclass to provide a GtkMenuToolButton in a toolbar.
@@ -76,21 +78,6 @@ static void delegate_button_activated(GtkAction *action)
 }
 
 
-static void set_arrow_tooltip(GtkMenuToolButton *button, const gchar *tooltip)
-{
-#if GTK_CHECK_VERSION(2, 12, 0)
-	gtk_menu_tool_button_set_arrow_tooltip_text(button, tooltip);
-#else
-	static GtkTooltips *tooltips = NULL;
-
-	if (G_UNLIKELY(tooltips == NULL))
-		tooltips = gtk_tooltips_new();
-
-	gtk_menu_tool_button_set_arrow_tooltip(button, tooltips, tooltip, NULL);
-#endif
-}
-
-
 static void geany_menu_button_action_set_property(GObject *object, guint prop_id,
 												  const GValue *value, GParamSpec *pspec)
 {
@@ -116,7 +103,7 @@ static GtkWidget *geany_menu_button_action_create_tool_item(GtkAction *action)
 	GeanyMenubuttonActionPrivate *priv = GEANY_MENU_BUTTON_ACTION_GET_PRIVATE(action);
 
 	toolitem = g_object_new(GTK_TYPE_MENU_TOOL_BUTTON, NULL);
-	set_arrow_tooltip(GTK_MENU_TOOL_BUTTON(toolitem), priv->tooltip_arrow);
+	gtk_menu_tool_button_set_arrow_tooltip_text(GTK_MENU_TOOL_BUTTON(toolitem), priv->tooltip_arrow);
 
 	return toolitem;
 }
