@@ -1,8 +1,8 @@
 /*
  *      highlighting.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: highlighting.h 4552 2010-01-25 15:53:54Z ntrel $
+ * $Id: highlighting.h 5528 2011-01-19 19:39:09Z eht16 $
  */
 
 
@@ -51,4 +51,8 @@ const GeanyLexerStyle *highlighting_get_style(gint ft_id, gint style_id);
 
 void highlighting_free_styles(void);
 
+gboolean highlighting_is_string_style(gint lexer, gint style);
+gboolean highlighting_is_comment_style(gint lexer, gint style);
+gboolean highlighting_is_code_style(gint lexer, gint style);
+
 #endif
