@@ -1,8 +1,8 @@
 /*
  *      keybindings.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2006-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: keybindings.c 5498 2010-12-24 13:31:39Z ntrel $
+ * $Id: keybindings.c 5895 2011-08-21 17:15:19Z colombanw $
  */
 
 /**
@@ -86,9 +86,9 @@ static GtkWidget *switch_dialog_label = NULL;
 static gboolean on_key_press_event(GtkWidget *widget, GdkEventKey *event, gpointer user_data);
 static gboolean on_key_release_event(GtkWidget *widget, GdkEventKey *event, gpointer user_data);
 
-static gboolean check_current_word(GeanyDocument *doc);
-static gboolean read_current_word(GeanyDocument *doc);
-static gchar *get_current_word_or_sel(GeanyDocument *doc);
+static gboolean check_current_word(GeanyDocument *doc, gboolean sci_word);
+static gboolean read_current_word(GeanyDocument *doc, gboolean sci_word);
+static gchar *get_current_word_or_sel(GeanyDocument *doc, gboolean sci_word);
 
 static gboolean cb_func_file_action(guint key_id);
 static gboolean cb_func_project_action(guint key_id);
@@ -531,6 +531,8 @@ static void init_default_kb(void)
 		0, 0, "remove_markers", _("Remove Markers"), LW(remove_markers1));
 	keybindings_set_item(group, GEANY_KEYS_DOCUMENT_REMOVE_ERROR_INDICATORS, NULL,
 		0, 0, "remove_error_indicators", _("Remove Error Indicators"), LW(menu_remove_indicators1));
+	keybindings_set_item(group, GEANY_KEYS_DOCUMENT_REMOVE_MARKERS_INDICATORS, NULL,
+		0, 0, "remove_markers_and_indicators", _("Remove Markers and Error Indicators"), NULL);
 
 	group = ADD_KB_GROUP(BUILD, _("Build"), cb_func_build_action);
 
@@ -864,7 +866,7 @@ static GtkWidget *create_dialog(void)
 	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(tree), FALSE);
 
 	text_renderer = gtk_cell_renderer_text_new();
-    /* we can't use "weight-set", see http://bugzilla.gnome.org/show_bug.cgi?id=355214 */
+	/* we can't use "weight-set", see http://bugzilla.gnome.org/show_bug.cgi?id=355214 */
 	column = gtk_tree_view_column_new_with_attributes(
 		NULL, text_renderer, "text", 0, "weight", 2, NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);
@@ -1074,10 +1076,13 @@ static gboolean check_vte(GdkModifierType state, guint keyval)
 	GeanyKeyGroup *group;
 	GtkWidget *widget;
 
-	if (! vc->enable_bash_keys)
-		return FALSE;
 	if (gtk_window_get_focus(GTK_WINDOW(main_widgets.window)) != vc->vte)
 		return FALSE;
+	/* let VTE copy/paste override any user keybinding */
+	if (state == (GDK_CONTROL_MASK | GDK_SHIFT_MASK) && (keyval == GDK_c || keyval == GDK_v))
+		return TRUE;
+	if (! vc->enable_bash_keys)
+		return FALSE;
 	/* prevent menubar flickering: */
 	if (state == GDK_SHIFT_MASK && (keyval >= GDK_a && keyval <= GDK_z))
 		return FALSE;
@@ -1410,16 +1415,12 @@ static gboolean cb_func_search_action(guint key_id)
 		case GEANY_KEYS_SEARCH_PREVIOUSMESSAGE:
 			on_previous_message1_activate(NULL, NULL); break;
 		case GEANY_KEYS_SEARCH_FINDUSAGE:
-			read_current_word(doc);
-			on_find_usage1_activate(NULL, NULL);
-			break;
+			on_find_usage1_activate(NULL, NULL); break;
 		case GEANY_KEYS_SEARCH_FINDDOCUMENTUSAGE:
-			read_current_word(doc);
-			on_find_document_usage1_activate(NULL, NULL);
-			break;
+			on_find_document_usage1_activate(NULL, NULL); break;
 		case GEANY_KEYS_SEARCH_MARKALL:
 		{
-			gchar *text = get_current_word_or_sel(doc);
+			gchar *text = get_current_word_or_sel(doc, TRUE);
 
 			if (sci_has_selection(sci))
 				search_mark_all(doc, text, SCFIND_MATCHCASE);
@@ -1541,25 +1542,29 @@ static gboolean cb_func_build_action(guint key_id)
 }
 
 
-static gboolean read_current_word(GeanyDocument *doc)
+static gboolean read_current_word(GeanyDocument *doc, gboolean sci_word)
 {
-	gint pos;
-
 	if (doc == NULL)
 		return FALSE;
 
-	pos = sci_get_current_position(doc->editor->sci);
-
-	editor_find_current_word(doc->editor, pos,
-		editor_info.current_word, GEANY_MAX_WORD_LENGTH, NULL);
+	if (sci_word)
+	{
+		editor_find_current_word_sciwc(doc->editor, -1,
+			editor_info.current_word, GEANY_MAX_WORD_LENGTH);
+	}
+	else
+	{
+		editor_find_current_word(doc->editor, -1,
+			editor_info.current_word, GEANY_MAX_WORD_LENGTH, NULL);
+	}
 
 	return (*editor_info.current_word != 0);
 }
 
 
-static gboolean check_current_word(GeanyDocument *doc)
+static gboolean check_current_word(GeanyDocument *doc, gboolean sci_word)
 {
-	if (!read_current_word(doc))
+	if (! read_current_word(doc, sci_word))
 	{
 		utils_beep();
 		return FALSE;
@@ -1568,14 +1573,14 @@ static gboolean check_current_word(GeanyDocument *doc)
 }
 
 
-static gchar *get_current_word_or_sel(GeanyDocument *doc)
+static gchar *get_current_word_or_sel(GeanyDocument *doc, gboolean sci_word)
 {
 	ScintillaObject *sci = doc->editor->sci;
 
 	if (sci_has_selection(sci))
 		return sci_get_selection_contents(sci);
 
-	return read_current_word(doc) ? g_strdup(editor_info.current_word) : NULL;
+	return read_current_word(doc, sci_word) ? g_strdup(editor_info.current_word) : NULL;
 }
 
 
@@ -1775,7 +1780,7 @@ static GtkWidget *create_switch_dialog(void)
 }
 
 
-static void update_filename_label()
+static void update_filename_label(void)
 {
 	if (!switch_dialog)
 	{
@@ -1783,8 +1788,7 @@ static void update_filename_label()
 		gtk_widget_show_all(switch_dialog);
 	}
 
-	geany_wrap_label_set_text(GTK_LABEL(switch_dialog_label),
-		DOC_FILENAME(document_get_current()));
+	gtk_label_set_text(GTK_LABEL(switch_dialog_label), DOC_FILENAME(document_get_current()));
 }
 
 
@@ -1930,7 +1934,7 @@ static gboolean cb_func_clipboard_action(guint key_id)
 
 static void goto_tag(GeanyDocument *doc, gboolean definition)
 {
-	gchar *text = get_current_word_or_sel(doc);
+	gchar *text = get_current_word_or_sel(doc, FALSE);
 
 	if (text)
 		symbols_goto_tag(text, definition);
@@ -2144,7 +2148,7 @@ static gboolean cb_func_editor_action(guint key_id)
 			editor_show_macro_list(doc->editor);
 			break;
 		case GEANY_KEYS_EDITOR_CONTEXTACTION:
-			if (check_current_word(doc))
+			if (check_current_word(doc, FALSE))
 				on_context_action1_activate(GTK_MENU_ITEM(ui_lookup_widget(main_widgets.editor_menu,
 					"context_action1")), NULL);
 			break;
@@ -2540,6 +2544,10 @@ static gboolean cb_func_document_action(guint key_id)
 		case GEANY_KEYS_DOCUMENT_REMOVE_ERROR_INDICATORS:
 			on_menu_remove_indicators1_activate(NULL, NULL);
 			break;
+		case GEANY_KEYS_DOCUMENT_REMOVE_MARKERS_INDICATORS:
+			on_remove_markers1_activate(NULL, NULL);
+			on_menu_remove_indicators1_activate(NULL, NULL);
+			break;
 	}
 	return TRUE;
 }
