@@ -1,8 +1,8 @@
 /*
  *      keyfile.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: keyfile.c 5471 2010-12-07 13:16:57Z ntrel $
+ * $Id: keyfile.c 5947 2011-09-22 15:55:10Z ntrel $
  */
 
 /*
@@ -83,6 +83,7 @@
 #define GEANY_DEFAULT_FONT_EDITOR		"Monospace 10"
 #define GEANY_TOGGLE_MARK				"~ "
 #define GEANY_MAX_AUTOCOMPLETE_WORDS	30
+#define GEANY_MAX_SYMBOLS_UPDATE_FREQ	250
 
 
 static gchar *scribble_text = NULL;
@@ -91,10 +92,20 @@ static GPtrArray *session_files = NULL;
 static gint session_notebook_page;
 static gint hpan_position;
 static gint vpan_position;
+static const gchar atomic_file_saving_key[] = "use_atomic_file_saving";
 
 static GPtrArray *keyfile_groups = NULL;
+
 GPtrArray *pref_groups = NULL;
 
+static struct
+{
+	int number_ft_menu_items;
+	int number_non_ft_menu_items;
+	int number_exec_menu_items;
+}
+build_menu_prefs;
+
 
 /* The group will be free'd on quitting.
  * @param for_prefs_dialog is whether the group also has Prefs dialog items. */
@@ -107,6 +118,14 @@ void configuration_add_pref_group(struct StashGroup *group, gboolean for_prefs_d
 }
 
 
+/* The group will be free'd on quitting. */
+void configuration_add_various_pref_group(struct StashGroup *group)
+{
+	configuration_add_pref_group(group, TRUE);
+	stash_group_set_various(group, TRUE);
+}
+
+
 static void init_pref_groups(void)
 {
 	StashGroup *group;
@@ -139,7 +158,9 @@ static void init_pref_groups(void)
 
 	/* editor */
 	stash_group_add_toggle_button(group, &editor_prefs.indentation->detect_type,
-		"check_detect_indent", FALSE, "check_detect_indent");
+		"check_detect_indent", FALSE, "check_detect_indent_type");
+	stash_group_add_toggle_button(group, &editor_prefs.indentation->detect_width,
+		"detect_indent_width", FALSE, "check_detect_indent_width");
 	stash_group_add_toggle_button(group, &editor_prefs.use_tab_to_indent,
 		"use_tab_to_indent", TRUE, "check_tab_key_indents");
 	stash_group_add_spin_button_integer(group, &editor_prefs.indentation->width,
@@ -165,6 +186,8 @@ static void init_pref_groups(void)
 	stash_group_add_spin_button_integer(group, (gint*)&editor_prefs.autocompletion_max_entries,
 		"autocompletion_max_entries", GEANY_MAX_AUTOCOMPLETE_WORDS,
 		"spin_autocompletion_max_entries");
+	stash_group_add_spin_button_integer(group, (gint*)&editor_prefs.autocompletion_update_freq,
+		"autocompletion_update_freq", GEANY_MAX_SYMBOLS_UPDATE_FREQ, "spin_symbol_update_freq");
 	stash_group_add_string(group, &editor_prefs.color_scheme,
 		"color_scheme", NULL);
 
@@ -174,10 +197,9 @@ static void init_pref_groups(void)
 	stash_group_add_spin_button_integer(group, &file_prefs.disk_check_timeout,
 		"disk_check_timeout", GEANY_DISK_CHECK_TIMEOUT, "spin_disk_check");
 
-	/* hidden prefs (don't overwrite them so users can edit them manually) */
+	/* various geany prefs */
 	group = stash_group_new(PACKAGE);
-	configuration_add_pref_group(group, FALSE);
-	stash_group_set_write_once(group, TRUE);
+	configuration_add_various_pref_group(group);
 
 	stash_group_add_boolean(group, &editor_prefs.show_scrollbars,
 		"show_editor_scrollbars", TRUE);
@@ -188,16 +210,29 @@ static void init_pref_groups(void)
 	stash_group_add_boolean(group, &editor_prefs.complete_snippets_whilst_editing,
 		"complete_snippets_whilst_editing", FALSE);
 	stash_group_add_boolean(group, &file_prefs.use_safe_file_saving,
-		"use_safe_file_saving", FALSE);
+		atomic_file_saving_key, FALSE);
 	stash_group_add_boolean(group, &file_prefs.gio_unsafe_save_backup,
 		"gio_unsafe_save_backup", FALSE);
+	stash_group_add_boolean(group, &file_prefs.use_gio_unsafe_file_saving,
+		"use_gio_unsafe_file_saving", TRUE);
 	/* for backwards-compatibility */
 	stash_group_add_integer(group, &editor_prefs.indentation->hard_tab_width,
 		"indent_hard_tab_width", 8);
 	stash_group_add_integer(group, (gint*)&search_prefs.find_selection_type,
 		"find_selection_type", GEANY_FIND_SEL_CURRENT_WORD);
 
-	/* Note: Interface-related hidden prefs are in ui_init_prefs() */
+	/* Note: Interface-related various prefs are in ui_init_prefs() */
+
+	/* various build-menu prefs */
+	group = stash_group_new("build-menu");
+	configuration_add_various_pref_group(group);
+
+	stash_group_add_integer(group, &build_menu_prefs.number_ft_menu_items,
+		"number_ft_menu_items", 0);
+	stash_group_add_integer(group, &build_menu_prefs.number_non_ft_menu_items,
+		"number_non_ft_menu_items", 0);
+	stash_group_add_integer(group, &build_menu_prefs.number_exec_menu_items,
+		"number_exec_menu_items", 0);
 }
 
 
@@ -330,6 +365,14 @@ void configuration_save_session_files(GKeyFile *config)
 			i++;
 		}
 	}
+
+#ifdef HAVE_VTE
+	if (vte_info.have_vte)
+	{
+		vte_get_working_directory();	/* refresh vte_info.dir */
+		g_key_file_set_string(config, "VTE", "last_dir", vte_info.dir);
+	}
+#endif
 }
 
 
@@ -363,10 +406,6 @@ static void save_dialog_prefs(GKeyFile *config)
 	g_key_file_set_boolean(config, PACKAGE, "tab_order_beside", file_prefs.tab_order_beside);
 	g_key_file_set_integer(config, PACKAGE, "tab_pos_editor", interface_prefs.tab_pos_editor);
 	g_key_file_set_integer(config, PACKAGE, "tab_pos_msgwin", interface_prefs.tab_pos_msgwin);
-	g_key_file_set_boolean(config, PACKAGE, "msgwin_status_visible", interface_prefs.msgwin_status_visible);
-	g_key_file_set_boolean(config, PACKAGE, "msgwin_compiler_visible", interface_prefs.msgwin_compiler_visible);
-	g_key_file_set_boolean(config, PACKAGE, "msgwin_messages_visible", interface_prefs.msgwin_messages_visible);
-	g_key_file_set_boolean(config, PACKAGE, "msgwin_scribble_visible", interface_prefs.msgwin_scribble_visible);
 	g_key_file_set_boolean(config, PACKAGE, "use_native_windows_dialogs", interface_prefs.use_native_windows_dialogs);
 
 	/* display */
@@ -477,8 +516,6 @@ static void save_dialog_prefs(GKeyFile *config)
 		tmp_string = utils_get_hex_from_color(vc->colour_back);
 		g_key_file_set_string(config, "VTE", "colour_back", tmp_string);
 		g_free(tmp_string);
-		vte_get_working_directory();	/* refresh vte_info.dir */
-		g_key_file_set_string(config, "VTE", "last_dir", vte_info.dir);
 	}
 #endif
 }
@@ -554,6 +591,13 @@ void configuration_save(void)
 
 	if (cl_options.load_session)
 		configuration_save_session_files(config);
+#ifdef HAVE_VTE
+	else if (vte_info.have_vte)
+	{
+		vte_get_working_directory();	/* refresh vte_info.dir */
+		g_key_file_set_string(config, "VTE", "last_dir", vte_info.dir);
+	}
+#endif
 
 	/* write the file */
 	data = g_key_file_to_data(config, NULL, NULL);
@@ -623,15 +667,33 @@ void configuration_load_session_files(GKeyFile *config, gboolean read_recent_fil
 		g_ptr_array_add(session_files, tmp_array);
 		i++;
 	}
+
+#ifdef HAVE_VTE
+	/* BUG: after loading project at startup, closing project doesn't restore old VTE path */
+	if (vte_info.have_vte)
+	{
+		gchar *tmp_string = utils_get_setting_string(config, "VTE", "last_dir", NULL);
+		vte_cwd(tmp_string,TRUE);
+		g_free(tmp_string);
+	}
+#endif
 }
 
 
+/* note: new settings should be added in init_pref_groups() */
 static void load_dialog_prefs(GKeyFile *config)
 {
 	gchar *tmp_string, *tmp_string2;
 	const gchar *default_charset = NULL;
 
-	/* new settings should be added in init_pref_groups() */
+	/* compatibility with Geany 0.20 */
+	if (!g_key_file_has_key(config, PACKAGE, atomic_file_saving_key, NULL))
+	{
+		g_key_file_set_boolean(config, PACKAGE, atomic_file_saving_key,
+			utils_get_setting_boolean(config, PACKAGE, "use_safe_file_saving", FALSE));
+	}
+	
+	/* read stash prefs */
 	settings_action(config, SETTING_READ);
 
 	/* general */
@@ -658,10 +720,6 @@ static void load_dialog_prefs(GKeyFile *config)
 	interface_prefs.editor_font = utils_get_setting_string(config, PACKAGE, "editor_font", GEANY_DEFAULT_FONT_EDITOR);
 	interface_prefs.tagbar_font = utils_get_setting_string(config, PACKAGE, "tagbar_font", GEANY_DEFAULT_FONT_SYMBOL_LIST);
 	interface_prefs.msgwin_font = utils_get_setting_string(config, PACKAGE, "msgwin_font", GEANY_DEFAULT_FONT_MSG_WINDOW);
-	interface_prefs.msgwin_status_visible = utils_get_setting_boolean(config, PACKAGE, "msgwin_status_visible", TRUE);
-	interface_prefs.msgwin_compiler_visible = utils_get_setting_boolean(config, PACKAGE, "msgwin_compiler_visible", TRUE);
-	interface_prefs.msgwin_messages_visible = utils_get_setting_boolean(config, PACKAGE, "msgwin_messages_visible", TRUE);
-	interface_prefs.msgwin_scribble_visible = utils_get_setting_boolean(config, PACKAGE, "msgwin_scribble_visible", TRUE);
 	interface_prefs.use_native_windows_dialogs = utils_get_setting_boolean(config, PACKAGE, "use_native_windows_dialogs", FALSE);
 
 	/* display, editor */
@@ -810,9 +868,9 @@ static void load_dialog_prefs(GKeyFile *config)
 	tool_prefs.context_action_cmd = utils_get_setting_string(config, PACKAGE, "context_action_cmd", "");
 
 	/* build menu */
-	build_set_group_count(GEANY_GBG_FT, utils_get_setting_integer(config, "build-menu", "number_ft_menu_items", 0));
-	build_set_group_count(GEANY_GBG_NON_FT, utils_get_setting_integer(config, "build-menu", "number_non_ft_menu_items", 0));
-	build_set_group_count(GEANY_GBG_EXEC, utils_get_setting_integer(config, "build-menu", "number_exec_menu_items", 0));
+	build_set_group_count(GEANY_GBG_FT, build_menu_prefs.number_ft_menu_items);
+	build_set_group_count(GEANY_GBG_NON_FT, build_menu_prefs.number_non_ft_menu_items);
+	build_set_group_count(GEANY_GBG_EXEC, build_menu_prefs.number_exec_menu_items);
 	build_load_menu(config, GEANY_BCS_PREF, NULL);
 
 	/* printing */
@@ -827,11 +885,7 @@ static void load_dialog_prefs(GKeyFile *config)
 	g_free(tmp_string);
 	g_free(tmp_string2);
 
-#if GTK_CHECK_VERSION(2, 10, 0)
 	printing_prefs.use_gtk_printing = utils_get_setting_boolean(config, "printing", "use_gtk_printing", TRUE);
-#else
-	printing_prefs.use_gtk_printing = utils_get_setting_boolean(config, "printing", "use_gtk_printing", FALSE);
-#endif
 	printing_prefs.print_line_numbers = utils_get_setting_boolean(config, "printing", "print_line_numbers", TRUE);
 	printing_prefs.print_page_numbers = utils_get_setting_boolean(config, "printing", "print_page_numbers", TRUE);
 	printing_prefs.print_page_header = utils_get_setting_boolean(config, "printing", "print_page_header", TRUE);
@@ -921,7 +975,7 @@ void configuration_save_default_session(void)
  */
 void configuration_reload_default_session(void)
 {
-	gchar *configfile = g_strconcat(app->configdir, G_DIR_SEPARATOR_S, "geany.conf", NULL);
+	const gchar *configfile = utils_build_path(app->configdir, "geany.conf", NULL);
 	GKeyFile *config = g_key_file_new();
 
 	g_key_file_load_from_file(config, configfile, G_KEY_FILE_NONE, NULL);
@@ -929,23 +983,22 @@ void configuration_reload_default_session(void)
 	configuration_load_session_files(config, FALSE);
 
 	g_key_file_free(config);
-	g_free(configfile);
 }
 
 
 gboolean configuration_load(void)
 {
-	gchar *configfile = g_strconcat(app->configdir, G_DIR_SEPARATOR_S, "geany.conf", NULL);
+	gchar *configfile = utils_build_path(app->configdir, "geany.conf", NULL);
 	GKeyFile *config = g_key_file_new();
 
 	if (! g_file_test(configfile, G_FILE_TEST_IS_REGULAR))
 	{	/* config file does not (yet) exist, so try to load a global config file which may be */
 		/* created by distributors */
 		geany_debug("No user config file found, trying to use global configuration.");
-		setptr(configfile, g_strconcat(app->datadir, G_DIR_SEPARATOR_S "geany.conf", NULL));
+		setptr(configfile, utils_build_path(app->datadir, "geany.conf", NULL));
 	}
-
 	g_key_file_load_from_file(config, configfile, G_KEY_FILE_NONE, NULL);
+	g_free(configfile);
 
 	load_dialog_prefs(config);
 	load_ui_prefs(config);
@@ -956,7 +1009,6 @@ gboolean configuration_load(void)
 	g_signal_emit_by_name(geany_object, "load-settings", config);
 
 	g_key_file_free(config);
-	g_free(configfile);
 	return TRUE;
 }
 
