@@ -1,8 +1,8 @@
 /*
  *      main.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: main.c 5472 2010-12-07 13:25:11Z ntrel $
+ * $Id: main.c 5886 2011-08-06 18:34:14Z colombanw $
  */
 
 /**
@@ -96,9 +96,6 @@ CommandLineOptions cl_options;	/* fields initialised in parse_command_line_optio
 
 
 static const gchar geany_lib_versions[] = "GTK %u.%u.%u, GLib %u.%u.%u"
-#ifdef HAVE_GIO
-	", GIO"
-#endif
 #ifdef USE_INCLUDED_REGEX
 	", built-in regex"
 #endif
@@ -232,8 +229,6 @@ static void main_init(void)
 	app->project			= NULL;
 	ui_widgets.open_fontsel		= NULL;
 	ui_widgets.open_colorsel	= NULL;
-	ui_widgets.open_filesel		= NULL;
-	ui_widgets.save_filesel		= NULL;
 	ui_widgets.prefs_dialog		= NULL;
 	main_status.main_window_realized = FALSE;
 	file_prefs.tab_order_ltr		= FALSE;
@@ -274,11 +269,6 @@ static void main_init(void)
 	gtk_widget_set_name(ui_lookup_widget(main_widgets.window, "menubar1"), "GeanyMenubar");
 	gtk_widget_set_name(main_widgets.toolbar, "GeanyToolbar");
 
-#if ! GTK_CHECK_VERSION(2, 10, 0)
-	/* hide Page setup menu item, it isn't supported with non-GTK printing */
-	gtk_widget_hide(ui_widgets.print_page_setup);
-#endif
-
 	/* for some reason we need to set the initial size request,
 	 * otherwise the main window gets crazy dimensions */
 	gtk_widget_set_size_request(main_widgets.window, 10, 10);
@@ -331,7 +321,7 @@ static void get_line_and_column_from_filename(gchar *filename, gint *line, gint
 
 	g_assert(*line == -1 && *column == -1);
 
-	if (! NZV(filename))
+	if (G_UNLIKELY(! NZV(filename)))
 		return;
 
 	/* allow to open files like "test:0" */
@@ -435,6 +425,9 @@ gboolean main_is_realized(void)
  *  to adjust the build system of your plugin to get internationalisation support
  *  working properly.
  *
+ *  If you have already used @ref PLUGIN_SET_TRANSLATABLE_INFO() you
+ *  don't need to call main_locale_init() again as it has already been done.
+ *
  *  @param locale_dir The location where the translation files should be searched. This is
  *                    usually the @c LOCALEDIR macro, defined by the build system.
  *                    E.g. @c $prefix/share/locale.
@@ -472,14 +465,17 @@ void main_locale_init(const gchar *locale_dir, const gchar *package)
 
 static void print_filetypes(void)
 {
-	guint i;
+	const GSList *list, *node;
 
 	filetypes_init_types();
-	printf("Geany's internal filetype names:\n");
+	printf("Geany's filetype names:\n");
 
-	for (i = 0; i < filetypes_array->len; i++)
+	list = filetypes_get_sorted_by_name();
+	foreach_slist(node, list)
 	{
-		printf("%s\n", filetypes[i]->name);
+		GeanyFiletype *ft = node->data;
+
+		printf("%s\n", ft->name);
 	}
 	filetypes_free_types();
 }
@@ -561,12 +557,9 @@ static void parse_command_line_options(gint *argc, gchar ***argv)
 	{
 		gboolean ret;
 
-		document_init_doclist();
 		filetypes_init_types();
-		filetypes_read_extensions();	/* needed for *.lang.tags filetype matching */
 		ret = symbols_generate_global_tags(*argc, *argv, ! no_preprocessing);
 		filetypes_free_types();
-		document_finalize();
 		exit(ret);
 	}
 
@@ -652,7 +645,7 @@ static gint create_config_dir(void)
 
 	if (saved_errno == 0 && ! g_file_test(conf_file, G_FILE_TEST_EXISTS))
 	{	/* check whether geany.conf can be written */
-		saved_errno = utils_is_file_writeable(app->configdir);
+		saved_errno = utils_is_file_writable(app->configdir);
 	}
 
 	/* make subdir for filetype definitions */
@@ -832,7 +825,7 @@ static void load_session_project_file(void)
 
 	locale_filename = utils_get_locale_from_utf8(project_prefs.session_file);
 
-	if (NZV(locale_filename))
+	if (G_LIKELY(NZV(locale_filename)))
 		project_load_file(locale_filename);
 
 	g_free(locale_filename);
@@ -957,6 +950,11 @@ gint main(gint argc, gchar **argv)
 #endif
 	parse_command_line_options(&argc, &argv);
 
+	/* Initialize GLib's thread system in case any plugins want to use it or their
+	 * dependencies (e.g. WebKit, Soup, ...) */
+	if (!g_thread_supported())
+		g_thread_init(NULL);
+
 	signal(SIGTERM, signal_cb);
 #ifdef G_OS_UNIX
 	/* SIGQUIT is used to kill spawned children and we get also this signal, so ignore */
@@ -967,7 +965,7 @@ gint main(gint argc, gchar **argv)
 
 	config_dir_result = setup_config_dir();
 #ifdef HAVE_SOCKET
-    /* check and create (unix domain) socket for remote operation */
+	/* check and create (unix domain) socket for remote operation */
 	if (! socket_info.ignore_socket)
 	{
 		socket_info.lock_socket = -1;
@@ -1030,14 +1028,12 @@ gint main(gint argc, gchar **argv)
 	ui_create_insert_menu_items();
 	ui_create_insert_date_menu_items();
 	keybindings_init();
-	tools_create_insert_custom_command_menu_items();
 	notebook_init();
 	filetypes_init();
 	templates_init();
 	navqueue_init();
 	document_init_doclist();
 	symbols_init();
-	filetypes_read_extensions();
 	editor_snippets_init();
 
 	/* set window icon */
@@ -1080,6 +1076,10 @@ gint main(gint argc, gchar **argv)
 	/* load keybinding settings after plugins have added their groups */
 	keybindings_load_keyfile();
 
+	/* create the custom command menu after the keybindings have been loaded to have the proper
+	 * accelerator shown for the menu items */
+	tools_create_insert_custom_command_menu_items();
+
 	/* load any command line files or session files */
 	main_status.opening_session_files = TRUE;
 	load_startup_files(argc, argv);
@@ -1164,7 +1164,6 @@ void main_quit()
 
 	navqueue_free();
 	keybindings_free();
-	filetypes_save_commands();
 	highlighting_free_styles();
 	templates_free_templates();
 	msgwin_finalize();
@@ -1213,8 +1212,6 @@ void main_quit()
 	queue_free(ui_prefs.recent_projects_queue);
 
 	if (ui_widgets.prefs_dialog && GTK_IS_WIDGET(ui_widgets.prefs_dialog)) gtk_widget_destroy(ui_widgets.prefs_dialog);
-	if (ui_widgets.save_filesel && GTK_IS_WIDGET(ui_widgets.save_filesel)) gtk_widget_destroy(ui_widgets.save_filesel);
-	if (ui_widgets.open_filesel && GTK_IS_WIDGET(ui_widgets.open_filesel)) gtk_widget_destroy(ui_widgets.open_filesel);
 	if (ui_widgets.open_fontsel && GTK_IS_WIDGET(ui_widgets.open_fontsel)) gtk_widget_destroy(ui_widgets.open_fontsel);
 	if (ui_widgets.open_colorsel && GTK_IS_WIDGET(ui_widgets.open_colorsel)) gtk_widget_destroy(ui_widgets.open_colorsel);
 #ifdef HAVE_VTE
@@ -1266,9 +1263,7 @@ void main_reload_configuration(void)
 	editor_snippets_free();
 	editor_snippets_init();
 
-	/* reload filetype extensions */
-	filetypes_read_extensions();
-
+	filetypes_reload_extensions();
 	filetypes_reload();
 
 	/* C tag names to ignore */
