@@ -1,8 +1,8 @@
 /*
  *      main.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2006-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: main.h 3454 2009-01-09 18:21:12Z eht16 $
+ * $Id: main.h 5528 2011-01-19 19:39:09Z eht16 $
  */
 
 
@@ -28,10 +28,12 @@
 typedef struct
 {
 	gboolean	new_instance;
+	gchar	   *socket_filename;
 	gboolean	load_session;
 	gint		goto_line;
 	gint		goto_column;
 	gboolean	ignore_global_tags;
+	gboolean	list_documents;
 }
 CommandLineOptions;
 
@@ -63,4 +65,8 @@ void main_reload_configuration(void);
 
 void main_locale_init(const gchar *locale_dir, const gchar *gettext_package);
 
+gboolean main_is_realized(void);
+
+void main_load_project_from_command_line(const gchar *locale_filename, gboolean use_session);
+
 #endif
