@@ -1,8 +1,8 @@
 /*
  *      msgwindow.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: msgwindow.c 5237 2010-09-16 15:31:23Z ntrel $
+ * $Id: msgwindow.c 5887 2011-08-13 20:44:39Z colombanw $
  */
 
 /**
@@ -71,7 +71,6 @@ static void prepare_msg_tree_view(void);
 static void prepare_status_tree_view(void);
 static void prepare_compiler_tree_view(void);
 static GtkWidget *create_message_popup_menu(gint type);
-static void msgwin_parse_grep_line(const gchar *string, gchar **filename, gint *line);
 static gboolean on_msgwin_button_press_event(GtkWidget *widget, GdkEventButton *event,
 																			gpointer user_data);
 static void on_scribble_populate(GtkTextView *textview, GtkMenu *arg1, gpointer user_data);
@@ -125,18 +124,20 @@ void msgwin_finalize(void)
 
 static gboolean on_msgwin_key_press_event(GtkWidget *widget, GdkEventKey *event, gpointer data)
 {
-	if (ui_is_keyval_enter_or_return(event->keyval) || event->keyval == GDK_space)
+	gboolean enter_or_return = ui_is_keyval_enter_or_return(event->keyval);
+
+	if (enter_or_return || event->keyval == GDK_space)
 	{
 		switch (GPOINTER_TO_INT(data))
 		{
 			case MSG_COMPILER:
-			{	/* single click in the compiler treeview */
-				msgwin_goto_compiler_file_line(event->keyval);
+			{	/* key press in the compiler treeview */
+				msgwin_goto_compiler_file_line(enter_or_return);
 				break;
 			}
 			case MSG_MESSAGE:
-			{	/* single click in the message treeview (results of 'Find usage') */
-				msgwin_goto_messages_file_line(event->keyval);
+			{	/* key press in the message treeview (results of 'Find usage') */
+				msgwin_goto_messages_file_line(enter_or_return);
 				break;
 			}
 		}
@@ -195,6 +196,9 @@ static void prepare_msg_tree_view(void)
 	 * (connect_after button-press-event doesn't work) */
 	g_signal_connect(msgwindow.tree_msg, "button-release-event",
 					G_CALLBACK(on_msgwin_button_press_event), GINT_TO_POINTER(MSG_MESSAGE));
+	/* for double-clicking only, after the first release */
+	g_signal_connect(msgwindow.tree_msg, "button-press-event",
+					G_CALLBACK(on_msgwin_button_press_event), GINT_TO_POINTER(MSG_MESSAGE));
 	g_signal_connect(msgwindow.tree_msg, "key-press-event",
 		G_CALLBACK(on_msgwin_key_press_event), GINT_TO_POINTER(MSG_MESSAGE));
 
@@ -228,6 +232,9 @@ static void prepare_compiler_tree_view(void)
 	 * (connect_after button-press-event doesn't work) */
 	g_signal_connect(msgwindow.tree_compiler, "button-release-event",
 					G_CALLBACK(on_msgwin_button_press_event), GINT_TO_POINTER(MSG_COMPILER));
+	/* for double-clicking only, after the first release */
+	g_signal_connect(msgwindow.tree_compiler, "button-press-event",
+					G_CALLBACK(on_msgwin_button_press_event), GINT_TO_POINTER(MSG_COMPILER));
 	g_signal_connect(msgwindow.tree_compiler, "key-press-event",
 		G_CALLBACK(on_msgwin_key_press_event), GINT_TO_POINTER(MSG_COMPILER));
 
@@ -425,8 +432,7 @@ void msgwin_status_add(const gchar *format, ...)
 
 
 static void
-on_message_treeview_clear_activate     (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+on_message_treeview_clear_activate(GtkMenuItem *menuitem, gpointer user_data)
 {
 	gint tabnum = GPOINTER_TO_INT(user_data);
 
@@ -435,8 +441,7 @@ on_message_treeview_clear_activate     (GtkMenuItem     *menuitem,
 
 
 static void
-on_compiler_treeview_copy_activate     (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+on_compiler_treeview_copy_activate(GtkMenuItem *menuitem, gpointer user_data)
 {
 	GtkWidget *tv = NULL;
 	GtkTreeSelection *selection;
@@ -532,8 +537,7 @@ static void on_compiler_treeview_copy_all_activate(GtkMenuItem *menuitem, gpoint
 
 
 static void
-on_hide_message_window                 (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+on_hide_message_window(GtkMenuItem *menuitem, gpointer user_data)
 {
 	msgwin_show_hide(FALSE);
 }
@@ -623,7 +627,7 @@ find_prev_build_dir(GtkTreePath *cur, GtkTreeModel *model, gchar **prefix)
 }
 
 
-static gboolean goto_compiler_file_line(const gchar *filename, gint line, guint keyval)
+static gboolean goto_compiler_file_line(const gchar *filename, gint line, gboolean focus_editor)
 {
 	if (!filename || line <= -1)
 		return FALSE;
@@ -673,7 +677,7 @@ static gboolean goto_compiler_file_line(const gchar *filename, gint line, guint
 				editor_indicator_set_on_line(doc->editor, GEANY_INDICATOR_ERROR, line - 1);
 
 			ret = navqueue_goto_line(old_doc, doc, line);
-			if (ret && ui_is_keyval_enter_or_return(keyval))
+			if (ret && focus_editor)
 				gtk_widget_grab_focus(GTK_WIDGET(doc->editor->sci));
 
 			return ret;
@@ -683,7 +687,7 @@ static gboolean goto_compiler_file_line(const gchar *filename, gint line, guint
 }
 
 
-gboolean msgwin_goto_compiler_file_line(guint keyval)
+gboolean msgwin_goto_compiler_file_line(gboolean focus_editor)
 {
 	GtkTreeIter iter;
 	GtkTreeModel *model;
@@ -719,7 +723,7 @@ gboolean msgwin_goto_compiler_file_line(guint keyval)
 			g_free(string);
 			g_free(dir);
 
-			ret = goto_compiler_file_line(filename, line, keyval);
+			ret = goto_compiler_file_line(filename, line, focus_editor);
 			g_free(filename);
 			return ret;
 		}
@@ -996,7 +1000,52 @@ void msgwin_parse_compiler_error_line(const gchar *string, const gchar *dir,
 }
 
 
-gboolean msgwin_goto_messages_file_line(guint keyval)
+/* Tries to parse strings of the file:line style, allowing line field to be missing
+ * * filename is filled with the filename, should be freed
+ * * line is filled with the line number or -1 */
+static void msgwin_parse_generic_line(const gchar *string, gchar **filename, gint *line)
+{
+	gchar **fields;
+	gboolean incertain = TRUE; /* whether we're reasonably certain of the result */
+
+	*filename = NULL;
+	*line = -1;
+
+	fields = g_strsplit(string, ":", 2);
+	/* extract the filename */
+	if (fields[0] != NULL)
+	{
+		*filename = g_strdup(fields[0]);
+		if (msgwindow.messages_dir != NULL)
+			make_absolute(filename, msgwindow.messages_dir);
+
+		/* now the line */
+		if (fields[1] != NULL)
+		{
+			gchar *end;
+
+			*line = strtol(fields[1], &end, 10);
+			if (end == fields[1])
+				*line = -1;
+			else if (*end == ':' || g_ascii_isspace(*end))
+			{	/* if we have a blank or a separator right after the number, assume we really got a
+				 * filename (it's a grep-like syntax) */
+				incertain = FALSE;
+			}
+		}
+
+		/* if we aren't sure we got a supposedly correct filename, check it */
+		if (incertain && ! g_file_test(*filename, G_FILE_TEST_EXISTS))
+		{
+			setptr(*filename, NULL);
+			*line = -1;
+		}
+	}
+	g_strfreev(fields);
+}
+
+
+gboolean msgwin_goto_messages_file_line(gboolean focus_editor)
 {
 	GtkTreeIter iter;
 	GtkTreeModel *model;
@@ -1016,22 +1065,23 @@ gboolean msgwin_goto_messages_file_line(guint keyval)
 		if (line >= 0 && DOC_VALID(doc))
 		{
 			ret = navqueue_goto_line(old_doc, doc, line);
-			if (ret && ui_is_keyval_enter_or_return(keyval))
+			if (ret && focus_editor)
 				gtk_widget_grab_focus(GTK_WIDGET(doc->editor->sci));
 		}
 		else if (line < 0 && string != NULL)
 		{
 			gchar *filename;
 
-			msgwin_parse_grep_line(string, &filename, &line);
-			if (filename != NULL && line > -1)
+			/* try with a file:line parsing */
+			msgwin_parse_generic_line(string, &filename, &line);
+			if (filename != NULL)
 			{
 				/* use document_open_file to find an already open file, or open it in place */
 				doc = document_open_file(filename, FALSE, NULL, NULL);
 				if (doc != NULL)
 				{
-					ret = navqueue_goto_line(old_doc, doc, line);
-					if (ret && ui_is_keyval_enter_or_return(keyval))
+					ret = (line < 0) ? TRUE : navqueue_goto_line(old_doc, doc, line);
+					if (ret && focus_editor)
 						gtk_widget_grab_focus(GTK_WIDGET(doc->editor->sci));
 				}
 			}
@@ -1043,56 +1093,28 @@ gboolean msgwin_goto_messages_file_line(guint keyval)
 }
 
 
-/* Try to parse the file and line number for string and when something useful is
- * found, store the line number in *line and the relevant file with the error in
- * *filename.
- * *line will be -1 if no error was found in string.
- * *filename must be freed unless NULL. */
-static void msgwin_parse_grep_line(const gchar *string, gchar **filename, gint *line)
-{
-	ParseData data;
-
-	*filename = NULL;
-	*line = -1;
-
-	if (string == NULL)
-		return;
-
-	/* conflict:3:conflicting types for `foo' */
-	data.string = string;
-	data.pattern = ":";
-	data.min_fields = 3;
-	data.line_idx = 1;
-	data.file_idx = 0;
-
-	parse_file_line(&data, filename, line);
-
-	/* FIF dir should be set, but a plugin might not have set it */
-	if (msgwindow.messages_dir != NULL)
-		make_absolute(filename, msgwindow.messages_dir);
-}
-
-
 static gboolean on_msgwin_button_press_event(GtkWidget *widget, GdkEventButton *event,
 											 gpointer user_data)
 {
 	/* user_data might be NULL, GPOINTER_TO_INT returns 0 if called with NULL */
+	gboolean double_click = event->type == GDK_2BUTTON_PRESS;
 
-	if (event->button == 1)
+	if (event->button == 1 && (event->type == GDK_BUTTON_RELEASE || double_click))
 	{
 		switch (GPOINTER_TO_INT(user_data))
 		{
 			case MSG_COMPILER:
-			{	/* single click in the compiler treeview */
-				msgwin_goto_compiler_file_line(0);
+			{	/* mouse click in the compiler treeview */
+				msgwin_goto_compiler_file_line(double_click);
 				break;
 			}
 			case MSG_MESSAGE:
-			{	/* single click in the message treeview (results of 'Find usage') */
-				msgwin_goto_messages_file_line(0);
+			{	/* mouse click in the message treeview (results of 'Find usage') */
+				msgwin_goto_messages_file_line(double_click);
 				break;
 			}
 		}
+		return double_click;	/* TRUE prevents message window re-focusing */
 	}
 
 	if (event->button == 3)
