@@ -1,8 +1,8 @@
 /*
  *      plugindata.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2007-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2007-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2007-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2007-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: plugindata.h 5132 2010-08-11 12:50:38Z ntrel $
+ * $Id: plugindata.h 5859 2011-06-18 14:54:50Z colombanw $
  */
 
 /**
@@ -43,21 +43,26 @@
 #include "editor.h"	/* GeanyIndentType */
 
 
-/* Note: We use enum instead of 'static const gint' to allow its use in global variable
- * initializing, otherwise we get errors like:
- * error: initializer element is not constant */
-/** Versioning data */
-enum {
-	/** The Application Programming Interface (API) version, incremented
-	 * whenever any plugin data types are modified or appended to. */
-	GEANY_API_VERSION = 189,
+/** The Application Programming Interface (API) version, incremented
+ * whenever any plugin data types are modified or appended to.
+ *
+ * You can protect code that needs a higher API than e.g. 200 with:
+ * @code #if GEANY_API_VERSION >= 200
+ * 	some_newer_function();
+ * #endif @endcode
+ *
+ * @warning You should not test for values below 200 as previously
+ * @c GEANY_API_VERSION was defined as an enum value, not a macro.
+ */
+#define GEANY_API_VERSION 211
+
+/** The Application Binary Interface (ABI) version, incremented whenever
+ * existing fields in the plugin data types have to be changed or reordered.
+ * Changing this forces all plugins to be recompiled before Geany can load them. */
+/* This should usually stay the same if fields are only appended, assuming only pointers to
+ * structs and not structs themselves are declared by plugins. */
+#define GEANY_ABI_VERSION 68
 
-	/** The Application Binary Interface (ABI) version, incremented whenever
-	 * existing fields in the plugin data types have to be changed or reordered. */
-	/* This should usually stay the same if fields are only appended, assuming only pointers to
-	 * structs and not structs themselves are declared by plugins. */
-	GEANY_ABI_VERSION = 66
-};
 
 /** Defines a function to check the plugin is safe to load.
  * This performs runtime checks that try to ensure:
@@ -113,7 +118,7 @@ GeanyPlugin;
  * extra PluginInfo features (such as an icon), so we don't need to break API
  * compatibility. Alternatively just add a new macro, PLUGIN_SET_INFO_FULL(). -ntrel */
 #define PLUGIN_SET_INFO(p_name, p_description, p_version, p_author) \
-	void plugin_set_info(PluginInfo* info) \
+	void plugin_set_info(PluginInfo *info) \
 	{ \
 		info->name = (p_name); \
 		info->description = (p_description); \
@@ -131,15 +136,25 @@ GeanyPlugin;
  *
  * @since 0.19 */
 #define PLUGIN_SET_TRANSLATABLE_INFO(localedir, package, p_name, p_description, p_version, p_author) \
-	void plugin_set_info(PluginInfo* info) \
+	void plugin_set_info(PluginInfo *info) \
 	{ \
-		main_locale_init(localedir, package); \
+		main_locale_init((localedir), (package)); \
 		info->name = (p_name); \
 		info->description = (p_description); \
 		info->version = (p_version); \
 		info->author = (p_author); \
 	}
 
+
+#ifndef GEANY_PRIVATE
+
+/* Prototypes for building plugins with -Wmissing-prototypes */
+gint plugin_version_check(gint abi_ver);
+void plugin_set_info(PluginInfo *info);
+
+#endif
+
+
 /** @deprecated - use plugin_set_key_group() instead.
  * @see PLUGIN_KEY_GROUP() macro. */
 typedef struct GeanyKeyGroupInfo
@@ -173,7 +188,7 @@ GeanyKeyGroupInfo;
 typedef struct PluginCallback
 {
 	/** The name of signal, must be an existing signal. For a list of available signals,
-	 *  please see the @link signals Signal documentation @endlink. */
+	 *  please see the @link pluginsignals.c Signal documentation @endlink. */
 	const gchar	*signal_name;
 	/** A callback function which is called when the signal is emitted. */
 	GCallback	callback;
@@ -295,6 +310,9 @@ typedef struct DocumentFuncs
 	const GdkColor*	(*document_get_status_color) (struct GeanyDocument *doc);
 	gchar*		(*document_get_basename_for_display) (struct GeanyDocument *doc, gint length);
 	gint		(*document_get_notebook_page) (struct GeanyDocument *doc);
+	gint		(*document_compare_by_display_name) (gconstpointer a, gconstpointer b);
+	gint		(*document_compare_by_tab_order) (gconstpointer a, gconstpointer b);
+	gint		(*document_compare_by_tab_order_reverse) (gconstpointer a, gconstpointer b);
 }
 DocumentFuncs;
 
@@ -305,7 +323,7 @@ struct _ScintillaObject;
 typedef struct ScintillaFuncs
 {
 	/** Send Scintilla a message. */
-	long int	(*scintilla_send_message) (struct _ScintillaObject* sci, unsigned int iMessage,
+	long int	(*scintilla_send_message) (struct _ScintillaObject *sci, unsigned int iMessage,
 			long unsigned int wParam, long int lParam);
 	/** Create a new ScintillaObject widget. */
 	GtkWidget*	(*scintilla_new)(void);
@@ -373,6 +391,7 @@ typedef struct SciFuncs
 	gint	(*sci_find_text) (struct _ScintillaObject *sci, gint flags, struct Sci_TextToFind *ttf);
 	void	(*sci_set_line_indentation) (struct _ScintillaObject *sci, gint line, gint indent);
 	gint	(*sci_get_line_indentation) (struct _ScintillaObject *sci, gint line);
+	gint	(*sci_get_lexer) (struct _ScintillaObject *sci);
 }
 SciFuncs;
 
@@ -419,7 +438,8 @@ typedef struct UtilsFuncs
 	GSList*		(*utils_get_file_list_full)(const gchar *path, gboolean full_path, gboolean sort,
 				GError **error);
 	gchar**		(*utils_copy_environment)(const gchar **exclude_vars, const gchar *first_varname, ...);
-
+	gchar*		(*utils_find_open_xml_tag) (const gchar sel[], gint size);
+	const gchar*	(*utils_find_open_xml_tag_pos) (const gchar sel[], gint size);
 }
 UtilsFuncs;
 
@@ -455,6 +475,10 @@ typedef struct UIUtilsFuncs
 	void		(*ui_widget_modify_font_from_string) (GtkWidget *widget, const gchar *str);
 	gboolean	(*ui_is_keyval_enter_or_return) (guint keyval);
 	gint		(*ui_get_gtk_settings_integer) (const gchar *property_name, gint default_value);
+	void		(*ui_combo_box_add_to_history) (GtkComboBoxEntry *combo_entry,
+				const gchar *text, gint history_len);
+	void		(*ui_menu_add_document_items_sorted) (GtkMenu *menu, struct GeanyDocument *active,
+				GCallback callback, GCompareFunc compare_func);
 }
 UIUtilsFuncs;
 
@@ -467,6 +491,8 @@ typedef struct DialogFuncs
 	gboolean	(*dialogs_show_save_as) (void);
 	gboolean	(*dialogs_show_input_numeric) (const gchar *title, const gchar *label_text,
 				 gdouble *value, gdouble min, gdouble max, gdouble step);
+	gchar*		(*dialogs_show_input)(const gchar *title, GtkWindow *parent, const gchar *label_text,
+				const gchar *default_text);
 }
 DialogFuncs;
 
@@ -489,6 +515,7 @@ typedef struct MsgWinFuncs
 				 const gchar *format, ...) G_GNUC_PRINTF (4, 5);
 	void		(*msgwin_clear_tab) (gint tabnum);
 	void		(*msgwin_switch_tab) (gint tabnum, gboolean show);
+	void		(*msgwin_set_messages_dir) (const gchar *messages_dir);
 }
 MsgWinFuncs;
 
@@ -526,6 +553,9 @@ typedef struct HighlightingFuncs
 {
 	const struct GeanyLexerStyle* (*highlighting_get_style) (gint ft_id, gint style_id);
 	void		(*highlighting_set_styles) (struct _ScintillaObject *sci, struct GeanyFiletype *ft);
+	gboolean	(*highlighting_is_string_style) (gint lexer, gint style);
+	gboolean	(*highlighting_is_comment_style) (gint lexer, gint style);
+	gboolean	(*highlighting_is_code_style) (gint lexer, gint style);
 }
 HighlightingFuncs;
 
@@ -536,6 +566,8 @@ typedef struct FiletypeFuncs
 	GeanyFiletype*	(*filetypes_detect_from_file) (const gchar *utf8_filename);
 	GeanyFiletype*	(*filetypes_lookup_by_name) (const gchar *name);
 	GeanyFiletype*	(*filetypes_index)(gint idx);
+	const gchar*	(*filetypes_get_display_name)(GeanyFiletype *ft);
+	const GSList*	(*filetypes_get_sorted_by_name)(void);
 	/* Remember to convert any filetype_id arguments to GeanyFiletype pointers in any
 	 * appended functions */
 }
@@ -595,6 +627,12 @@ typedef struct EditorFuncs
 	void	(*editor_insert_text_block) (struct GeanyEditor *editor, const gchar *text,
 			 gint insert_pos, gint cursor_index, gint newline_indent_size,
 			 gboolean replace_newlines);
+
+	gint	(*editor_get_eol_char_mode) (struct GeanyEditor *editor);
+	gboolean (*editor_goto_pos) (struct GeanyEditor *editor, gint pos, gboolean mark);
+
+	const gchar* (*editor_find_snippet) (struct GeanyEditor *editor, const gchar *snippet_name);
+	void	(*editor_insert_snippet) (struct GeanyEditor *editor, gint pos, const gchar *snippet);
 }
 EditorFuncs;
 
@@ -613,6 +651,11 @@ typedef struct PluginFuncs
 	struct GeanyKeyGroup* (*plugin_set_key_group)(GeanyPlugin *plugin,
 		const gchar *section_name, gsize count, _GeanyKeyGroupCallback callback);
 	void	(*plugin_show_configure)(GeanyPlugin *plugin);
+	guint	(*plugin_timeout_add) (GeanyPlugin *plugin, guint interval, GSourceFunc function,
+		gpointer data);
+	guint	(*plugin_timeout_add_seconds) (GeanyPlugin *plugin, guint interval,
+		GSourceFunc function, gpointer data);
+	guint	(*plugin_idle_add) (GeanyPlugin *plugin, GSourceFunc function, gpointer data);
 }
 PluginFuncs;
 
@@ -638,20 +681,20 @@ typedef struct StashFuncs
 	gint (*stash_group_save_to_file)(struct StashGroup *group, const gchar *filename,
 			GKeyFileFlags flags);
 	void (*stash_group_add_toggle_button)(struct StashGroup *group, gboolean *setting,
-			const gchar *key_name, gboolean default_value, gpointer widget_id);
+			const gchar *key_name, gboolean default_value, gconstpointer widget_id);
 	void (*stash_group_add_radio_buttons)(struct StashGroup *group, gint *setting,
 			const gchar *key_name, gint default_value,
-			gpointer widget_id, gint enum_id, ...) G_GNUC_NULL_TERMINATED;
+			gconstpointer widget_id, gint enum_id, ...) G_GNUC_NULL_TERMINATED;
 	void (*stash_group_add_spin_button_integer)(struct StashGroup *group, gint *setting,
-			const gchar *key_name, gint default_value, gpointer widget_id);
+			const gchar *key_name, gint default_value, gconstpointer widget_id);
 	void (*stash_group_add_combo_box)(struct StashGroup *group, gint *setting,
-			const gchar *key_name, gint default_value, gpointer widget_id);
+			const gchar *key_name, gint default_value, gconstpointer widget_id);
 	void (*stash_group_add_combo_box_entry)(struct StashGroup *group, gchar **setting,
-			const gchar *key_name, const gchar *default_value, gpointer widget_id);
+			const gchar *key_name, const gchar *default_value, gconstpointer widget_id);
 	void (*stash_group_add_entry)(struct StashGroup *group, gchar **setting,
-			const gchar *key_name, const gchar *default_value, gpointer widget_id);
+			const gchar *key_name, const gchar *default_value, gconstpointer widget_id);
 	void (*stash_group_add_widget_property)(struct StashGroup *group, gpointer setting,
-			const gchar *key_name, gpointer default_value, gpointer widget_id,
+			const gchar *key_name, gpointer default_value, gconstpointer widget_id,
 			const gchar *property_name, GType type);
 	void (*stash_group_display)(struct StashGroup *group, GtkWidget *owner);
 	void (*stash_group_update)(struct StashGroup *group, GtkWidget *owner);
