@@ -1,8 +1,8 @@
 /*
  *      plugins.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2007-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2007-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2007-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2007-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: plugins.c 5507 2011-01-04 12:41:22Z ntrel $
+ * $Id: plugins.c 5886 2011-08-06 18:34:14Z colombanw $
  */
 
 /* Code to manage, load and unload plugins. */
@@ -85,7 +85,10 @@ static PluginFuncs plugin_funcs = {
 	&plugin_module_make_resident,
 	&plugin_signal_connect,
 	&plugin_set_key_group,
-	&plugin_show_configure
+	&plugin_show_configure,
+	&plugin_timeout_add,
+	&plugin_timeout_add_seconds,
+	&plugin_idle_add
 };
 
 static DocumentFuncs doc_funcs = {
@@ -108,7 +111,10 @@ static DocumentFuncs doc_funcs = {
 	&document_rename_file,
 	&document_get_status_color,
 	&document_get_basename_for_display,
-	&document_get_notebook_page
+	&document_get_notebook_page,
+	&document_compare_by_display_name,
+	&document_compare_by_tab_order,
+	&document_compare_by_tab_order_reverse
 };
 
 static EditorFuncs editor_funcs = {
@@ -219,7 +225,8 @@ static UtilsFuncs utils_funcs = {
 	&utils_str_remove_chars,
 	&utils_get_file_list_full,
 	&utils_copy_environment,
-	&utils_find_open_xml_tag
+	&utils_find_open_xml_tag,
+	&utils_find_open_xml_tag_pos
 };
 
 static UIUtilsFuncs uiutils_funcs = {
@@ -240,7 +247,8 @@ static UIUtilsFuncs uiutils_funcs = {
 	&ui_widget_modify_font_from_string,
 	&ui_is_keyval_enter_or_return,
 	&ui_get_gtk_settings_integer,
-	&ui_combo_box_add_to_history
+	&ui_combo_box_add_to_history,
+	&ui_menu_add_document_items_sorted
 };
 
 static DialogFuncs dialog_funcs = {
@@ -305,7 +313,8 @@ static FiletypeFuncs filetype_funcs = {
 	&filetypes_detect_from_file,
 	&filetypes_lookup_by_name,
 	&filetypes_index,
-	&filetypes_get_display_name
+	&filetypes_get_display_name,
+	&filetypes_get_sorted_by_name
 };
 
 static NavQueueFuncs navqueue_funcs = {
@@ -397,7 +406,7 @@ geany_data_init(void)
 
 
 /* Prevent the same plugin filename being loaded more than once.
- * Note: g_module_name always returns the .so name, even when Plugin::filename is an .la file. */
+ * Note: g_module_name always returns the .so name, even when Plugin::filename is a .la file. */
 static gboolean
 plugin_loaded(GModule *module)
 {
@@ -554,6 +563,15 @@ static void read_key_group(Plugin *plugin)
 }
 
 
+static gint cmp_plugin_names(gconstpointer a, gconstpointer b)
+{
+	const Plugin *pa = a;
+	const Plugin *pb = b;
+
+	return strcmp(pa->info.name, pb->info.name);
+}
+
+
 static void
 plugin_init(Plugin *plugin)
 {
@@ -607,8 +625,10 @@ plugin_init(Plugin *plugin)
 	if (callbacks)
 		add_callbacks(plugin, callbacks);
 
-	/* remember which plugins are active */
-	active_plugin_list = g_list_append(active_plugin_list, plugin);
+	/* remember which plugins are active.
+	 * keep list sorted so tools menu items and plugin preference tabs are
+	 * sorted by plugin name */
+	active_plugin_list = g_list_insert_sorted(active_plugin_list, plugin, cmp_plugin_names);
 
 	geany_debug("Loaded:   %s (%s)", plugin->filename,
 		NVL(plugin->info.name, "<Unknown>"));
@@ -639,7 +659,13 @@ plugin_new(const gchar *fname, gboolean init_plugin, gboolean add_to_list)
 	{
 		geany_debug("Plugin \"%s\" already loaded.", fname);
 		if (add_to_list)
-			plugin_list = g_list_append(plugin_list, plugin);
+		{
+			/* do not add the to list twice */
+			if (g_list_find(plugin_list, plugin) != NULL)
+				return NULL;
+
+			plugin_list = g_list_prepend(plugin_list, plugin);
+		}
 		return plugin;
 	}
 
@@ -694,7 +720,7 @@ plugin_new(const gchar *fname, gboolean init_plugin, gboolean add_to_list)
 
 	/* read plugin name, etc. */
 	plugin_set_info(&plugin->info);
-	if (!NZV(plugin->info.name))
+	if (G_UNLIKELY(! NZV(plugin->info.name)))
 	{
 		geany_debug("No plugin name set in plugin_set_info() for \"%s\" - ignoring plugin!",
 			fname);
@@ -727,7 +753,7 @@ plugin_new(const gchar *fname, gboolean init_plugin, gboolean add_to_list)
 		plugin_init(plugin);
 
 	if (add_to_list)
-		plugin_list = g_list_append(plugin_list, plugin);
+		plugin_list = g_list_prepend(plugin_list, plugin);
 
 	return plugin;
 }
@@ -748,6 +774,22 @@ static void remove_callbacks(Plugin *plugin)
 }
 
 
+static void remove_sources(Plugin *plugin)
+{
+	GList *item;
+
+	item = plugin->sources;
+	while (item != NULL)
+	{
+		GList *next = item->next; /* cache the next pointer because current item will be freed */
+
+		g_source_destroy(item->data);
+		item = next;
+	}
+	/* don't free the list here, it is allocated inside each source's data */
+}
+
+
 static gboolean is_active_plugin(Plugin *plugin)
 {
 	return (g_list_find(active_plugin_list, plugin) != NULL);
@@ -764,6 +806,7 @@ plugin_cleanup(Plugin *plugin)
 		plugin->cleanup();
 
 	remove_callbacks(plugin);
+	remove_sources(plugin);
 
 	if (plugin->key_group)
 		keybindings_free_group(plugin->key_group);
@@ -814,7 +857,7 @@ load_active_plugins(void)
 		if (NZV(fname) && g_file_test(fname, G_FILE_TEST_EXISTS))
 		{
 			if (plugin_new(fname, TRUE, FALSE) == NULL)
-				failed_plugins_list = g_list_append(failed_plugins_list, g_strdup(fname));
+				failed_plugins_list = g_list_prepend(failed_plugins_list, g_strdup(fname));
 		}
 	}
 }
@@ -849,42 +892,65 @@ load_plugins_from_path(const gchar *path)
 }
 
 
-#ifdef G_OS_WIN32
-static gchar *get_plugin_path()
+static gchar *get_plugin_path(void)
 {
-	gchar *install_dir = win32_get_installation_dir();
+#ifdef G_OS_WIN32
 	gchar *path;
+	gchar *install_dir = win32_get_installation_dir();
 
 	path = g_strconcat(install_dir, "\\lib", NULL);
 	g_free(install_dir);
 
 	return path;
-}
+#else
+	return g_strconcat(GEANY_LIBDIR, G_DIR_SEPARATOR_S "geany", NULL);
 #endif
+}
+
+
+static gboolean validate_custom_plugin_path(const gchar *plugin_path_custom,
+											const gchar *plugin_path_config,
+											const gchar *plugin_path_system)
+{
+	/* check whether the custom plugin path is one of the system or user plugin paths
+	 * and abort if so */
+	if (utils_str_equal(plugin_path_custom, plugin_path_config) ||
+		utils_str_equal(plugin_path_custom, plugin_path_system))
+		return FALSE;
+
+	return TRUE;
+}
 
 
 /* Load (but don't initialize) all plugins for the Plugin Manager dialog */
 static void load_all_plugins(void)
 {
-	gchar *path;
+	gchar *plugin_path_config;
+	gchar *plugin_path_system;
+
+	plugin_path_config = g_strconcat(app->configdir, G_DIR_SEPARATOR_S, "plugins", NULL);
+	plugin_path_system = get_plugin_path();
 
-	path = g_strconcat(app->configdir, G_DIR_SEPARATOR_S, "plugins", NULL);
 	/* first load plugins in ~/.config/geany/plugins/ */
-	load_plugins_from_path(path);
-	g_free(path);
+	load_plugins_from_path(plugin_path_config);
 
 	/* load plugins from a custom path */
 	if (NZV(prefs.custom_plugin_path))
-		load_plugins_from_path(prefs.custom_plugin_path);
+	{
+		gchar *plugin_path_custom = utils_get_locale_from_utf8(prefs.custom_plugin_path);
+		utils_tidy_path(plugin_path_custom);
+
+		if (validate_custom_plugin_path(plugin_path_custom, plugin_path_config, plugin_path_system))
+			load_plugins_from_path(plugin_path_custom);
+
+		g_free(plugin_path_custom);
+	}
 
 	/* finally load plugins from $prefix/lib/geany */
-#ifdef G_OS_WIN32
-	path = get_plugin_path();
-#else
-	path = g_strconcat(GEANY_LIBDIR, G_DIR_SEPARATOR_S "geany", NULL);
-#endif
-	load_plugins_from_path(path);
-	g_free(path);
+	load_plugins_from_path(plugin_path_system);
+
+	g_free(plugin_path_config);
+	g_free(plugin_path_system);
 }
 
 
@@ -938,15 +1004,6 @@ void plugins_load_active(void)
 }
 
 
-static gint cmp_plugin_names(gconstpointer a, gconstpointer b)
-{
-	const Plugin *pa = a;
-	const Plugin *pb = b;
-
-	return strcmp(pa->info.name, pb->info.name);
-}
-
-
 static void update_active_plugins_pref(void)
 {
 	gint i = 0;
@@ -961,10 +1018,6 @@ static void update_active_plugins_pref(void)
 		return;
 	}
 
-	/* sort the list so next time tools menu items are sorted by plugin name
-	 * (not ideal to do here, but better than nothing) */
-	active_plugin_list = g_list_sort(active_plugin_list, cmp_plugin_names);
-
 	active_plugins_pref = g_new0(gchar*, count + 1);
 
 	for (list = g_list_first(active_plugin_list); list != NULL; list = list->next)
@@ -1065,7 +1118,9 @@ typedef struct
 	GtkWidget *dialog;
 	GtkWidget *tree;
 	GtkListStore *store;
+	GtkWidget *plugin_label;
 	GtkWidget *description_label;
+	GtkWidget *author_label;
 	GtkWidget *configure_button;
 	GtkWidget *help_button;
 }
@@ -1101,11 +1156,11 @@ static void pm_selection_changed(GtkTreeSelection *selection, gpointer user_data
 			PluginInfo *pi;
 
 			pi = &p->info;
-			text = g_strdup_printf(
-				_("Plugin: %s %s\nDescription: %s\nAuthor(s): %s"),
-				pi->name, pi->version, pi->description, pi->author);
-
-			geany_wrap_label_set_text(GTK_LABEL(pm_widgets.description_label), text);
+			/* Translators: <plugin name> <plugin version> */
+			text = g_strdup_printf(_("%s %s"), pi->name, pi->version);
+			gtk_label_set_text(GTK_LABEL(pm_widgets.plugin_label), text);
+			gtk_label_set_text(GTK_LABEL(pm_widgets.description_label), pi->description);
+			gtk_label_set_text(GTK_LABEL(pm_widgets.author_label), pi->author);
 			g_free(text);
 
 			pm_update_buttons(p);
@@ -1177,19 +1232,19 @@ static void pm_prepare_treeview(GtkWidget *tree, GtkListStore *store)
 	GtkTreeSelection *sel;
 
 	checkbox_renderer = gtk_cell_renderer_toggle_new();
-    column = gtk_tree_view_column_new_with_attributes(
+	column = gtk_tree_view_column_new_with_attributes(
 		_("Active"), checkbox_renderer, "active", PLUGIN_COLUMN_CHECK, NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);
 	g_signal_connect(checkbox_renderer, "toggled", G_CALLBACK(pm_plugin_toggled), NULL);
 
 	text_renderer = gtk_cell_renderer_text_new();
-    column = gtk_tree_view_column_new_with_attributes(
+	column = gtk_tree_view_column_new_with_attributes(
 		_("Plugin"), text_renderer, "text", PLUGIN_COLUMN_NAME, NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);
 
 	text_renderer = gtk_cell_renderer_text_new();
 	g_object_set(text_renderer, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
-    column = gtk_tree_view_column_new_with_attributes(
+	column = gtk_tree_view_column_new_with_attributes(
 		_("File"), text_renderer, "text", PLUGIN_COLUMN_FILE, NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);
 
@@ -1281,9 +1336,25 @@ static void pm_dialog_response(GtkDialog *dialog, gint response, gpointer user_d
 }
 
 
+static GtkWidget *create_table_label(const gchar *text)
+{
+	GtkWidget *label;
+	PangoAttrList *attrs;
+
+	attrs = pango_attr_list_new();
+	pango_attr_list_insert(attrs, pango_attr_weight_new(PANGO_WEIGHT_BOLD));
+	label = gtk_label_new(text);
+	gtk_label_set_attributes(GTK_LABEL(label), attrs);
+	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.0);
+	pango_attr_list_unref(attrs);
+
+	return label;
+}
+
+
 static void pm_show_dialog(GtkMenuItem *menuitem, gpointer user_data)
 {
-	GtkWidget *vbox, *vbox2, *label_vbox, *hbox, *swin, *label, *label2, *desc_win;
+	GtkWidget *vbox, *vbox2, *label_vbox, *hbox, *swin, *label, *label2, *desc_win, *table, *paned;
 
 	/* before showing the dialog, we need to create the list of available plugins */
 	load_all_plugins();
@@ -1324,12 +1395,22 @@ static void pm_show_dialog(GtkMenuItem *menuitem, gpointer user_data)
 	gtk_label_set_use_markup(GTK_LABEL(label2), TRUE);
 	gtk_misc_set_alignment(GTK_MISC(label2), 0, 0.5);
 
-	pm_widgets.description_label = geany_wrap_label_new("");
+	table = gtk_table_new(3, 2, FALSE);
+	gtk_table_set_col_spacings(GTK_TABLE(table), 6);
+	pm_widgets.plugin_label = geany_wrap_label_new(NULL);
+	pm_widgets.description_label = geany_wrap_label_new(NULL);
+	pm_widgets.author_label = geany_wrap_label_new(NULL);
+	gtk_table_attach(GTK_TABLE(table), create_table_label(_("Plugin:")), 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
+	gtk_table_attach(GTK_TABLE(table), create_table_label(_("Description:")), 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
+	gtk_table_attach(GTK_TABLE(table), create_table_label(_("Author(s):")), 0, 1, 2, 3, GTK_FILL, GTK_FILL, 0, 0);
+	gtk_table_attach(GTK_TABLE(table), pm_widgets.plugin_label, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
+	gtk_table_attach(GTK_TABLE(table), pm_widgets.description_label, 1, 2, 1, 2, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
+	gtk_table_attach(GTK_TABLE(table), pm_widgets.author_label, 1, 2, 2, 3, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
+
 	desc_win = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(desc_win),
 		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(desc_win),
-		pm_widgets.description_label);
+	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(desc_win), table);
 
 	hbox = gtk_hbox_new(FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(hbox), label2, TRUE, TRUE, 0);
@@ -1338,12 +1419,15 @@ static void pm_show_dialog(GtkMenuItem *menuitem, gpointer user_data)
 
 	label_vbox = gtk_vbox_new(FALSE, 3);
 	gtk_box_pack_start(GTK_BOX(label_vbox), hbox, FALSE, FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(label_vbox), desc_win, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(label_vbox), desc_win, TRUE, TRUE, 0);
+
+	paned = gtk_vpaned_new();
+	gtk_paned_pack1(GTK_PANED(paned), swin, TRUE, FALSE);
+	gtk_paned_pack2(GTK_PANED(paned), label_vbox, FALSE, FALSE);
 
 	vbox2 = gtk_vbox_new(FALSE, 3);
 	gtk_box_pack_start(GTK_BOX(vbox2), label, FALSE, FALSE, 5);
-	gtk_box_pack_start(GTK_BOX(vbox2), swin, TRUE, TRUE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox2), label_vbox, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox2), paned, TRUE, TRUE, 0);
 
 	g_signal_connect(pm_widgets.dialog, "response", G_CALLBACK(pm_dialog_response), NULL);
 
