@@ -55,8 +55,8 @@ static char *br_last_value = (char*)NULL;
 static void
 br_free_last_value ()
 {
-    if (br_last_value)
-        free (br_last_value);
+	if (br_last_value)
+		free (br_last_value);
 }
 
 
@@ -152,7 +152,7 @@ br_locate (void *symbol)
 		if (symbol >= (void *) start && symbol < (void *) end)
 		{
 			char *tmp;
-			size_t len;
+			gsize len;
 
 			/* Extract the filename; it is always an absolute path */
 			path = strchr (line, '/');
