@@ -1,8 +1,8 @@
 /*
  *      prefs.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: prefs.c 5432 2010-11-23 17:36:29Z ntrel $
+ * $Id: prefs.c 5943 2011-09-21 16:03:11Z ntrel $
  */
 
 /*
@@ -59,6 +59,7 @@
 #include "templates.h"
 #include "search.h"
 #include "toolbar.h"
+#include "tools.h"
 #include "stash.h"
 #include "keyfile.h"
 #include "filetypes.h"
@@ -73,12 +74,15 @@ GeanyPrefs prefs;
 GeanyToolPrefs tool_prefs;
 
 
+/* keybinding globals, should be put in a struct */
 static GtkTreeIter g_iter;
 static GtkTreeStore *store = NULL;
 static GtkTreeView *tree = NULL;
 static GtkWidget *dialog_label;
 static gboolean edited = FALSE;
 
+static GtkTreeView *various_treeview = NULL;
+
 static void kb_cell_edited_cb(GtkCellRendererText *cellrenderertext, gchar *path, gchar *new_text, gpointer user_data);
 static gboolean kb_grab_key_dialog_key_press_cb(GtkWidget *dialog, GdkEventKey *event, gpointer user_data);
 static void kb_grab_key_dialog_response_cb(GtkWidget *dialog, gint response, gpointer user_data);
@@ -92,6 +96,7 @@ static void on_open_encoding_toggled(GtkToggleButton *togglebutton, gpointer use
 static void on_sidebar_visible_toggled(GtkToggleButton *togglebutton, gpointer user_data);
 static void on_prefs_print_radio_button_toggled(GtkToggleButton *togglebutton, gpointer user_data);
 static void on_prefs_print_page_header_toggled(GtkToggleButton *togglebutton, gpointer user_data);
+static void open_preferences_help(void);
 
 
 typedef enum PrefCallbackAction
@@ -120,6 +125,16 @@ static void prefs_action(PrefCallbackAction action)
 				break;
 		}
 	}
+
+	switch (action)
+	{
+		case PREF_DISPLAY:
+			stash_tree_display(pref_groups, various_treeview);
+			break;
+		case PREF_UPDATE:
+			stash_tree_update(pref_groups, various_treeview);
+			break;
+	}
 }
 
 
@@ -127,7 +142,8 @@ enum
 {
 	KB_TREE_ACTION,
 	KB_TREE_SHORTCUT,
-	KB_TREE_INDEX
+	KB_TREE_INDEX,
+	KB_TREE_EDITABLE
 };
 
 
@@ -265,7 +281,7 @@ static void kb_init_tree(void)
 
 	tree = GTK_TREE_VIEW(ui_lookup_widget(ui_widgets.prefs_dialog, "treeview7"));
 
-	store = gtk_tree_store_new(3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT);
+	store = gtk_tree_store_new(4, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, G_TYPE_BOOLEAN);
 	gtk_tree_view_set_model(GTK_TREE_VIEW(tree), GTK_TREE_MODEL(store));
 	g_object_unref(store);
 
@@ -274,8 +290,8 @@ static void kb_init_tree(void)
 	gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);
 
 	renderer = gtk_cell_renderer_text_new();
-	g_object_set(renderer, "editable", TRUE, NULL);
-	column = gtk_tree_view_column_new_with_attributes(_("Shortcut"), renderer, "text", KB_TREE_SHORTCUT, NULL);
+	column = gtk_tree_view_column_new_with_attributes(_("Shortcut"), renderer,
+		"text", KB_TREE_SHORTCUT, "editable", KB_TREE_EDITABLE, NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);
 
 	/* set policy settings for the scrolled window around the treeview again, because glade
@@ -315,7 +331,7 @@ static void kb_init(void)
 			key_string = gtk_accelerator_name(kb->key, kb->mods);
 			gtk_tree_store_append(store, &iter, &parent);
 			gtk_tree_store_set(store, &iter, KB_TREE_ACTION, label,
-				KB_TREE_SHORTCUT, key_string, KB_TREE_INDEX, kb->id, -1);
+				KB_TREE_SHORTCUT, key_string, KB_TREE_EDITABLE, TRUE, KB_TREE_INDEX, kb->id, -1);
 			g_free(key_string);
 			g_free(label);
 		}
@@ -605,8 +621,8 @@ static void prefs_init_dialog(void)
 
 	/* Tools Settings */
 
-    if (tool_prefs.term_cmd)
-            gtk_entry_set_text(GTK_ENTRY(ui_lookup_widget(ui_widgets.prefs_dialog, "entry_com_term")), tool_prefs.term_cmd);
+	if (tool_prefs.term_cmd)
+		gtk_entry_set_text(GTK_ENTRY(ui_lookup_widget(ui_widgets.prefs_dialog, "entry_com_term")), tool_prefs.term_cmd);
 
 	if (tool_prefs.browser_cmd)
 		gtk_entry_set_text(GTK_ENTRY(ui_lookup_widget(ui_widgets.prefs_dialog, "entry_browser")), tool_prefs.browser_cmd);
@@ -652,13 +668,6 @@ static void prefs_init_dialog(void)
 		else
 			widget = ui_lookup_widget(ui_widgets.prefs_dialog, "radio_print_external");
 
-#if GTK_CHECK_VERSION(2, 10, 0)
-		if (gtk_check_version(2, 10, 0) != NULL)
-#endif
-		{
-			gtk_widget_set_sensitive(widget_gtk, FALSE); /* disable the whole option block */
-			widget = ui_lookup_widget(ui_widgets.prefs_dialog, "radio_print_external");
-		}
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), TRUE);
 
 		on_prefs_print_radio_button_toggled(GTK_TOGGLE_BUTTON(widget_gtk), NULL);
@@ -802,6 +811,7 @@ on_prefs_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 		guint autoclose_brackets[5];
 		gboolean old_invert_all = interface_prefs.highlighting_invert_all;
 		gboolean old_sidebar_pos = interface_prefs.sidebar_pos;
+		GeanyDocument *doc = document_get_current();
 
 		/* Synchronize Stash settings */
 		prefs_action(PREF_UPDATE);
@@ -1120,6 +1130,7 @@ on_prefs_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 		if (edited)
 		{
 			kb_update();
+			tools_create_insert_custom_command_menu_items();
 			keybindings_write_to_file();
 		}
 
@@ -1217,11 +1228,21 @@ on_prefs_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 		ui_document_show_hide(NULL);
 		ui_update_view_editor_menu_items();
 
+		/* various preferences */
+		doc = document_get_current();
+		ui_save_buttons_toggle((doc != NULL) ? doc->changed : FALSE);
+		msgwin_show_hide_tabs();
+		ui_update_statusbar(doc, -1);
+
 		/* store all settings */
 		configuration_save();
 	}
 
-	if (response != GTK_RESPONSE_APPLY)
+	if (response == GTK_RESPONSE_HELP)
+	{
+		open_preferences_help();
+	}
+	else if (response != GTK_RESPONSE_APPLY)
 	{
 		gtk_tree_store_clear(store);
 		gtk_widget_hide(GTK_WIDGET(dialog));
@@ -1342,7 +1363,7 @@ static gboolean kb_grab_key_dialog_key_press_cb(GtkWidget *dialog, GdkEventKey *
 	gchar *str;
 	gint state;
 
-    state = event->state & gtk_accelerator_get_default_mod_mask();
+	state = event->state & gtk_accelerator_get_default_mod_mask();
 
 	if (event->keyval == GDK_Escape)
 		return FALSE;	/* close the dialog, don't allow escape when detecting keybindings. */
@@ -1566,6 +1587,8 @@ static void open_preferences_help(void)
 		suffix = "#keybinding-preferences";
 	else if (utils_str_equal(label, _("Printing")))
 		suffix = "#printing-preferences";
+	else if (utils_str_equal(label, _("Various")))
+		suffix = "#various-preferences";
 	else if (utils_str_equal(label, _("Terminal")))
 		suffix = "#terminal-vte-preferences";
 
@@ -1589,18 +1612,6 @@ static gboolean prefs_dialog_key_press_response_cb(GtkWidget *dialog, GdkEventKe
 }
 
 
-static gboolean prefs_dialog_button_press_event_cb(GtkWidget *dialog, GdkEventButton *event,
-												   gpointer data)
-{
-	if (event->button == 1)
-	{
-		open_preferences_help();
-		return TRUE;
-	}
-	return FALSE;
-}
-
-
 void prefs_show_dialog(void)
 {
 	if (ui_widgets.prefs_dialog == NULL)
@@ -1648,22 +1659,11 @@ void prefs_show_dialog(void)
 		gtk_widget_show(label);
 		gtk_box_pack_start(GTK_BOX(ui_lookup_widget(ui_widgets.prefs_dialog, "vbox31")),
 			label, FALSE, TRUE, 5);
-		label = geany_wrap_label_new(_("<i>Notice: For all changes you make here to take effect, you need to restart Geany or force the reload of the settings using Tools->Reload Configuration.</i>"));
-		gtk_widget_show(label);
-		gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
-		gtk_box_pack_start(GTK_BOX(ui_lookup_widget(ui_widgets.prefs_dialog, "vbox9")),
-			label, FALSE, TRUE, 5);
 		/* page Keybindings */
 		label = geany_wrap_label_new(_("Here you can change keyboard shortcuts for various actions. Select one and press the Change button to enter a new shortcut, or double click on an action to edit the string representation of the shortcut directly."));
 		gtk_widget_show(label);
 		gtk_box_pack_start(GTK_BOX(ui_lookup_widget(ui_widgets.prefs_dialog, "vbox32")),
 			label, FALSE, TRUE, 5);
-		/* page Printing */
-		label = geany_wrap_label_new(_("<i>Notice: Native GTK printing is only available if Geany was built against GTK 2.10 (or above) <b>and</b> Geany is running with GTK 2.10 (or above).</i>"));
-		gtk_widget_show(label);
-		gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
-		gtk_box_pack_start(GTK_BOX(ui_lookup_widget(ui_widgets.prefs_dialog, "vbox27")),
-			label, FALSE, TRUE, 5);
 		/* page Editor->Indentation */
 		label = geany_wrap_label_new(_("<i>Warning: these settings are overridden by the current project. See <b>Project->Properties</b>.</i>"));
 		gtk_widget_show(label);
@@ -1700,6 +1700,11 @@ void prefs_show_dialog(void)
 				ui_entry_add_clear_icon(GTK_ENTRY(ui_lookup_widget(ui_widgets.prefs_dialog, *name)));
 		}
 
+		/* page Various */
+		various_treeview = GTK_TREE_VIEW(ui_lookup_widget(ui_widgets.prefs_dialog,
+			"various_treeview"));
+		stash_tree_setup(pref_groups, various_treeview);
+
 #ifdef HAVE_VTE
 		vte_append_preferences_tab();
 #endif
@@ -1769,8 +1774,6 @@ void prefs_show_dialog(void)
 		g_signal_connect(ui_lookup_widget(ui_widgets.prefs_dialog, "check_sidebar_visible"),
 				"toggled", G_CALLBACK(on_sidebar_visible_toggled), NULL);
 
-		g_signal_connect(ui_lookup_widget(ui_widgets.prefs_dialog, "button_help"),
-				"button-press-event", G_CALLBACK(prefs_dialog_button_press_event_cb), NULL);
 		g_signal_connect(ui_widgets.prefs_dialog,
 				"key-press-event", G_CALLBACK(prefs_dialog_key_press_response_cb), NULL);
 	}
