@@ -1,8 +1,8 @@
 /*
  *      prefs.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -20,35 +20,35 @@
  *
  */
 
-
 #ifndef GEANY_PREFS_H
 #define GEANY_PREFS_H 1
 
-/* General Preferences dialog settings. */
+/** General Preferences dialog settings. */
 typedef struct GeanyPrefs
 {
 	gboolean		load_session;
 	gboolean		load_plugins;
 	gboolean		save_winpos;
 	gboolean		confirm_exit;
-	gboolean		beep_on_errors;
+	gboolean		beep_on_errors;		/* use utils_beep() instead */
 	gboolean		suppress_status_messages;
 	gboolean		switch_to_status;
 	gboolean		auto_focus;
-	gchar			*default_open_path;
+	gchar			*default_open_path;	/**< Default path to look for files when no other path is appropriate. */
+	gchar			*custom_plugin_path;
 }
 GeanyPrefs;
 
 extern GeanyPrefs prefs;
 
 
+/** Tools preferences */
 typedef struct GeanyToolPrefs
 {
-	gchar			*browser_cmd;
-	gchar			*make_cmd;
-	gchar			*term_cmd;
-	gchar			*grep_cmd;
-	gchar			*context_action_cmd;
+	gchar			*browser_cmd;			/**< web browser command */
+	gchar			*term_cmd;				/**< terminal emulator command */
+	gchar			*grep_cmd;				/**< grep command */
+	gchar			*context_action_cmd;	/**< context action command */
 }
 GeanyToolPrefs;
 
@@ -57,10 +57,4 @@ extern GeanyToolPrefs tool_prefs;
 
 void prefs_show_dialog(void);
 
-void on_prefs_font_choosed(GtkFontButton *widget, gpointer user_data);
-
-void on_prefs_color_choosed(GtkColorButton *widget, gpointer user_data);
-
-void on_prefs_tools_button_clicked(GtkButton *button, gpointer user_data);
-
 #endif
