@@ -1,8 +1,8 @@
 /*
  *      project.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2007-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2007-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2007-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2007-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: project.c 5432 2010-11-23 17:36:29Z ntrel $
+ * $Id: project.c 5840 2011-06-13 18:41:50Z colombanw $
  */
 
 /** @file project.h
@@ -29,6 +29,7 @@
 
 #include <string.h>
 #include <unistd.h>
+#include <errno.h>
 
 #include "project.h"
 #include "projectprivate.h"
@@ -80,7 +81,7 @@ typedef struct _PropertyDialogElements
 } PropertyDialogElements;
 
 
-static gboolean update_config(const PropertyDialogElements *e);
+static gboolean update_config(const PropertyDialogElements *e, gboolean new_project);
 static void on_file_save_button_clicked(GtkButton *button, PropertyDialogElements *e);
 static gboolean load_config(const gchar *filename);
 static gboolean write_config(gboolean emit_signal);
@@ -166,7 +167,7 @@ void project_new(void)
 
 	e->base_path = gtk_entry_new();
 	ui_entry_add_clear_icon(GTK_ENTRY(e->base_path));
-	ui_widget_set_tooltip_text(e->base_path,
+	gtk_widget_set_tooltip_text(e->base_path,
 		_("Base directory of all files that make up the project. "
 		"This can be a new path, or an existing directory tree. "
 		"You can use paths relative to the project filename."));
@@ -189,13 +190,17 @@ void project_new(void)
 
 	while (gtk_dialog_run(GTK_DIALOG(e->dialog)) == GTK_RESPONSE_OK)
 	{
-		if (update_config(e))
+		if (update_config(e, TRUE))
 		{
-			write_config(TRUE);
-			ui_set_statusbar(TRUE, _("Project \"%s\" created."), app->project->name);
+			if (!write_config(TRUE))
+				SHOW_ERR(_("Project file could not be written"));
+			else
+			{
+				ui_set_statusbar(TRUE, _("Project \"%s\" created."), app->project->name);
 
-			ui_add_recent_project_file(app->project->file_name);
-			break;
+				ui_add_recent_project_file(app->project->file_name);
+				break;
+			}
 		}
 	}
 	gtk_widget_destroy(e->dialog);
@@ -212,6 +217,7 @@ gboolean project_load_file_with_session(const gchar *locale_file_name)
 			configuration_open_files();
 			/* open a new file if no other file was opened */
 			document_new_file_if_non_open();
+			ui_focus_current_document();
 		}
 		return TRUE;
 	}
@@ -342,7 +348,8 @@ void project_close(gboolean open_default)
 	ui_set_statusbar(TRUE, _("Project \"%s\" closed."), app->project->name);
 
 	/* use write_config() to save project session files */
-	write_config(FALSE);
+	if (!write_config(FALSE))
+		g_warning("Project file \"%s\" could not be written", app->project->file_name);
 
 	/* remove project filetypes build entries */
 	if (app->project->build_filetypes_list != NULL)
@@ -377,6 +384,7 @@ void project_close(gboolean open_default)
 			configuration_open_files();
 			/* open a new file if no other file was opened */
 			document_new_file_if_non_open();
+			ui_focus_current_document();
 		}
 	}
 	g_signal_emit_by_name(geany_object, "project-close");
@@ -460,7 +468,7 @@ static void create_properties_dialog(PropertyDialogElements *e)
 
 	e->base_path = gtk_entry_new();
 	ui_entry_add_clear_icon(GTK_ENTRY(e->base_path));
-	ui_widget_set_tooltip_text(e->base_path,
+	gtk_widget_set_tooltip_text(e->base_path,
 		_("Base directory of all files that make up the project. "
 		"This can be a new path, or an existing directory tree. "
 		"You can use paths relative to the project filename."));
@@ -481,24 +489,19 @@ static void create_properties_dialog(PropertyDialogElements *e)
 	g_signal_connect(ui_lookup_widget(e->dialog, "radio_long_line_custom"), "toggled",
 		G_CALLBACK(on_radio_long_line_custom_toggled), ui_lookup_widget(e->dialog, "spin_long_line"));
 
-#if 0
 	label = gtk_label_new(_("File patterns:"));
-	/* <small>Separate multiple patterns by a new line</small> */
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 6, 7,
+	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 4, 5,
 					(GtkAttachOptions) (GTK_FILL),
-					(GtkAttachOptions) (GTK_FILL), 0, 0);
+					(GtkAttachOptions) (0), 0, 0);
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
 
-	e->patterns = gtk_text_view_new();
-	swin = gtk_scrolled_window_new(NULL, NULL);
-	gtk_widget_set_size_request(swin, -1, 80);
-	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(swin),
-				GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(swin), GTK_WIDGET(e->patterns));
-	gtk_table_attach(GTK_TABLE(table), swin, 1, 2, 4, 5,
+	e->patterns = gtk_entry_new();
+	gtk_widget_set_tooltip_text(e->patterns,
+		_("Space separated list of file patterns used for the find in files dialog "
+		  "(e.g. *.c *.h)"));
+	gtk_table_attach(GTK_TABLE(table), e->patterns, 1, 2, 4, 5,
 					(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
 					(GtkAttachOptions) (0), 0, 0);
-#endif
 
 	label = gtk_label_new(_("Project"));
 	gtk_notebook_insert_page(GTK_NOTEBOOK(notebook), table, label, 0);
@@ -545,26 +548,14 @@ static void show_project_properties(gboolean show_build)
 		gtk_text_buffer_set_text(buffer, p->description, -1);
 	}
 
-#if 0
 	if (p->file_patterns != NULL)
 	{	/* set the file patterns */
-		gint i;
-		gint len = g_strv_length(p->file_patterns);
-		GtkTextBuffer *buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(e->patterns));
-		GString *str = g_string_sized_new(len * 4);
+		gchar *str;
 
-		for (i = 0; i < len; i++)
-		{
-			if (p->file_patterns[i] != NULL)
-			{
-				g_string_append(str, p->file_patterns[i]);
-				g_string_append_c(str, '\n');
-			}
-		}
-		gtk_text_buffer_set_text(buffer, str->str, -1);
-		g_string_free(str, TRUE);
+		str = g_strjoinv(" ", p->file_patterns);
+		gtk_entry_set_text(GTK_ENTRY(e->patterns), str);
+		g_free(str);
 	}
-#endif
 
 	g_signal_emit_by_name(geany_object, "project-dialog-create", e->notebook);
 	gtk_widget_show_all(e->dialog);
@@ -577,12 +568,16 @@ static void show_project_properties(gboolean show_build)
 
 	while (gtk_dialog_run(GTK_DIALOG(e->dialog)) == GTK_RESPONSE_OK)
 	{
-		if (update_config(e))
+		if (update_config(e, FALSE))
 		{
 			g_signal_emit_by_name(geany_object, "project-dialog-confirmed", e->notebook);
-			write_config(TRUE);
-			ui_set_statusbar(TRUE, _("Project \"%s\" saved."), app->project->name);
-			break;
+			if (!write_config(TRUE))
+				SHOW_ERR(_("Project file could not be written"));
+			else
+			{
+				ui_set_statusbar(TRUE, _("Project \"%s\" saved."), app->project->name);
+				break;
+			}
 		}
 	}
 	build_free_fields(e->build_properties);
@@ -634,6 +629,8 @@ static GeanyProject *create_project(void)
 	priv.indentation = &indentation;
 	project->priv = &priv;
 
+	project->file_patterns = NULL;
+
 	project->long_line_behaviour = 1 /* use global settings */;
 	project->long_line_column = editor_prefs.long_line_column;
 
@@ -644,13 +641,12 @@ static GeanyProject *create_project(void)
 
 /* Verifies data for New & Properties dialogs.
  * Returns: FALSE if the user needs to change any data. */
-static gboolean update_config(const PropertyDialogElements *e)
+static gboolean update_config(const PropertyDialogElements *e, gboolean new_project)
 {
 	const gchar *name, *file_name, *base_path;
 	gchar *locale_filename;
 	gint name_len;
 	gint err_code = 0;
-	gboolean new_project = FALSE;
 	GeanyProject *p;
 
 	g_return_val_if_fail(e != NULL, TRUE);
@@ -670,12 +666,12 @@ static gboolean update_config(const PropertyDialogElements *e)
 		return FALSE;
 	}
 
-	if (app->project == NULL)
+	if (new_project)
 		file_name = gtk_entry_get_text(GTK_ENTRY(e->file_name));
 	else
 		file_name = gtk_label_get_text(GTK_LABEL(e->file_name));
 
-	if (! NZV(file_name))
+	if (G_UNLIKELY(! NZV(file_name)))
 	{
 		SHOW_ERR(_("You have specified an invalid project filename."));
 		gtk_widget_grab_focus(e->file_name);
@@ -720,7 +716,8 @@ static gboolean update_config(const PropertyDialogElements *e)
 		g_free(locale_path);
 	}
 	/* finally test whether the given project file can be written */
-	if ((err_code = utils_is_file_writeable(locale_filename)) != 0)
+	if ((err_code = utils_is_file_writable(locale_filename)) != 0 ||
+		(err_code = g_file_test(locale_filename, G_FILE_TEST_IS_DIR) ? EISDIR : 0) != 0)
 	{
 		SHOW_ERR1(_("Project file could not be written (%s)."), g_strerror(err_code));
 		gtk_widget_grab_focus(e->file_name);
@@ -749,6 +746,8 @@ static gboolean update_config(const PropertyDialogElements *e)
 		GeanyBuildCommand *oldvalue;
 		GeanyFiletype *ft = doc ? doc->file_type : NULL;
 		GtkWidget *widget;
+		gchar *tmp;
+		GString *str;
 
 		/* get and set the project description */
 		buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(e->description));
@@ -788,16 +787,15 @@ static gboolean update_config(const PropertyDialogElements *e)
 		p->long_line_column = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
 		apply_editor_prefs();
 
-#if 0
 		/* get and set the project file patterns */
-		buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(e->patterns));
-		gtk_text_buffer_get_start_iter(buffer, &start);
-		gtk_text_buffer_get_end_iter(buffer, &end);
-		tmp = gtk_text_buffer_get_text(buffer, &start, &end, FALSE);
+		tmp = g_strdup(gtk_entry_get_text(GTK_ENTRY(e->patterns)));
 		g_strfreev(p->file_patterns);
-		p->file_patterns = g_strsplit(tmp, "\n", -1);
+		g_strstrip(tmp);
+		str = g_string_new(tmp);
+		do {} while (utils_string_replace_all(str, "  ", " "));
+		p->file_patterns = g_strsplit(str->str, " ", -1);
+		g_string_free(str, TRUE);
 		g_free(tmp);
-#endif
 	}
 
 	update_ui();
@@ -1010,6 +1008,7 @@ static gboolean load_config(const gchar *filename)
 		document_close_all();
 		/* read session files so they can be opened with configuration_open_files() */
 		configuration_load_session_files(config, FALSE);
+		ui_focus_current_document();
 	}
 	g_signal_emit_by_name(geany_object, "project-open", config);
 	g_key_file_free(config);
@@ -1199,7 +1198,9 @@ void project_init(void)
 	stash_group_add_integer(group, &indentation.hard_tab_width,
 		"indent_hard_tab_width", 8);
 	stash_group_add_toggle_button(group, &indentation.detect_type,
-		"detect_indent", FALSE, "check_detect_indent");
+		"detect_indent", FALSE, "check_detect_indent_type");
+	stash_group_add_toggle_button(group, &indentation.detect_width,
+		"detect_indent_width", FALSE, "check_detect_indent_width");
 	stash_group_add_combo_box(group, (gint*)(gpointer)&indentation.auto_indent_mode,
 		"indent_mode", GEANY_AUTOINDENT_CURRENTCHARS, "combo_auto_indent_mode");
 }
