@@ -1,8 +1,8 @@
 /*
  *      search.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2006-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: search.c 5299 2010-10-16 12:37:20Z eht16 $
+ * $Id: search.c 5840 2011-06-13 18:41:50Z colombanw $
  */
 
 /*
@@ -75,6 +75,14 @@ enum
 };
 
 
+enum
+{
+	FILES_MODE_ALL,
+	FILES_MODE_PROJECT,
+	FILES_MODE_CUSTOM
+};
+
+
 GeanySearchData search_data;
 GeanySearchPrefs search_prefs;
 
@@ -88,12 +96,27 @@ static struct
 	gboolean fif_recursive;
 	gboolean fif_use_extra_options;
 	gchar *fif_extra_options;
-	gboolean fif_use_files;
+	gint fif_files_mode;
 	gchar *fif_files;
+	gboolean find_regexp;
+	gboolean find_escape_sequences;
+	gboolean find_case_sensitive;
+	gboolean find_match_whole_word;
+	gboolean find_match_word_start;
+	gboolean find_close_dialog;
+	gboolean replace_regexp;
+	gboolean replace_escape_sequences;
+	gboolean replace_case_sensitive;
+	gboolean replace_match_whole_word;
+	gboolean replace_match_word_start;
+	gboolean replace_search_backwards;
+	gboolean replace_close_dialog;
 }
-settings = {FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, NULL, FALSE, NULL};
+settings;
 
 static StashGroup *fif_prefs = NULL;
+static StashGroup *find_prefs = NULL;
+static StashGroup *replace_prefs = NULL;
 
 
 static struct
@@ -122,9 +145,10 @@ static struct
 	GtkWidget	*files_combo;
 	GtkWidget	*search_combo;
 	GtkWidget	*encoding_combo;
+	GtkWidget	*files_mode_combo;
 	gint		position[2]; /* x, y */
 }
-fif_dlg = {NULL, NULL, NULL, NULL, NULL, {0, 0}};
+fif_dlg = {NULL, NULL, NULL, NULL, NULL, NULL, {0, 0}};
 
 
 static gboolean search_read_io(GIOChannel *source, GIOCondition condition, gpointer data);
@@ -144,6 +168,9 @@ on_find_dialog_response(GtkDialog *dialog, gint response, gpointer user_data);
 static void
 on_find_entry_activate(GtkEntry *entry, gpointer user_data);
 
+static void
+on_find_entry_activate_backward(GtkEntry *entry, gpointer user_data);
+
 static void
 on_replace_dialog_response(GtkDialog *dialog, gint response, gpointer user_data);
 
@@ -176,6 +203,8 @@ static void init_prefs(void)
 	stash_group_add_integer(group, &fif_dlg.position[0], "position_fif_x", -1);
 	stash_group_add_integer(group, &fif_dlg.position[1], "position_fif_y", -1);
 
+	memset(&settings, '\0', sizeof(settings));
+
 	group = stash_group_new("search");
 	fif_prefs = group;
 	configuration_add_pref_group(group, FALSE);
@@ -195,14 +224,49 @@ static void init_prefs(void)
 		"fif_use_extra_options", FALSE, "check_extra");
 	stash_group_add_entry(group, &settings.fif_files,
 		"fif_files", "", "entry_files");
-	stash_group_add_toggle_button(group, &settings.fif_use_files,
-		"fif_use_files", FALSE, "check_files");
+	stash_group_add_combo_box(group, &settings.fif_files_mode,
+		"fif_files_mode", FILES_MODE_ALL, "combo_files_mode");
+
+	group = stash_group_new("search");
+	find_prefs = group;
+	configuration_add_pref_group(group, FALSE);
+	stash_group_add_toggle_button(group, &settings.find_regexp,
+		"find_regexp", FALSE, "check_regexp");
+	stash_group_add_toggle_button(group, &settings.find_case_sensitive,
+		"find_case_sensitive", FALSE, "check_case");
+	stash_group_add_toggle_button(group, &settings.find_escape_sequences,
+		"find_escape_sequences", FALSE, "check_escape");
+	stash_group_add_toggle_button(group, &settings.find_match_whole_word,
+		"find_match_whole_word", FALSE, "check_word");
+	stash_group_add_toggle_button(group, &settings.find_match_word_start,
+		"find_match_word_start", FALSE, "check_wordstart");
+	stash_group_add_toggle_button(group, &settings.find_close_dialog,
+		"find_close_dialog", TRUE, "check_close");
+
+	group = stash_group_new("search");
+	replace_prefs = group;
+	configuration_add_pref_group(group, FALSE);
+	stash_group_add_toggle_button(group, &settings.replace_regexp,
+		"replace_regexp", FALSE, "check_regexp");
+	stash_group_add_toggle_button(group, &settings.replace_case_sensitive,
+		"replace_case_sensitive", FALSE, "check_case");
+	stash_group_add_toggle_button(group, &settings.replace_escape_sequences,
+		"replace_escape_sequences", FALSE, "check_escape");
+	stash_group_add_toggle_button(group, &settings.replace_match_whole_word,
+		"replace_match_whole_word", FALSE, "check_word");
+	stash_group_add_toggle_button(group, &settings.replace_match_word_start,
+		"replace_match_word_start", FALSE, "check_wordstart");
+	stash_group_add_toggle_button(group, &settings.replace_search_backwards,
+		"replace_search_backwards", FALSE, "check_back");
+	stash_group_add_toggle_button(group, &settings.replace_close_dialog,
+		"replace_close_dialog", TRUE, "check_close");
 }
 
 
 void search_init(void)
 {
 	search_data.text = NULL;
+	search_data.original_text = NULL;
 	init_prefs();
 }
 
@@ -216,6 +280,7 @@ void search_finalize(void)
 	FREE_WIDGET(replace_dlg.dialog);
 	FREE_WIDGET(fif_dlg.dialog);
 	g_free(search_data.text);
+	g_free(search_data.original_text);
 }
 
 
@@ -227,10 +292,10 @@ static GtkWidget *add_find_checkboxes(GtkDialog *dialog)
 	check_regexp = gtk_check_button_new_with_mnemonic(_("_Use regular expressions"));
 	ui_hookup_widget(dialog, check_regexp, "check_regexp");
 	gtk_button_set_focus_on_click(GTK_BUTTON(check_regexp), FALSE);
-	ui_widget_set_tooltip_text(check_regexp, _("Use POSIX-like regular expressions. "
+	gtk_widget_set_tooltip_text(check_regexp, _("Use POSIX-like regular expressions. "
 		"For detailed information about using regular expressions, please read the documentation."));
 	g_signal_connect(check_regexp, "toggled",
-		G_CALLBACK(on_find_replace_checkbutton_toggled), GTK_WIDGET(dialog));
+		G_CALLBACK(on_find_replace_checkbutton_toggled), dialog);
 
 	if (dialog != GTK_DIALOG(find_dlg.dialog))
 	{
@@ -250,7 +315,7 @@ static GtkWidget *add_find_checkboxes(GtkDialog *dialog)
 	checkbox7 = gtk_check_button_new_with_mnemonic(_("Use _escape sequences"));
 	ui_hookup_widget(dialog, checkbox7, "check_escape");
 	gtk_button_set_focus_on_click(GTK_BUTTON(checkbox7), FALSE);
-	ui_widget_set_tooltip_text(checkbox7,
+	gtk_widget_set_tooltip_text(checkbox7,
 		_("Replace \\\\, \\t, \\n, \\r and \\uXXXX (Unicode chararacters) with the "
 		  "corresponding control characters"));
 
@@ -295,7 +360,9 @@ static void send_find_dialog_response(GtkButton *button, gpointer user_data)
 static void setup_find_next(const gchar *text)
 {
 	g_free(search_data.text);
+	g_free(search_data.original_text);
 	search_data.text = g_strdup(text);
+	search_data.original_text = g_strdup(text);
 	search_data.flags = 0;
 	search_data.backwards = FALSE;
 	search_data.search_bar = FALSE;
@@ -342,7 +409,7 @@ void search_find_selection(GeanyDocument *doc, gboolean search_backwards)
 	{
 		setup_find_next(s);	/* allow find next/prev */
 
-		if (document_find_text(doc, s, 0, search_backwards, FALSE, NULL) > -1)
+		if (document_find_text(doc, s, NULL, 0, search_backwards, FALSE, NULL) > -1)
 			editor_display_current_line(doc->editor, 0.3F);
 		g_free(s);
 	}
@@ -424,6 +491,9 @@ static void create_find_dialog(void)
 
 	g_signal_connect(gtk_bin_get_child(GTK_BIN(entry)), "activate",
 			G_CALLBACK(on_find_entry_activate), NULL);
+	ui_entry_add_activate_backward_signal(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(entry))));
+	g_signal_connect(gtk_bin_get_child(GTK_BIN(entry)), "activate-backward",
+			G_CALLBACK(on_find_entry_activate_backward), NULL);
 	g_signal_connect(find_dlg.dialog, "response",
 			G_CALLBACK(on_find_dialog_response), entry);
 	g_signal_connect(find_dlg.dialog, "delete-event",
@@ -446,7 +516,7 @@ static void create_find_dialog(void)
 	bbox = gtk_hbutton_box_new();
 
 	button = gtk_button_new_with_mnemonic(_("_Mark"));
-	ui_widget_set_tooltip_text(button,
+	gtk_widget_set_tooltip_text(button,
 			_("Mark all matches in the current document"));
 	gtk_container_add(GTK_CONTAINER(bbox), button);
 	g_signal_connect(button, "clicked", G_CALLBACK(send_find_dialog_response),
@@ -466,9 +536,8 @@ static void create_find_dialog(void)
 	check_close = gtk_check_button_new_with_mnemonic(_("Close _dialog"));
 	ui_hookup_widget(find_dlg.dialog, check_close, "check_close");
 	gtk_button_set_focus_on_click(GTK_BUTTON(check_close), FALSE);
-	ui_widget_set_tooltip_text(check_close,
+	gtk_widget_set_tooltip_text(check_close,
 			_("Disable this option to keep the dialog open"));
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_close), TRUE);
 	gtk_container_add(GTK_CONTAINER(bbox), check_close);
 	gtk_button_box_set_child_secondary(GTK_BUTTON_BOX(bbox), check_close, TRUE);
 
@@ -499,6 +568,7 @@ void search_show_find_dialog(void)
 	if (find_dlg.dialog == NULL)
 	{
 		create_find_dialog();
+		stash_group_display(find_prefs, find_dlg.dialog);
 		if (sel)
 			gtk_entry_set_text(GTK_ENTRY(find_dlg.entry), sel);
 
@@ -640,7 +710,7 @@ static void create_replace_dialog(void)
 		GINT_TO_POINTER(GEANY_RESPONSE_REPLACE_IN_FILE));
 
 	button = gtk_button_new_with_mnemonic(_("In Se_lection"));
-	ui_widget_set_tooltip_text(button,
+	gtk_widget_set_tooltip_text(button,
 		_("Replace all matches found in the currently selected text"));
 	gtk_container_add(GTK_CONTAINER(bbox), button);
 	g_signal_connect(button, "clicked", G_CALLBACK(send_replace_dialog_response),
@@ -650,9 +720,8 @@ static void create_replace_dialog(void)
 	check_close = gtk_check_button_new_with_mnemonic(_("Close _dialog"));
 	ui_hookup_widget(replace_dlg.dialog, check_close, "check_close");
 	gtk_button_set_focus_on_click(GTK_BUTTON(check_close), FALSE);
-	ui_widget_set_tooltip_text(check_close,
+	gtk_widget_set_tooltip_text(check_close,
 			_("Disable this option to keep the dialog open"));
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_close), TRUE);
 	gtk_container_add(GTK_CONTAINER(bbox), check_close);
 	gtk_button_box_set_child_secondary(GTK_BUTTON_BOX(bbox), check_close, TRUE);
 
@@ -677,6 +746,7 @@ void search_show_replace_dialog(void)
 	if (replace_dlg.dialog == NULL)
 	{
 		create_replace_dialog();
+		stash_group_display(replace_prefs, replace_dlg.dialog);
 		if (sel)
 			gtk_entry_set_text(GTK_ENTRY(replace_dlg.find_entry), sel);
 
@@ -696,6 +766,7 @@ void search_show_replace_dialog(void)
 		/* bring the dialog back in the foreground in case it is already open but the focus is away */
 		gtk_window_present(GTK_WINDOW(replace_dlg.dialog));
 	}
+
 	g_free(sel);
 }
 
@@ -708,11 +779,92 @@ static void on_widget_toggled_set_sensitive(GtkToggleButton *togglebutton, gpoin
 }
 
 
+static void update_file_patterns(GtkWidget *mode_combo, GtkWidget *fcombo)
+{
+	gint selection;
+	GtkWidget *entry;
+
+	entry = gtk_bin_get_child(GTK_BIN(fcombo));
+
+	selection = gtk_combo_box_get_active(GTK_COMBO_BOX(mode_combo));
+
+	if (selection == FILES_MODE_ALL)
+	{
+		gtk_entry_set_text(GTK_ENTRY(entry), "");
+		gtk_widget_set_sensitive(fcombo, FALSE);
+	}
+	else if (selection == FILES_MODE_CUSTOM)
+	{
+		gtk_widget_set_sensitive(fcombo, TRUE);
+	}
+	else if (selection == FILES_MODE_PROJECT)
+	{
+		if (app->project && NZV(app->project->file_patterns))
+		{
+			gchar *patterns;
+
+			patterns = g_strjoinv(" ", app->project->file_patterns);
+			gtk_entry_set_text(GTK_ENTRY(entry), patterns);
+			g_free(patterns);
+		}
+		else
+		{
+			gtk_entry_set_text(GTK_ENTRY(entry), "");
+		}
+
+		gtk_widget_set_sensitive(fcombo, FALSE);
+	}
+}
+
+
+/* creates the combo to choose which files include in the search */
+static GtkWidget *create_fif_file_mode_combo(void)
+{
+	GtkWidget *combo;
+	GtkCellRenderer *renderer;
+	GtkListStore *store;
+	GtkTreeIter iter;
+
+	/* text/sensitive */
+	store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_BOOLEAN);
+	gtk_list_store_append(store, &iter);
+	gtk_list_store_set(store, &iter, 0, _("all"), 1, TRUE, -1);
+	gtk_list_store_append(store, &iter);
+	gtk_list_store_set(store, &iter, 0, _("project"), 1, app->project != NULL, -1);
+	gtk_list_store_append(store, &iter);
+	gtk_list_store_set(store, &iter, 0, _("custom"), 1, TRUE, -1);
+
+	combo = gtk_combo_box_new_with_model(GTK_TREE_MODEL(store));
+	g_object_unref(store);
+	gtk_widget_set_tooltip_text(combo, _("All: search all files in the directory\n"
+										"Project: use file patterns defined in the project settings\n"
+										"Custom: specify file patterns manually"));
+
+	renderer = gtk_cell_renderer_text_new();
+	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(combo), renderer, TRUE);
+	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(combo), renderer, "text", 0, "sensitive", 1, NULL);
+
+	return combo;
+}
+
+
+/* updates the sensitivity of the project combo item */
+static void update_fif_file_mode_combo(void)
+{
+	GtkTreeModel *model = gtk_combo_box_get_model(GTK_COMBO_BOX(fif_dlg.files_mode_combo));
+	GtkTreeIter iter;
+
+	/* "1" refers to the second list entry, project  */
+	if (gtk_tree_model_get_iter_from_string(model, &iter, "1"))
+		gtk_list_store_set(GTK_LIST_STORE(model), &iter, 1, app->project != NULL, -1);
+}
+
+
 static void create_fif_dialog(void)
 {
 	GtkWidget *dir_combo, *combo, *fcombo, *e_combo, *entry;
-	GtkWidget *label, *label1, *label2, *checkbox1, *checkbox2, *check_wholeword,
-		*check_recursive, *check_extra, *entry_extra, *check_regexp, *check;
+	GtkWidget *label, *label1, *label2, *label3, *checkbox1, *checkbox2, *check_wholeword,
+		*check_recursive, *check_extra, *entry_extra, *check_regexp, *combo_files_mode;
 	GtkWidget *dbox, *sbox, *lbox, *rbox, *hbox, *vbox, *ebox;
 	GtkSizeGroup *size_group;
 	gchar *encoding_string;
@@ -751,30 +903,28 @@ static void create_fif_dialog(void)
 	size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
 	gtk_size_group_add_widget(size_group, label);
 
-	check = gtk_check_button_new_with_mnemonic(_("Fi_les:"));
-	ui_hookup_widget(fif_dlg.dialog, check, "check_files");
-	gtk_button_set_focus_on_click(GTK_BUTTON(check), FALSE);
-	gtk_size_group_add_widget(size_group, check);
+	label3 = gtk_label_new_with_mnemonic(_("Fi_les:"));
+	gtk_misc_set_alignment(GTK_MISC(label3), 0, 0.5);
 
-	/* tab from search to the files checkbox */
-	g_signal_connect(entry, "key-press-event",
-		G_CALLBACK(on_widget_key_pressed_set_focus), check);
+	combo_files_mode = create_fif_file_mode_combo();
+	gtk_label_set_mnemonic_widget(GTK_LABEL(label3), combo_files_mode);
+	ui_hookup_widget(fif_dlg.dialog, combo_files_mode, "combo_files_mode");
+	fif_dlg.files_mode_combo = combo_files_mode;
 
 	fcombo = gtk_combo_box_entry_new_text();
-	gtk_widget_set_sensitive(fcombo, FALSE);
 	entry = gtk_bin_get_child(GTK_BIN(fcombo));
 	ui_entry_add_clear_icon(GTK_ENTRY(entry));
 	gtk_entry_set_activates_default(GTK_ENTRY(entry), TRUE);
-	ui_widget_set_tooltip_text(entry, _("File patterns, e.g. *.c *.h"));
+	gtk_widget_set_tooltip_text(entry, _("File patterns, e.g. *.c *.h"));
 	ui_hookup_widget(fif_dlg.dialog, entry, "entry_files");
 	fif_dlg.files_combo = fcombo;
 
-	/* enable entry when check is checked */
-	g_signal_connect(check, "toggled",
-		G_CALLBACK(on_widget_toggled_set_sensitive), fcombo);
+	/* update the entry when selection is changed */
+	g_signal_connect(combo_files_mode, "changed", G_CALLBACK(update_file_patterns), fcombo);
 
 	hbox = gtk_hbox_new(FALSE, 6);
-	gtk_box_pack_start(GTK_BOX(hbox), check, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), label3, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), combo_files_mode, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(hbox), fcombo, TRUE, TRUE, 0);
 
 	label1 = gtk_label_new_with_mnemonic(_("_Directory:"));
@@ -816,6 +966,7 @@ static void create_fif_dialog(void)
 
 	gtk_size_group_add_widget(size_group, label1);
 	gtk_size_group_add_widget(size_group, label2);
+	gtk_size_group_add_widget(size_group, label3);
 	g_object_unref(G_OBJECT(size_group));	/* auto destroy the size group */
 
 	gtk_box_pack_start(GTK_BOX(vbox), sbox, TRUE, FALSE, 0);
@@ -826,7 +977,7 @@ static void create_fif_dialog(void)
 	check_regexp = gtk_check_button_new_with_mnemonic(_("_Use regular expressions"));
 	ui_hookup_widget(fif_dlg.dialog, check_regexp, "check_regexp");
 	gtk_button_set_focus_on_click(GTK_BUTTON(check_regexp), FALSE);
-	ui_widget_set_tooltip_text(check_regexp, _("See grep's manual page for more information"));
+	gtk_widget_set_tooltip_text(check_regexp, _("See grep's manual page for more information"));
 
 	check_recursive = gtk_check_button_new_with_mnemonic(_("_Recurse in subfolders"));
 	ui_hookup_widget(fif_dlg.dialog, check_recursive, "check_recursive");
@@ -844,7 +995,7 @@ static void create_fif_dialog(void)
 	checkbox2 = gtk_check_button_new_with_mnemonic(_("_Invert search results"));
 	ui_hookup_widget(fif_dlg.dialog, checkbox2, "check_invert");
 	gtk_button_set_focus_on_click(GTK_BUTTON(checkbox2), FALSE);
-	ui_widget_set_tooltip_text(checkbox2,
+	gtk_widget_set_tooltip_text(checkbox2,
 			_("Invert the sense of matching, to select non-matching lines"));
 
 	lbox = gtk_vbox_new(FALSE, 0);
@@ -869,7 +1020,7 @@ static void create_fif_dialog(void)
 	entry_extra = gtk_entry_new();
 	ui_entry_add_clear_icon(GTK_ENTRY(entry_extra));
 	gtk_widget_set_sensitive(entry_extra, FALSE);
-	ui_widget_set_tooltip_text(entry_extra, _("Other options to pass to Grep"));
+	gtk_widget_set_tooltip_text(entry_extra, _("Other options to pass to Grep"));
 	ui_hookup_widget(fif_dlg.dialog, entry_extra, "entry_extra");
 
 	/* enable entry_extra when check_extra is checked */
@@ -949,6 +1100,9 @@ void search_show_find_in_files_dialog(const gchar *dir)
 		g_free(cur_dir);
 	}
 
+	update_fif_file_mode_combo();
+	update_file_patterns(fif_dlg.files_mode_combo, fif_dlg.files_combo);
+
 	/* set the encoding of the current file */
 	if (doc != NULL)
 		enc_idx = encodings_get_idx_from_charset(doc->encoding);
@@ -981,34 +1135,15 @@ on_find_replace_checkbutton_toggled(GtkToggleButton *togglebutton, gpointer user
 		gboolean regex_set = gtk_toggle_button_get_active(chk_regexp);
 		GtkWidget *check_word = ui_lookup_widget(dialog, "check_word");
 		GtkWidget *check_wordstart = ui_lookup_widget(dialog, "check_wordstart");
-		GtkToggleButton *check_case = GTK_TOGGLE_BUTTON(
-			ui_lookup_widget(dialog, "check_case"));
 		GtkWidget *check_escape = ui_lookup_widget(dialog, "check_escape");
-		static gboolean case_state = FALSE; /* state before regex enabled */
+		gboolean replace = (dialog != find_dlg.dialog);
+		const char *back_button[2] = { "btn_previous" , "check_back" };
 
 		/* hide options that don't apply to regex searches */
 		gtk_widget_set_sensitive(check_escape, ! regex_set);
-
-		if (dialog == find_dlg.dialog)
-			gtk_widget_set_sensitive(ui_lookup_widget(dialog, "btn_previous"), ! regex_set);
-		else
-			gtk_widget_set_sensitive(ui_lookup_widget(dialog, "check_back"), ! regex_set);
-
+		gtk_widget_set_sensitive(ui_lookup_widget(dialog, back_button[replace]), ! regex_set);
 		gtk_widget_set_sensitive(check_word, ! regex_set);
 		gtk_widget_set_sensitive(check_wordstart, ! regex_set);
-
-		if (regex_set)	/* regex enabled */
-		{
-			/* Enable case sensitive but remember original case toggle state */
-			case_state = gtk_toggle_button_get_active(check_case);
-			gtk_toggle_button_set_active(check_case, TRUE);
-		}
-		else	/* regex disabled */
-		{
-			/* If case sensitive is still enabled, revert to what it was before we enabled it */
-			if (gtk_toggle_button_get_active(check_case) == TRUE)
-				gtk_toggle_button_set_active(check_case, case_state);
-		}
 	}
 }
 
@@ -1026,7 +1161,7 @@ gint search_mark_all(GeanyDocument *doc, const gchar *search_text, gint flags)
 	/* clear previous search indicators */
 	editor_indicator_clear(doc->editor, GEANY_INDICATOR_SEARCH);
 
-	if (!NZV(search_text))
+	if (G_UNLIKELY(! NZV(search_text)))
 		return 0;
 
 	ttf.chrg.cpMin = 0;
@@ -1056,73 +1191,73 @@ on_find_entry_activate(GtkEntry *entry, gpointer user_data)
 }
 
 
-static gint get_search_flags(GtkWidget *dialog)
+static void
+on_find_entry_activate_backward(GtkEntry *entry, gpointer user_data)
 {
-	gboolean fl1, fl2, fl3, fl4;
-
-	fl1 = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
-				ui_lookup_widget(dialog, "check_case")));
-	fl2 = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
-				ui_lookup_widget(dialog, "check_word")));
-	fl3 = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
-				ui_lookup_widget(dialog, "check_regexp")));
-	fl4 = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
-				ui_lookup_widget(dialog, "check_wordstart")));
-
-	return (fl1 ? SCFIND_MATCHCASE : 0) |
-		(fl2 ? SCFIND_WHOLEWORD : 0) |
-		(fl3 ? SCFIND_REGEXP | SCFIND_POSIX : 0) |
-		(fl4 ? SCFIND_WORDSTART : 0);
+	/* can't search backwards with a regexp */
+	if (search_data.flags & SCFIND_REGEXP)
+		utils_beep();
+	else
+		on_find_dialog_response(NULL, GEANY_RESPONSE_FIND_PREVIOUS,
+					ui_lookup_widget(GTK_WIDGET(entry), "entry"));
 }
 
 
+#define int_search_flags(match_case, whole_word, regexp, word_start) \
+	((match_case ? SCFIND_MATCHCASE : 0) | \
+	(whole_word ? SCFIND_WHOLEWORD : 0) | \
+	(regexp ? SCFIND_REGEXP | SCFIND_POSIX : 0) | \
+	(word_start ? SCFIND_WORDSTART : 0))
+
+
 static void
 on_find_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 {
 	gtk_window_get_position(GTK_WINDOW(find_dlg.dialog),
 		&find_dlg.position[0], &find_dlg.position[1]);
 
+	stash_group_update(find_prefs, find_dlg.dialog);
+
 	if (response == GTK_RESPONSE_CANCEL || response == GTK_RESPONSE_DELETE_EVENT)
 		gtk_widget_hide(find_dlg.dialog);
 	else
 	{
 		GeanyDocument *doc = document_get_current();
-		gboolean search_replace_escape;
-		gboolean check_close = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
-						ui_lookup_widget(GTK_WIDGET(find_dlg.dialog), "check_close")));
+		gboolean check_close = settings.find_close_dialog;
 
 		if (doc == NULL)
 			return;
 
-		search_replace_escape = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
-						ui_lookup_widget(GTK_WIDGET(find_dlg.dialog), "check_escape")));
 		search_data.backwards = FALSE;
 		search_data.search_bar = FALSE;
 
 		g_free(search_data.text);
+		g_free(search_data.original_text);
 		search_data.text = g_strdup(gtk_entry_get_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(user_data)))));
-		search_data.flags = get_search_flags(find_dlg.dialog);
+		search_data.original_text = g_strdup(search_data.text);
+		search_data.flags = int_search_flags(settings.find_case_sensitive,
+			settings.find_match_whole_word, settings.find_regexp, settings.find_match_word_start);
 
-		if (strlen(search_data.text) == 0)
+		if (! NZV(search_data.text))
 		{
 			fail:
 			utils_beep();
 			gtk_widget_grab_focus(find_dlg.entry);
 			return;
 		}
-		if (search_replace_escape || search_data.flags & SCFIND_REGEXP)
+		if (settings.find_escape_sequences || search_data.flags & SCFIND_REGEXP)
 		{
 			if (! utils_str_replace_escape(search_data.text, search_data.flags & SCFIND_REGEXP))
 				goto fail;
 		}
-		ui_combo_box_add_to_history(GTK_COMBO_BOX_ENTRY(user_data), search_data.text, 0);
+		ui_combo_box_add_to_history(GTK_COMBO_BOX_ENTRY(user_data), search_data.original_text, 0);
 
 		switch (response)
 		{
 			case GEANY_RESPONSE_FIND:
 			case GEANY_RESPONSE_FIND_PREVIOUS:
 			{
-				gint result = document_find_text(doc, search_data.text, search_data.flags,
+				gint result = document_find_text(doc, search_data.text, search_data.original_text, search_data.flags,
 					(response == GEANY_RESPONSE_FIND_PREVIOUS), TRUE, GTK_WIDGET(find_dlg.dialog));
 				ui_set_search_entry_background(find_dlg.entry, (result > -1));
 				check_close = FALSE;
@@ -1131,11 +1266,11 @@ on_find_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 				break;
 			}
 			case GEANY_RESPONSE_FIND_IN_FILE:
-				search_find_usage(search_data.text, search_data.flags, FALSE);
+				search_find_usage(search_data.text, search_data.original_text, search_data.flags, FALSE);
 				break;
 
 			case GEANY_RESPONSE_FIND_IN_SESSION:
-				search_find_usage(search_data.text, search_data.flags, TRUE);
+				search_find_usage(search_data.text, search_data.original_text, search_data.flags, TRUE);
 				break;
 
 			case GEANY_RESPONSE_MARK:
@@ -1143,12 +1278,12 @@ on_find_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 				gint count = search_mark_all(doc, search_data.text, search_data.flags);
 
 				if (count == 0)
-					ui_set_statusbar(FALSE, _("No matches found for \"%s\"."), search_data.text);
+					ui_set_statusbar(FALSE, _("No matches found for \"%s\"."), search_data.original_text);
 				else
 					ui_set_statusbar(FALSE,
 						ngettext("Found %d match for \"%s\".",
 								 "Found %d matches for \"%s\".", count),
-						count, search_data.text);
+						count, search_data.original_text);
 			}
 			break;
 		}
@@ -1167,7 +1302,8 @@ on_replace_entry_activate(GtkEntry *entry, gpointer user_data)
 
 static void replace_in_session(GeanyDocument *doc,
 		gint search_flags_re, gboolean search_replace_escape_re,
-		const gchar *find, const gchar *replace)
+		const gchar *find, const gchar *replace,
+		const gchar *original_find, const gchar *original_replace)
 {
 	guint n, page_count, rep_count = 0, file_count = 0;
 
@@ -1178,8 +1314,7 @@ static void replace_in_session(GeanyDocument *doc,
 		GeanyDocument *tmp_doc = document_get_from_page(n);
 		gint reps = 0;
 
-		reps = document_replace_all(tmp_doc, find, replace, search_flags_re,
-			search_replace_escape_re);
+		reps = document_replace_all(tmp_doc, find, replace, original_find, original_replace, search_flags_re);
 		rep_count += reps;
 		if (reps)
 			file_count++;
@@ -1187,7 +1322,7 @@ static void replace_in_session(GeanyDocument *doc,
 	if (file_count == 0)
 	{
 		utils_beep();
-		ui_set_statusbar(FALSE, _("No matches found for \"%s\"."), find);
+		ui_set_statusbar(FALSE, _("No matches found for \"%s\"."), original_find);
 		return;
 	}
 	/* if only one file was changed, don't override that document's status message
@@ -1209,39 +1344,34 @@ on_replace_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 	GeanyDocument *doc = document_get_current();
 	gint search_flags_re;
 	gboolean search_backwards_re, search_replace_escape_re;
-	gboolean close_window;
-	gchar *find, *replace;
+	gchar *find, *replace, *original_find = NULL, *original_replace = NULL;
 
 	gtk_window_get_position(GTK_WINDOW(replace_dlg.dialog),
 		&replace_dlg.position[0], &replace_dlg.position[1]);
 
+	stash_group_update(replace_prefs, replace_dlg.dialog);
+
 	if (response == GTK_RESPONSE_CANCEL || response == GTK_RESPONSE_DELETE_EVENT)
 	{
 		gtk_widget_hide(replace_dlg.dialog);
 		return;
 	}
 
-	close_window = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
-				ui_lookup_widget(GTK_WIDGET(replace_dlg.dialog), "check_close")));
-	search_backwards_re = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
-				ui_lookup_widget(GTK_WIDGET(replace_dlg.dialog), "check_back")));
-	search_replace_escape_re = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
-				ui_lookup_widget(GTK_WIDGET(replace_dlg.dialog), "check_escape")));
+	search_backwards_re = settings.replace_search_backwards;
+	search_replace_escape_re = settings.replace_escape_sequences;
 	find = g_strdup(gtk_entry_get_text(GTK_ENTRY(replace_dlg.find_entry)));
 	replace = g_strdup(gtk_entry_get_text(GTK_ENTRY(replace_dlg.replace_entry)));
 
-	search_flags_re = get_search_flags(replace_dlg.dialog);
+	search_flags_re = int_search_flags(settings.replace_case_sensitive,
+		settings.replace_match_whole_word, settings.replace_regexp,
+		settings.replace_match_word_start);
 
 	if ((response != GEANY_RESPONSE_FIND) && (search_flags_re & SCFIND_MATCHCASE)
 		&& (strcmp(find, replace) == 0))
-	{
-		fail:
-		utils_beep();
-		gtk_widget_grab_focus(replace_dlg.find_entry);
-		g_free(find);
-		g_free(replace);
-		return;
-	}
+		goto fail;
+
+	original_find = g_strdup(find);
+	original_replace = g_strdup(replace);
 	if (search_flags_re & SCFIND_REGEXP)
 	{
 		if (! utils_str_replace_escape(find, TRUE) ||
@@ -1256,45 +1386,44 @@ on_replace_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 	}
 
 	ui_combo_box_add_to_history(GTK_COMBO_BOX_ENTRY(
-		gtk_widget_get_parent(replace_dlg.find_entry)), find, 0);
+		gtk_widget_get_parent(replace_dlg.find_entry)), original_find, 0);
 	ui_combo_box_add_to_history(GTK_COMBO_BOX_ENTRY(
-		gtk_widget_get_parent(replace_dlg.replace_entry)), replace, 0);
+		gtk_widget_get_parent(replace_dlg.replace_entry)), original_replace, 0);
 
 	switch (response)
 	{
 		case GEANY_RESPONSE_REPLACE_AND_FIND:
 		{
-			gint rep = document_replace_text(doc, find, replace, search_flags_re,
+			gint rep = document_replace_text(doc, find, original_find, replace, search_flags_re,
 				search_backwards_re);
 			if (rep != -1)
-				document_find_text(doc, find, search_flags_re, search_backwards_re,
+				document_find_text(doc, find, original_find, search_flags_re, search_backwards_re,
 					TRUE, NULL);
 			break;
 		}
 		case GEANY_RESPONSE_REPLACE:
-			document_replace_text(doc, find, replace, search_flags_re,
+			document_replace_text(doc, find, original_find, replace, search_flags_re,
 				search_backwards_re);
 			break;
 
 		case GEANY_RESPONSE_FIND:
 		{
-			gint result = document_find_text(doc, find, search_flags_re,
+			gint result = document_find_text(doc, find, original_find, search_flags_re,
 								search_backwards_re, TRUE, GTK_WIDGET(dialog));
 			ui_set_search_entry_background(replace_dlg.find_entry, (result > -1));
 			break;
 		}
 		case GEANY_RESPONSE_REPLACE_IN_FILE:
-			if (! document_replace_all(doc, find, replace, search_flags_re,
-				search_replace_escape_re))
+			if (! document_replace_all(doc, find, replace, original_find, original_replace, search_flags_re))
 				utils_beep();
 			break;
 
 		case GEANY_RESPONSE_REPLACE_IN_SESSION:
-			replace_in_session(doc, search_flags_re, search_replace_escape_re, find, replace);
+			replace_in_session(doc, search_flags_re, search_replace_escape_re, find, replace, original_find, original_replace);
 			break;
 
 		case GEANY_RESPONSE_REPLACE_IN_SEL:
-			document_replace_sel(doc, find, replace, search_flags_re, search_replace_escape_re);
+			document_replace_sel(doc, find, replace, original_find, original_replace, search_flags_re);
 			break;
 	}
 	switch (response)
@@ -1302,11 +1431,22 @@ on_replace_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 		case GEANY_RESPONSE_REPLACE_IN_SEL:
 		case GEANY_RESPONSE_REPLACE_IN_FILE:
 		case GEANY_RESPONSE_REPLACE_IN_SESSION:
-			if (close_window)
+			if (settings.replace_close_dialog)
 				gtk_widget_hide(replace_dlg.dialog);
 	}
 	g_free(find);
 	g_free(replace);
+	g_free(original_find);
+	g_free(original_replace);
+	return;
+
+fail:
+	utils_beep();
+	gtk_widget_grab_focus(replace_dlg.find_entry);
+	g_free(find);
+	g_free(replace);
+	g_free(original_find);
+	g_free(original_replace);
 }
 
 
@@ -1339,7 +1479,7 @@ static GString *get_grep_options(void)
 		}
 	}
 	g_strstrip(settings.fif_files);
-	if (settings.fif_use_files && *settings.fif_files)
+	if (settings.fif_files_mode != FILES_MODE_ALL && *settings.fif_files)
 	{
 		GString *tmp;
 
@@ -1374,7 +1514,7 @@ on_find_in_files_dialog_response(GtkDialog *dialog, gint response,
 		GeanyEncodingIndex enc_idx = gtk_combo_box_get_active(
 			GTK_COMBO_BOX(fif_dlg.encoding_combo));
 
-		if (!NZV(utf8_dir))
+		if (G_UNLIKELY(! NZV(utf8_dir)))
 			ui_set_statusbar(FALSE, _("Invalid directory for find in files."));
 		else if (NZV(search_text))
 		{
@@ -1514,14 +1654,27 @@ search_find_in_files(const gchar *utf8_search_text, const gchar *dir, const gcha
 }
 
 
+static gboolean pattern_list_match(GSList *patterns, const gchar *str)
+{
+	GSList *item;
+
+	foreach_slist(item, patterns)
+	{
+		if (g_pattern_match_string(item->data, str))
+			return TRUE;
+	}
+	return FALSE;
+}
+
+
 /* Creates an argument vector of strings, copying argv_prefix[] values for
  * the first arguments, then followed by filenames found in dir.
  * Returns NULL if no files were found, otherwise returned vector should be fully freed. */
 static gchar **search_get_argv(const gchar **argv_prefix, const gchar *dir)
 {
-	guint prefix_len, list_len, i;
+	guint prefix_len, list_len, i, j;
 	gchar **argv;
-	GSList *list, *item;
+	GSList *list, *item, *patterns = NULL;
 	GError *error = NULL;
 
 	g_return_val_if_fail(dir != NULL, NULL);
@@ -1539,13 +1692,40 @@ static gchar **search_get_argv(const gchar **argv_prefix, const gchar *dir)
 
 	argv = g_new(gchar*, prefix_len + list_len + 1);
 
-	for (i = 0; i < prefix_len; i++)
-		argv[i] = g_strdup(argv_prefix[i]);
+	for (i = 0, j = 0; i < prefix_len; i++)
+	{
+		if (g_str_has_prefix(argv_prefix[i], "--include="))
+		{
+			const gchar *pat = &(argv_prefix[i][10]); /* the pattern part of the argument */
 
-	foreach_slist(item, list)
-		argv[i++] = item->data;
+			patterns = g_slist_prepend(patterns, g_pattern_spec_new(pat));
+		}
+		else
+			argv[j++] = g_strdup(argv_prefix[i]);
+	}
 
-	argv[i] = NULL;
+	if (patterns)
+	{
+		GSList *pat;
+
+		foreach_slist(item, list)
+		{
+			if (pattern_list_match(patterns, item->data))
+				argv[j++] = item->data;
+			else
+				g_free(item->data);
+		}
+		foreach_slist(pat, patterns)
+			g_pattern_spec_free(pat->data);
+		g_slist_free(patterns);
+	}
+	else
+	{
+		foreach_slist(item, list)
+			argv[j++] = item->data;
+	}
+
+	argv[j] = NULL;
 	g_slist_free(list);
 	return argv;
 }
@@ -1844,7 +2024,8 @@ static gint find_document_usage(GeanyDocument *doc, const gchar *search_text, gi
 }
 
 
-void search_find_usage(const gchar *search_text, gint flags, gboolean in_session)
+void search_find_usage(const gchar *search_text, const gchar *original_search_text,
+		gint flags, gboolean in_session)
 {
 	GeanyDocument *doc;
 	gint count = 0;
@@ -1852,7 +2033,7 @@ void search_find_usage(const gchar *search_text, gint flags, gboolean in_session
 	doc = document_get_current();
 	g_return_if_fail(doc != NULL);
 
-	if (!NZV(search_text))
+	if (G_UNLIKELY(! NZV(search_text)))
 	{
 		utils_beep();
 		return;
@@ -1879,17 +2060,17 @@ void search_find_usage(const gchar *search_text, gint flags, gboolean in_session
 
 	if (count == 0) /* no matches were found */
 	{
-		ui_set_statusbar(FALSE, _("No matches found for \"%s\"."), search_text);
-		msgwin_msg_add(COLOR_BLUE, -1, NULL, _("No matches found for \"%s\"."), search_text);
+		ui_set_statusbar(FALSE, _("No matches found for \"%s\"."), original_search_text);
+		msgwin_msg_add(COLOR_BLUE, -1, NULL, _("No matches found for \"%s\"."), original_search_text);
 	}
 	else
 	{
 		ui_set_statusbar(FALSE, ngettext(
 			"Found %d match for \"%s\".", "Found %d matches for \"%s\".", count),
-			count, search_text);
+			count, original_search_text);
 		msgwin_msg_add(COLOR_BLUE, -1, NULL, ngettext(
 			"Found %d match for \"%s\".", "Found %d matches for \"%s\".", count),
-			count, search_text);
+			count, original_search_text);
 	}
 }
 
@@ -1967,7 +2148,7 @@ void search_find_again(gboolean change_direction)
 	if (search_data.text)
 	{
 		gboolean forward = ! search_data.backwards;
-		gint result = document_find_text(doc, search_data.text, search_data.flags,
+		gint result = document_find_text(doc, search_data.text, search_data.original_text, search_data.flags,
 			change_direction ? forward : !forward, FALSE, NULL);
 
 		if (result > -1)
@@ -1978,5 +2159,3 @@ void search_find_again(gboolean change_direction)
 				toolbar_get_widget_child_by_name("SearchEntry"), (result > -1));
 	}
 }
-
-
