@@ -1,8 +1,8 @@
 /*
  *      sidebar.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: sidebar.c 5422 2010-11-22 13:49:38Z ntrel $
+ * $Id: sidebar.c 5856 2011-06-17 22:52:43Z colombanw $
  */
 
 /*
@@ -62,12 +62,6 @@ static struct
 }
 doc_items = {NULL, NULL, NULL, NULL, NULL};
 
-static struct
-{
-	GtkTreeSelection *selection;
-	guint keyval;
-} selection_change = {NULL, 0};
-
 enum
 {
 	TREEVIEW_SYMBOL = 0,
@@ -97,9 +91,7 @@ static gboolean documents_show_paths;
 static GtkWidget *tag_window;	/* scrolled window that holds the symbol list GtkTreeView */
 
 /* callback prototypes */
-static gboolean on_openfiles_tree_selection_changed(gpointer data);
 static void on_openfiles_document_action(GtkMenuItem *menuitem, gpointer user_data);
-static gboolean on_taglist_tree_selection_changed(gpointer data);
 static gboolean sidebar_button_press_cb(GtkWidget *widget, GdkEventButton *event,
 		gpointer user_data);
 static gboolean sidebar_key_press_cb(GtkWidget *widget, GdkEventKey *event,
@@ -144,16 +136,11 @@ static void prepare_taglist(GtkWidget *tree, GtkTreeStore *store)
 	g_signal_connect(tree, "key-press-event",
 		G_CALLBACK(sidebar_key_press_cb), NULL);
 
-	if (gtk_check_version(2, 12, 0) == NULL)
-	{
-		g_object_set(tree, "show-expanders", interface_prefs.show_symbol_list_expanders, NULL);
-		if (! interface_prefs.show_symbol_list_expanders)
-			g_object_set(tree, "level-indentation", 10, NULL);
-		/* Tooltips */
-		g_object_set(tree,
-			"has-tooltip", TRUE,
-			"tooltip-column", SYMBOLS_COLUMN_TOOLTIP, NULL);
-	}
+	gtk_tree_view_set_show_expanders(GTK_TREE_VIEW(tree), interface_prefs.show_symbol_list_expanders);
+	if (! interface_prefs.show_symbol_list_expanders)
+		gtk_tree_view_set_level_indentation(GTK_TREE_VIEW(tree), 10);
+	/* Tooltips */
+	gtk_tree_view_set_tooltip_column(GTK_TREE_VIEW(tree), SYMBOLS_COLUMN_TOOLTIP);
 
 	/* selection handling */
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree));
@@ -198,8 +185,21 @@ static void create_default_tag_tree(void)
 /* update = rescan the tags for doc->filename */
 void sidebar_update_tag_list(GeanyDocument *doc, gboolean update)
 {
-	if (gtk_bin_get_child(GTK_BIN(tag_window)))
-		gtk_container_remove(GTK_CONTAINER(tag_window), gtk_bin_get_child(GTK_BIN(tag_window)));
+	GtkWidget *child = gtk_bin_get_child(GTK_BIN(tag_window));
+
+	/* changes the tree view to the given one, trying not to do useless changes */
+	#define CHANGE_TREE(new_child) \
+		G_STMT_START { \
+			/* only change the tag tree if it's actually not the same (to avoid flickering) and if
+			 * it's the one of the current document (to avoid problems when e.g. reloading
+			 * configuration files */ \
+			if (child != new_child && doc == document_get_current()) \
+			{ \
+				if (child) \
+					gtk_container_remove(GTK_CONTAINER(tag_window), child); \
+				gtk_container_add(GTK_CONTAINER(tag_window), new_child); \
+			} \
+		} G_STMT_END
 
 	if (tv.default_tag_tree == NULL)
 		create_default_tag_tree();
@@ -207,7 +207,7 @@ void sidebar_update_tag_list(GeanyDocument *doc, gboolean update)
 	/* show default empty tag tree if there are no tags */
 	if (doc == NULL || doc->file_type == NULL || ! filetype_has_tags(doc->file_type))
 	{
-		gtk_container_add(GTK_CONTAINER(tag_window), tv.default_tag_tree);
+		CHANGE_TREE(tv.default_tag_tree);
 		return;
 	}
 
@@ -216,7 +216,7 @@ void sidebar_update_tag_list(GeanyDocument *doc, gboolean update)
 		if (doc->priv->tag_tree == NULL)
 		{
 			doc->priv->tag_store = gtk_tree_store_new(
-				SYMBOLS_N_COLUMNS, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_POINTER, G_TYPE_STRING);
+				SYMBOLS_N_COLUMNS, GDK_TYPE_PIXBUF, G_TYPE_STRING, TM_TYPE_TAG, G_TYPE_STRING, G_TYPE_BOOLEAN);
 			doc->priv->tag_tree = gtk_tree_view_new();
 			prepare_taglist(doc->priv->tag_tree, doc->priv->tag_store);
 			gtk_widget_show(doc->priv->tag_tree);
@@ -228,12 +228,14 @@ void sidebar_update_tag_list(GeanyDocument *doc, gboolean update)
 
 	if (doc->has_tags)
 	{
-		gtk_container_add(GTK_CONTAINER(tag_window), doc->priv->tag_tree);
+		CHANGE_TREE(doc->priv->tag_tree);
 	}
 	else
 	{
-		gtk_container_add(GTK_CONTAINER(tag_window), tv.default_tag_tree);
+		CHANGE_TREE(tv.default_tag_tree);
 	}
+
+	#undef CHANGE_TREE
 }
 
 
@@ -304,9 +306,8 @@ static void prepare_openfiles(void)
 
 	ui_widget_modify_font_from_string(tv.tree_openfiles, interface_prefs.tagbar_font);
 
-	/* GTK 2.12 tooltips */
-	if (gtk_check_version(2, 12, 0) == NULL)
-		g_object_set(tv.tree_openfiles, "has-tooltip", TRUE, "tooltip-column", DOCUMENTS_FILENAME, NULL);
+	/* tooltips */
+	gtk_tree_view_set_tooltip_column(GTK_TREE_VIEW(tv.tree_openfiles), DOCUMENTS_FILENAME);
 
 	/* selection handling */
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tv.tree_openfiles));
@@ -704,6 +705,7 @@ static void create_openfiles_popup_menu(void)
 	gtk_container_add(GTK_CONTAINER(openfiles_popup_menu), item);
 
 	doc_items.show_paths = gtk_check_menu_item_new_with_mnemonic(_("Show _Paths"));
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(doc_items.show_paths), documents_show_paths);
 	gtk_widget_show(doc_items.show_paths);
 	gtk_container_add(GTK_CONTAINER(openfiles_popup_menu), doc_items.show_paths);
 	g_signal_connect(doc_items.show_paths, "activate",
@@ -824,14 +826,14 @@ static void change_focus_to_editor(GeanyDocument *doc, GtkWidget *source_widget)
 }
 
 
-static gboolean on_openfiles_tree_selection_changed(gpointer data)
+static gboolean openfiles_go_to_selection(GtkTreeSelection *selection, guint keyval)
 {
 	GtkTreeIter iter;
 	GtkTreeModel *model;
 	GeanyDocument *doc = NULL;
 
 	/* use switch_notebook_page to ignore changing the notebook page because it is already done */
-	if (gtk_tree_selection_get_selected(selection_change.selection, &model, &iter) && ! ignore_callback)
+	if (gtk_tree_selection_get_selected(selection, &model, &iter) && ! ignore_callback)
 	{
 		gtk_tree_model_get(model, &iter, DOCUMENTS_DOCUMENT, &doc, -1);
 		if (! doc)
@@ -841,22 +843,22 @@ static gboolean on_openfiles_tree_selection_changed(gpointer data)
 		gtk_notebook_set_current_page(GTK_NOTEBOOK(main_widgets.notebook),
 			gtk_notebook_page_num(GTK_NOTEBOOK(main_widgets.notebook),
 			(GtkWidget*) doc->editor->sci));
-		if (selection_change.keyval != GDK_space)
+		if (keyval != GDK_space)
 			change_focus_to_editor(doc, tv.tree_openfiles);
 	}
 	return FALSE;
 }
 
 
-static gboolean on_taglist_tree_selection_changed(gpointer data)
+static gboolean taglist_go_to_selection(GtkTreeSelection *selection, guint keyval)
 {
 	GtkTreeIter iter;
 	GtkTreeModel *model;
 	gint line = 0;
 
-	if (gtk_tree_selection_get_selected(selection_change.selection, &model, &iter))
+	if (gtk_tree_selection_get_selected(selection, &model, &iter))
 	{
-		const TMTag *tag;
+		TMTag *tag;
 
 		gtk_tree_model_get(model, &iter, SYMBOLS_COLUMN_TAG, &tag, -1);
 		if (! tag)
@@ -870,38 +872,38 @@ static gboolean on_taglist_tree_selection_changed(gpointer data)
 			if (doc != NULL)
 			{
 				navqueue_goto_line(doc, doc, line);
-				if (selection_change.keyval != GDK_space)
+				if (keyval != GDK_space)
 					change_focus_to_editor(doc, NULL);
 			}
 		}
+		tm_tag_unref(tag);
 	}
 	return FALSE;
 }
 
 
-static void update_selection_change(GtkTreeSelection *selection, guint keyval)
-{
-	selection_change.selection = selection;
-	selection_change.keyval = keyval;
-}
-
-
 static gboolean sidebar_key_press_cb(GtkWidget *widget, GdkEventKey *event,
 											 gpointer user_data)
 {
 	may_steal_focus = FALSE;
 	if (ui_is_keyval_enter_or_return(event->keyval) || event->keyval == GDK_space)
 	{
+		GtkWidgetClass *widget_class = GTK_WIDGET_GET_CLASS(widget);
 		GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
 		may_steal_focus = TRUE;
-		/* delay the query of selection state because this callback is executed before GTK
-		 * changes the selection (g_signal_connect_after would be better but it doesn't work) */
-		update_selection_change(selection, event->keyval);
 
-		if (widget ==  tv.tree_openfiles) /* tag and doc list have separate handlers */
-			g_idle_add(on_openfiles_tree_selection_changed, NULL);
+		/* force the TreeView handler to run before us for it to do its job (selection & stuff).
+		 * doing so will prevent further handlers to be run in most cases, but the only one is our
+		 * own, so guess it's fine. */
+		if (widget_class->key_press_event)
+			widget_class->key_press_event(widget, event);
+
+		if (widget == tv.tree_openfiles) /* tag and doc list have separate handlers */
+			openfiles_go_to_selection(selection, event->keyval);
 		else
-			g_idle_add(on_taglist_tree_selection_changed, NULL);
+			taglist_go_to_selection(selection, event->keyval);
+
+		return TRUE;
 	}
 	return FALSE;
 }
@@ -911,6 +913,14 @@ static gboolean sidebar_button_press_cb(GtkWidget *widget, GdkEventButton *event
 		G_GNUC_UNUSED gpointer user_data)
 {
 	GtkTreeSelection *selection;
+	GtkWidgetClass *widget_class = GTK_WIDGET_GET_CLASS(widget);
+	gboolean handled = FALSE;
+
+	/* force the TreeView handler to run before us for it to do its job (selection & stuff).
+	 * doing so will prevent further handlers to be run in most cases, but the only one is our own,
+	 * so guess it's fine. */
+	if (widget_class->button_press_event)
+		handled = widget_class->button_press_event(widget, event);
 
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
 	may_steal_focus = TRUE;
@@ -938,14 +948,11 @@ static gboolean sidebar_button_press_cb(GtkWidget *widget, GdkEventButton *event
 	}
 	else if (event->button == 1)
 	{	/* allow reclicking of taglist treeview item */
-		/* delay the query of selection state because this callback is executed before GTK
-		 * changes the selection (g_signal_connect_after would be better but it doesn't work) */
-		update_selection_change(selection, 0);
-
 		if (widget == tv.tree_openfiles)
-			g_idle_add(on_openfiles_tree_selection_changed, NULL);
+			openfiles_go_to_selection(selection, 0);
 		else
-			g_idle_add(on_taglist_tree_selection_changed, NULL);
+			taglist_go_to_selection(selection, 0);
+		handled = TRUE;
 	}
 	else if (event->button == 3)
 	{
@@ -964,8 +971,9 @@ static gboolean sidebar_button_press_cb(GtkWidget *widget, GdkEventButton *event
 			gtk_menu_popup(GTK_MENU(tv.popup_taglist), NULL, NULL, NULL, NULL,
 					event->button, event->time);
 		}
+		handled = TRUE;
 	}
-	return FALSE;
+	return handled;
 }
 
 
@@ -1035,19 +1043,14 @@ void sidebar_init(void)
 	g_signal_connect(geany_object, "load-settings", on_load_settings, NULL);
 	g_signal_connect(geany_object, "save-settings", on_save_settings, NULL);
 
+	g_signal_connect(main_widgets.sidebar_notebook, "page-added",
+		G_CALLBACK(sidebar_tabs_show_hide), NULL);
+	g_signal_connect(main_widgets.sidebar_notebook, "page-removed",
+		G_CALLBACK(sidebar_tabs_show_hide), NULL);
+	/* tabs may have changed when sidebar is reshown */
+	g_signal_connect(main_widgets.sidebar_notebook, "show",
+		G_CALLBACK(sidebar_tabs_show_hide), NULL);
 
-	if (gtk_check_version(2, 10, 0) == NULL)
-	{
-		g_signal_connect(main_widgets.sidebar_notebook, "page-added",
-			G_CALLBACK(sidebar_tabs_show_hide), NULL);
-
-		g_signal_connect(main_widgets.sidebar_notebook, "page-removed",
-			G_CALLBACK(sidebar_tabs_show_hide), NULL);
-
-		/* tabs may have changed when sidebar is reshown */
-		g_signal_connect(main_widgets.sidebar_notebook, "show",
-			G_CALLBACK(sidebar_tabs_show_hide), NULL);
-	}
 	sidebar_tabs_show_hide(GTK_NOTEBOOK(main_widgets.sidebar_notebook), NULL, 0, NULL);
 }
 
@@ -1098,15 +1101,12 @@ void sidebar_focus_symbols_tab(void)
 static void sidebar_tabs_show_hide(GtkNotebook *notebook, GtkWidget *child,
 								   guint page_num, gpointer data)
 {
-	if (gtk_check_version(2, 10, 0) == NULL)
-	{
-		gint tabs = gtk_notebook_get_n_pages(notebook);
+	gint tabs = gtk_notebook_get_n_pages(notebook);
 
-		if (interface_prefs.sidebar_symbol_visible == FALSE)
-			tabs--;
-		if (interface_prefs.sidebar_openfiles_visible == FALSE)
-			tabs--;
+	if (interface_prefs.sidebar_symbol_visible == FALSE)
+		tabs--;
+	if (interface_prefs.sidebar_openfiles_visible == FALSE)
+		tabs--;
 
-		gtk_notebook_set_show_tabs(notebook, (tabs > 1));
-	}
+	gtk_notebook_set_show_tabs(notebook, (tabs > 1));
 }
