@@ -1,8 +1,8 @@
 /*
  *      socket.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2006-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *      Copyright 2006 Hiroyuki Yamamoto (author of Sylpheed)
  *
  *      This program is free software; you can redistribute it and/or modify
@@ -19,7 +19,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: socket.c 5292 2010-10-07 14:10:32Z ntrel $
+ * $Id: socket.c 5975 2011-09-30 11:14:23Z ntrel $
  */
 
 /*
@@ -93,9 +93,9 @@
 
 #ifdef G_OS_WIN32
 #define REMOTE_CMD_PORT		49876
-#define SOCKET_IS_VALID(s)	(G_LIKELY((s) != INVALID_SOCKET))
+#define SOCKET_IS_VALID(s)	((s) != INVALID_SOCKET)
 #else
-#define SOCKET_IS_VALID(s)	(G_LIKELY((s) >= 0))
+#define SOCKET_IS_VALID(s)	((s) >= 0)
 #define INVALID_SOCKET		(-1)
 #endif
 #define BUFFER_LENGTH 4096
@@ -410,7 +410,7 @@ static gint socket_fd_open_unix(const gchar *path)
 	real_path = g_strdup_printf("%s%cgeany_socket.%08x",
 		g_get_tmp_dir(), G_DIR_SEPARATOR, g_random_int());
 
-	if (utils_is_file_writeable(real_path) != 0)
+	if (utils_is_file_writable(real_path) != 0)
 	{	/* if real_path is not writable for us, fall back to ~/.config/geany/geany_socket_*_* */
 		/* instead of creating a symlink and print a warning */
 		g_warning("Socket %s could not be written, using %s as fallback.", real_path, path);
@@ -468,7 +468,7 @@ static gint socket_fd_open_inet(gushort port)
 	gchar val;
 
 	sock = socket(AF_INET, SOCK_STREAM, 0);
-	if (! SOCKET_IS_VALID(sock))
+	if (G_UNLIKELY(! SOCKET_IS_VALID(sock)))
 	{
 		geany_debug("fd_open_inet(): socket() failed: %d\n", WSAGetLastError());
 		return -1;
@@ -511,7 +511,7 @@ static gint socket_fd_connect_inet(gushort port)
 	struct sockaddr_in addr;
 
 	sock = socket(AF_INET, SOCK_STREAM, 0);
-	if (! SOCKET_IS_VALID(sock))
+	if (G_UNLIKELY(! SOCKET_IS_VALID(sock)))
 	{
 		geany_debug("fd_connect_inet(): socket() failed: %d\n", WSAGetLastError());
 		return -1;
