@@ -1,8 +1,8 @@
 /*
  *      symbols.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2006-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: symbols.c 5428 2010-11-23 13:17:42Z ntrel $
+ * $Id: symbols.c 5949 2011-09-22 18:04:03Z colombanw $
  */
 
 /**
@@ -437,10 +437,10 @@ static gint compare_symbol(const TMTag *tag_a, const TMTag *tag_b)
 		return 0;
 
 	if (tag_a->name == NULL)
-        return -(tag_a->name != tag_b->name);
+		return -(tag_a->name != tag_b->name);
 
 	if (tag_b->name == NULL)
-        return tag_a->name != tag_b->name;
+		return tag_a->name != tag_b->name;
 
 	ret = strcmp(tag_a->name, tag_b->name);
 	if (ret == 0)
@@ -451,16 +451,27 @@ static gint compare_symbol(const TMTag *tag_a, const TMTag *tag_b)
 }
 
 
-/* sort by line only */
+/* sort by line, then scope */
 static gint compare_symbol_lines(gconstpointer a, gconstpointer b)
 {
 	const TMTag *tag_a = TM_TAG(a);
 	const TMTag *tag_b = TM_TAG(b);
+	gint ret;
 
 	if (a == NULL || b == NULL)
 		return 0;
 
-	return tag_a->atts.entry.line - tag_b->atts.entry.line;
+	ret = tag_a->atts.entry.line - tag_b->atts.entry.line;
+	if (ret == 0)
+	{
+		if (tag_a->atts.entry.scope == NULL)
+			return -(tag_a->atts.entry.scope != tag_b->atts.entry.scope);
+		if (tag_b->atts.entry.scope == NULL)
+			return tag_a->atts.entry.scope != tag_b->atts.entry.scope;
+		else
+			return strcmp(tag_a->atts.entry.scope, tag_b->atts.entry.scope);
+	}
+	return ret;
 }
 
 
@@ -483,7 +494,7 @@ static GList *get_tag_list(GeanyDocument *doc, guint tag_types)
 
 		if (tag->type & tag_types)
 		{
-			tag_names = g_list_append(tag_names, tag);
+			tag_names = g_list_prepend(tag_names, tag);
 		}
 	}
 	tag_names = g_list_sort(tag_names, compare_symbol_lines);
@@ -547,6 +558,71 @@ static GdkPixbuf *get_tag_icon(const gchar *icon_name)
 }
 
 
+/* finds the next iter at any level
+ * @param iter in/out, the current iter, will be changed to the next one
+ * @return TRUE if there @p iter was set, or FALSE if there is no next iter */
+static gboolean next_iter(GtkTreeModel *model, GtkTreeIter *iter)
+{
+	GtkTreeIter guess;
+	GtkTreeIter copy = *iter;
+
+	/* go down if the item has children */
+	if (gtk_tree_model_iter_children(model, &guess, iter))
+		*iter = guess;
+	/* or to the next item at the same level */
+	else if (gtk_tree_model_iter_next(model, &copy))
+		*iter = copy;
+	/* or to the next item at a parent level */
+	else if (gtk_tree_model_iter_parent(model, &guess, iter))
+	{
+		copy = guess;
+		while(TRUE)
+		{
+			if (gtk_tree_model_iter_next(model, &copy))
+			{
+				*iter = copy;
+				return TRUE;
+			}
+			else if (gtk_tree_model_iter_parent(model, &copy, &guess))
+				guess = copy;
+			else
+				return FALSE;
+		}
+	}
+	else
+		return FALSE;
+
+	return TRUE;
+}
+
+
+static gboolean find_toplevel_iter(GtkTreeStore *store, GtkTreeIter *iter, const gchar *title)
+{
+	GtkTreeModel *model = GTK_TREE_MODEL(store);
+
+	if (!gtk_tree_model_get_iter_first(model, iter))
+		return FALSE;
+	do
+	{
+		gchar *candidate;
+
+		gtk_tree_model_get(model, iter, SYMBOLS_COLUMN_NAME, &candidate, -1);
+		/* FIXME: what if 2 different items have the same name?
+		 * this should never happen, but might be caused by a typo in a translation */
+		if (utils_str_equal(candidate, title))
+		{
+			g_free(candidate);
+			return TRUE;
+		}
+		else
+			g_free(candidate);
+	}
+	while (gtk_tree_model_iter_next(model, iter));
+
+	return FALSE;
+}
+
+
 /* Adds symbol list groups in (iter*, title) pairs.
  * The list must be ended with NULL. */
 static void G_GNUC_NULL_TERMINATED
@@ -557,9 +633,9 @@ tag_list_add_groups(GtkTreeStore *tree_store, ...)
 
 	g_return_if_fail(top_level_iter_names);
 
-    va_start(args, tree_store);
-    for (; iter = va_arg(args, GtkTreeIter*), iter != NULL;)
-    {
+	va_start(args, tree_store);
+	for (; iter = va_arg(args, GtkTreeIter*), iter != NULL;)
+	{
 		gchar *title = va_arg(args, gchar*);
 		gchar *icon_name = va_arg(args, gchar *);
 		GdkPixbuf *icon = NULL;
@@ -569,17 +645,18 @@ tag_list_add_groups(GtkTreeStore *tree_store, ...)
 			icon = get_tag_icon(icon_name);
 		}
 
-    	g_assert(title != NULL);
+		g_assert(title != NULL);
 		g_ptr_array_add(top_level_iter_names, title);
 
-		gtk_tree_store_append(tree_store, iter, NULL);
+		if (!find_toplevel_iter(tree_store, iter, title))
+			gtk_tree_store_append(tree_store, iter, NULL);
 
 		if (G_IS_OBJECT(icon))
 		{
 			gtk_tree_store_set(tree_store, iter, SYMBOLS_COLUMN_ICON, icon, -1);
 			g_object_unref(icon);
 		}
-		gtk_tree_store_set(tree_store, iter, SYMBOLS_COLUMN_NAME, title, -1);
+		gtk_tree_store_set(tree_store, iter, SYMBOLS_COLUMN_NAME, title, SYMBOLS_COLUMN_VALID, TRUE, -1);
 	}
 	va_end(args);
 }
@@ -626,6 +703,16 @@ static void add_top_level_items(GeanyDocument *doc)
 				&tv_iters.tag_function, _("Functions"), "classviewer-method",
 				NULL);
 			break;
+		case GEANY_FILETYPES_COBOL:
+			tag_list_add_groups(tag_store,
+				&tv_iters.tag_class, _("Program"), "classviewer-class",
+				&tv_iters.tag_function, _("File"), "classviewer-method",
+				&tv_iters.tag_namespace, _("Sections"), "classviewer-namespace",
+				&tv_iters.tag_macro, _("Paragraph"), "classviewer-other",
+				&tv_iters.tag_struct, _("Group"), "classviewer-struct",
+				&tv_iters.tag_variable, _("Data"), "classviewer-var",
+				NULL);
+			break;
 		case GEANY_FILETYPES_CONF:
 			tag_list_add_groups(tag_store,
 				&tv_iters.tag_namespace, _("Sections"), "classviewer-other",
@@ -859,6 +946,19 @@ static void add_top_level_items(GeanyDocument *doc)
 				&tv_iters.tag_macro, _("Macros"), "classviewer-macro",
 				NULL);
 			break;
+		case GEANY_FILETYPES_SQL:
+		{
+			tag_list_add_groups(tag_store,
+				&(tv_iters.tag_function), _("Functions"), "classviewer-method",
+				&(tv_iters.tag_namespace), _("Procedures"), "classviewer-namespace",
+				&(tv_iters.tag_struct), _("Indexes"), "classviewer-struct",
+				&(tv_iters.tag_class), _("Tables"), "classviewer-class",
+				&(tv_iters.tag_macro), _("Triggers"), "classviewer-macro",
+				&(tv_iters.tag_member), _("Views"), "classviewer-var",
+				&(tv_iters.tag_other), _("Other"), "classviewer-other",
+				NULL);
+			break;
+		}
 		case GEANY_FILETYPES_D:
 		default:
 		{
@@ -890,30 +990,21 @@ static void add_top_level_items(GeanyDocument *doc)
 }
 
 
-/* the following code surely can be improved, at the moment it collects some iters
- * for removal and after that the actual removal is done. I didn't find a way to find and remove
- * an empty row in one loop (next iter fails then) */
+/* removes toplevel items that have no children */
 static void hide_empty_rows(GtkTreeStore *store)
 {
-	GtkTreeIter iter, *iters[MAX_SYMBOL_TYPES] = { NULL };
-	guint i = 0;
+	GtkTreeIter iter;
+	gboolean cont = TRUE;
 
 	if (! gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter))
 		return; /* stop when first iter is invalid, i.e. no elements */
 
-	do /* first collect the iters we need to delete empty rows */
+	while (cont)
 	{
 		if (! gtk_tree_model_iter_has_child(GTK_TREE_MODEL(store), &iter))
-			iters[i++] = gtk_tree_iter_copy(&iter);
-	} while (gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter));
-
-	/* now actually delete the collected iters */
-	for (i = 0; i < MAX_SYMBOL_TYPES; i++)
-	{
-		if (G_UNLIKELY(iters[i] == NULL))
-			break;
-		gtk_tree_store_remove(store, iters[i]);
-		gtk_tree_iter_free(iters[i]);
+			cont = gtk_tree_store_remove(store, &iter);
+		else
+			cont = gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
 	}
 }
 
@@ -1094,7 +1185,97 @@ static GdkPixbuf *get_child_icon(GtkTreeStore *tree_store, GtkTreeIter *parent)
 }
 
 
-static void add_tree_tag(GeanyDocument *doc, const TMTag *tag, GHashTable *parent_hash)
+static gboolean tag_equal(gconstpointer v1, gconstpointer v2)
+{
+	const TMTag *t1 = v1;
+	const TMTag *t2 = v2;
+
+	return (t1->type == t2->type && strcmp(t1->name, t2->name) == 0 &&
+			utils_str_equal(t1->atts.entry.scope, t2->atts.entry.scope) &&
+			/* include arglist in match to support e.g. C++ overloading */
+			utils_str_equal(t1->atts.entry.arglist, t2->atts.entry.arglist));
+}
+
+
+/* inspired from g_str_hash() */
+static guint tag_hash(gconstpointer v)
+{
+	const TMTag *tag = v;
+	const gchar *p;
+	guint32 h = 5381;
+
+	h = (h << 5) + h + tag->type;
+	for (p = tag->name; *p != '\0'; p++)
+		h = (h << 5) + h + *p;
+	if (tag->atts.entry.scope)
+	{
+		for (p = tag->atts.entry.scope; *p != '\0'; p++)
+			h = (h << 5) + h + *p;
+	}
+	/* for e.g. C++ overloading */
+	if (tag->atts.entry.arglist)
+	{
+		for (p = tag->atts.entry.arglist; *p != '\0'; p++)
+			h = (h << 5) + h + *p;
+	}
+
+	return h;
+}
+
+
+static GHashTable *build_iter_table(GtkTreeStore *store)
+{
+	GtkTreeModel *model = GTK_TREE_MODEL(store);
+	GHashTable *table;
+	GtkTreeIter iter;
+
+	table = g_hash_table_new_full(tag_hash, tag_equal,
+			(GDestroyNotify)tm_tag_unref, (GDestroyNotify)gtk_tree_path_free);
+
+	if (!gtk_tree_model_get_iter_first(model, &iter))
+		return table;
+	do
+	{
+		TMTag *tag;
+
+		gtk_tree_model_get(model, &iter, SYMBOLS_COLUMN_TAG, &tag, -1);
+		if (tag)
+			g_hash_table_insert(table, tag, gtk_tree_model_get_path(model, &iter));
+	}
+	while (next_iter(model, &iter));
+
+	return table;
+}
+
+
+static gboolean find_iter(GtkTreeStore *store, GHashTable *iter_hash, const TMTag *tag,
+		GtkTreeIter *iter)
+{
+	GtkTreePath *path;
+
+	path = g_hash_table_lookup(iter_hash, tag);
+	if (path)
+	{
+		GtkTreeIter tmp;
+		gboolean valid;
+
+		if (!gtk_tree_model_get_iter(GTK_TREE_MODEL(store), &tmp, path))
+			return FALSE;
+		gtk_tree_model_get(GTK_TREE_MODEL(store), &tmp, SYMBOLS_COLUMN_VALID, &valid, -1);
+		/* if the row is valid it has already been updated, so it's simply a duplicate */
+		if (!valid)
+		{
+			*iter = tmp;
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
+
+static void add_tree_tag(GeanyDocument *doc, const TMTag *tag, GHashTable *parent_hash,
+		GHashTable *iter_hash)
 {
 	filetype_id ft_id = doc->file_type->id;
 	GtkTreeStore *tree_store = doc->priv->tag_store;
@@ -1109,6 +1290,7 @@ static void add_tree_tag(GeanyDocument *doc, const TMTag *tag, GHashTable *paren
 		GtkTreeIter iter;
 		GtkTreeIter *child = NULL;
  		GdkPixbuf *icon = NULL;
+		gchar *tooltip;
 
  		child = &iter;
 		icon = get_child_icon(tree_store, parent);
@@ -1136,22 +1318,35 @@ static void add_tree_tag(GeanyDocument *doc, const TMTag *tag, GHashTable *paren
 			child = new_iter;
 		}
 
-		gtk_tree_store_append(tree_store, child, parent);
+		if (!find_iter(tree_store, iter_hash, tag, child))
+		{
+			gboolean expand = FALSE;
+
+			/* only expand to the iter if the parent was empty, otherwise we let the
+			 * folding as it was before (already expanded, or closed by the user) */
+			expand = !gtk_tree_model_iter_has_child(GTK_TREE_MODEL(tree_store), parent);
+			gtk_tree_store_append(tree_store, child, parent);
+			if (expand)
+			{
+				GtkTreePath *path;
+
+				path = gtk_tree_model_get_path(GTK_TREE_MODEL(tree_store), child);
+				gtk_tree_view_expand_to_path(GTK_TREE_VIEW(doc->priv->tag_tree), path);
+				gtk_tree_path_free(path);
+			}
+		}
 
 		name = get_symbol_name(doc, tag, (parent_name != NULL));
+		tooltip = get_symbol_tooltip(doc, tag);
 		gtk_tree_store_set(tree_store, child,
 			SYMBOLS_COLUMN_ICON, icon,
 			SYMBOLS_COLUMN_NAME, name,
 			SYMBOLS_COLUMN_TAG, tag,
+			SYMBOLS_COLUMN_VALID, TRUE,
+			SYMBOLS_COLUMN_TOOLTIP, tooltip,
 			-1);
 
-		if (gtk_check_version(2, 12, 0) == NULL)
-		{
-			gchar *tooltip = get_symbol_tooltip(doc, tag);
-			gtk_tree_store_set(tree_store, child, SYMBOLS_COLUMN_TOOLTIP, tooltip, -1);
-			g_free(tooltip);
-		}
-
+		g_free(tooltip);
 		if (G_LIKELY(G_IS_OBJECT(icon)))
 			g_object_unref(icon);
 	}
@@ -1164,9 +1359,11 @@ static void add_tree_tags(GeanyDocument *doc, const GList *tags)
 {
 	const GList *item;
 	GHashTable *parent_hash;
+	GHashTable *iter_hash;
 
 	/* Create a hash table "parent_tag_name":(GtkTreeIter*) */
 	parent_hash = g_hash_table_new_full(g_str_hash, g_str_equal, NULL, g_free);
+	iter_hash = build_iter_table(doc->priv->tag_store);
 
 	/* find and store all parent names in the hash table */
 	for (item = tags; item; item = g_list_next(item))
@@ -1181,8 +1378,9 @@ static void add_tree_tags(GeanyDocument *doc, const GList *tags)
 	{
 		const TMTag *tag = item->data;
 
-		add_tree_tag(doc, tag, parent_hash);
+		add_tree_tag(doc, tag, parent_hash, iter_hash);
 	}
+	g_hash_table_destroy(iter_hash);
 	g_hash_table_destroy(parent_hash);
 }
 
@@ -1223,7 +1421,8 @@ static gint tree_sort_func(GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b,
 		gpointer user_data)
 {
 	gboolean sort_by_name = GPOINTER_TO_INT(user_data);
-	const TMTag *tag_a, *tag_b;
+	TMTag *tag_a, *tag_b;
+	gint cmp;
 
 	gtk_tree_model_get(model, a, SYMBOLS_COLUMN_TAG, &tag_a, -1);
 	gtk_tree_model_get(model, b, SYMBOLS_COLUMN_TAG, &tag_b, -1);
@@ -1233,13 +1432,12 @@ static gint tree_sort_func(GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b,
 	if (tag_a && !tag_has_missing_parent(tag_a, GTK_TREE_STORE(model), a) &&
 		tag_b && !tag_has_missing_parent(tag_b, GTK_TREE_STORE(model), b))
 	{
-		return sort_by_name ? compare_symbol(tag_a, tag_b) :
+		cmp = sort_by_name ? compare_symbol(tag_a, tag_b) :
 			compare_symbol_lines(tag_a, tag_b);
 	}
 	else
 	{
 		gchar *astr, *bstr;
-		gint cmp;
 
 		gtk_tree_model_get(model, a, SYMBOLS_COLUMN_NAME, &astr, -1);
 		gtk_tree_model_get(model, b, SYMBOLS_COLUMN_NAME, &bstr, -1);
@@ -1253,23 +1451,28 @@ static gint tree_sort_func(GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b,
 		{
 			/* this is what g_strcmp0() does */
 			if (! astr)
-				return -(astr != bstr);
+				cmp = -(astr != bstr);
 			if (! bstr)
-				return astr != bstr;
+				cmp = astr != bstr;
+			else
+			{
+				cmp = strcmp(astr, bstr);
 
-			cmp = strcmp(astr, bstr);
-
-			/* sort duplicate 'ScopeName::OverloadedTagName' items by line as well */
-			if (tag_a && tag_b)
-				if (!sort_by_name ||
-					(utils_str_equal(tag_a->name, tag_b->name) &&
-						utils_str_equal(tag_a->atts.entry.scope, tag_b->atts.entry.scope)))
-					cmp = compare_symbol_lines(tag_a, tag_b);
+				/* sort duplicate 'ScopeName::OverloadedTagName' items by line as well */
+				if (tag_a && tag_b)
+					if (!sort_by_name ||
+						(utils_str_equal(tag_a->name, tag_b->name) &&
+							utils_str_equal(tag_a->atts.entry.scope, tag_b->atts.entry.scope)))
+						cmp = compare_symbol_lines(tag_a, tag_b);
+			}
 		}
 		g_free(astr);
 		g_free(bstr);
-		return cmp;
 	}
+	tm_tag_unref(tag_a);
+	tm_tag_unref(tag_b);
+
+	return cmp;
 }
 
 
@@ -1282,6 +1485,53 @@ static void sort_tree(GtkTreeStore *store, gboolean sort_by_name)
 }
 
 
+static void invalidate_rows(GtkTreeStore *store)
+{
+	GtkTreeIter iter;
+	GtkTreeModel *model = GTK_TREE_MODEL(store);
+
+	if (!gtk_tree_model_get_iter_first(model, &iter))
+		return;
+	do
+	{
+		gtk_tree_store_set(store, &iter, SYMBOLS_COLUMN_VALID, FALSE, -1);
+	}
+	while (next_iter(model, &iter));
+}
+
+
+static void remove_invalid_rows(GtkTreeStore *store)
+{
+	GtkTreeIter iter;
+	GtkTreeModel *model = GTK_TREE_MODEL(store);
+	gboolean cont;
+
+	cont = gtk_tree_model_get_iter_first(model, &iter);
+	while (cont)
+	{
+		gboolean valid;
+
+		gtk_tree_model_get(model, &iter, SYMBOLS_COLUMN_VALID, &valid, -1);
+		if (!valid)
+		{
+			GtkTreeIter parent;
+			gboolean have_parent;
+
+			have_parent = gtk_tree_model_iter_parent(model, &parent, &iter);
+			cont = gtk_tree_store_remove(store, &iter);
+			/* if there is no next at this level but there is a parent iter, continue from it */
+			if (!cont && have_parent)
+			{
+				iter = parent;
+				cont = next_iter(model, &iter);
+			}
+		}
+		else
+			cont = next_iter(model, &iter);
+	}
+}
+
+
 gboolean symbols_recreate_tag_list(GeanyDocument *doc, gint sort_mode)
 {
 	GList *tags;
@@ -1292,12 +1542,14 @@ gboolean symbols_recreate_tag_list(GeanyDocument *doc, gint sort_mode)
 	if (tags == NULL)
 		return FALSE;
 
-	/* Make sure the model stays with us after the tree view unrefs it */
-	g_object_ref(GTK_TREE_MODEL(doc->priv->tag_store));
-	/* Detach model from view */
-	gtk_tree_view_set_model(GTK_TREE_VIEW(doc->priv->tag_tree), NULL);
-	/* Clear all contents */
-	gtk_tree_store_clear(doc->priv->tag_store);
+	/* FIXME: Not sure why we detached the model here? */
+
+	/* disable sorting during update because the code doesn't support correctly
+	 * models that are currently being built */
+	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(doc->priv->tag_store), GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID, 0);
+
+	/* invalidate all content */
+	invalidate_rows(doc->priv->tag_store);
 
 	/* add grandparent type iters */
 	add_top_level_items(doc);
@@ -1305,6 +1557,7 @@ gboolean symbols_recreate_tag_list(GeanyDocument *doc, gint sort_mode)
 	add_tree_tags(doc, tags);
 	g_list_free(tags);
 
+	remove_invalid_rows(doc->priv->tag_store);
 	hide_empty_rows(doc->priv->tag_store);
 
 	if (sort_mode == SYMBOLS_SORT_USE_PREVIOUS)
@@ -1313,12 +1566,6 @@ gboolean symbols_recreate_tag_list(GeanyDocument *doc, gint sort_mode)
 	sort_tree(doc->priv->tag_store, sort_mode == SYMBOLS_SORT_BY_NAME);
 	doc->priv->symbol_list_sort_mode = sort_mode;
 
-	/* Re-attach model to view */
-	gtk_tree_view_set_model(GTK_TREE_VIEW(doc->priv->tag_tree),
-		GTK_TREE_MODEL(doc->priv->tag_store));
-	g_object_unref(GTK_TREE_MODEL(doc->priv->tag_store));
-	gtk_tree_view_expand_all(GTK_TREE_VIEW(doc->priv->tag_tree));
-
 	return TRUE;
 }
 
@@ -1385,8 +1632,7 @@ int symbols_generate_global_tags(int argc, char **argv, gboolean want_preprocess
 
 		geany_debug("Generating %s tags file.", ft->name);
 		tm_get_workspace();
-		status = tm_workspace_create_global_tags(app->configdir, command,
-												 (const char **) (argv + 2),
+		status = tm_workspace_create_global_tags(command, (const char **) (argv + 2),
 												 argc - 2, tags_file, ft->lang);
 		g_free(command);
 		symbols_finalize(); /* free c_tags_ignore data */
@@ -1487,7 +1733,7 @@ static GHashTable *init_user_tags(void)
 {
 	GSList *file_list = NULL, *list = NULL;
 	GHashTable *lang_hash;
-	const gchar *dir;
+	gchar *dir;
 
 	dir = utils_build_path(app->configdir, "tags", NULL);
 	/* create the user tags dir for next time if it doesn't exist */
@@ -1497,8 +1743,10 @@ static GHashTable *init_user_tags(void)
 	}
 	file_list = utils_get_file_list_full(dir, TRUE, TRUE, NULL);
 
-	dir = utils_build_path(app->datadir, "tags", NULL);
-	list =  utils_get_file_list_full(dir, TRUE, TRUE, NULL);
+	setptr(dir, utils_build_path(app->datadir, "tags", NULL));
+	list = utils_get_file_list_full(dir, TRUE, TRUE, NULL);
+	g_free(dir);
+
 	file_list = g_slist_concat(file_list, list);
 
 	lang_hash = get_tagfile_hash(file_list);
@@ -1893,12 +2141,12 @@ static void create_taglist_popup_menu(void)
 	symbol_menu.expand_all = item = ui_image_menu_item_new(GTK_STOCK_ADD, _("_Expand All"));
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
-	g_signal_connect(item, "activate", G_CALLBACK(on_expand_collapse), GTK_STOCK_ADD);
+	g_signal_connect(item, "activate", G_CALLBACK(on_expand_collapse), (gpointer)GTK_STOCK_ADD);
 
 	symbol_menu.collapse_all = item = ui_image_menu_item_new(GTK_STOCK_REMOVE, _("_Collapse All"));
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
-	g_signal_connect(item, "activate", G_CALLBACK(on_expand_collapse), GTK_STOCK_REMOVE);
+	g_signal_connect(item, "activate", G_CALLBACK(on_expand_collapse), (gpointer)GTK_STOCK_REMOVE);
 
 	item = gtk_separator_menu_item_new();
 	gtk_widget_show(item);
@@ -1926,20 +2174,27 @@ static void create_taglist_popup_menu(void)
 
 static void on_document_save(G_GNUC_UNUSED GObject *object, GeanyDocument *doc)
 {
+	gchar *f = utils_build_path(app->configdir, "ignore.tags", NULL);
+
 	g_return_if_fail(NZV(doc->real_path));
 
-	if (utils_str_equal(doc->real_path,
-		utils_build_path(app->configdir, "ignore.tags", NULL)))
+	if (utils_str_equal(doc->real_path, f))
 		load_c_ignore_tags();
+
+	g_free(f);
 }
 
 
 void symbols_init(void)
 {
+	gchar *f;
+
 	create_taglist_popup_menu();
 
-	ui_add_config_file_menu_item(utils_build_path(app->configdir, "ignore.tags", NULL),
-		NULL, NULL);
+	f = utils_build_path(app->configdir, "ignore.tags", NULL);
+	ui_add_config_file_menu_item(f, NULL, NULL);
+	g_free(f);
+
 	g_signal_connect(geany_object, "document-save", G_CALLBACK(on_document_save), NULL);
 }
 
