@@ -1,8 +1,8 @@
 /*
  *      templates.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,11 +18,11 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: templates.c 4917 2010-05-15 13:14:38Z eht16 $
+ * $Id: templates.c 5872 2011-07-28 17:59:40Z colombanw $
  */
 
 /*
- * Templates to insert into the current document, or filetype templates to create a new
+ * Templates to insert into the current document, or file templates to create a new
  * document from.
  */
 
@@ -35,6 +35,8 @@
 #include "support.h"
 #include "utils.h"
 #include "document.h"
+#include "encodings.h"
+#include "editor.h"
 #include "filetypes.h"
 #include "ui_utils.h"
 #include "toolbar.h"
@@ -46,86 +48,9 @@ GeanyTemplatePrefs template_prefs;
 
 static GtkWidget *new_with_template_menu = NULL;	/* submenu used for both file menu and toolbar */
 
-
-/* TODO: implement custom insertion templates, put these into files in data/templates */
-
-/* default templates, only for initial tempate file creation on first start of Geany */
-static const gchar templates_gpl_notice[] = "\
-This program is free software; you can redistribute it and/or modify\n\
-it under the terms of the GNU General Public License as published by\n\
-the Free Software Foundation; either version 2 of the License, or\n\
-(at your option) any later version.\n\
-\n\
-This program is distributed in the hope that it will be useful,\n\
-but WITHOUT ANY WARRANTY; without even the implied warranty of\n\
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n\
-GNU General Public License for more details.\n\
-\n\
-You should have received a copy of the GNU General Public License\n\
-along with this program; if not, write to the Free Software\n\
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,\n\
-MA 02110-1301, USA.\n\
-";
-
-static const gchar templates_bsd_notice[] = "\
-Redistribution and use in source and binary forms, with or without\n\
-modification, are permitted provided that the following conditions are\n\
-met:\n\
-\n\
-* Redistributions of source code must retain the above copyright\n\
-  notice, this list of conditions and the following disclaimer.\n\
-* Redistributions in binary form must reproduce the above\n\
-  copyright notice, this list of conditions and the following disclaimer\n\
-  in the documentation and/or other materials provided with the\n\
-  distribution.\n\
-* Neither the name of the {company} nor the names of its\n\
-  contributors may be used to endorse or promote products derived from\n\
-  this software without specific prior written permission.\n\
-\n\
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS\n\
-\"AS IS\" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT\n\
-LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR\n\
-A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT\n\
-OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,\n\
-SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT\n\
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,\n\
-DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY\n\
-THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT\n\
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE\n\
-OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.\n\
-";
-
-static const gchar templates_function_description[] = "\
-\n\
-name: {functionname}\n\
-@param\n\
-@return\n\
-";
-
-static const gchar templates_multiline[] = "\
- \n\
- \n\
-";
-
-static const gchar templates_fileheader[] = "\
-{filename}\n\
-\n\
-Copyright {year} {developer} <{mail}>\n\
-\n\
-{gpl}\
-";
-
-static const gchar templates_changelog[] = "\
-{date}  {developer}  <{mail}>\n\
-\n\
- * \n\n\n";
-
+/* TODO: implement custom insertion templates instead? */
 static gchar *templates[GEANY_MAX_TEMPLATES];
 
-/* We should probably remove filetype templates support soon - users can use custom
- * file templates instead. */
-static gchar *ft_templates[GEANY_MAX_BUILT_IN_FILETYPES] = {NULL};
-
 
 static void replace_static_values(GString *text);
 static gchar *get_template_fileheader(GeanyFiletype *ft);
@@ -136,158 +61,72 @@ static void templates_replace_command(GString *text, const gchar *file_name,
 	const gchar *file_type, const gchar *func_name);
 
 
-/* some simple macros to reduce code size and make the code readable */
-#define TEMPLATES_GET_FILENAME(shortname) \
-	g_strconcat(app->configdir, \
-		G_DIR_SEPARATOR_S GEANY_TEMPLATES_SUBDIR G_DIR_SEPARATOR_S, shortname, NULL)
-
-#define TEMPLATES_READ_FILE(fname, contents_ptr) \
-	g_file_get_contents(fname, contents_ptr, NULL, NULL);
-
-
-static void create_template_file_if_necessary(const gchar *filename, const gchar *content)
-{
-	if (! g_file_test(filename, G_FILE_TEST_EXISTS))
-	{
-		if (file_prefs.default_eol_character != SC_EOL_LF)
-		{
-			/* Replace the \n characters in the default template text by the proper
-			 * platform-specific line ending characters. */
-			GString *tmp = g_string_new(content);
-			const gchar *eol_str = (file_prefs.default_eol_character == SC_EOL_CR) ? "\r" : "\r\n";
-
-			utils_string_replace_all(tmp, "\n", eol_str);
-			utils_write_file(filename, tmp->str);
-			g_string_free(tmp, TRUE);
-		}
-		else
-			utils_write_file(filename, content);
-	}
-}
-
-
-/* FIXME the callers should use GStrings instead of char arrays */
-static gchar *replace_all(gchar *text, const gchar *year, const gchar *date, const gchar *datetime)
+static gchar *read_file(const gchar *locale_fname)
 {
+	gchar *contents;
+	gsize length;
 	GString *str;
 
-	if (text == NULL)
+	if (! g_file_get_contents(locale_fname, &contents, &length, NULL))
 		return NULL;
 
-	str = g_string_new(text);
+	if (! encodings_convert_to_utf8_auto(&contents, &length, NULL, NULL, NULL, NULL))
+	{
+		gchar *utf8_fname = utils_get_utf8_from_locale(locale_fname);
 
-	g_free(text);
-	templates_replace_valist(str,
-		"{year}", year,
-		"{date}", date,
-		"{datetime}", datetime,
-		NULL);
+		ui_set_statusbar(TRUE, _("Failed to convert template file \"%s\" to UTF-8"), utf8_fname);
+		g_free(utf8_fname);
+		g_free(contents);
+		return NULL;
+	}
 
+	str = g_string_new(contents);
+	g_free(contents);
+
+	/* convert to LF endings for consistency in mixing templates */
+	utils_ensure_same_eol_characters(str, SC_EOL_LF);
 	return g_string_free(str, FALSE);
 }
 
 
-static void init_general_templates(const gchar *year, const gchar *date, const gchar *datetime)
+static void read_template(const gchar *name, gint id)
 {
-	gchar *template_filename_fileheader = TEMPLATES_GET_FILENAME("fileheader");
-	gchar *template_filename_gpl = TEMPLATES_GET_FILENAME("gpl");
-	gchar *template_filename_bsd = TEMPLATES_GET_FILENAME("bsd");
-	gchar *template_filename_function = TEMPLATES_GET_FILENAME("function");
-	gchar *template_filename_changelog = TEMPLATES_GET_FILENAME("changelog");
+	gchar *fname = g_build_path(G_DIR_SEPARATOR_S, app->configdir,
+		GEANY_TEMPLATES_SUBDIR, name, NULL);
 
-	/* create the template files in the configuration directory, if they don't exist */
-	create_template_file_if_necessary(template_filename_fileheader, templates_fileheader);
-	create_template_file_if_necessary(template_filename_gpl, templates_gpl_notice);
-	create_template_file_if_necessary(template_filename_bsd, templates_bsd_notice);
-	create_template_file_if_necessary(template_filename_function, templates_function_description);
-	create_template_file_if_necessary(template_filename_changelog, templates_changelog);
+	/* try system if user template doesn't exist */
+	if (!g_file_test(fname, G_FILE_TEST_EXISTS))
+		setptr(fname, g_build_path(G_DIR_SEPARATOR_S, app->datadir,
+			GEANY_TEMPLATES_SUBDIR, name, NULL));
 
+	templates[id] = read_file(fname);
+	g_free(fname);
+}
+
+
+/* called when inserting templates into an existing document */
+static void convert_eol_characters(GString *template, GeanyDocument *doc)
+{
+	gint doc_eol_mode;
+
+	if (doc == NULL)
+		doc = document_get_current();
+
+	g_return_if_fail(doc != NULL);
+
+	doc_eol_mode = editor_get_eol_char_mode(doc->editor);
+	utils_ensure_same_eol_characters(template, doc_eol_mode);
+}
+
+
+static void init_general_templates(void)
+{
 	/* read the contents */
-	TEMPLATES_READ_FILE(template_filename_fileheader, &templates[GEANY_TEMPLATE_FILEHEADER]);
-	templates[GEANY_TEMPLATE_FILEHEADER] = replace_all(templates[GEANY_TEMPLATE_FILEHEADER], year, date, datetime);
-
-	TEMPLATES_READ_FILE(template_filename_gpl, &templates[GEANY_TEMPLATE_GPL]);
-	templates[GEANY_TEMPLATE_GPL] = replace_all(templates[GEANY_TEMPLATE_GPL], year, date, datetime);
-
-	TEMPLATES_READ_FILE(template_filename_bsd, &templates[GEANY_TEMPLATE_BSD]);
-	templates[GEANY_TEMPLATE_BSD] = replace_all(templates[GEANY_TEMPLATE_BSD], year, date, datetime);
-
-	TEMPLATES_READ_FILE(template_filename_function, &templates[GEANY_TEMPLATE_FUNCTION]);
-	templates[GEANY_TEMPLATE_FUNCTION] = replace_all(templates[GEANY_TEMPLATE_FUNCTION], year, date, datetime);
-
-	TEMPLATES_READ_FILE(template_filename_changelog, &templates[GEANY_TEMPLATE_CHANGELOG]);
-	templates[GEANY_TEMPLATE_CHANGELOG] = replace_all(templates[GEANY_TEMPLATE_CHANGELOG], year, date, datetime);
-
-	/* free the whole stuff */
-	g_free(template_filename_fileheader);
-	g_free(template_filename_gpl);
-	g_free(template_filename_bsd);
-	g_free(template_filename_function);
-	g_free(template_filename_changelog);
-}
-
-
-static void init_ft_templates(const gchar *year, const gchar *date, const gchar *datetime)
-{
-	filetype_id ft_id;
-
-	for (ft_id = 0; ft_id < GEANY_MAX_BUILT_IN_FILETYPES; ft_id++)
-	{
-		gchar *ext = filetypes_get_conf_extension(ft_id);
-		gchar *shortname = g_strconcat("filetype.", ext, NULL);
-		gchar *fname = TEMPLATES_GET_FILENAME(shortname);
-
-		TEMPLATES_READ_FILE(fname, &ft_templates[ft_id]);
-		ft_templates[ft_id] = replace_all(ft_templates[ft_id], year, date, datetime);
-
-		g_free(fname);
-		g_free(shortname);
-		g_free(ext);
-	}
-}
-
-
-static void
-on_new_with_filetype_template(GtkMenuItem *menuitem, gpointer user_data)
-{
-	GeanyFiletype *ft = user_data;
-	gchar *template = templates_get_template_new_file(ft);
-
-	document_new_file(NULL, ft, template);
-	g_free(template);
-}
-
-
-/* TODO: remove filetype template support after 0.19 */
-static gboolean create_new_filetype_items(void)
-{
-	GSList *node;
-	gboolean ret = FALSE;
-	GtkWidget *menu = NULL;
-
-	foreach_slist(node, filetypes_by_title)
-	{
-		GeanyFiletype *ft = node->data;
-		GtkWidget *item;
-
-		if (ft->id >= GEANY_MAX_BUILT_IN_FILETYPES || ft_templates[ft->id] == NULL)
-			continue;
-
-		if (!menu)
-		{
-			item = gtk_menu_item_new_with_label(_("Old"));
-			menu = gtk_menu_new();
-			gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), menu);
-			gtk_widget_show_all(item);
-			gtk_container_add(GTK_CONTAINER(new_with_template_menu), item);
-		}
-		item = gtk_menu_item_new_with_label(ft->title);
-		gtk_widget_show(item);
-		gtk_container_add(GTK_CONTAINER(menu), item);
-		g_signal_connect(item, "activate", G_CALLBACK(on_new_with_filetype_template), ft);
-		ret = TRUE;
-	}
-	return ret;
+	read_template("fileheader", GEANY_TEMPLATE_FILEHEADER);
+	read_template("gpl", GEANY_TEMPLATE_GPL);
+	read_template("bsd", GEANY_TEMPLATE_BSD);
+	read_template("function", GEANY_TEMPLATE_FUNCTION);
+	read_template("changelog", GEANY_TEMPLATE_CHANGELOG);
 }
 
 
@@ -330,7 +169,7 @@ static gchar *get_template_from_file(const gchar *locale_fname, const gchar *doc
 	gchar *content;
 	GString *template = NULL;
 
-	g_file_get_contents(locale_fname, &content, NULL, NULL);
+	content = read_file(locale_fname);
 
 	if (content != NULL)
 	{
@@ -377,7 +216,10 @@ on_new_with_file_template(GtkMenuItem *menuitem, G_GNUC_UNUSED gpointer user_dat
 		template = get_template_from_file(path, new_filename, ft);
 	}
 	if (template)
+	{
+		/* line endings will be converted */
 		document_new_file(new_filename, ft, template);
+	}
 	else
 	{
 		setptr(fname, utils_get_utf8_from_locale(fname));
@@ -428,19 +270,9 @@ static gboolean add_custom_template_items(void)
 
 static void create_file_template_menu(void)
 {
-	GtkWidget *sep = NULL;
-
 	new_with_template_menu = gtk_menu_new();
+	add_custom_template_items();
 
-	if (add_custom_template_items())
-	{
-		sep = gtk_separator_menu_item_new();
-		gtk_container_add(GTK_CONTAINER(new_with_template_menu), sep);
-	}
-	if (create_new_filetype_items() && sep)
-	{
-		gtk_widget_show(sep);
-	}
 	/* unless the file menu is showing, menu should be in the toolbar widget */
 	geany_menu_button_action_set_menu(GEANY_MENU_BUTTON_ACTION(
 		toolbar_get_action_by_name("New")), new_with_template_menu);
@@ -468,7 +300,7 @@ static void on_file_menu_hide(GtkWidget *item)
 /* reload templates if any file in the templates path is saved */
 static void on_document_save(G_GNUC_UNUSED GObject *object, GeanyDocument *doc)
 {
-	const gchar *path = utils_build_path(app->configdir, GEANY_TEMPLATES_SUBDIR, NULL);
+	gchar *path = utils_build_path(app->configdir, GEANY_TEMPLATES_SUBDIR, NULL);
 
 	g_return_if_fail(NZV(doc->real_path));
 
@@ -478,23 +310,16 @@ static void on_document_save(G_GNUC_UNUSED GObject *object, GeanyDocument *doc)
 		templates_free_templates();
 		templates_init();
 	}
+	g_free(path);
 }
 
 
 /* warning: also called when reloading template settings */
 void templates_init(void)
 {
-	gchar *year = utils_get_date_time(template_prefs.year_format, NULL);
-	gchar *date = utils_get_date_time(template_prefs.date_format, NULL);
-	gchar *datetime = utils_get_date_time(template_prefs.datetime_format, NULL);
 	static gboolean init_done = FALSE;
 
-	init_general_templates(year, date, datetime);
-	init_ft_templates(year, date, datetime);
-
-	g_free(date);
-	g_free(datetime);
-	g_free(year);
+	init_general_templates();
 
 	create_file_template_menu();
 	/* we hold our own ref for the menu as it has no parent whilst being moved */
@@ -520,40 +345,52 @@ void templates_init(void)
  * e.g. indent = 8 prints " *     here comes the text of the line"
  * indent is meant to be the whole amount of characters before the real line content follows, i.e.
  * 6 characters are filled with whitespace when the comment characters include " *" */
-/* TODO make this function operating on a GString */
-static gchar *make_comment_block(const gchar *comment_text, gint filetype_idx, guint indent)
+static void make_comment_block(GString *comment_text, gint filetype_idx, guint indent)
 {
 	gchar *frame_start;			/* to add before comment_text */
 	gchar *frame_end;			/* to add after comment_text */
 	const gchar *line_prefix;	/* to add before every line in comment_text */
-	gchar *result;
 	gchar *tmp;
 	gchar *prefix;
 	gchar **lines;
 	guint i, len;
+	gint template_eol_mode;
+	const gchar *template_eol_char;
 	GeanyFiletype *ft = filetypes_index(filetype_idx);
+	const gchar *co;
+	const gchar *cc;
 
-	g_return_val_if_fail(ft != NULL, NULL);
+	g_return_if_fail(comment_text != NULL);
+	g_return_if_fail(ft != NULL);
 
-	if (NZV(ft->comment_open))
+	template_eol_mode = utils_get_line_endings(comment_text->str, comment_text->len);
+	template_eol_char = utils_get_eol_char(template_eol_mode);
+
+	co = ft->comment_open;
+	cc = NULL;
+	if (NZV(co))
+		cc = ft->comment_close;
+	else
+		co = ft->comment_single;
+	if (NZV(co))
 	{
-		if (NZV(ft->comment_close))
+		if (NZV(cc))
 		{
-			frame_start = g_strconcat(ft->comment_open, "\n", NULL);
-			frame_end = g_strconcat(ft->comment_close, "\n", NULL);
+			frame_start = g_strconcat(co, template_eol_char, NULL);
+			frame_end = g_strconcat(cc, template_eol_char, NULL);
 			line_prefix = "";
 		}
 		else
 		{
 			frame_start = NULL;
 			frame_end = NULL;
-			line_prefix = ft->comment_open;
+			line_prefix = co;
 		}
 	}
 	else
 	{	/* use C-like multi-line comments as fallback */
-		frame_start = g_strdup("/*\n");
-		frame_end = g_strdup("*/\n");
+		frame_start = g_strconcat("/*", template_eol_char, NULL);
+		frame_end = g_strconcat("*/", template_eol_char, NULL);
 		line_prefix = "";
 	}
 
@@ -572,32 +409,39 @@ static gchar *make_comment_block(const gchar *comment_text, gint filetype_idx, g
 	g_free(tmp);
 
 	/* add line_prefix to every line of comment_text */
-	lines = g_strsplit(comment_text, "\n", -1);
-	len = g_strv_length(lines) - 1;
-	for (i = 0; i < len; i++)
+	lines = g_strsplit(comment_text->str, template_eol_char, -1);
+	len = g_strv_length(lines);
+	if (len > 0)	/* prevent unsigned wraparound if comment_text is empty */
 	{
-		tmp = lines[i];
-		lines[i] = g_strconcat(prefix, tmp, NULL);
-		g_free(tmp);
+		for (i = 0; i < len - 1; i++)
+		{
+			tmp = lines[i];
+			lines[i] = g_strconcat(prefix, tmp, NULL);
+			g_free(tmp);
+		}
 	}
-	tmp = g_strjoinv("\n", lines);
+	tmp = g_strjoinv(template_eol_char, lines);
 
-	/* add frame_start and frame_end */
+	/* clear old contents */
+	g_string_erase(comment_text, 0, -1);
+
+	/* add frame_end */
 	if (frame_start != NULL)
-		result = g_strconcat(frame_start, tmp, frame_end, NULL);
-	else
-		result = g_strconcat(tmp, frame_end, NULL);
+		g_string_append(comment_text, frame_start);
+	/* add the new main content */
+	g_string_append(comment_text, tmp);
+	/* add frame_start  */
+	if (frame_end != NULL)
+		g_string_append(comment_text, frame_end);
 
 	utils_free_pointers(4, prefix, tmp, frame_start, frame_end, NULL);
 	g_strfreev(lines);
-	return result;
 }
 
 
 gchar *templates_get_template_licence(GeanyDocument *doc, gint licence_type)
 {
 	GString *template;
-	gchar *result = NULL;
 
 	g_return_val_if_fail(doc != NULL, NULL);
 	g_return_val_if_fail(licence_type == GEANY_TEMPLATE_GPL || licence_type == GEANY_TEMPLATE_BSD, NULL);
@@ -607,18 +451,16 @@ gchar *templates_get_template_licence(GeanyDocument *doc, gint licence_type)
 	templates_replace_default_dates(template);
 	templates_replace_command(template, DOC_FILENAME(doc), doc->file_type->name, NULL);
 
-	result = make_comment_block(template->str, FILETYPE_ID(doc->file_type), 8);
+	make_comment_block(template, doc->file_type->id, GEANY_TEMPLATES_INDENT);
+	convert_eol_characters(template, doc);
 
-	g_string_free(template, TRUE);
-
-	return result;
+	return g_string_free(template, FALSE);
 }
 
 
 static gchar *get_template_fileheader(GeanyFiletype *ft)
 {
 	GString *template = g_string_new(templates[GEANY_TEMPLATE_FILEHEADER]);
-	gchar *result;
 
 	filetypes_load_config(ft->id, FALSE);	/* load any user extension setting */
 
@@ -628,9 +470,8 @@ static gchar *get_template_fileheader(GeanyFiletype *ft)
 		NULL);
 
 	/* we don't replace other wildcards here otherwise they would get done twice for files */
-	result = make_comment_block(template->str, ft->id, 8);
-	g_string_free(template, TRUE);
-	return result;
+	make_comment_block(template, ft->id, GEANY_TEMPLATES_INDENT);
+	return g_string_free(template, FALSE);
 }
 
 
@@ -643,38 +484,13 @@ gchar *templates_get_template_fileheader(gint filetype_idx, const gchar *fname)
 
 	g_free(str);
 	templates_replace_common(template, fname, ft, NULL);
+	convert_eol_characters(template, NULL);
 	return g_string_free(template, FALSE);
 }
 
 
-gchar *templates_get_template_new_file(GeanyFiletype *ft)
-{
-	GString *ft_template;
-	gchar *file_header = NULL;
-
-	g_return_val_if_fail(ft != NULL, NULL);
-	g_return_val_if_fail(ft->id < GEANY_MAX_BUILT_IN_FILETYPES, NULL);
-
-	ft_template = g_string_new(ft_templates[ft->id]);
-	if (FILETYPE_ID(ft) == GEANY_FILETYPES_NONE)
-	{
-		replace_static_values(ft_template);
-	}
-	else
-	{	/* file template only used for new files */
-		file_header = get_template_fileheader(ft);
-		templates_replace_valist(ft_template, "{fileheader}", file_header, NULL);
-	}
-	templates_replace_common(ft_template, NULL, ft, NULL);
-
-	g_free(file_header);
-	return g_string_free(ft_template, FALSE);
-}
-
-
 gchar *templates_get_template_function(GeanyDocument *doc, const gchar *func_name)
 {
-	gchar *result;
 	GString *text;
 
 	func_name = (func_name != NULL) ? func_name : "";
@@ -684,10 +500,10 @@ gchar *templates_get_template_function(GeanyDocument *doc, const gchar *func_nam
 	templates_replace_default_dates(text);
 	templates_replace_command(text, DOC_FILENAME(doc), doc->file_type->name, func_name);
 
-	result = make_comment_block(text->str, doc->file_type->id, 3);
+	make_comment_block(text, doc->file_type->id, GEANY_TEMPLATES_INDENT);
+	convert_eol_characters(text, doc);
 
-	g_string_free(text, TRUE);
-	return result;
+	return g_string_free(text, FALSE);
 }
 
 
@@ -699,6 +515,7 @@ gchar *templates_get_template_changelog(GeanyDocument *doc)
 	replace_static_values(result);
 	templates_replace_default_dates(result);
 	templates_replace_command(result, DOC_FILENAME(doc), file_type_name, NULL);
+	convert_eol_characters(result, doc);
 
 	return g_string_free(result, FALSE);
 }
@@ -718,10 +535,6 @@ void templates_free_templates(void)
 	{
 		g_free(templates[i]);
 	}
-	for (i = 0; i < GEANY_MAX_BUILT_IN_FILETYPES; i++)
-	{
-		g_free(ft_templates[i]);
-	}
 	/* destroy "New with template" sub menu items (in case we want to reload the templates) */
 	children = gtk_container_get_children(GTK_CONTAINER(new_with_template_menu));
 	foreach_list(item, children)
@@ -855,6 +668,7 @@ static void templates_replace_command(GString *text, const gchar *file_name,
 		result = run_command(cmd, file_name, file_type, func_name);
 		if (result != NULL)
 		{
+			result = g_strstrip(result);
 			utils_string_replace_first(text, wildcard, result);
 			g_free(result);
 		}
