@@ -1,8 +1,8 @@
 /*
  *      templates.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: templates.h 4852 2010-04-22 13:01:45Z ntrel $
+ * $Id: templates.h 5674 2011-04-01 16:27:38Z ntrel $
  */
 
 /**
@@ -30,6 +30,8 @@
 #ifndef GEANY_TEMPLATES_H
 #define GEANY_TEMPLATES_H 1
 
+#define GEANY_TEMPLATES_INDENT 3
+
 enum
 {
 	GEANY_TEMPLATE_GPL = 0,
@@ -64,8 +66,6 @@ void templates_init(void);
 
 gchar *templates_get_template_fileheader(gint filetype_idx, const gchar *fname);
 
-gchar *templates_get_template_new_file(GeanyFiletype *ft);
-
 gchar *templates_get_template_changelog(GeanyDocument *doc);
 
 gchar *templates_get_template_function(GeanyDocument *doc, const gchar *func_name);
