@@ -1,8 +1,8 @@
 /*
  *      toolbar.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2009-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2009-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2009-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2009-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: toolbar.c 4941 2010-05-22 19:39:03Z eht16 $
+ * $Id: toolbar.c 5721 2011-04-13 12:59:20Z ntrel $
  */
 
 /**
@@ -54,6 +54,7 @@ static GSList *plugin_items = NULL;
 const GtkActionEntry ui_entries[] = {
 	/* custom actions defined in toolbar_init(): "New", "Open", "SearchEntry", "GotoEntry", "Build" */
 	{ "Save", GTK_STOCK_SAVE, NULL, NULL, N_("Save the current file"), G_CALLBACK(on_toolbutton_save_clicked) },
+	{ "SaveAs", GTK_STOCK_SAVE_AS, NULL, NULL, N_("Save as"), G_CALLBACK(on_save_as1_activate) },
 	{ "SaveAll", GEANY_STOCK_SAVE_ALL, NULL, NULL, N_("Save all open files"), G_CALLBACK(on_save_all1_activate) },
 	{ "Reload", GTK_STOCK_REVERT_TO_SAVED, NULL, NULL, N_("Reload the current file from disk"), G_CALLBACK(on_toolbutton_reload_clicked) },
 	{ "Close", GTK_STOCK_CLOSE, NULL, NULL, N_("Close the current file"), G_CALLBACK(on_toolbutton_close_clicked) },
@@ -176,7 +177,7 @@ static GtkWidget *toolbar_reload(const gchar *markup)
 	GSList *l;
 	GtkWidget *entry;
 	GError *error = NULL;
-	const gchar *filename;
+	gchar *filename;
 	static guint merge_id = 0;
 	GtkWidget *toolbar_new_file_menu = NULL;
 	GtkWidget *toolbar_recent_files_menu = NULL;
@@ -225,9 +226,10 @@ static GtkWidget *toolbar_reload(const gchar *markup)
 			g_error_free(error);
 			error = NULL;
 
-			filename = utils_build_path(app->datadir, "ui_toolbar.xml", NULL);
+			setptr(filename, utils_build_path(app->datadir, "ui_toolbar.xml", NULL));
 			merge_id = gtk_ui_manager_add_ui_from_file(uim, filename, &error);
 		}
+		g_free(filename);
 	}
 	if (error != NULL)
 	{
@@ -312,7 +314,6 @@ static GtkWidget *toolbar_reload(const gchar *markup)
 	if (entry != NULL)
 		g_signal_connect(entry, "motion-notify-event", G_CALLBACK(on_motion_event), NULL);
 
-
 	return main_widgets.toolbar;
 }
 
@@ -380,9 +381,11 @@ GtkWidget *toolbar_init(void)
 	action_searchentry = geany_entry_action_new(
 		"SearchEntry", _("Search"), _("Find the entered text in the current file"), FALSE);
 	g_signal_connect(action_searchentry, "entry-activate",
-		G_CALLBACK(on_toolbar_search_entry_changed), GINT_TO_POINTER(FALSE));
+		G_CALLBACK(on_toolbar_search_entry_activate), GINT_TO_POINTER(FALSE));
+	g_signal_connect(action_searchentry, "entry-activate-backward",
+		G_CALLBACK(on_toolbar_search_entry_activate), GINT_TO_POINTER(TRUE));
 	g_signal_connect(action_searchentry, "entry-changed",
-		G_CALLBACK(on_toolbar_search_entry_changed), GINT_TO_POINTER(TRUE));
+		G_CALLBACK(on_toolbar_search_entry_changed), NULL);
 	gtk_action_group_add_action(group, action_searchentry);
 
 	action_gotoentry = geany_entry_action_new(
@@ -479,7 +482,8 @@ void toolbar_update_ui(void)
 	/* we need to adjust the packing flags for the menubar to expand it if it is alone in the
 	 * hbox and not expand it if the toolbar is appended */
 	gtk_box_set_child_packing(GTK_BOX(hbox_menubar), menubar,
-		! toolbar_prefs.append_to_menu, ! toolbar_prefs.append_to_menu, 0, GTK_PACK_START);
+		! (toolbar_prefs.visible && toolbar_prefs.append_to_menu),
+		! (toolbar_prefs.visible && toolbar_prefs.append_to_menu), 0, GTK_PACK_START);
 }
 
 
@@ -789,7 +793,7 @@ static void tb_editor_drag_data_get_cb(GtkWidget *widget, GdkDragContext *contex
 		return;
 
 	gtk_tree_model_get(model, &iter, TB_EDITOR_COL_ACTION, &name, -1);
-	if (! NZV(name))
+	if (G_UNLIKELY(! NZV(name)))
 		return;
 
 	atom = gdk_atom_intern(tb_editor_dnd_targets[0].target, FALSE);
@@ -865,7 +869,7 @@ static gboolean tb_editor_foreach_used(GtkTreeModel *model, GtkTreePath *path,
 
 	if (utils_str_equal(action_name, TB_EDITOR_SEPARATOR))
 		g_string_append_printf(data, "\t\t<separator/>\n");
-	else if (NZV(action_name))
+	else if (G_LIKELY(NZV(action_name)))
 		g_string_append_printf(data, "\t\t<toolitem action='%s' />\n", action_name);
 
 	g_free(action_name);
@@ -887,7 +891,7 @@ editor in Geany.\n\n\
 A list of available actions can be found in the documentation included with Geany or\n\
 at http://www.geany.org/manual/current/index.html#customizing-the-toolbar.\n-->\n\
 \t<toolbar name='GeanyToolbar'>\n";
-	const gchar *filename = utils_build_path(app->configdir, "ui_toolbar.xml", NULL);
+	gchar *filename;
 	GString *str = g_string_new(template);
 
 	gtk_tree_model_foreach(GTK_TREE_MODEL(tbw->store_used), tb_editor_foreach_used, str);
@@ -896,7 +900,9 @@ at http://www.geany.org/manual/current/index.html#customizing-the-toolbar.\n-->\
 
 	toolbar_reload(str->str);
 
+	filename = utils_build_path(app->configdir, "ui_toolbar.xml", NULL);
 	utils_write_file(filename, str->str);
+	g_free(filename);
 
 	g_string_free(str, TRUE);
 }
