@@ -1,8 +1,8 @@
 /*
  *      tools.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2006-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: tools.h 3446 2009-01-04 18:30:42Z eht16 $
+ * $Id: tools.h 5528 2011-01-19 19:39:09Z eht16 $
  */
 
 
@@ -29,9 +29,8 @@ void tools_create_insert_custom_command_menu_items(void);
 
 void tools_execute_custom_command(GeanyDocument *doc, const gchar *command);
 
-
 void tools_word_count(void);
 
-void tools_color_chooser(gchar *color);
+void tools_color_chooser(const gchar *color);
 
 #endif
