@@ -1,8 +1,8 @@
 /*
  *      ui_utils.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2006-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: ui_utils.c 5383 2010-11-08 13:13:33Z ntrel $
+ * $Id: ui_utils.c 5944 2011-09-22 11:52:58Z ntrel $
  */
 
 /** @file ui_utils.h
@@ -661,7 +661,7 @@ void ui_save_buttons_toggle(gboolean enable)
 	gboolean dirty_tabs = FALSE;
 
 	if (ui_prefs.allow_always_save)
-		return;
+		enable = gtk_notebook_get_n_pages(GTK_NOTEBOOK(main_widgets.notebook)) > 0 ? TRUE : FALSE;
 
 	ui_widget_set_sensitive(widgets.save_buttons[0], enable);
 	ui_widget_set_sensitive(widgets.save_buttons[1], enable);
@@ -720,6 +720,7 @@ static void init_document_widgets(void)
 	add_doc_widget("menu_reload_as1");
 	add_doc_widget("menu_select_all1");
 	add_doc_widget("insert_date1");
+	add_doc_widget("insert_alternative_white_space1");
 	add_doc_widget("menu_format1");
 	add_doc_widget("commands2");
 	add_doc_widget("menu_open_selected_file1");
@@ -727,14 +728,18 @@ static void init_document_widgets(void)
 	add_doc_widget("find1");
 	add_doc_widget("find_next1");
 	add_doc_widget("find_previous1");
+	add_doc_widget("go_to_next_marker1");
+	add_doc_widget("go_to_previous_marker1");
 	add_doc_widget("replace1");
 	add_doc_widget("find_nextsel1");
 	add_doc_widget("find_prevsel1");
 	add_doc_widget("find_usage1");
 	add_doc_widget("find_document_usage1");
+	add_doc_widget("mark_all1");
 	add_doc_widget("go_to_line1");
 	add_doc_widget("goto_tag_definition1");
 	add_doc_widget("goto_tag_declaration1");
+	add_doc_widget("reset_indentation1");
 	add_doc_toolitem("Close");
 	add_doc_toolitem("CloseAll");
 	add_doc_toolitem("Search");
@@ -750,6 +755,7 @@ static void init_document_widgets(void)
 	add_doc_toolitem("Paste");
 	add_doc_toolitem("Delete");
 	add_doc_toolitem("Save");
+	add_doc_toolitem("SaveAs");
 	add_doc_toolitem("SaveAll");
 	add_doc_toolitem("Compile");
 	add_doc_toolitem("Run");
@@ -1435,6 +1441,35 @@ void ui_entry_add_clear_icon(GtkEntry *entry)
 }
 
 
+/* Adds a :activate-backwards signal emitted by default when <Shift>Return is pressed */
+void ui_entry_add_activate_backward_signal(GtkEntry *entry)
+{
+	static gboolean installed = FALSE;
+
+	g_return_if_fail(GTK_IS_ENTRY(entry));
+
+	if (G_UNLIKELY(! installed))
+	{
+		GtkBindingSet *binding_set;
+
+		installed = TRUE;
+
+		/* try to handle the unexpected case where GTK would already have installed the signal */
+		if (g_signal_lookup("activate-backward", G_TYPE_FROM_INSTANCE(entry)))
+		{
+			g_warning("Signal GtkEntry:activate-backward is unexpectedly already installed");
+			return;
+		}
+
+		g_signal_new("activate-backward", G_TYPE_FROM_INSTANCE(entry),
+			G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION, 0, NULL, NULL,
+			g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
+		binding_set = gtk_binding_set_by_class(GTK_ENTRY_GET_CLASS(entry));
+		gtk_binding_entry_add_signal(binding_set, GDK_Return, GDK_SHIFT_MASK, "activate-backward", 0);
+	}
+}
+
+
 static void add_to_size_group(GtkWidget *widget, gpointer size_group)
 {
 	g_return_if_fail(GTK_IS_SIZE_GROUP(size_group));
@@ -1589,7 +1624,7 @@ static gboolean tree_view_find(GtkTreeView *treeview, TVMatchCallback cb, gboole
 	while (TRUE)
 	{
 		gtk_tree_selection_select_iter(treesel, &iter);
-		if (cb(0))
+		if (cb(FALSE))
 			break;	/* found next message */
 
 		if (! tree_model_iter_get_next(model, &iter, down))
@@ -1907,9 +1942,9 @@ void ui_init_stock_items(void)
 	gsize i, len;
 	GtkStockItem items[] =
 	{
-		{ GEANY_STOCK_SAVE_ALL, _("Save All"), 0, 0, GETTEXT_PACKAGE },
-		{ GEANY_STOCK_CLOSE_ALL, _("Close All"), 0, 0, GETTEXT_PACKAGE },
-		{ GEANY_STOCK_BUILD, _("Build"), 0, 0, GETTEXT_PACKAGE }
+		{ GEANY_STOCK_SAVE_ALL, N_("Save All"), 0, 0, GETTEXT_PACKAGE },
+		{ GEANY_STOCK_CLOSE_ALL, N_("Close All"), 0, 0, GETTEXT_PACKAGE },
+		{ GEANY_STOCK_BUILD, N_("Build"), 0, 0, GETTEXT_PACKAGE }
 	};
 
 	len = G_N_ELEMENTS(items);
@@ -1985,9 +2020,9 @@ static void ui_menu_move(GtkWidget *menu, GtkWidget *old, GtkWidget *new)
 
 typedef struct GeanySharedMenu
 {
-	gchar *menu;
-	gchar *menubar_item;
-	gchar *popup_item;
+	const gchar *menu;
+	const gchar *menubar_item;
+	const gchar *popup_item;
 }
 GeanySharedMenu;
 
@@ -2030,9 +2065,8 @@ void ui_init_prefs(void)
 {
 	StashGroup *group = stash_group_new(PACKAGE);
 
-	/* hidden prefs (don't overwrite them so users can edit them manually) */
-	configuration_add_pref_group(group, FALSE);
-	stash_group_set_write_once(group, TRUE);
+	/* various prefs */
+	configuration_add_various_pref_group(group);
 
 	stash_group_add_boolean(group, &interface_prefs.show_symbol_list_expanders,
 		"show_symbol_list_expanders", TRUE);
@@ -2044,6 +2078,14 @@ void ui_init_prefs(void)
 		"statusbar_template", "");
 	stash_group_add_boolean(group, &ui_prefs.new_document_after_close,
 		"new_document_after_close", FALSE);
+	stash_group_add_boolean(group, &interface_prefs.msgwin_status_visible,
+		"msgwin_status_visible", TRUE);
+	stash_group_add_boolean(group, &interface_prefs.msgwin_compiler_visible,
+		"msgwin_compiler_visible", TRUE);
+	stash_group_add_boolean(group, &interface_prefs.msgwin_messages_visible,
+		"msgwin_messages_visible", TRUE);
+	stash_group_add_boolean(group, &interface_prefs.msgwin_scribble_visible,
+		"msgwin_scribble_visible", TRUE);
 }
 
 
@@ -2160,25 +2202,17 @@ void ui_auto_separator_add_ref(GeanyAutoSeparator *autosep, GtkWidget *item)
 
 
 /**
- *  Sets @a text as the contents of the tooltip for @a widget.
+ * Sets @a text as the contents of the tooltip for @a widget.
  *
- *  @param widget The widget the tooltip should be set for.
- *  @param text The text for the tooltip.
+ * @param widget The widget the tooltip should be set for.
+ * @param text The text for the tooltip.
  *
- *  @since 0.16
+ * @since 0.16
+ * @deprecated 0.21 use gtk_widget_set_tooltip_text() instead
  */
 void ui_widget_set_tooltip_text(GtkWidget *widget, const gchar *text)
 {
-#if GTK_CHECK_VERSION(2, 12, 0)
 	gtk_widget_set_tooltip_text(widget, text);
-#else
-	static GtkTooltips *tooltips = NULL;
-
-	if (G_UNLIKELY(tooltips == NULL))
-		tooltips = GTK_TOOLTIPS(ui_lookup_widget(main_widgets.window, "tooltips"));
-
-	gtk_tooltips_set_tip(tooltips, widget, text, NULL);
-#endif
 }
 
 
@@ -2228,9 +2262,9 @@ static GtkWidget *progress_bar_create(void)
 {
 	GtkWidget *bar = gtk_progress_bar_new();
 
-    /* Set the progressbar's height to 1 to fit it in the statusbar */
-    gtk_widget_set_size_request(bar, -1, 1);
-    gtk_box_pack_start (GTK_BOX(ui_widgets.statusbar), bar, FALSE, FALSE, 3);
+	/* Set the progressbar's height to 1 to fit it in the statusbar */
+	gtk_widget_set_size_request(bar, -1, 1);
+	gtk_box_pack_start (GTK_BOX(ui_widgets.statusbar), bar, FALSE, FALSE, 3);
 
 	return bar;
 }
@@ -2363,19 +2397,52 @@ GtkWidget *ui_label_new_bold(const gchar *text)
  * @warning You should check @c doc->is_valid in the callback.
  * @since 0.19 */
 void ui_menu_add_document_items(GtkMenu *menu, GeanyDocument *active, GCallback callback)
+{
+	ui_menu_add_document_items_sorted(menu, active, callback, NULL);
+}
+
+
+/** Adds a list of document items to @a menu.
+ *
+ * @a compare_func might be NULL to not sort the documents in the menu. In this case,
+ * the order of the document tabs is used.
+ *
+ * See document_sort_by_display_name() for an example sort function.
+ *
+ * @param menu Menu.
+ * @param active Which document to highlight, or @c NULL.
+ * @param callback is used for each menu item's @c "activate" signal and will be passed
+ * the corresponding document pointer as @c user_data.
+ * @param compare_func is used to sort the list. Might be @c NULL to not sort the list.
+ * @warning You should check @c doc->is_valid in the callback.
+ * @since 0.21 */
+void ui_menu_add_document_items_sorted(GtkMenu *menu, GeanyDocument *active,
+	GCallback callback, GCompareFunc compare_func)
 {
 	GtkWidget *menu_item, *menu_item_label, *image;
 	const GdkColor *color;
 	GeanyDocument *doc;
 	guint i, len;
 	gchar *base_name, *label;
+	GPtrArray *sorted_documents;
 
 	len = gtk_notebook_get_n_pages(GTK_NOTEBOOK(main_widgets.notebook));
-	for (i = 0; i < len; i++)
+
+	sorted_documents = g_ptr_array_sized_new(len);
+	/* copy the documents_array into the new one */
+	foreach_document(i)
 	{
-		doc = document_get_from_page(i);
-		if (! DOC_VALID(doc))
-			continue;
+		g_ptr_array_add(sorted_documents, documents[i]);
+	}
+	if (compare_func == NULL)
+		compare_func = document_compare_by_tab_order;
+
+	/* and now sort it */
+	g_ptr_array_sort(sorted_documents, compare_func);
+
+	for (i = 0; i < sorted_documents->len; i++)
+	{
+		doc = g_ptr_array_index(sorted_documents, i);
 
 		base_name = g_path_get_basename(DOC_FILENAME(doc));
 		menu_item = gtk_image_menu_item_new_with_label(base_name);
@@ -2400,6 +2467,7 @@ void ui_menu_add_document_items(GtkMenu *menu, GeanyDocument *active, GCallback
 
 		g_free(base_name);
 	}
+	g_ptr_array_free(sorted_documents, TRUE);
 }
 
 
@@ -2461,7 +2529,7 @@ void ui_editable_insert_text_callback(GtkEditable *editable, gchar *new_text,
 GdkPixbuf *ui_get_mime_icon(const gchar *mime_type, GtkIconSize size)
 {
 	GdkPixbuf *icon = NULL;
-#if defined(HAVE_GIO) && GLIB_CHECK_VERSION(2, 18, 0)
+#if GTK_CHECK_VERSION(2, 14, 0)
 	gchar *ctype;
 	GIcon *gicon;
 	GtkIconInfo *info;
@@ -2488,7 +2556,7 @@ GdkPixbuf *ui_get_mime_icon(const gchar *mime_type, GtkIconSize size)
 		}
 	}
 #endif
-	/* fallback for builds without GIO or if icon lookup failed, like it might happen on Windows */
+	/* fallback for builds with GIO < 2.18 or if icon lookup failed, like it might happen on Windows */
 	if (icon == NULL)
 	{
 		const gchar *stock_id = GTK_STOCK_FILE;
@@ -2508,3 +2576,11 @@ GdkPixbuf *ui_get_mime_icon(const gchar *mime_type, GtkIconSize size)
 	return icon;
 }
 
+
+void ui_focus_current_document(void)
+{
+	GeanyDocument *doc = document_get_current();
+
+	if (doc != NULL)
+		document_grab_focus(doc);
+}
