@@ -1,8 +1,8 @@
 /*
  *      utils.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: utils.c 5454 2010-11-30 18:25:16Z ntrel $
+ * $Id: utils.c 5966 2011-09-28 19:49:45Z colombanw $
  */
 
 /*
@@ -44,9 +44,7 @@
 
 #include <glib/gstdio.h>
 
-#ifdef HAVE_GIO
-# include <gio/gio.h>
-#endif
+#include <gio/gio.h>
 
 #include "prefs.h"
 #include "support.h"
@@ -55,6 +53,7 @@
 #include "dialogs.h"
 #include "win32.h"
 #include "project.h"
+#include "ui_utils.h"
 
 #include "utils.h"
 
@@ -63,8 +62,7 @@
  *  Tries to open the given URI in a browser.
  *  On Windows, the system's default browser is opened.
  *  On non-Windows systems, the browser command set in the preferences dialog is used. In case
- *  that fails or it is unset, @c xdg-open is used as fallback as well as some other known
- *  browsers.
+ *  that fails or it is unset, the user is asked to correct or fill it.
  *
  *  @param uri The URI to open in the web browser.
  *
@@ -76,61 +74,41 @@ void utils_open_browser(const gchar *uri)
 	g_return_if_fail(uri != NULL);
 	win32_open_browser(uri);
 #else
-	gchar *cmdline;
+	gboolean again = TRUE;
 
 	g_return_if_fail(uri != NULL);
 
-	cmdline = g_strconcat(tool_prefs.browser_cmd, " \"", uri, "\"", NULL);
-	if (! g_spawn_command_line_async(cmdline, NULL))
+	while (again)
 	{
-		const gchar *argv[3];
+		gchar *cmdline = g_strconcat(tool_prefs.browser_cmd, " \"", uri, "\"", NULL);
 
-		argv[0] = "xdg-open";
-		argv[1] = uri;
-		argv[2] = NULL;
-		if (! g_spawn_async(NULL, (gchar**)argv, NULL, G_SPAWN_SEARCH_PATH,
-				NULL, NULL, NULL, NULL))
+		if (g_spawn_command_line_async(cmdline, NULL))
+			again = FALSE;
+		else
 		{
-			argv[0] = "firefox";
-			if (! g_spawn_async(NULL, (gchar**)argv, NULL, G_SPAWN_SEARCH_PATH,
-					NULL, NULL, NULL, NULL))
-			{
-				argv[0] = "mozilla";
-				if (! g_spawn_async(NULL, (gchar**)argv, NULL, G_SPAWN_SEARCH_PATH, NULL,
-						NULL, NULL, NULL))
-				{
-					argv[0] = "opera";
-					if (! g_spawn_async(NULL, (gchar**)argv, NULL, G_SPAWN_SEARCH_PATH,
-							NULL, NULL, NULL, NULL))
-					{
-						argv[0] = "konqueror";
-						if (! g_spawn_async(NULL, (gchar**)argv, NULL, G_SPAWN_SEARCH_PATH,
-								NULL, NULL, NULL, NULL))
-						{
-							argv[0] = "netscape";
-							g_spawn_async(NULL, (gchar**)argv, NULL, G_SPAWN_SEARCH_PATH,
-								NULL, NULL, NULL, NULL);
-						}
-					}
-				}
-			}
+			gchar *new_cmd = dialogs_show_input(_("Select Browser"), GTK_WINDOW(main_widgets.window),
+				_("Failed to spawn the configured browser command. "
+				  "Please correct it or enter another one."),
+				tool_prefs.browser_cmd);
+
+			if (new_cmd == NULL) /* user canceled */
+				again = FALSE;
+			else
+				setptr(tool_prefs.browser_cmd, new_cmd);
 		}
+		g_free(cmdline);
 	}
-	g_free(cmdline);
 #endif
 }
 
 
 /* taken from anjuta, to determine the EOL mode of the file */
-gint utils_get_line_endings(const gchar* buffer, glong size)
+gint utils_get_line_endings(const gchar* buffer, gsize size)
 {
-	gint i;
+	gsize i;
 	guint cr, lf, crlf, max_mode;
 	gint mode;
 
-	if (size == -1)
-		size = strlen(buffer);
-
 	cr = lf = crlf = 0;
 
 	for (i = 0; i < size ; i++)
@@ -249,7 +227,7 @@ gint utils_write_file(const gchar *filename, const gchar *text)
 	else
 	{
 		FILE *fp;
-		gint bytes_written, len;
+		gsize bytes_written, len;
 		gboolean fail = FALSE;
 
 		if (filename == NULL)
@@ -268,7 +246,7 @@ gint utils_write_file(const gchar *filename, const gchar *text)
 			{
 				fail = TRUE;
 				geany_debug(
-					"utils_write_file(): written only %d bytes, had to write %d bytes to %s",
+					"utils_write_file(): written only %"G_GSIZE_FORMAT" bytes, had to write %"G_GSIZE_FORMAT" bytes to %s",
 					bytes_written, len, filename);
 			}
 			if (fclose(fp) != 0)
@@ -285,12 +263,36 @@ gint utils_write_file(const gchar *filename, const gchar *text)
 }
 
 
-/** Searches backward through @a size bytes looking for a '<', then returns the tag, if any.
+/** Searches backward through @a size bytes looking for a '<'.
  * @param sel .
  * @param size .
- * @return The tag name.
+ * @return The tag name (newly allocated) or @c NULL if no opening tag was found.
  */
 gchar *utils_find_open_xml_tag(const gchar sel[], gint size)
+{
+	const gchar *cur, *begin;
+	gsize len;
+
+	cur = utils_find_open_xml_tag_pos(sel, size);
+	if (cur == NULL)
+		return NULL;
+
+	cur++; /* skip the bracket */
+	begin = cur;
+	while (strchr(":_-.", *cur) || isalnum(*cur))
+		cur++;
+
+	len = (gsize)(cur - begin);
+	return len ? g_strndup(begin, len) : NULL;
+}
+
+
+/** Searches backward through @a size bytes looking for a '<'.
+ * @param sel .
+ * @param size .
+ * @return pointer to '<' of the found opening tag within @a sel, or @c NULL if no opening tag was found.
+ */
+const gchar *utils_find_open_xml_tag_pos(const gchar sel[], gint size)
 {
 	/* stolen from anjuta and modified */
 	const gchar *begin, *cur;
@@ -319,45 +321,43 @@ gchar *utils_find_open_xml_tag(const gchar sel[], gint size)
 	{
 		if (*cur == '<')
 			break;
+		/* exit immediately if such non-valid XML/HTML is detected, e.g. "<script>if a >" */
 		else if (*cur == '>')
 			break;
 		--cur;
 	}
 
-	if (*cur == '<')
-	{
-		GString *result;
-
-		cur++;
-		if (*cur == '/')
-			return NULL; /* we found a closing tag */
-
-		result = g_string_sized_new(64);
-		while (strchr(":_-.", *cur) || isalnum(*cur))
-		{
-			g_string_append_c(result, *cur);
-			cur++;
-		}
-		return g_string_free(result, FALSE);
-	}
+	/* if the found tag is an opening, not a closing tag or empty <> */
+	if (*cur == '<' && *(cur + 1) != '/' && *(cur + 1) != '>')
+		return cur;
 
 	return NULL;
 }
 
 
-/* Returns true if specified tag doesn't usually contain any content and can be left unclosed */
+/* Returns true if tag_name is a self-closing tag */
 gboolean utils_is_short_html_tag(const gchar *tag_name)
 {
-	return utils_str_equal(tag_name, "br")
-	 || utils_str_equal(tag_name, "hr")
-	 || utils_str_equal(tag_name, "img")
-	 || utils_str_equal(tag_name, "base")
-	 || utils_str_equal(tag_name, "basefont")	/* < or not < */
-	 || utils_str_equal(tag_name, "frame")
-	 || utils_str_equal(tag_name, "input")
-	 || utils_str_equal(tag_name, "link")
-	 || utils_str_equal(tag_name, "area")
-	 || utils_str_equal(tag_name, "meta");
+	const gchar names[][20] = {
+		"area",
+		"base",
+		"basefont",	/* < or not < */
+		"br",
+		"frame",
+		"hr",
+		"img",
+		"input",
+		"link",
+		"meta"
+	};
+
+	if (tag_name)
+	{
+		if (bsearch(tag_name, names, G_N_ELEMENTS(names), 20,
+			(GCompareFunc)g_ascii_strcasecmp))
+				return TRUE;
+	}
+	return FALSE;
 }
 
 
@@ -413,7 +413,7 @@ gboolean utils_atob(const gchar *str)
 /* NULL-safe version of g_path_is_absolute(). */
 gboolean utils_is_absolute_path(const gchar *path)
 {
-	if (! NZV(path))
+	if (G_UNLIKELY(! NZV(path)))
 		return FALSE;
 
 	return g_path_is_absolute(path);
@@ -643,7 +643,7 @@ gchar *utils_get_hostname(void)
 
 /* Checks whether the given file can be written. locale_filename is expected in locale encoding.
  * Returns 0 if it can be written, otherwise it returns errno */
-gint utils_is_file_writeable(const gchar *locale_filename)
+gint utils_is_file_writable(const gchar *locale_filename)
 {
 	gchar *file;
 	gint ret;
@@ -822,7 +822,7 @@ gint utils_get_setting_integer(GKeyFile *config, const gchar *section, const gch
 		return default_value;
 
 	tmp = g_key_file_get_integer(config, section, key, &error);
-	if (G_UNLIKELY(error))
+	if (error)
 	{
 		g_error_free(error);
 		return default_value;
@@ -853,7 +853,7 @@ gboolean utils_get_setting_boolean(GKeyFile *config, const gchar *section, const
 		return default_value;
 
 	tmp = g_key_file_get_boolean(config, section, key, &error);
-	if (G_UNLIKELY(error))
+	if (error)
 	{
 		g_error_free(error);
 		return default_value;
@@ -883,7 +883,7 @@ gchar *utils_get_setting_string(GKeyFile *config, const gchar *section, const gc
 		return g_strdup(default_value);
 
 	tmp = g_key_file_get_string(config, section, key, NULL);
-	if (G_UNLIKELY(!tmp))
+	if (!tmp)
 	{
 		return g_strdup(default_value);
 	}
@@ -1375,17 +1375,17 @@ gchar **utils_strv_new(const gchar *first, ...)
 
 	strvlen = 1;	/* for first argument */
 
-    /* count other arguments */
-    va_start(args, first);
-    for (; va_arg(args, gchar*) != NULL; strvlen++);
+	/* count other arguments */
+	va_start(args, first);
+	for (; va_arg(args, gchar*) != NULL; strvlen++);
 	va_end(args);
 
 	strv = g_new(gchar*, strvlen + 1);	/* +1 for NULL terminator */
 	strv[0] = g_strdup(first);
 
-    va_start(args, first);
-    for (i = 1; str = va_arg(args, gchar*), str != NULL; i++)
-    {
+	va_start(args, first);
+	for (i = 1; str = va_arg(args, gchar*), str != NULL; i++)
+	{
 		strv[i] = g_strdup(str);
 	}
 	va_end(args);
@@ -1459,13 +1459,15 @@ GSList *utils_get_file_list_full(const gchar *path, gboolean full_path, gboolean
 
 	foreach_dir(filename, dir)
 	{
-		list = g_slist_append(list, full_path ?
+		list = g_slist_prepend(list, full_path ?
 			g_build_path(G_DIR_SEPARATOR_S, path, filename, NULL) : g_strdup(filename));
 	}
 	g_dir_close(dir);
 	/* sorting last is quicker than on insertion */
 	if (sort)
 		list = g_slist_sort(list, (GCompareFunc) utils_str_casecmp);
+	else
+		list = g_slist_reverse(list);
 	return list;
 }
 
@@ -1518,39 +1520,49 @@ gboolean utils_str_has_upper(const gchar *str)
 }
 
 
-static guint utils_string_replace_helper(GString *haystack, const gchar *needle,
-										 const gchar *replace, const guint max_replaces)
+/* end can be -1 for haystack->len.
+ * returns: position of found text or -1. */
+gint utils_string_find(GString *haystack, gint start, gint end, const gchar *needle)
 {
-	const gchar *stack, *match;
-	guint ret = 0;
-	gssize pos;
+	gint pos;
 
-	g_return_val_if_fail(haystack != NULL, 0);
+	g_return_val_if_fail(haystack != NULL, -1);
 	if (haystack->len == 0)
-		return FALSE;
-	g_return_val_if_fail(NZV(needle), 0);
+		return -1;
 
-	stack = haystack->str;
-	if (! (match = strstr(stack, needle)))
-		return 0;
-	do
+	g_return_val_if_fail(start >= 0, -1);
+	if (start >= (gint)haystack->len)
+		return -1;
+
+	g_return_val_if_fail(NZV(needle), -1);
+
+	if (end < 0)
+		end = haystack->len;
+
+	pos = utils_strpos(haystack->str + start, needle);
+	if (pos == -1)
+		return -1;
+
+	pos += start;
+	if (pos >= end)
+		return -1;
+	return pos;
+}
+
+
+/* Replaces @len characters from offset @a pos.
+ * len can be -1 for str->len.
+ * returns: pos + strlen(replace). */
+gint utils_string_replace(GString *str, gint pos, gint len, const gchar *replace)
+{
+	g_string_erase(str, pos, len);
+	if (replace)
 	{
-		pos = match - haystack->str;
-		g_string_erase(haystack, pos, strlen(needle));
-
-		/* make next search after removed matching text.
-		 * (we have to be careful to only use haystack->str as its address may change) */
-		stack = haystack->str + pos;
-
-		if (G_LIKELY(replace))
-		{
-			g_string_insert(haystack, pos, replace);
-			stack = haystack->str + pos + strlen(replace);	/* skip past replacement */
-		}
+		g_string_insert(str, pos, replace);
+		pos += strlen(replace);
 	}
-	while (++ret != max_replaces && (match = strstr(stack, needle)));
 
-	return ret;
+	return pos;
 }
 
 
@@ -1567,7 +1579,21 @@ static guint utils_string_replace_helper(GString *haystack, const gchar *needle,
  **/
 guint utils_string_replace_all(GString *haystack, const gchar *needle, const gchar *replace)
 {
-	return utils_string_replace_helper(haystack, needle, replace, 0);
+	guint count = 0;
+	gint pos = 0;
+	gsize needle_length = strlen(needle);
+
+	while (1)
+	{
+		pos = utils_string_find(haystack, pos, -1, needle);
+
+		if (pos == -1)
+			break;
+
+		pos = utils_string_replace(haystack, pos, needle_length, replace);
+		count++;
+	}
+	return count;
 }
 
 
@@ -1586,7 +1612,13 @@ guint utils_string_replace_all(GString *haystack, const gchar *needle, const gch
  */
 guint utils_string_replace_first(GString *haystack, const gchar *needle, const gchar *replace)
 {
-	return utils_string_replace_helper(haystack, needle, replace, 1);
+	gint pos = utils_string_find(haystack, 0, -1, needle);
+
+	if (pos == -1)
+		return 0;
+
+	utils_string_replace(haystack, pos, strlen(needle), replace);
+	return 1;
 }
 
 
@@ -1710,15 +1742,29 @@ gboolean utils_spawn_async(const gchar *dir, gchar **argv, gchar **env, GSpawnFl
 }
 
 
-/* Similar to g_build_path() but (re)using a fixed buffer, so never free it.
- * This assumes a small enough resulting string length to be kept without freeing,
- * but this should be the case for filenames.
- * @warning As the buffer is reused, you can't call this recursively, e.g. for a
- * function argument and within the function called. */
-const gchar *utils_build_path(const gchar *first, ...)
+static gboolean utils_string_vappend(GString *buffer, const gchar *sep, va_list args)
+{
+	const gchar *str = va_arg(args, const gchar *);
+
+	if (!str)
+		return FALSE;
+	do
+	{
+		g_string_append(buffer, str);
+		str = va_arg(args, const gchar *);
+		if (str && sep)
+			g_string_append(buffer, sep);
+	}
+	while (str);
+
+	return TRUE;
+}
+
+
+/* Like g_build_path() but without first argument. */
+gchar *utils_build_path(const gchar *first, ...)
 {
 	static GString *buffer = NULL;
-	const gchar *str;
 	va_list args;
 
 	if (! buffer)
@@ -1726,19 +1772,36 @@ const gchar *utils_build_path(const gchar *first, ...)
 	else
 		g_string_assign(buffer, first);
 
+	g_string_append_c(buffer, G_DIR_SEPARATOR);
+
 	va_start(args, first);
-	while (1)
-	{
-		str = va_arg(args, const gchar *);
-		if (!str)
-			break;
-
-		g_string_append_c(buffer, G_DIR_SEPARATOR);
-		g_string_append(buffer, str);
-	}
+	utils_string_vappend(buffer, G_DIR_SEPARATOR_S, args);
 	va_end(args);
+	return g_strdup(buffer->str);
+}
 
-	return buffer->str;
+
+/* Concatenates a path with other strings.
+ * @param path A path, which will have a separator added before the other strings.
+ * @param ... Strings to concatenate (no directory separators will be
+ * 	inserted between them).
+ * E.g. filename = utils_make_filename(app->datadir, "filetypes.", ext, NULL); */
+gchar *utils_make_filename(const gchar *path, ...)
+{
+	static GString *buffer = NULL;
+	va_list args;
+
+	if (! buffer)
+		buffer = g_string_new(path);
+	else
+		g_string_assign(buffer, path);
+
+	g_string_append_c(buffer, G_DIR_SEPARATOR);
+
+	va_start(args, path);
+	utils_string_vappend(buffer, NULL, args);
+	va_end(args);
+	return g_strdup(buffer->str);
 }
 
 
@@ -1758,7 +1821,6 @@ gchar *utils_get_path_from_uri(const gchar *uri)
 
 	/* this will work only for 'file://' URIs */
 	locale_filename = g_filename_from_uri(uri, NULL, NULL);
-#ifdef HAVE_GIO
 	/* g_filename_from_uri() failed, so we probably have a non-local URI */
 	if (locale_filename == NULL)
 	{
@@ -1769,13 +1831,8 @@ gchar *utils_get_path_from_uri(const gchar *uri)
 		{
 			geany_debug("The URI '%s' could not be resolved to a local path. This means "
 				"that the URI is invalid or that you don't have gvfs-fuse installed.", uri);
-			return NULL;
 		}
 	}
-#endif
-	if (locale_filename == NULL)
-		geany_debug("The URI '%s' could not be resolved to a local path. This means that the "
-			"URI is invalid or that Geany can't use GVFS (maybe it is not installed).", uri);
 
 	return locale_filename;
 }
@@ -1892,7 +1949,7 @@ gchar *utils_str_remove_chars(gchar *string, const gchar *chars)
 	gchar *w = string;
 
 	g_return_val_if_fail(string, NULL);
-	if (!NZV(chars))
+	if (G_UNLIKELY(! NZV(chars)))
 		return string;
 
 	foreach_str(r, string)
