@@ -1,8 +1,8 @@
 /*
  *      win32.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: win32.c 4098 2009-08-16 17:27:01Z eht16 $
+ *  $Id: win32.c 5946 2011-09-22 15:53:14Z ntrel $
  */
 
 /*
@@ -57,6 +57,7 @@
 #include "editor.h"
 
 #define BUFSIZE 4096
+#define CMDSIZE 32768
 
 struct _geany_win32_spawn
 {
@@ -84,19 +85,16 @@ static wchar_t *get_file_filters(void)
 {
 	gchar *string;
 	guint i, j, len;
-	static wchar_t title[1024];
+	static wchar_t title[4096];
 	GString *str = g_string_sized_new(100);
 	GString *all_patterns = g_string_sized_new(100);
+	GSList *node;
 	gchar *tmp;
 
-	for (i = 0; i < filetypes_array->len; i++)
-	{
-		tmp = g_strjoinv(";", filetypes[i]->pattern);
-		g_string_append_printf(str, "%s\t%s\t", filetypes[i]->title, tmp);
-		g_free(tmp);
-	}
-	/* create meta file filter "All Source" */
-	for (i = 0; i < filetypes_array->len; i++)
+	/* create meta file filter "All files" */
+	g_string_append_printf(str, "%s\t*\t", _("All files"));
+	/* create meta file filter "All Source" (skip GEANY_FILETYPES_NONE) */
+	for (i = GEANY_FILETYPES_NONE + 1; i < filetypes_array->len; i++)
 	{
 		for (j = 0; filetypes[i]->pattern[j] != NULL; j++)
 		{
@@ -106,17 +104,25 @@ static wchar_t *get_file_filters(void)
 	}
 	g_string_append_printf(str, "%s\t%s\t", _("All Source"), all_patterns->str);
 	g_string_free(all_patterns, TRUE);
+	/* add 'usual' filetypes */
+	foreach_slist(node, filetypes_by_title)
+	{
+		GeanyFiletype *ft = node->data;
 
+		if (G_UNLIKELY(ft->id == GEANY_FILETYPES_NONE))
+			continue;
+		tmp = g_strjoinv(";", ft->pattern);
+		g_string_append_printf(str, "%s\t%s\t", ft->title, tmp);
+		g_free(tmp);
+	}
 	g_string_append_c(str, '\t'); /* the final \0 byte to mark the end of the string */
 	string = str->str;
 	g_string_free(str, FALSE);
 
 	/* replace all "\t"s by \0 */
 	len = strlen(string);
-	for(i = 0; i < len; i++)
-	{
-		if (string[i] == '\t') string[i] = '\0';
-	}
+	g_strdelimit(string, "\t", '\0');
+	g_assert(string[len - 1] == 0x0);
 	MultiByteToWideChar(CP_UTF8, 0, string, len, title, sizeof(title));
 	g_free(string);
 
@@ -124,10 +130,29 @@ static wchar_t *get_file_filters(void)
 }
 
 
+static wchar_t *get_file_filter_all_files(void)
+{
+	guint len;
+	static wchar_t title[4096];
+	gchar *filter;
+
+	/* create meta file filter "All files" */
+	filter = g_strdup_printf("%s\t*\t", _("All files"));
+
+	len = strlen(filter);
+	g_strdelimit(filter, "\t", '\0');
+	g_assert(filter[len - 1] == 0x0);
+	MultiByteToWideChar(CP_UTF8, 0, filter, len, title, sizeof(title));
+	g_free(filter);
+
+	return title;
+}
+
+
 static wchar_t *get_filters(gboolean project_files)
 {
 	gchar *string;
-	gint i, len;
+	gint len;
 	static wchar_t title[1024];
 
 	if (project_files)
@@ -145,10 +170,8 @@ static wchar_t *get_filters(gboolean project_files)
 
 	/* replace all "\t"s by \0 */
 	len = strlen(string);
-	for(i = 0; i < len; i++)
-	{
-		if (string[i] == '\t') string[i] = '\0';
-	}
+	g_strdelimit(string, "\t", '\0');
+	g_assert(string[len - 1] == 0x0);
 	MultiByteToWideChar(CP_UTF8, 0, string, len, title, sizeof(title));
 	g_free(string);
 
@@ -183,7 +206,7 @@ static wchar_t *get_dir_for_path(const gchar *utf8_filename)
  * folder when the dialog is initialised. Yeah, I like Windows. */
 INT CALLBACK BrowseCallbackProc(HWND hwnd, UINT uMsg, LPARAM lp, LPARAM pData)
 {
-	switch(uMsg)
+	switch (uMsg)
 	{
 		case BFFM_INITIALIZED:
 		{
@@ -208,8 +231,7 @@ INT CALLBACK BrowseCallbackProc(HWND hwnd, UINT uMsg, LPARAM lp, LPARAM pData)
 /* Shows a folder selection dialog.
  * initial_dir is expected in UTF-8
  * The selected folder name is returned. */
-gchar *win32_show_project_folder_dialog(GtkWidget *parent, const gchar *title,
-										const gchar *initial_dir)
+gchar *win32_show_folder_dialog(GtkWidget *parent, const gchar *title, const gchar *initial_dir)
 {
 	BROWSEINFOW bi;
 	LPCITEMIDLIST pidl;
@@ -314,19 +336,23 @@ gchar *win32_show_project_open_dialog(GtkWidget *parent, const gchar *title,
 
 /* initial_dir can be NULL to use the current working directory.
  * Returns: TRUE if the dialog was not cancelled. */
-gboolean win32_show_file_dialog(gboolean file_open, const gchar *initial_dir)
+gboolean win32_show_document_open_dialog(GtkWindow *parent, const gchar *title, const gchar *initial_dir)
 {
 	OPENFILENAMEW of;
 	gint retval;
+	guint x;
 	gchar tmp[MAX_PATH];
 	wchar_t fname[MAX_PATH];
 	wchar_t w_dir[MAX_PATH];
+	wchar_t w_title[512];
 
 	fname[0] = '\0';
 
 	if (initial_dir != NULL)
 		MultiByteToWideChar(CP_UTF8, 0, initial_dir, -1, w_dir, sizeof(w_dir));
 
+	MultiByteToWideChar(CP_UTF8, 0, title, -1, w_title, sizeof(w_title));
+
 	/* initialise file dialog info struct */
 	memset(&of, 0, sizeof of);
 #ifdef OPENFILENAME_SIZE_VERSION_400
@@ -334,7 +360,7 @@ gboolean win32_show_file_dialog(gboolean file_open, const gchar *initial_dir)
 #else
 	of.lStructSize = sizeof of;
 #endif
-	of.hwndOwner = GDK_WINDOW_HWND(main_widgets.window->window);
+	of.hwndOwner = GDK_WINDOW_HWND(GTK_WIDGET(parent)->window);
 	of.lpstrFilter = get_file_filters();
 
 	of.lpstrCustomFilter = NULL;
@@ -343,18 +369,11 @@ gboolean win32_show_file_dialog(gboolean file_open, const gchar *initial_dir)
 	of.lpstrInitialDir = (initial_dir != NULL) ? w_dir : NULL;
 	of.nMaxFile = 2048;
 	of.lpstrFileTitle = NULL;
-	of.lpstrTitle = NULL;
+	of.lpstrTitle = w_title;
 	of.lpstrDefExt = L"";
-	if (file_open)
-	{
-		of.Flags = OFN_ALLOWMULTISELECT | OFN_FILEMUSTEXIST | OFN_EXPLORER;
-		retval = GetOpenFileNameW(&of);
-	}
-	else
-	{
-		of.Flags = OFN_OVERWRITEPROMPT | OFN_PATHMUSTEXIST;
-		retval = GetSaveFileNameW(&of);
-	}
+	of.Flags = OFN_ALLOWMULTISELECT | OFN_FILEMUSTEXIST | OFN_EXPLORER;
+
+	retval = GetOpenFileNameW(&of);
 
 	if (!retval)
 	{
@@ -367,63 +386,162 @@ gboolean win32_show_file_dialog(gboolean file_open, const gchar *initial_dir)
 		return FALSE;
 	}
 
-	if (file_open)
-	{
-		guint x;
-
-		x = of.nFileOffset - 1;
-		if (x != wcslen(fname))
-		{	/* open a single file */
-			WideCharToMultiByte(CP_UTF8, 0, fname, -1, tmp, sizeof(tmp), NULL, NULL);
-			document_open_file(tmp, of.Flags & OFN_READONLY, NULL, NULL);
-		}
-		else
-		{	/* open multiple files */
-			gchar file_name[MAX_PATH];
-			gchar dir_name[MAX_PATH];
-
-			WideCharToMultiByte(CP_UTF8, 0, fname, of.nFileOffset,
-				dir_name, sizeof(dir_name), NULL, NULL);
-			for (; ;)
-			{
-				if (! fname[x])
-				{
-					if (! fname[x+1])
-						break;
-
-					WideCharToMultiByte(CP_UTF8, 0, fname + x + 1, -1,
-						tmp, sizeof(tmp), NULL, NULL);
-					g_snprintf(file_name, 511, "%s\\%s", dir_name, tmp);
-
-					/* convert the resulting filename into UTF-8 */
-					document_open_file(file_name, of.Flags & OFN_READONLY, NULL, NULL);
-				}
-				x++;
-			}
-		}
+	x = of.nFileOffset - 1;
+	if (x != wcslen(fname))
+	{	/* open a single file */
+		WideCharToMultiByte(CP_UTF8, 0, fname, -1, tmp, sizeof(tmp), NULL, NULL);
+		document_open_file(tmp, of.Flags & OFN_READONLY, NULL, NULL);
 	}
 	else
-	{
-		GeanyDocument *doc = document_get_current();
+	{	/* open multiple files */
+		gchar file_name[MAX_PATH];
+		gchar dir_name[MAX_PATH];
 
-		WideCharToMultiByte(CP_UTF8, 0, fname, -1, tmp, sizeof(tmp), NULL, NULL);
-		document_save_file_as(doc, tmp);
+		WideCharToMultiByte(CP_UTF8, 0, fname, of.nFileOffset,
+			dir_name, sizeof(dir_name), NULL, NULL);
+		for (; ;)
+		{
+			if (! fname[x])
+			{
+				if (! fname[x + 1])
+					break;
+
+				WideCharToMultiByte(CP_UTF8, 0, fname + x + 1, -1,
+					tmp, sizeof(tmp), NULL, NULL);
+				g_snprintf(file_name, 511, "%s\\%s", dir_name, tmp);
+
+				/* convert the resulting filename into UTF-8 */
+				document_open_file(file_name, of.Flags & OFN_READONLY, NULL, NULL);
+			}
+			x++;
+		}
 	}
 	return (retval != 0);
 }
 
 
+gchar *win32_show_document_save_as_dialog(GtkWindow *parent, const gchar *title,
+										  const gchar *initial_file)
+{
+	OPENFILENAMEW of;
+	gint retval;
+	gchar tmp[MAX_PATH];
+	wchar_t w_file[MAX_PATH];
+	wchar_t w_title[512];
+
+	w_file[0] = '\0';
+
+	if (initial_file != NULL)
+		MultiByteToWideChar(CP_UTF8, 0, initial_file, -1, w_file, sizeof(w_file));
+
+	MultiByteToWideChar(CP_UTF8, 0, title, -1, w_title, sizeof(w_title));
+
+	/* initialise file dialog info struct */
+	memset(&of, 0, sizeof of);
+#ifdef OPENFILENAME_SIZE_VERSION_400
+	of.lStructSize = OPENFILENAME_SIZE_VERSION_400;
+#else
+	of.lStructSize = sizeof of;
+#endif
+	of.hwndOwner = GDK_WINDOW_HWND(GTK_WIDGET(parent)->window);
+
+	of.lpstrFilter = get_file_filter_all_files();
+	of.lpstrCustomFilter = NULL;
+	of.nFilterIndex = 0;
+
+	of.lpstrFile = w_file;
+	of.nMaxFile = 2048;
+	of.lpstrFileTitle = NULL;
+	of.lpstrTitle = w_title;
+	of.lpstrDefExt = L"";
+	of.Flags = OFN_FILEMUSTEXIST | OFN_EXPLORER;
+	retval = GetSaveFileNameW(&of);
+
+	if (! retval)
+	{
+		if (CommDlgExtendedError())
+		{
+			gchar *error = g_strdup_printf(
+				"File dialog box error (%x)", (gint) CommDlgExtendedError());
+			win32_message_dialog(NULL, GTK_MESSAGE_ERROR, error);
+			g_free(error);
+		}
+		return NULL;
+	}
+
+	WideCharToMultiByte(CP_UTF8, 0, w_file, -1, tmp, sizeof(tmp), NULL, NULL);
+
+	return g_strdup(tmp);
+}
+
+
+/* initial_dir can be NULL to use the current working directory.
+ * Returns: the selected filename */
+gchar *win32_show_file_dialog(GtkWindow *parent, const gchar *title, const gchar *initial_file)
+{
+	OPENFILENAMEW of;
+	gint retval;
+	gchar tmp[MAX_PATH];
+	wchar_t w_file[MAX_PATH];
+	wchar_t w_title[512];
+
+	w_file[0] = '\0';
+
+	if (initial_file != NULL)
+		MultiByteToWideChar(CP_UTF8, 0, initial_file, -1, w_file, sizeof(w_file));
+
+	MultiByteToWideChar(CP_UTF8, 0, title, -1, w_title, sizeof(w_title));
+
+	/* initialise file dialog info struct */
+	memset(&of, 0, sizeof of);
+#ifdef OPENFILENAME_SIZE_VERSION_400
+	of.lStructSize = OPENFILENAME_SIZE_VERSION_400;
+#else
+	of.lStructSize = sizeof of;
+#endif
+	of.hwndOwner = GDK_WINDOW_HWND(GTK_WIDGET(parent)->window);
+
+	of.lpstrFile = w_file;
+	of.nMaxFile = 2048;
+	of.lpstrFileTitle = NULL;
+	of.lpstrTitle = w_title;
+	of.lpstrDefExt = L"";
+	of.Flags = OFN_FILEMUSTEXIST | OFN_EXPLORER;
+	retval = GetOpenFileNameW(&of);
+
+	if (! retval)
+	{
+		if (CommDlgExtendedError())
+		{
+			gchar *error = g_strdup_printf(
+				"File dialog box error (%x)", (gint) CommDlgExtendedError());
+			win32_message_dialog(NULL, GTK_MESSAGE_ERROR, error);
+			g_free(error);
+		}
+		return NULL;
+	}
+
+	WideCharToMultiByte(CP_UTF8, 0, w_file, -1, tmp, sizeof(tmp), NULL, NULL);
+
+	return g_strdup(tmp);
+}
+
+
 void win32_show_font_dialog(void)
 {
 	gint retval;
 	CHOOSEFONT cf;
 	LOGFONT lf;        /* logical font structure */
 
+	memset(&lf, 0, sizeof lf);
+	/* TODO: init lf members */
+	
 	memset(&cf, 0, sizeof cf);
 	cf.lStructSize = sizeof cf;
 	cf.hwndOwner = GDK_WINDOW_HWND(main_widgets.window->window);
 	cf.lpLogFont = &lf;
-	cf.Flags = CF_APPLY | CF_NOSCRIPTSEL | CF_FORCEFONTEXIST | CF_INITTOLOGFONTSTRUCT | CF_SCREENFONTS;
+	/* support CF_APPLY? */
+	cf.Flags = CF_NOSCRIPTSEL | CF_FORCEFONTEXIST | CF_INITTOLOGFONTSTRUCT | CF_SCREENFONTS;
 
 	retval = ChooseFont(&cf);
 
@@ -630,7 +748,7 @@ gint win32_message_dialog_unsaved(const gchar *msg)
 		parent_hwnd = GDK_WINDOW_HWND(main_widgets.window->window);
 
 	ret = MessageBoxW(parent_hwnd, w_msg, w_title, MB_YESNOCANCEL | MB_ICONQUESTION);
-	switch(ret)
+	switch (ret)
 	{
 		case IDYES: ret = GTK_RESPONSE_YES; break;
 		case IDNO: ret = GTK_RESPONSE_NO; break;
@@ -640,14 +758,18 @@ gint win32_message_dialog_unsaved(const gchar *msg)
 	return ret;
 }
 
+
 /* Just a simple wrapper function to open a browser window */
 void win32_open_browser(const gchar *uri)
 {
 	if (strncmp(uri, "file://", 7) == 0)
 	{
 		uri += 7;
-		while (*uri == '/')
-			uri++;
+		if (strchr(uri, ':') != NULL)
+		{
+			while (*uri == '/')
+				uri++;
+		}
 	}
 	ShellExecute(NULL, "open", uri, NULL, NULL, SW_SHOWNORMAL);
 }
@@ -726,12 +848,12 @@ gchar *win32_get_hostname(void)
 gboolean win32_spawn(const gchar *dir, gchar **argv, gchar **env, GSpawnFlags flags,
 					 gchar **std_out, gchar **std_err, gint *exit_status, GError **error)
 {
-	TCHAR  buffer[MAX_PATH]=TEXT("");
-	TCHAR  cmdline[MAX_PATH] = TEXT("");
-	TCHAR* lpPart[MAX_PATH]={NULL};
-	DWORD  retval=0;
+	TCHAR  buffer[CMDSIZE]=TEXT("");
+	TCHAR  cmdline[CMDSIZE] = TEXT("");
+	TCHAR* lpPart[CMDSIZE]={NULL};
+	DWORD  retval = 0;
 	gint argc = 0, i;
-	gint cmdpos=0;
+	gint cmdpos = 0;
 
 	SECURITY_ATTRIBUTES saAttr;
 	BOOL fSuccess;
@@ -755,9 +877,9 @@ gboolean win32_spawn(const gchar *dir, gchar **argv, gchar **env, GSpawnFlags fl
 
 	if (flags & G_SPAWN_SEARCH_PATH)
 	{
-		retval = SearchPath(NULL, argv[0], ".exe", MAX_PATH, buffer, lpPart);
+		retval = SearchPath(NULL, argv[0], ".exe", sizeof(buffer), buffer, lpPart);
 		if (retval > 0)
-			g_snprintf(cmdline, MAX_PATH, "\"%s\"", buffer);
+			g_snprintf(cmdline, sizeof(cmdline), "\"%s\"", buffer);
 		else
 			g_strlcpy(cmdline, argv[0], sizeof(cmdline));
 		cmdpos = 1;
@@ -765,7 +887,7 @@ gboolean win32_spawn(const gchar *dir, gchar **argv, gchar **env, GSpawnFlags fl
 
 	for (i = cmdpos; i < argc; i++)
 	{
-		g_snprintf(cmdline, MAX_PATH, "%s %s", cmdline, argv[i]);
+		g_snprintf(cmdline, sizeof(cmdline), "%s %s", cmdline, argv[i]);
 		/*MessageBox(NULL, cmdline, cmdline, MB_OK);*/
 	}
 
@@ -954,15 +1076,17 @@ gchar *win32_expand_environment_variables(const gchar *str)
 static gboolean CreateChildProcess(geany_win32_spawn *gw_spawn, TCHAR *szCmdline, const TCHAR *dir, GError **error)
 {
 	PROCESS_INFORMATION piProcInfo;
-	STARTUPINFO siStartInfo;
+	STARTUPINFOW siStartInfo;
 	BOOL bFuncRetn = FALSE;
 	gchar *expandedCmdline;
+	wchar_t w_commandline[CMDSIZE];
+	wchar_t w_dir[MAX_PATH];
 
 	/* Set up members of the PROCESS_INFORMATION structure. */
-	ZeroMemory(&piProcInfo, sizeof(PROCESS_INFORMATION) );
+	ZeroMemory(&piProcInfo, sizeof(PROCESS_INFORMATION));
 
 	/* Set up members of the STARTUPINFO structure.*/
-	ZeroMemory(&siStartInfo, sizeof(STARTUPINFO) );
+	ZeroMemory(&siStartInfo, sizeof(STARTUPINFO));
 
 	siStartInfo.cb         = sizeof(STARTUPINFO);
 	siStartInfo.hStdError  = gw_spawn->hChildStderrWr;
@@ -973,15 +1097,18 @@ static gboolean CreateChildProcess(geany_win32_spawn *gw_spawn, TCHAR *szCmdline
 	/* Expand environment variables like %blah%. */
 	expandedCmdline = win32_expand_environment_variables(szCmdline);
 
+	MultiByteToWideChar(CP_UTF8, 0, expandedCmdline, -1, w_commandline, sizeof(w_commandline));
+	MultiByteToWideChar(CP_UTF8, 0, dir, -1, w_dir, sizeof(w_dir));
+
 	/* Create the child process. */
-	bFuncRetn = CreateProcess(NULL,
-		expandedCmdline,             /* command line */
+	bFuncRetn = CreateProcessW(NULL,
+		w_commandline,             /* command line */
 		NULL,          /* process security attributes */
 		NULL,          /* primary thread security attributes */
 		TRUE,          /* handles are inherited */
 		CREATE_NO_WINDOW,             /* creation flags */
 		NULL,          /* use parent's environment */
-		dir,           /* use parent's current directory */
+		w_dir,           /* use parent's current directory */
 		&siStartInfo,  /* STARTUPINFO pointer */
 		&piProcInfo);  /* receives PROCESS_INFORMATION */
 
@@ -1055,7 +1182,7 @@ static VOID ReadFromPipe(HANDLE hRead, HANDLE hWrite, HANDLE hFile, GError **err
 static HANDLE GetTempFileHandle(GError **error)
 {
 	/* Temp file */
-	DWORD dwBufSize=BUFSIZE;
+	DWORD dwBufSize = BUFSIZE;
 	UINT uRetVal;
 	TCHAR szTempName[BUFSIZE];
 	TCHAR lpPathBuffer[BUFSIZE];
@@ -1195,18 +1322,17 @@ gchar *win32_get_shortcut_target(const gchar *file_name)
 		return path;
 }
 
+
 void win32_set_working_directory(const gchar *dir)
 {
 	SetCurrentDirectory(dir);
 }
 
+
 gchar *win32_get_installation_dir(void)
 {
-#if GLIB_CHECK_VERSION(2, 16, 0)
 	return g_win32_get_package_installation_directory_of_module(NULL);
-#else
-	return g_win32_get_package_installation_directory(NULL, NULL);
-#endif
 }
 
+
 #endif
