@@ -1,5 +1,5 @@
 /*
- *   $Id: basic.c 3449 2009-01-06 14:53:31Z eht16 $
+ *   $Id: basic.c 5281 2010-10-05 14:44:00Z ntrel $
  *
  *   Copyright (c) 2000-2006, Darren Hiebert, Elias Pschernig
  *
@@ -55,6 +55,9 @@ static KeyWord freebasic_keywords[] = {
 	{"const", K_CONST}, /* must always be the third */
 	{"function", K_FUNCTION},
 	{"sub", K_FUNCTION},
+	{"property", K_FUNCTION},
+	{"constructor", K_FUNCTION},
+	{"destructor", K_FUNCTION},
 	{"private sub", K_FUNCTION},
 	{"public sub", K_FUNCTION},
 	{"private function", K_FUNCTION},
