@@ -16,6 +16,7 @@
 
 #include <string.h>
 #include <setjmp.h>
+#include <mio/mio.h>
 
 #include "entry.h"
 #include "get.h"
@@ -180,8 +181,7 @@ typedef struct sTokenInfo
 	keywordId		keyword;
 	vString*		name;			/* the name of the token */
 	unsigned long	lineNumber;		/* line number of tag */
-	fpos_t			filePosition;	/* file position of line containing name */
-	int				bufferPosition;	/* buffer position of line containing name */
+	MIOPos			filePosition;	/* file position of line containing name */
 } tokenInfo;
 
 typedef enum eImplementation
@@ -602,10 +602,7 @@ static void initToken (tokenInfo* const token)
 	token->type			= TOKEN_NONE;
 	token->keyword		= KEYWORD_NONE;
 	token->lineNumber	= getSourceLineNumber();
-	if (useFile())
-		token->filePosition   = getInputFilePosition();
-	else
-		token->bufferPosition = getInputBufferPosition();
+	token->filePosition	= getInputFilePosition();
 	vStringClear(token->name);
 }
 
@@ -664,7 +661,7 @@ static void deleteToken (tokenInfo *const token)
 
 static const char *accessString (const accessType laccess)
 {
-	static const char *const names [] ={
+	static const char *const names [] = {
 		"?", "private", "protected", "public", "default"
 	};
 	Assert (sizeof (names) / sizeof (names [0]) == ACCESS_COUNT);
@@ -674,7 +671,7 @@ static const char *accessString (const accessType laccess)
 
 static const char *implementationString (const impType imp)
 {
-	static const char *const names [] ={
+	static const char *const names [] = {
 		"?", "abstract", "virtual", "pure virtual"
 	};
 	Assert (sizeof (names) / sizeof (names [0]) == IMP_COUNT);
@@ -694,7 +691,7 @@ static const char *tokenString (const tokenType type)
 {
 	static const char *const names [] = {
 		"none", "args", "}", "{", "comma", "double colon", "keyword", "name",
-		"package", "paren-name", "semicolon", "specifier", "*", "[]", "count"
+		"package", "paren-name", "semicolon", "specifier", "*", "[]"
 	};
 	Assert (sizeof (names) / sizeof (names [0]) == TOKEN_COUNT);
 	Assert ((int) type < TOKEN_COUNT);
@@ -713,13 +710,14 @@ static const char *scopeString (const tagScope scope)
 
 static const char *declString (const declType declaration)
 {
-		static const char *const names [] = {
-			"?", "base", "class", "enum", "function", "ignore", "interface",
-			"namespace", "no mangle", "package", "struct", "union",
-		};
-		Assert (sizeof (names) / sizeof (names [0]) == DECL_COUNT);
-		Assert ((int) declaration < DECL_COUNT);
-		return names[(int) declaration];
+	static const char *const names [] = {
+		"?", "base", "class", "enum", "event", "signal", "function",
+		"function template", "ignore", "interface", "module", "namespace",
+		"no mangle", "package", "struct", "union",
+	};
+	Assert (sizeof (names) / sizeof (names [0]) == DECL_COUNT);
+	Assert ((int) declaration < DECL_COUNT);
+	return names[(int) declaration];
 }
 
 static const char *keywordString (const keywordId keyword)
@@ -956,8 +954,7 @@ static void reinitStatement (statementInfo *const st, const boolean partial)
 	initToken (st->blockName);
 	vStringClear (st->parentClasses);
 
-	/*  Init member info.
-	 */
+	/* Init member info. */
 	if (! partial)
 		st->member.access = st->member.accessDefault;
 
@@ -984,7 +981,7 @@ static void initStatement (statementInfo *const st, statementInfo *const parent)
 	st->parent = parent;
 	initMemberInfo (st);
 	reinitStatement (st, FALSE);
-	if(parent)
+	if (parent)
 	{
 		const tokenInfo *const src = activeToken (parent);
 		tokenInfo *const dst = activeToken (st);
@@ -1185,11 +1182,11 @@ static const char* accessField (const statementInfo *const st)
 static void addOtherFields (tagEntryInfo* const tag, const tagType type,
 							const statementInfo *const st, vString *const scope)
 {
-    /*  For selected tag types, append an extension flag designating the
-     *  parent object in which the tag is defined.
-     */
-    switch (type)
-    {
+	/*  For selected tag types, append an extension flag designating the
+	 *  parent object in which the tag is defined.
+	 */
+	switch (type)
+	{
 		default: break;
 
 		case TAG_CLASS:
@@ -1218,7 +1215,6 @@ static void addOtherFields (tagEntryInfo* const tag, const tagType type,
 			if ((type == TAG_CLASS  ||  type == TAG_INTERFACE  ||
 				 type == TAG_STRUCT) && vStringLength (st->parentClasses) > 0)
 			{
-
 				tag->extensionFields.inheritance =
 						vStringValue (st->parentClasses);
 			}
@@ -1236,16 +1232,8 @@ static void addOtherFields (tagEntryInfo* const tag, const tagType type,
             if ((TRUE == st->gotArgs) && (TRUE == Option.extensionFields.argList) &&
 				((TAG_FUNCTION == type) || (TAG_METHOD == type) || (TAG_PROTOTYPE == type)))
 			{
-				if (useFile())
-				{
-					tag->extensionFields.arglist = getArglistFromFilePos(
+				tag->extensionFields.arglist = getArglistFromFilePos(
 						tag->filePosition, tag->name);
-				}
-				else
-				{
-					tag->extensionFields.arglist = getArglistFromBufferPos(
-						tag->bufferPosition, tag->name);
-				}
 			}
 			break;
 		}
@@ -1424,10 +1412,7 @@ static void makeTag (const tokenInfo *const token,
 		initTagEntry (&e, vStringValue (token->name));
 
 		e.lineNumber	= token->lineNumber;
-		if (useFile())
-			e.filePosition	= token->filePosition;
-		else
-			e.bufferPosition = token->bufferPosition;
+		e.filePosition	= token->filePosition;
 		e.isFileScope = isFileScope;
 		e.kindName	= tagName (type);
 		e.kind		= tagLetter (type);
@@ -1561,7 +1546,8 @@ static void qualifyVariableTag (const statementInfo *const st,
 			else if (st->scope == SCOPE_GLOBAL  ||  st->scope == SCOPE_STATIC)
 				makeTag (nameToken, st, TRUE, TAG_MEMBER);
 		}
-		else if (isLanguage (Lang_java) || isLanguage (Lang_csharp) || isLanguage (Lang_vala));
+		else if (isLanguage (Lang_java) || isLanguage (Lang_csharp) || isLanguage (Lang_vala))
+			;
 		else
 		{
 			if (st->scope == SCOPE_EXTERN  ||  ! st->haveQualifyingName)
@@ -1894,10 +1880,7 @@ static void copyToken (tokenInfo *const dest, const tokenInfo *const src)
 {
 	dest->type         = src->type;
 	dest->keyword      = src->keyword;
-	if (useFile())
-		dest->filePosition = src->filePosition;
-	else
-		dest->bufferPosition = src->bufferPosition;
+	dest->filePosition = src->filePosition;
 	dest->lineNumber   = src->lineNumber;
 	vStringCopy (dest->name, src->name);
 }
@@ -2485,15 +2468,8 @@ static int parseParens (statementInfo *const st, parenInfo *const info)
 		skipToMatch ("()");
 		--depth;
 	}
-		if (st->argEndPosition == 0)
-		{
-			if (useFile())
-				st->argEndPosition = ftell(File.fp);
-			else
-				/* FIXME File.fpBufferPosition is wrong here, this breaks function signatures and
-				 * so Geany's calltips */
-				st->argEndPosition = File.fpBufferPosition;
-		}
+	if (st->argEndPosition == 0)
+		st->argEndPosition = mio_tell (File.mio);
 
 	if (! info->isNameCandidate)
 		initToken (token);
@@ -2541,7 +2517,8 @@ static void analyzeParens (statementInfo *const st)
 			token->type = TOKEN_NAME;
 			processName (st);
 			st->gotParenName = TRUE;
-			if (isLanguage(Lang_d) && c == '(' && isType (prev, TOKEN_NAME)) {
+			if (isLanguage(Lang_d) && c == '(' && isType (prev, TOKEN_NAME))
+			{
 				st->declaration = DECL_FUNCTION_TEMPLATE;
 				copyToken (st->blockName, prev);
 			}
@@ -2876,12 +2853,12 @@ static void nest (statementInfo *const st, const unsigned int nestLevel)
 
 static void tagCheck (statementInfo *const st)
 {
-    const tokenInfo *const token = activeToken (st);
-    const tokenInfo *const prev  = prevToken (st, 1);
-    const tokenInfo *const prev2 = prevToken (st, 2);
+	const tokenInfo *const token = activeToken (st);
+	const tokenInfo *const prev  = prevToken (st, 1);
+	const tokenInfo *const prev2 = prevToken (st, 2);
 
-    switch (token->type)
-    {
+	switch (token->type)
+	{
 		case TOKEN_NAME:
 		{
 			if (insideEnumBody (st))
@@ -2913,6 +2890,7 @@ static void tagCheck (statementInfo *const st)
 			else if (isContextualStatement (st))
 			{
 				tokenInfo *name_token = (tokenInfo *)prev;
+				boolean free_name_token = FALSE;
 
 				if (isType (name_token, TOKEN_NAME))
 				{
@@ -2949,6 +2927,7 @@ static void tagCheck (statementInfo *const st)
 						char buffer[64];
 
 						name_token = newToken ();
+						free_name_token = TRUE;
 						copyToken (name_token, contextual_token);
 
 						sprintf(buffer, "anon_%s_%d", name_token->name->buffer, contextual_fake_count++);
@@ -2967,6 +2946,8 @@ static void tagCheck (statementInfo *const st)
 					}
 				}
 				qualifyBlockTag (st, name_token);
+				if (free_name_token)
+					deleteToken (name_token);
 			}
 			break;
 		}
@@ -3044,6 +3025,8 @@ static boolean findCTags (const unsigned int passCount)
 	exception_t exception;
 	boolean retry;
 
+	contextual_fake_count = 0;
+
 	Assert (passCount < 3);
 	cppInit ((boolean) (passCount > 1), isLanguage (Lang_csharp));
 
@@ -3082,56 +3065,48 @@ static void buildKeywordHash (const langType language, unsigned int idx)
 
 static void initializeCParser (const langType language)
 {
-	contextual_fake_count = 0;
 	Lang_c = language;
 	buildKeywordHash (language, 0);
 }
 
 static void initializeCppParser (const langType language)
 {
-	contextual_fake_count = 0;
 	Lang_cpp = language;
 	buildKeywordHash (language, 1);
 }
 
 static void initializeJavaParser (const langType language)
 {
-	contextual_fake_count = 0;
 	Lang_java = language;
 	buildKeywordHash (language, 3);
 }
 
 static void initializeDParser (const langType language)
 {
-	contextual_fake_count = 0;
 	Lang_d = language;
 	buildKeywordHash (language, 6);
 }
 
 static void initializeGLSLParser (const langType language)
 {
-	contextual_fake_count = 0;
 	Lang_glsl = language;
 	buildKeywordHash (language, 0); /* C keywords */
 }
 
 static void initializeFeriteParser (const langType language)
 {
-	contextual_fake_count = 0;
 	Lang_ferite = language;
 	buildKeywordHash (language, 1);	/* C++ keywords */
 }
 
 static void initializeCsharpParser (const langType language)
 {
-	contextual_fake_count = 0;
 	Lang_csharp = language;
 	buildKeywordHash (language, 2);
 }
 
 static void initializeValaParser (const langType language)
 {
-	contextual_fake_count = 0;
 	Lang_vala = language;
 	buildKeywordHash (language, 5);
 }
