@@ -219,7 +219,7 @@ char *CurrentDirectory = NULL;
 #ifdef HAVE_MKSTEMP
 static const char *ExecutableProgram = NULL;
 #endif
-static const char *ExecutableName = NULL;
+static const char *ExecutableName = "geany";
 static stringList* Excluded = NULL;
 
 static struct { long files, lines, bytes; } Totals = { 0, 0, 0 };
@@ -285,11 +285,7 @@ extern void error (const errorSelection selection,
 	    selected (selection, WARNING) ? "Warning: " : "");
     vfprintf (errout, format, ap);
     if (selected (selection, PERROR))
-#ifdef HAVE_STRERROR
-	fprintf (errout, " : %s", strerror (errno));
-#else
-	perror (" ");
-#endif
+	fprintf (errout, " : %s", g_strerror (errno));
     fputs ("\n", errout);
     va_end (ap);
     if (selected (selection, FATAL))
@@ -377,8 +373,8 @@ extern void *eRealloc (void *const ptr, const size_t size)
 
 extern void eFree (void *const ptr)
 {
-    Assert (ptr != NULL);
-    free (ptr);
+    if (ptr != NULL)
+	free (ptr);
 }
 
 extern void toLowerString (char* str)
@@ -1396,4 +1392,10 @@ extern int ctags_main (int __unused__ argc, char **argv)
 }
 #endif
 
+/* wrap g_warning so we don't include glib.h for all parsers, to keep compat with CTags */
+void utils_warn(const char *msg)
+{
+    g_warning("%s", msg);
+}
+
 /* vi:set tabstop=8 shiftwidth=4: */
