@@ -1,5 +1,5 @@
 /*
-*   $Id: fortran.c 3527 2009-01-29 17:24:31Z eht16 $
+*   $Id: fortran.c 5555 2011-03-05 22:40:50Z colombanw $
 *
 *   Copyright (c) 1998-2003, Darren Hiebert
 *
@@ -19,6 +19,7 @@
 #include <limits.h>
 #include <ctype.h>	/* to define tolower () */
 #include <setjmp.h>
+#include <mio/mio.h>
 
 #include "entry.h"
 #include "keyword.h"
@@ -83,6 +84,7 @@ typedef enum eKeywordId {
 	KEYWORD_end,
 	KEYWORD_entry,
 	KEYWORD_equivalence,
+	KEYWORD_extends,
 	KEYWORD_external,
 	KEYWORD_format,
 	KEYWORD_function,
@@ -182,8 +184,7 @@ typedef struct sTokenInfo {
 	vString* string;
 	struct sTokenInfo *secondary;
 	unsigned long lineNumber;
-	fpos_t filePosition;
-	int bufferPosition;	/* buffer position of line containing name */
+	MIOPos filePosition;
 } tokenInfo;
 
 /*
@@ -248,6 +249,7 @@ static const keywordDesc FortranKeywordTable [] = {
 	{ "end",            KEYWORD_end          },
 	{ "entry",          KEYWORD_entry        },
 	{ "equivalence",    KEYWORD_equivalence  },
+	{ "extends",        KEYWORD_extends      },
 	{ "external",       KEYWORD_external     },
 	{ "format",         KEYWORD_format       },
 	{ "function",       KEYWORD_function     },
@@ -421,10 +423,7 @@ static tokenInfo *newToken (void)
 	token->string       = vStringNew ();
 	token->secondary    = NULL;
 	token->lineNumber   = getSourceLineNumber ();
-	if (useFile())
-	    token->filePosition = getInputFilePosition ();
-	else
-	    token->bufferPosition = getInputBufferPosition ();
+	token->filePosition = getInputFilePosition ();
 
 	return token;
 }
@@ -478,10 +477,7 @@ static void makeFortranTag (tokenInfo *const token, tagType tag)
 			e.lineNumberEntry = (boolean) (Option.locate != EX_PATTERN);
 
 		e.lineNumber	= token->lineNumber;
-		if (useFile())
-		    e.filePosition   = token->filePosition;
-		else
-		    e.bufferPosition = token->bufferPosition;
+		e.filePosition	= token->filePosition;
 		e.isFileScope	= isFileScope (token->tag);
 		e.kindName		= FortranKinds [token->tag].name;
 		e.kind			= FortranKinds [token->tag].letter;
@@ -957,10 +953,7 @@ getNextChar:
 	c = getChar ();
 
 	token->lineNumber	= getSourceLineNumber ();
-	if (useFile())
-	    token->filePosition = getInputFilePosition ();
-	else
-	    token->bufferPosition = getInputBufferPosition ();
+	token->filePosition = getInputFilePosition ();
 
 	switch (c)
 	{
@@ -1271,6 +1264,7 @@ static boolean skipStatementIfKeyword (tokenInfo *const token, keywordId keyword
  *  component-attr-spec
  *      is POINTER
  *      or DIMENSION ( component-array-spec )
+ *      or EXTENDS ( type name )
  */
 static void parseQualifierSpecList (tokenInfo *const token)
 {
@@ -1293,6 +1287,7 @@ static void parseQualifierSpecList (tokenInfo *const token)
 				break;
 
 			case KEYWORD_dimension:
+			case KEYWORD_extends:
 			case KEYWORD_intent:
 				readToken (token);
 				skipOverParens (token);
@@ -1615,6 +1610,7 @@ static boolean parseSpecificationStmt (tokenInfo *const token)
 		case KEYWORD_data:
 		case KEYWORD_dimension:
 		case KEYWORD_equivalence:
+		case KEYWORD_extends:
 		case KEYWORD_external:
 		case KEYWORD_intent:
 		case KEYWORD_intrinsic:
