@@ -1,5 +1,5 @@
 /*
-*   $Id: get.c 5295 2010-10-15 12:17:22Z ntrel $
+*   $Id: get.c 5856 2011-06-17 22:52:43Z colombanw $
 *
 *   Copyright (c) 1996-2002, Darren Hiebert
 *
@@ -314,14 +314,8 @@ static void makeDefineTag (const char *const name, boolean parameterized)
 		e.kind         = 'd';
 		if (parameterized)
 		{
-			if (useFile()) {
-				e.extensionFields.arglist = getArglistFromFilePos(getInputFilePosition()
-		  			, e.name);
-			}
-			else {
-				e.extensionFields.arglist = getArglistFromBufferPos(getInputBufferPosition()
-		  			, e.name);
-			}
+			e.extensionFields.arglist = getArglistFromFilePos(getInputFilePosition()
+					, e.name);
 		}
 		makeTagEntry (&e);
 		if (parameterized)
@@ -500,7 +494,7 @@ static int skipOverCplusComment (void)
 /* Skips over a D style comment.
  * Really we should match nested /+ comments. At least they're less common.
  */
-int skipOverDComment (void)
+static int skipOverDComment (void)
 {
 	int c = fileGetc ();
 
@@ -715,63 +709,30 @@ process:
 	return c;
 }
 
-extern char *getArglistFromBufferPos(int startPosition, const char *tokenName)
+extern char *getArglistFromFilePos(MIOPos startPosition, const char *tokenName)
 {
-	int bufferOriginalPosition;
+	MIOPos originalPosition;
 	char *result = NULL;
 	char *arglist = NULL;
 	long pos1, pos2;
 
-	/* FIXME startPosition as well as getBufPos() are mostly wrong here */
-	pos2 = getBufPos();
+	pos2 = mio_tell(File.mio);
 
-	if (!useFile()) {
-		bufferOriginalPosition = getBufPos ();
-		setBufPos(startPosition);
-		pos1 = File.fpBufferPosition;
-	}
-	else
-		return NULL;
+	mio_getpos(File.mio, &originalPosition);
+	mio_setpos(File.mio, &startPosition);
+	pos1 = mio_tell(File.mio);
 
 	if (pos2 > pos1)
 	{
 		result = (char *) g_malloc(sizeof(char ) * (pos2 - pos1 + 2));
-		if (result != NULL)
-		{
-			memcpy(result, &File.fpBuffer[getBufPos()], pos2 - pos1 + 1);
-			result[pos2-pos1+1] = '\0';
-			arglist = getArglistFromStr(result, tokenName);
-			g_free(result);
-		}
-	}
-	setBufPos (bufferOriginalPosition);
-	return arglist;
-}
-
-extern char *getArglistFromFilePos(fpos_t startPosition, const char *tokenName)
-{
-	fpos_t originalPosition;
-	char *result = NULL;
-	char *arglist = NULL;
-	long pos1, pos2;
-
-	pos2 = ftell(File.fp);
-
-	fgetpos(File.fp, &originalPosition);
-	fsetpos(File.fp, &startPosition);
-	pos1 = ftell(File.fp);
-
-	if (pos2 > pos1)
-	{
-		result = (char *) g_malloc(sizeof(char ) * (pos2 - pos1 + 2));
-		if (result != NULL && fread(result, sizeof(char), pos2 - pos1 + 1, File.fp) > 0)
+		if (result != NULL && mio_read(File.mio, result, sizeof(char), pos2 - pos1 + 1) > 0)
 		{
 			result[pos2-pos1+1] = '\0';
 			arglist = getArglistFromStr(result, tokenName);
 		}
 		g_free(result);
 	}
-	fsetpos(File.fp, &originalPosition);
+	mio_setpos(File.mio, &originalPosition);
 	return arglist;
 }
 
