@@ -1,5 +1,5 @@
 /*
-*   $Id: get.h 3352 2008-12-09 19:00:45Z eht16 $
+*   $Id: get.h 5555 2011-03-05 22:40:50Z colombanw $
 *
 *   Copyright (c) 1998-2002, Darren Hiebert
 *
@@ -45,8 +45,7 @@ extern void cppEndStatement (void);
 extern void cppUngetc (const int c);
 extern int cppGetc (void);
 extern int skipOverCComment (void);
-extern char *getArglistFromFilePos(fpos_t startPosition, const char *tokenName);
-extern char *getArglistFromBufferPos(int startPosition, const char *tokenName);
+extern char *getArglistFromFilePos(MIOPos startPosition, const char *tokenName);
 extern char *getArglistFromStr(char *buf, const char *name);
 
 #endif  /* _GET_H */
