@@ -1,5 +1,5 @@
 /*
- *	 $Id: js.c 2504 2008-04-19 07:56:56Z eht16 $
+ *	 $Id: js.c 5555 2011-03-05 22:40:50Z colombanw $
  *
  *	 Copyright (c) 2003, Darren Hiebert
  *
@@ -21,6 +21,7 @@
 #include "general.h"	/* must always come first */
 #include <ctype.h>	/* to define isalpha () */
 #include <setjmp.h>
+#include <mio/mio.h>
 #ifdef DEBUG
 #include <stdio.h>
 #endif
@@ -107,7 +108,7 @@ typedef struct sTokenInfo {
 	vString *		string;
 	vString *		scope;
 	unsigned long 	lineNumber;
-	fpos_t 			filePosition;
+	MIOPos			filePosition;
 	int				nestLevel;
 	boolean			ignoreTag;
 } tokenInfo;
