@@ -131,8 +131,7 @@ extern const char *fileExtension (const char *const fileName)
     const char *extension;
     const char *pDelimiter = NULL;
 
-    if (pDelimiter == NULL)
-        pDelimiter = strrchr (fileName, '.');
+    pDelimiter = strrchr (fileName, '.');
 
     if (pDelimiter == NULL)
 	extension = "";
@@ -151,7 +150,7 @@ extern boolean isIncludeFile (const char *const fileName)
     return FALSE;
 }
 
-/* tags_ignore is a NULL-terminated array of strings, read from ~/.geany/ignore.tags.
+/* tags_ignore is a NULL-terminated array of strings, read from ~/.config/geany/ignore.tags.
  * This file contains a space or newline separated list of symbols which should be ignored
  * by the C/C++ parser, see -I command line option of ctags for details. */
 gchar **c_tags_ignore = NULL;
