@@ -1,5 +1,5 @@
 /*
-*   $Id: php.c 5417 2010-11-18 18:20:12Z ntrel $
+*   $Id: php.c 5856 2011-06-17 22:52:43Z colombanw $
 *
 *   Copyright (c) 2000, Jesus Castagnetto <jmcastagnetto@zkey.com>
 *
@@ -102,7 +102,7 @@ static void function_cb(const char *line, const regexMatch *matches, unsigned in
 {
 	char *name, *arglist;
 	char kind = 'f';
-	static char *kindName = "function";
+	static const char *kindName = "function";
 	tagEntryInfo e;
 	const regexMatch *match_funcname = NULL;
 	const regexMatch *match_arglist = NULL;
@@ -127,6 +127,9 @@ static void function_cb(const char *line, const regexMatch *matches, unsigned in
 		e.kindName = kindName;
 		e.extensionFields.arglist = arglist;
 		makeTagEntry (&e);
+
+		eFree(name);
+		eFree(arglist);
 	}
 }
 
