@@ -1,5 +1,5 @@
 /*
-*   $Id: python.c 5224 2010-09-13 15:03:18Z ntrel $
+*   $Id: python.c 5851 2011-06-15 21:01:23Z eht16 $
 *
 *   Copyright (c) 2000-2003, Darren Hiebert
 *
@@ -195,9 +195,30 @@ static const char *skipString (const char *cp)
 /* Skip everything up to an identifier start. */
 static const char *skipEverything (const char *cp)
 {
+	int match;
 	for (; *cp; cp++)
 	{
+		match = 0;
 		if (*cp == '"' || *cp == '\'' || *cp == '#')
+			match = 1;
+
+		/* these checks find unicode, binary (Python 3) and raw strings */
+		if (!match && (
+			!strncasecmp(cp, "u'", 2) || !strncasecmp(cp, "u\"", 2) ||
+			!strncasecmp(cp, "r'", 2) || !strncasecmp(cp, "r\"", 2) ||
+			!strncasecmp(cp, "b'", 2) || !strncasecmp(cp, "b\"", 2)))
+		{
+			match = 1;
+			cp += 1;
+		}
+		if (!match && (
+			!strncasecmp(cp, "ur'", 3) || !strncasecmp(cp, "ur\"", 3) ||
+			!strncasecmp(cp, "br'", 3) || !strncasecmp(cp, "br\"", 3)))
+		{
+			match = 1;
+			cp += 2;
+		}
+		if (match)
 		{
 			cp = skipString(cp);
 			if (!*cp) break;
@@ -457,6 +478,9 @@ static char const *find_triple_start(char const *string, char const **which)
 
 	for (; *cp; cp++)
 	{
+		if (*cp == '#')
+			break;
+
 		if (*cp == '"' || *cp == '\'')
 		{
 			if (strncmp(cp, doubletriple, 3) == 0)
@@ -653,14 +677,14 @@ static void findPythonTags (void)
 		indent = cp - line;
 		line_skip = 0;
 
-		checkParent(nesting_levels, indent, parent);
-
 		/* Deal with multiline string ending. */
 		if (longStringLiteral)
 		{
 			find_triple_end(cp, &longStringLiteral);
 			continue;
 		}
+		
+		checkParent(nesting_levels, indent, parent);
 
 		/* Deal with multiline string start. */
 		longstring = find_triple_start(cp, &longStringLiteral);
