@@ -1,5 +1,5 @@
 /*
-*   $Id: sql.c 2287 2008-02-27 13:17:29Z eht16 $
+*   $Id: sql.c 5856 2011-06-17 22:52:43Z colombanw $
 *
 *   Copyright (c) 2002-2003, Darren Hiebert
 *
@@ -137,7 +137,7 @@ typedef struct sTokenInfo {
     vString *	string;
     vString *	scope;
     unsigned long lineNumber;
-    fpos_t filePosition;
+    MIOPos filePosition;
 } tokenInfo;
 
 /*
@@ -182,18 +182,18 @@ static kindOption SqlKinds [] = {
     { FALSE, 'l', "local",        "local variables"        },
     { TRUE,  'L', "label",        "block label"            },
     { TRUE,  'P', "package",      "packages"	           },
-    { TRUE,  'p', "procedure",    "procedures"  	   },
+    { TRUE,  'n', "namespace",    "procedures"  	   },
     { FALSE, 'r', "record",       "records"     	   },
     { TRUE,  's', "subtype",      "subtypes"    	   },
-    { TRUE,  't', "table",        "tables"		   },
-    { TRUE,  'T', "trigger",      "triggers"	           },
+    { TRUE,  't', "class",        "tables"		   },
+    { TRUE,  'T', "macro",        "triggers"	           },
     { TRUE,  'v', "variable",     "variables"   	   },
-    { TRUE,  'i', "index",        "indexes"     	   },
+    { TRUE,  'i', "struct",       "indexes"     	   },
     { TRUE,  'e', "event",        "events"	           },
     { TRUE,  'U', "publication",  "publications"           },
     { TRUE,  'R', "service",      "services"               },
     { TRUE,  'D', "domain",       "domains"                },
-    { TRUE,  'V', "view",         "views"                  },
+    { TRUE,  'm', "member",       "views"                  },
     { TRUE,  'n', "synonym",      "synonyms"               },
     { TRUE,  'x', "mltable",      "MobiLink Table Scripts" },
     { TRUE,  'y', "mlconn",       "MobiLink Conn Scripts"  }
@@ -267,37 +267,37 @@ static void makeConstTag (tokenInfo *const token, const sqlKind kind);
 *   DEBUG function
 */
 
-static void dispToken (tokenInfo *const token, char * location)
+static void dispToken (tokenInfo *const token, const char * location)
 {
 #ifdef DEBUG
     if ( isKeyword(token, KEYWORD_NONE) )
     {
         if ( isType(token, TOKEN_IDENTIFIER) || isType(token, TOKEN_STRING)  )
         {
-            printf( "\n%s: token string  t:%s  s:%s  l:%lu  p:%ld\n"
+            printf( "\n%s: token string  t:%s  s:%s  l:%lu  p:%d\n"
                     , location
                     , vStringValue(token->string)
                     , vStringValue(token->scope)
                     , token->lineNumber
-                    , token->filePosition
+                    , token->bufferPosition
                   );
         } else {
-            printf( "\n%s: token  t:%d  s:%s  l:%lu  p:%lu\n"
+            printf( "\n%s: token  t:%d  s:%s  l:%lu  p:%d\n"
                     , location
                     , token->type
                     , vStringValue(token->scope)
                     , token->lineNumber
-                    , token->filePosition
+                    , token->bufferPosition
                   );
         }
     } else {
-        printf( "\n%s: keyword:%s k:%d  s:%s  l:%lu  p:%ld\n"
+        printf( "\n%s: keyword:%s k:%d  s:%s  l:%lu  p:%d\n"
                 , location
                 , vStringValue(token->string)
                 , token->keyword
                 , vStringValue(token->scope)
                 , token->lineNumber
-                , token->filePosition
+                , token->bufferPosition
               );
     }
 #endif
@@ -491,7 +491,7 @@ getNextChar:
 	    token->type = TOKEN_STRING;
 	    parseString (token->string, c);
             token->lineNumber = getSourceLineNumber ();
-            token->filePosition = getInputFilePosition ();
+	    token->filePosition	= getInputFilePosition ();
 	    break;
 
 	case '-':
