@@ -9,6 +9,7 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <glib-object.h>
 
 #include "general.h"
 #include "entry.h"
@@ -18,26 +19,9 @@
 #include "tm_tag.h"
 
 
-#if GLIB_CHECK_VERSION (2, 10, 0)
-/* Use GSlices if present */
-
 #define TAG_NEW(T)	((T) = g_slice_new0(TMTag))
 #define TAG_FREE(T)	g_slice_free(TMTag, (T))
 
-#else /* GLib < 2.10 */
-
-static GMemChunk *s_tag_mem_chunk = NULL;
-
-#define TAG_NEW(T) {\
-	if (!s_tag_mem_chunk) \
-		s_tag_mem_chunk = g_mem_chunk_new("TMTag memChunk", sizeof(TMTag), 1024 \
-		  , G_ALLOC_AND_FREE); \
-	(T) = g_chunk_new0(TMTag, s_tag_mem_chunk);}
-
-#define TAG_FREE(T) g_mem_chunk_free(s_tag_mem_chunk, (T))
-
-#endif /* GLib version check */
-
 
 /* Note: To preserve binary compatibility, it is very important
 	that you only *append* to this list ! */
@@ -105,6 +89,17 @@ static int s_tag_types[] = {
 	tm_tag_other_t
 };
 
+GType tm_tag_get_type(void)
+{
+	static GType gtype = 0;
+	if (G_UNLIKELY (gtype == 0))
+	{
+		gtype = g_boxed_type_register_static("TMTag", (GBoxedCopyFunc)tm_tag_ref,
+											 (GBoxedFreeFunc)tm_tag_unref);
+	}
+	return gtype;
+}
+
 static int get_tag_type(const char *tag_name)
 {
 	unsigned int i;
@@ -129,6 +124,7 @@ static int get_tag_type(const char *tag_name)
 
 gboolean tm_tag_init(TMTag *tag, TMSourceFile *file, const tagEntryInfo *tag_entry)
 {
+	tag->refcount = 1;
 	if (NULL == tag_entry)
 	{
 		/* This is a file tag */
@@ -224,6 +220,7 @@ gboolean tm_tag_init_from_file(TMTag *tag, TMSourceFile *file, FILE *fp)
 	gboolean status;
 	guchar changed_char = TA_NAME;
 
+	tag->refcount = 1;
 	if ((NULL == fgets((gchar*)buf, BUFSIZ, fp)) || ('\0' == *buf))
 		return FALSE;
 	for (start = end = buf, status = TRUE; (TRUE == status); start = end, ++ end)
@@ -327,6 +324,7 @@ gboolean tm_tag_init_from_file_alt(TMTag *tag, TMSourceFile *file, FILE *fp)
 	gboolean status;
 	/*guchar changed_char = TA_NAME;*/
 
+	tag->refcount = 1;
 	if ((NULL == fgets((gchar*)buf, BUFSIZ, fp)) || ('\0' == *buf))
 		return FALSE;
 	{
@@ -436,15 +434,30 @@ static void tm_tag_destroy(TMTag *tag)
 	}
 }
 
+#if 0
 void tm_tag_free(gpointer tag)
 {
-	if (NULL != tag)
+	tm_tag_unref(tag);
+}
+#endif
+
+void tm_tag_unref(TMTag *tag)
+{
+	/* be NULL-proof because tm_tag_free() was NULL-proof and we indent to be a
+	 * drop-in replacment of it */
+	if (NULL != tag && g_atomic_int_dec_and_test(&tag->refcount))
 	{
-		tm_tag_destroy((TMTag *) tag);
+		tm_tag_destroy(tag);
 		TAG_FREE(tag);
 	}
 }
 
+TMTag *tm_tag_ref(TMTag *tag)
+{
+	g_atomic_int_inc(&tag->refcount);
+	return tag;
+}
+
 int tm_tag_compare(const void *ptr1, const void *ptr2)
 {
 	unsigned int *sort_attr;
@@ -603,7 +616,7 @@ void tm_tags_array_free(GPtrArray *tags_array, gboolean free_all)
 	{
 		guint i;
 		for (i = 0; i < tags_array->len; ++i)
-			tm_tag_free(tags_array->pdata[i]);
+			tm_tag_unref(tags_array->pdata[i]);
 		if (free_all)
 			g_ptr_array_free(tags_array, TRUE);
 		else
