@@ -82,7 +82,7 @@ void tm_workspace_free(gpointer workspace)
 		if (theWorkspace->global_tags)
 		{
 			for (i=0; i < theWorkspace->global_tags->len; ++i)
-				tm_tag_free(theWorkspace->global_tags->pdata[i]);
+				tm_tag_unref(theWorkspace->global_tags->pdata[i]);
 			g_ptr_array_free(theWorkspace->global_tags, TRUE);
 		}
 		tm_work_object_destroy(TM_WORK_OBJECT(theWorkspace));
@@ -223,11 +223,10 @@ static void write_includes_file(FILE *fp, GList *includes_files)
 	{
 		char *str = g_strdup_printf("#include \"%s\"\n", (char*)node->data);
 		int str_len = strlen(str);
-		size_t size;
 
-		size = fwrite(str, str_len, 1, fp);
+		fwrite(str, str_len, 1, fp);
 		g_free(str);
-		node = g_list_next (node);
+		node = g_list_next(node);
 	}
 }
 
@@ -251,17 +250,30 @@ static void append_to_temp_file(FILE *fp, GList *file_list)
 		}
 		else
 		{
-			size_t size;
-			size = fwrite(contents, length, 1, fp);
-			size = fwrite("\n", 1, 1, fp);	/* in case file doesn't end in newline (e.g. windows). */
+			fwrite(contents, length, 1, fp);
+			fwrite("\n", 1, 1, fp);	/* in case file doesn't end in newline (e.g. windows). */
 			g_free(contents);
 		}
 		node = g_list_next (node);
 	}
 }
 
-gboolean tm_workspace_create_global_tags(const char *config_dir, const char *pre_process,
-	const char **includes, int includes_count, const char *tags_file, int lang)
+static gchar *create_temp_file(const gchar *tpl)
+{
+	gchar *name;
+	gint fd;
+
+	fd = g_file_open_tmp(tpl, &name, NULL);
+	if (fd < 0)
+		name = NULL;
+	else
+		close(fd);
+
+	return name;
+}
+
+gboolean tm_workspace_create_global_tags(const char *pre_process, const char **includes,
+	int includes_count, const char *tags_file, int lang)
 {
 #ifdef HAVE_GLOB_H
 	glob_t globbuf;
@@ -275,15 +287,11 @@ gboolean tm_workspace_create_global_tags(const char *config_dir, const char *pre
 	GPtrArray *tags_array;
 	GHashTable *includes_files_hash;
 	GList *includes_files = NULL;
-#ifdef G_OS_WIN32
-	char *temp_file = g_strdup_printf("%s\\_%d_%ld_1.cpp", config_dir, getpid(), time(NULL));
-	char *temp_file2 = g_strdup_printf("%s\\_%d_%ld_2.cpp", config_dir, getpid(), time(NULL));
-#else
-	char *temp_file = g_strdup_printf("%s/%d_%ld_1.cpp", config_dir, getpid(), time(NULL));
-	char *temp_file2 = g_strdup_printf("%s/%d_%ld_2.cpp", config_dir, getpid(), time(NULL));
-#endif
+	gchar *temp_file = create_temp_file("tmp_XXXXXX.cpp");
+	gchar *temp_file2 = create_temp_file("tmp_XXXXXX.cpp");
 
-	if (NULL == theWorkspace || NULL == (fp = g_fopen(temp_file, "w")))
+	if (NULL == temp_file || NULL == temp_file2 ||
+		NULL == theWorkspace || NULL == (fp = g_fopen(temp_file, "w")))
 	{
 		g_free(temp_file);
 		g_free(temp_file2);
@@ -376,13 +384,12 @@ gboolean tm_workspace_create_global_tags(const char *config_dir, const char *pre
 	 */
 	if (pre_process != NULL)
 	{
-		int ret;
 		command = g_strdup_printf("%s %s | grep -v -E '^\\s*(G_BEGIN_DECLS|G_END_DECLS)\\s*$' > %s",
 							  pre_process, temp_file, temp_file2);
 #ifdef TM_DEBUG
 		g_message("Executing: %s", command);
 #endif
-		ret = system(command);
+		system(command);
 		g_free(command);
 		g_unlink(temp_file);
 		g_free(temp_file);
@@ -390,6 +397,7 @@ gboolean tm_workspace_create_global_tags(const char *config_dir, const char *pre
 	else
 	{
 		/* no pre-processing needed, so temp_file2 = temp_file */
+		g_unlink(temp_file2);
 		g_free(temp_file2);
 		temp_file2 = temp_file;
 		temp_file = NULL;
